<?php

namespace Importers;

/**
 * Class that imports base database.
 *
 * @copyright YetiForce Sp. z o.o.
 * @license   YetiForce Public License 3.0 (licenses/LicenseEN.txt or yetiforce.com)
 * @author    Mariusz Krzaczkowski <m.krzaczkowski@yetiforce.com>
 */
class Base2 extends \App\Db\Importers\Base
{
	public $dbType = 'base';

	public function scheme()
	{
		$this->tables = [
			'vtiger_account' => [
				'columns' => [
					'accountid' => $this->integer(10)->notNull()->defaultValue(0),
					'account_no' => $this->stringType(100)->notNull(),
					'accountname' => $this->stringType(100)->notNull(),
					'parentid' => $this->integer(10)->defaultValue(0),
					'account_type' => $this->stringType(200),
					'industry' => $this->stringType(200),
					'annualrevenue' => $this->decimal('25,8'),
					'ownership' => $this->stringType(50),
					'siccode' => $this->stringType(),
					'phone' => $this->stringType(30),
					'otherphone' => $this->stringType(30),
					'email1' => $this->stringType(100),
					'email2' => $this->stringType(100),
					'website' => $this->stringType(100),
					'fax' => $this->stringType(30),
					'employees' => $this->integer(10)->defaultValue(0),
					'emailoptout' => $this->smallInteger(1)->defaultValue(0),
					'isconvertedfromlead' => $this->smallInteger(3)->defaultValue(0),
					'vat_id' => $this->stringType(30),
					'registration_number_1' => $this->stringType(30),
					'registration_number_2' => $this->stringType(30),
					'verification' => $this->text(),
					'no_approval' => $this->smallInteger(1)->defaultValue(0),
					'balance' => $this->decimal('25,8'),
					'payment_balance' => $this->decimal('25,8'),
					'legal_form' => $this->stringType(),
					'sum_time' => $this->decimal('10,2'),
					'inventorybalance' => $this->decimal('25,8')->defaultValue(0),
					'discount' => $this->decimal('5,2')->defaultValue(0),
					'creditlimit' => $this->integer(10),
					'products' => $this->text(),
					'services' => $this->text(),
					'last_invoice_date' => $this->date(),
					'active' => $this->smallInteger(1)->defaultValue(0),
					'accounts_status' => $this->stringType(),
					'phone_extra' => $this->stringType(100),
					'fax_extra' => $this->stringType(100),
					'otherphone_extra' => $this->stringType(100),
				],
				'columns_mysql' => [
					'active' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['account_account_type_idx', 'account_type'],
					['email_idx', ['email1', 'email2']],
					['accountname', 'accountname'],
					['parentid', 'parentid'],
				],
				'primaryKeys' => [
					['account_pk', 'accountid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_accountaddress' => [
				'columns' => [
					'accountaddressid' => $this->integer(10)->notNull(),
					'addresslevel1a' => $this->stringType(),
					'addresslevel1b' => $this->stringType(),
					'addresslevel1c' => $this->stringType(),
					'addresslevel2a' => $this->stringType(),
					'addresslevel2b' => $this->stringType(),
					'addresslevel2c' => $this->stringType(),
					'addresslevel3a' => $this->stringType(),
					'addresslevel3b' => $this->stringType(),
					'addresslevel3c' => $this->stringType(),
					'addresslevel4a' => $this->stringType(),
					'addresslevel4b' => $this->stringType(),
					'addresslevel4c' => $this->stringType(),
					'addresslevel5a' => $this->stringType(),
					'addresslevel5b' => $this->stringType(),
					'addresslevel5c' => $this->stringType(),
					'addresslevel6a' => $this->stringType(),
					'addresslevel6b' => $this->stringType(),
					'addresslevel6c' => $this->stringType(),
					'addresslevel7a' => $this->stringType(),
					'addresslevel7b' => $this->stringType(),
					'addresslevel7c' => $this->stringType(),
					'addresslevel8a' => $this->stringType(),
					'addresslevel8b' => $this->stringType(),
					'addresslevel8c' => $this->stringType(),
					'buildingnumbera' => $this->stringType(100),
					'localnumbera' => $this->stringType(100),
					'buildingnumberb' => $this->stringType(100),
					'localnumberb' => $this->stringType(100),
					'buildingnumberc' => $this->stringType(100),
					'localnumberc' => $this->stringType(100),
					'poboxa' => $this->stringType(50),
					'poboxb' => $this->stringType(50),
					'poboxc' => $this->stringType(50),
				],
				'primaryKeys' => [
					['accountaddress_pk', 'accountaddressid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_accounts_status' => [
				'columns' => [
					'accounts_statusid' => $this->primaryKey(10),
					'accounts_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_accountscf' => [
				'columns' => [
					'accountid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['accountscf_pk', 'accountid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_accounttype' => [
				'columns' => [
					'accounttypeid' => $this->primaryKey(10),
					'accounttype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['accounttype_accounttype_idx', 'accounttype', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_accounttype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_actionmapping' => [
				'columns' => [
					'actionid' => $this->smallInteger(5)->unsigned()->notNull(),
					'actionname' => $this->stringType(200)->notNull(),
					'securitycheck' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'securitycheck' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'index' => [
					['actionname', 'actionname'],
				],
				'primaryKeys' => [
					['actionmapping_pk', ['actionid', 'actionname']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity' => [
				'columns' => [
					'activityid' => $this->integer(10)->notNull()->defaultValue(0),
					'subject' => $this->stringType(100)->notNull(),
					'activitytype' => $this->stringType(200)->notNull(),
					'date_start' => $this->date()->notNull(),
					'due_date' => $this->date(),
					'time_start' => $this->time(),
					'time_end' => $this->time(),
					'sendnotification' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'duration_hours' => $this->smallInteger(5),
					'duration_minutes' => $this->stringType(3),
					'status' => $this->stringType(200),
					'priority' => $this->stringType(200),
					'location' => $this->stringType(150),
					'notime' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'visibility' => $this->stringType(50)->notNull()->defaultValue('all'),
					'deleted' => $this->smallInteger(1)->defaultValue(0),
					'smownerid' => $this->smallInteger(5)->unsigned(),
					'allday' => $this->smallInteger(1),
					'dav_status' => $this->smallInteger(1)->defaultValue(1),
					'state' => $this->stringType(),
					'link' => $this->integer(10),
					'process' => $this->integer(10),
					'subprocess' => $this->integer(10),
					'followup' => $this->integer(10),
					'reapeat' => $this->smallInteger(1),
					'recurrence' => $this->text(),
					'linkextend' => $this->integer(10),
				],
				'columns_mysql' => [
					'deleted' => $this->tinyInteger(1)->defaultValue(0),
					'allday' => $this->tinyInteger(1),
					'dav_status' => $this->tinyInteger(1)->defaultValue(1),
				],
				'index' => [
					['activity_activityid_subject_idx', ['activityid', 'subject']],
					['activity_activitytype_date_start_idx', ['activitytype', 'date_start']],
					['activity_date_start_due_date_idx', ['date_start', 'due_date']],
					['activity_date_start_time_start_idx', ['date_start', 'time_start']],
					['activity_status_idx', 'status'],
					['activitytype_2', ['activitytype', 'date_start', 'due_date', 'time_start', 'time_end', 'deleted', 'smownerid']],
					['link', 'link'],
					['process', 'process'],
					['followup', 'followup'],
					['subprocess', 'subprocess'],
					['activitytype_3', ['activitytype', 'status']],
					['smownerid', 'smownerid'],
					['linkextend', 'linkextend'],
				],
				'primaryKeys' => [
					['activity_pk', 'activityid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity_reminder' => [
				'columns' => [
					'activity_id' => $this->integer(10)->notNull(),
					'reminder_time' => $this->integer(10)->notNull(),
					'reminder_sent' => $this->integer(2)->notNull(),
				],
				'primaryKeys' => [
					['activity_reminder_pk', 'activity_id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity_reminder_popup' => [
				'columns' => [
					'reminderid' => $this->primaryKey(10),
					'recordid' => $this->integer(10)->notNull(),
					'datetime' => $this->dateTime()->notNull(),
					'status' => $this->integer(2)->notNull(),
				],
				'index' => [
					['recordid', 'recordid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity_update_dates' => [
				'columns' => [
					'activityid' => $this->integer(10)->notNull(),
					'parent' => $this->integer(10)->notNull(),
					'task_id' => $this->integer(10)->notNull(),
				],
				'index' => [
					['parent', 'parent'],
					['vtiger_activity_update_dates_ibfk_1', 'task_id'],
				],
				'primaryKeys' => [
					['activity_update_dates_pk', 'activityid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity_view' => [
				'columns' => [
					'activity_viewid' => $this->primaryKey(10),
					'activity_view' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activity_view_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activitycf' => [
				'columns' => [
					'activityid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['activitycf_pk', 'activityid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activityproductrel' => [
				'columns' => [
					'activityid' => $this->integer(10)->notNull()->defaultValue(0),
					'productid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['activityproductrel_activityid_idx', 'activityid'],
					['activityproductrel_productid_idx', 'productid'],
				],
				'primaryKeys' => [
					['activityproductrel_pk', ['activityid', 'productid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activityregister_status' => [
				'columns' => [
					'activityregister_statusid' => $this->primaryKey(),
					'activityregister_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activitystatus' => [
				'columns' => [
					'activitystatusid' => $this->primaryKey(10),
					'activitystatus' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activitystatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activitytype' => [
				'columns' => [
					'activitytypeid' => $this->primaryKey(10),
					'activitytype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
					'color' => $this->stringType(25),
				],
				'index' => [
					['activitytype_activitytype_idx', 'activitytype', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_activitytype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_announcementstatus' => [
				'columns' => [
					'announcementstatusid' => $this->primaryKey(10),
					'announcementstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_assets' => [
				'columns' => [
					'assetsid' => $this->integer(10)->notNull(),
					'asset_no' => $this->stringType(30)->notNull(),
					'product' => $this->integer(10)->notNull(),
					'serialnumber' => $this->stringType(200),
					'datesold' => $this->date(),
					'dateinservice' => $this->date(),
					'assetstatus' => $this->stringType(200)->defaultValue('PLL_DRAFT'),
					'assetname' => $this->stringType(100),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'parent_id' => $this->integer(10),
					'ordertime' => $this->decimal('10,2'),
					'pscategory' => $this->stringType()->defaultValue(''),
					'ssalesprocessesid' => $this->integer(10),
					'assets_renew' => $this->stringType(),
					'renewalinvoice' => $this->integer(10),
					'contactid' => $this->integer(19),
				],
				'index' => [
					['parent_id', 'parent_id'],
					['product', 'product'],
					['ssalesprocessesid', 'ssalesprocessesid'],
					['renewalinvoice', 'renewalinvoice'],
					['contactid_idx', 'contactid'],
				],
				'primaryKeys' => [
					['assets_pk', 'assetsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_assets_renew' => [
				'columns' => [
					'assets_renewid' => $this->primaryKey(10),
					'assets_renew' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_assetscf' => [
				'columns' => [
					'assetsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['assetscf_pk', 'assetsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_assetstatus' => [
				'columns' => [
					'assetstatusid' => $this->primaryKey(10),
					'assetstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_assetstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_asterisk' => [
				'columns' => [
					'server' => $this->stringType(30),
					'port' => $this->stringType(30),
					'username' => $this->stringType(50),
					'password' => $this->stringType(50),
					'version' => $this->stringType(50),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_asteriskextensions' => [
				'columns' => [
					'userid' => $this->smallInteger(5)->unsigned()->notNull(),
					'asterisk_extension' => $this->stringType(50),
					'use_asterisk' => $this->stringType(3),
				],
				'primaryKeys' => [
					['asteriskextensions_pk', 'userid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_asteriskincomingcalls' => [
				'columns' => [
					'from_number' => $this->stringType(50),
					'from_name' => $this->stringType(50),
					'to_number' => $this->stringType(50),
					'callertype' => $this->stringType(30),
					'flag' => $this->integer(10),
					'timer' => $this->integer(10),
					'refuid' => $this->stringType(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_asteriskincomingevents' => [
				'columns' => [
					'uid' => $this->stringType()->notNull(),
					'channel' => $this->stringType(100),
					'from_number' => $this->bigInteger(10),
					'from_name' => $this->stringType(100),
					'to_number' => $this->bigInteger(10),
					'callertype' => $this->stringType(100),
					'timer' => $this->integer(10),
					'flag' => $this->stringType(3),
					'pbxrecordid' => $this->integer(10),
					'relcrmid' => $this->integer(10),
				],
				'primaryKeys' => [
					['asteriskincomingevents_pk', 'uid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_attachments' => [
				'columns' => [
					'attachmentsid' => $this->integer(10)->notNull(),
					'name' => $this->stringType()->notNull(),
					'description' => $this->text(),
					'type' => $this->stringType(100),
					'path' => $this->text(),
					'subject' => $this->stringType(),
				],
				'primaryKeys' => [
					['attachments_pk', 'attachmentsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_audit_trial' => [
				'columns' => [
					'auditid' => $this->integer(10)->notNull(),
					'userid' => $this->integer(10),
					'module' => $this->stringType(),
					'action' => $this->stringType(),
					'recordid' => $this->stringType(20),
					'actiondate' => $this->dateTime(),
				],
				'primaryKeys' => [
					['audit_trial_pk', 'auditid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_auditregister_status' => [
				'columns' => [
					'auditregister_statusid' => $this->primaryKey(),
					'auditregister_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_auditregister_type' => [
				'columns' => [
					'auditregister_typeid' => $this->primaryKey(),
					'auditregister_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_authy_methods' => [
				'columns' => [
					'authy_methodsid' => $this->primaryKey(),
					'authy_methods' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_blocks' => [
				'columns' => [
					'blockid' => $this->primaryKey(10),
					'tabid' => $this->smallInteger(5)->notNull(),
					'blocklabel' => $this->stringType(100)->notNull(),
					'sequence' => $this->integer(10),
					'show_title' => $this->integer(2),
					'visible' => $this->integer(2)->notNull()->defaultValue(0),
					'create_view' => $this->integer(2)->notNull()->defaultValue(0),
					'edit_view' => $this->integer(2)->notNull()->defaultValue(0),
					'detail_view' => $this->integer(2)->notNull()->defaultValue(0),
					'display_status' => $this->integer(1)->notNull()->defaultValue(1),
					'iscustom' => $this->integer(1)->notNull()->defaultValue(0),
				],
				'index' => [
					['block_tabid_idx', 'tabid'],
					['block_sequence_idx', 'sequence'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_blocks_hide' => [
				'columns' => [
					'id' => $this->primaryKey(10)->unsigned(),
					'blockid' => $this->integer(10)->unsigned(),
					'conditions' => $this->text(),
					'enabled' => $this->smallInteger(1)->unsigned(),
					'view' => $this->stringType(100),
				],
				'columns_mysql' => [
					'enabled' => $this->tinyInteger(1)->unsigned(),
				],
				'index' => [
					['blockid', ['blockid', 'enabled']],
					['view', 'view'],
					['blockid_2', ['blockid', 'enabled', 'view']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_calendar_config' => [
				'columns' => [
					'type' => $this->stringType(10),
					'name' => $this->stringType(20),
					'label' => $this->stringType(20),
					'value' => $this->stringType(100),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callduration' => [
				'columns' => [
					'calldurationid' => $this->primaryKey(10),
					'callduration' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callduration_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callhistory' => [
				'columns' => [
					'callhistoryid' => $this->integer(10)->notNull(),
					'callhistorytype' => $this->stringType(),
					'from_number' => $this->stringType(30),
					'to_number' => $this->stringType(30),
					'location' => $this->stringType(200),
					'phonecallid' => $this->stringType(100),
					'duration' => $this->integer(10),
					'start_time' => $this->dateTime(),
					'end_time' => $this->dateTime(),
					'country' => $this->stringType(100),
					'imei' => $this->stringType(100),
					'ipaddress' => $this->stringType(100),
					'simserial' => $this->stringType(100),
					'subscriberid' => $this->stringType(100),
					'destination' => $this->integer(10),
					'source' => $this->integer(10),
					'from_number_extra' => $this->stringType(100),
					'to_number_extra' => $this->stringType(100),
				],
				'index' => [
					['source', 'source'],
					['destination', 'destination'],
				],
				'primaryKeys' => [
					['callhistory_pk', 'callhistoryid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callhistorycf' => [
				'columns' => [
					'callhistoryid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['callhistorycf_pk', 'callhistoryid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callhistorytype' => [
				'columns' => [
					'callhistorytypeid' => $this->primaryKey(10),
					'callhistorytype' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_callhistorytype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaign' => [
				'columns' => [
					'campaign_no' => $this->stringType(100)->notNull(),
					'campaignname' => $this->stringType(),
					'campaigntype' => $this->stringType(200),
					'campaignstatus' => $this->stringType(200),
					'expectedrevenue' => $this->decimal('25,8'),
					'budgetcost' => $this->decimal('25,8'),
					'actualcost' => $this->decimal('25,8'),
					'expectedresponse' => $this->stringType(200),
					'numsent' => $this->decimal('11,0'),
					'product_id' => $this->integer(10),
					'sponsor' => $this->stringType(),
					'targetaudience' => $this->stringType(),
					'targetsize' => $this->integer(10),
					'expectedresponsecount' => $this->integer(10),
					'expectedsalescount' => $this->integer(10),
					'expectedroi' => $this->decimal('25,8'),
					'actualresponsecount' => $this->integer(10),
					'actualsalescount' => $this->integer(10),
					'actualroi' => $this->decimal('25,8'),
					'campaignid' => $this->integer(10)->notNull(),
					'closingdate' => $this->date(),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
				],
				'index' => [
					['campaign_campaignstatus_idx', 'campaignstatus'],
					['campaign_campaignname_idx', 'campaignname'],
					['campaign_campaignid_idx', 'campaignid'],
				],
				'primaryKeys' => [
					['campaign_pk', 'campaignid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaign_records' => [
				'columns' => [
					'campaignid' => $this->integer(10)->notNull()->defaultValue(0),
					'crmid' => $this->integer(10)->notNull()->defaultValue(0),
					'campaignrelstatusid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['campaigncontrel_contractid_idx', 'crmid'],
				],
				'primaryKeys' => [
					['campaign_records_pk', ['campaignid', 'crmid', 'campaignrelstatusid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaignscf' => [
				'columns' => [
					'campaignid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['campaignscf_pk', 'campaignid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaignstatus' => [
				'columns' => [
					'campaignstatusid' => $this->primaryKey(10),
					'campaignstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['campaignstatus_campaignstatus_idx', 'campaignstatus'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaignstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaigntype' => [
				'columns' => [
					'campaigntypeid' => $this->primaryKey(10),
					'campaigntype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['campaigntype_campaigntype_idx', 'campaigntype', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_campaigntype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cmileage_logbook_status' => [
				'columns' => [
					'cmileage_logbook_statusid' => $this->primaryKey(10),
					'cmileage_logbook_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactaddress' => [
				'columns' => [
					'contactaddressid' => $this->integer(10)->notNull()->defaultValue(0),
					'addresslevel1a' => $this->stringType(),
					'addresslevel1b' => $this->stringType(),
					'addresslevel2a' => $this->stringType(),
					'addresslevel2b' => $this->stringType(),
					'addresslevel3a' => $this->stringType(),
					'addresslevel3b' => $this->stringType(),
					'addresslevel4a' => $this->stringType(),
					'addresslevel4b' => $this->stringType(),
					'addresslevel5a' => $this->stringType(),
					'addresslevel5b' => $this->stringType(),
					'addresslevel6a' => $this->stringType(),
					'addresslevel6b' => $this->stringType(),
					'addresslevel7a' => $this->stringType(),
					'addresslevel7b' => $this->stringType(),
					'addresslevel8a' => $this->stringType(),
					'addresslevel8b' => $this->stringType(),
					'buildingnumbera' => $this->stringType(100),
					'localnumbera' => $this->stringType(100),
					'buildingnumberb' => $this->stringType(100),
					'localnumberb' => $this->stringType(100),
					'poboxa' => $this->stringType(50),
					'poboxb' => $this->stringType(50),
				],
				'primaryKeys' => [
					['contactaddress_pk', 'contactaddressid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactdetails' => [
				'columns' => [
					'contactid' => $this->integer(10)->notNull()->defaultValue(0),
					'contact_no' => $this->stringType(100)->notNull(),
					'parentid' => $this->integer(10),
					'salutation' => $this->stringType(200),
					'firstname' => $this->stringType(40),
					'lastname' => $this->stringType(80)->notNull(),
					'email' => $this->stringType(100),
					'phone' => $this->stringType(50),
					'mobile' => $this->stringType(50),
					'reportsto' => $this->integer(),
					'training' => $this->stringType(50),
					'usertype' => $this->stringType(50),
					'contacttype' => $this->stringType(50),
					'otheremail' => $this->stringType(100),
					'donotcall' => $this->smallInteger(1),
					'emailoptout' => $this->smallInteger(1)->defaultValue(0),
					'imagename' => $this->text(),
					'isconvertedfromlead' => $this->smallInteger(1)->defaultValue(0),
					'verification' => $this->text(),
					'secondary_email' => $this->stringType(100)->defaultValue(''),
					'notifilanguage' => $this->stringType(100)->defaultValue(''),
					'contactstatus' => $this->stringType()->defaultValue(''),
					'dav_status' => $this->smallInteger(1)->defaultValue(1),
					'jobtitle' => $this->stringType(100)->defaultValue(''),
					'decision_maker' => $this->smallInteger(1)->defaultValue(0),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'active' => $this->smallInteger(1)->defaultValue(0),
					'phone_extra' => $this->stringType(100),
					'mobile_extra' => $this->stringType(100),
				],
				'columns_mysql' => [
					'dav_status' => $this->tinyInteger(1)->defaultValue(1),
					'decision_maker' => $this->tinyInteger(1)->defaultValue(0),
					'active' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['contactdetails_accountid_idx', 'parentid'],
					['email_idx', 'email'],
					['lastname', 'lastname'],
				],
				'primaryKeys' => [
					['contactdetails_pk', 'contactid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactscf' => [
				'columns' => [
					'contactid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['contactscf_pk', 'contactid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactstatus' => [
				'columns' => [
					'contactstatusid' => $this->primaryKey(10),
					'contactstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contactsubdetails' => [
				'columns' => [
					'contactsubscriptionid' => $this->integer(10)->notNull()->defaultValue(0),
					'birthday' => $this->date(),
					'laststayintouchrequest' => $this->integer(10)->defaultValue(0),
					'laststayintouchsavedate' => $this->integer(10)->defaultValue(0),
					'leadsource' => $this->stringType(200),
				],
				'primaryKeys' => [
					['contactsubdetails_pk', 'contactsubscriptionid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_priority' => [
				'columns' => [
					'contract_priorityid' => $this->primaryKey(10),
					'contract_priority' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_priority_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_status' => [
				'columns' => [
					'contract_statusid' => $this->primaryKey(10),
					'contract_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_type' => [
				'columns' => [
					'contract_typeid' => $this->primaryKey(10),
					'contract_type' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_contract_type_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_convertleadmapping' => [
				'columns' => [
					'cfmid' => $this->primaryKey(10),
					'leadfid' => $this->integer(10)->notNull(),
					'accountfid' => $this->integer(10),
					'editable' => $this->integer(10)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_crmentity' => [
				'columns' => [
					'crmid' => $this->primaryKey(10),
					'smcreatorid' => $this->smallInteger(5)->unsigned()->notNull()->defaultValue(0),
					'smownerid' => $this->smallInteger(5)->unsigned()->notNull()->defaultValue(0),
					'shownerid' => $this->smallInteger(1),
					'modifiedby' => $this->smallInteger(5)->unsigned()->notNull()->defaultValue(0),
					'setype' => $this->stringType(30)->notNull(),
					'description' => $this->text(),
					'attention' => $this->text(),
					'createdtime' => $this->dateTime()->notNull(),
					'modifiedtime' => $this->dateTime()->notNull(),
					'viewedtime' => $this->dateTime(),
					'closedtime' => $this->dateTime(),
					'status' => $this->stringType(50),
					'version' => $this->integer(10)->unsigned()->notNull()->defaultValue(0),
					'presence' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'deleted' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'was_read' => $this->smallInteger(1)->defaultValue(0),
					'private' => $this->smallInteger(1)->defaultValue(0),
					'users' => $this->text(),
				],
				'columns_mysql' => [
					'shownerid' => $this->tinyInteger(1),
					'presence' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'deleted' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'was_read' => $this->tinyInteger(1)->defaultValue(0),
					'private' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['crmentity_smcreatorid_idx', 'smcreatorid'],
					['crmentity_modifiedby_idx', 'modifiedby'],
					['crmentity_deleted_idx', 'deleted'],
					['crm_ownerid_del_setype_idx', ['smownerid', 'deleted', 'setype']],
					['crmid', ['crmid', 'deleted']],
					['crmid_2', ['crmid', 'setype']],
					['setypedeleted', ['setype', 'deleted']],
					['setype', 'setype'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_crmentityrel' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull(),
					'module' => $this->stringType(25)->notNull(),
					'relcrmid' => $this->integer(10)->notNull(),
					'relmodule' => $this->stringType(25)->notNull(),
					'rel_created_user' => $this->integer(10),
					'rel_created_time' => $this->dateTime(),
					'rel_comment' => $this->stringType(),
				],
				'index' => [
					['crmid', 'crmid'],
					['relcrmid', 'relcrmid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cron_task' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'name' => $this->stringType(100),
					'handler_file' => $this->stringType(100),
					'frequency' => $this->integer(10),
					'laststart' => $this->integer(10)->unsigned(),
					'lastend' => $this->integer(10)->unsigned(),
					'status' => $this->integer(10),
					'module' => $this->stringType(100),
					'sequence' => $this->integer(10),
					'description' => $this->text(),
				],
				'index' => [
					['name', 'name', true],
					['handler_file', 'handler_file', true],
					['vtiger_cron_task_status_idx', 'status'],
					['vtiger_cron_task_sequence_idx', 'sequence'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currencies' => [
				'columns' => [
					'currencyid' => $this->primaryKey(10),
					'currency_name' => $this->stringType(200),
					'currency_code' => $this->stringType(50),
					'currency_symbol' => $this->stringType(11),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currencies_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency' => [
				'columns' => [
					'currencyid' => $this->primaryKey(10),
					'currency' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'index' => [
					['currency_currency_idx', 'currency', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_decimal_separator' => [
				'columns' => [
					'currency_decimal_separatorid' => $this->primaryKey(10),
					'currency_decimal_separator' => $this->stringType(2)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_decimal_separator_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_grouping_pattern' => [
				'columns' => [
					'currency_grouping_patternid' => $this->primaryKey(10),
					'currency_grouping_pattern' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_grouping_pattern_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_grouping_separator' => [
				'columns' => [
					'currency_grouping_separatorid' => $this->primaryKey(10),
					'currency_grouping_separator' => $this->stringType(2)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_grouping_separator_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_info' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'currency_name' => $this->stringType(100),
					'currency_code' => $this->stringType(100),
					'currency_symbol' => $this->stringType(30),
					'conversion_rate' => $this->decimal('12,5'),
					'currency_status' => $this->stringType(25),
					'defaultid' => $this->smallInteger(3)->notNull()->defaultValue(0),
					'deleted' => $this->integer(1)->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'defaultid' => $this->tinyInteger(3)->notNull()->defaultValue(0),
				],
				'index' => [
					['deleted', 'deleted'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_symbol_placement' => [
				'columns' => [
					'currency_symbol_placementid' => $this->primaryKey(10),
					'currency_symbol_placement' => $this->stringType(30)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_currency_symbol_placement_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_customaction' => [
				'columns' => [
					'cvid' => $this->integer(10)->notNull(),
					'subject' => $this->stringType(250)->notNull(),
					'module' => $this->stringType(50)->notNull(),
					'content' => $this->text(),
				],
				'index' => [
					['customaction_cvid_idx', 'cvid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_customerdetails' => [
				'columns' => [
					'customerid' => $this->integer(10)->notNull(),
					'portal' => $this->smallInteger(1),
					'support_start_date' => $this->date(),
					'support_end_date' => $this->date(),
				],
				'primaryKeys' => [
					['customerdetails_pk', 'customerid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_customview' => [
				'columns' => [
					'cvid' => $this->primaryKey(10),
					'viewname' => $this->stringType(100)->notNull(),
					'setdefault' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'setmetrics' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'entitytype' => $this->stringType(25)->notNull(),
					'status' => $this->smallInteger(1)->notNull()->defaultValue(1),
					'userid' => $this->integer(10)->defaultValue(1),
					'privileges' => $this->smallInteger(2)->defaultValue(1),
					'featured' => $this->smallInteger(1)->defaultValue(0),
					'sequence' => $this->integer(10),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'description' => $this->text(),
					'sort' => $this->stringType(30)->defaultValue(''),
					'color' => $this->stringType(10)->defaultValue(''),
				],
				'columns_mysql' => [
					'setdefault' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'setmetrics' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'status' => $this->tinyInteger(1)->notNull()->defaultValue(1),
					'privileges' => $this->tinyInteger(2)->defaultValue(1),
					'featured' => $this->tinyInteger(1)->defaultValue(0),
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'index' => [
					['customview_entitytype_idx', 'entitytype'],
					['setdefault', ['setdefault', 'entitytype']],
					['customview_userid_idx', 'userid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cvadvfilter' => [
				'columns' => [
					'cvid' => $this->integer(10)->notNull(),
					'columnindex' => $this->integer(10)->notNull(),
					'columnname' => $this->stringType(250)->defaultValue(''),
					'comparator' => $this->stringType(20),
					'value' => $this->stringType(512),
					'groupid' => $this->integer(10)->defaultValue(1),
					'column_condition' => $this->stringType()->defaultValue('and'),
				],
				'index' => [
					['cvadvfilter_cvid_idx', 'cvid'],
					['cvadvfilter_groupid_idx', 'groupid'],
					['cvadvfilter_columnindex_idx', 'columnindex'],
				],
				'primaryKeys' => [
					['cvadvfilter_pk', ['cvid', 'columnindex']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cvadvfilter_grouping' => [
				'columns' => [
					'groupid' => $this->integer(10)->unsigned()->notNull(),
					'cvid' => $this->integer(10)->unsigned()->notNull(),
					'group_condition' => $this->stringType(),
					'condition_expression' => $this->text(),
				],
				'index' => [
					['cvid', 'cvid'],
				],
				'primaryKeys' => [
					['cvadvfilter_grouping_pk', ['groupid', 'cvid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cvcolumnlist' => [
				'columns' => [
					'cvid' => $this->integer(10)->notNull(),
					'columnindex' => $this->integer(10)->notNull(),
					'columnname' => $this->stringType(250)->defaultValue(''),
				],
				'index' => [
					['cvcolumnlist_columnindex_idx', 'columnindex'],
					['cvcolumnlist_cvid_idx', 'cvid'],
				],
				'primaryKeys' => [
					['cvcolumnlist_pk', ['cvid', 'columnindex']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_cvstdfilter' => [
				'columns' => [
					'cvid' => $this->integer(10)->notNull(),
					'columnname' => $this->stringType(250)->defaultValue(''),
					'stdfilter' => $this->stringType(250)->defaultValue(''),
					'startdate' => $this->date(),
					'enddate' => $this->date(),
				],
				'index' => [
					['cvstdfilter_cvid_idx', 'cvid'],
				],
				'primaryKeys' => [
					['cvstdfilter_pk', 'cvid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datasetregister_status' => [
				'columns' => [
					'datasetregister_statusid' => $this->primaryKey(),
					'datasetregister_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_grp2grp' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_groupid' => $this->integer(10),
					'to_groupid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_grp2grp_share_groupid_idx', 'share_groupid'],
					['datashare_grp2grp_to_groupid_idx', 'to_groupid'],
				],
				'primaryKeys' => [
					['datashare_grp2grp_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_grp2role' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_groupid' => $this->integer(10),
					'to_roleid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['idx_datashare_grp2role_share_groupid', 'share_groupid'],
					['idx_datashare_grp2role_to_roleid', 'to_roleid'],
				],
				'primaryKeys' => [
					['datashare_grp2role_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_grp2rs' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_groupid' => $this->integer(10),
					'to_roleandsubid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_grp2rs_share_groupid_idx', 'share_groupid'],
					['datashare_grp2rs_to_roleandsubid_idx', 'to_roleandsubid'],
				],
				'primaryKeys' => [
					['datashare_grp2rs_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_grp2us' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_groupid' => $this->integer(10),
					'to_userid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_grp2us_share_groupid_idx', 'share_groupid'],
					['datashare_grp2us_to_userid_idx', 'to_userid'],
				],
				'primaryKeys' => [
					['datashare_grp2us_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_module_rel' => [
				'columns' => [
					'shareid' => $this->primaryKey(10),
					'tabid' => $this->smallInteger(5)->notNull(),
					'relationtype' => $this->stringType(200),
				],
				'index' => [
					['idx_datashare_module_rel_tabid', 'tabid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_relatedmodule_permission' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'datashare_relatedmodule_id' => $this->integer(10)->notNull(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_relatedmodule_permission_shareid_permissions_idx', ['shareid', 'permission']],
				],
				'primaryKeys' => [
					['datashare_relatedmodule_permission_pk', ['shareid', 'datashare_relatedmodule_id']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_relatedmodules' => [
				'columns' => [
					'datashare_relatedmodule_id' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5),
					'relatedto_tabid' => $this->integer(10),
				],
				'index' => [
					['datashare_relatedmodules_tabid_idx', 'tabid'],
					['datashare_relatedmodules_relatedto_tabid_idx', 'relatedto_tabid'],
				],
				'primaryKeys' => [
					['datashare_relatedmodules_pk', 'datashare_relatedmodule_id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_relatedmodules_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_role2group' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleid' => $this->stringType(),
					'to_groupid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['idx_datashare_role2group_share_roleid', 'share_roleid'],
					['idx_datashare_role2group_to_groupid', 'to_groupid'],
				],
				'primaryKeys' => [
					['datashare_role2group_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_role2role' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleid' => $this->stringType(),
					'to_roleid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_role2role_share_roleid_idx', 'share_roleid'],
					['datashare_role2role_to_roleid_idx', 'to_roleid'],
				],
				'primaryKeys' => [
					['datashare_role2role_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_role2rs' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleid' => $this->stringType(),
					'to_roleandsubid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_role2s_share_roleid_idx', 'share_roleid'],
					['datashare_role2s_to_roleandsubid_idx', 'to_roleandsubid'],
				],
				'primaryKeys' => [
					['datashare_role2rs_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_role2us' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleid' => $this->stringType(),
					'to_userid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_role2us_share_roleid_idx', 'share_roleid'],
					['datashare_role2us_to_userid_idx', 'to_userid'],
				],
				'primaryKeys' => [
					['datashare_role2us_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_rs2grp' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleandsubid' => $this->stringType(),
					'to_groupid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_rs2grp_share_roleandsubid_idx', 'share_roleandsubid'],
					['datashare_rs2grp_to_groupid_idx', 'to_groupid'],
				],
				'primaryKeys' => [
					['datashare_rs2grp_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_rs2role' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleandsubid' => $this->stringType(),
					'to_roleid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_rs2role_share_roleandsubid_idx', 'share_roleandsubid'],
					['datashare_rs2role_to_roleid_idx', 'to_roleid'],
				],
				'primaryKeys' => [
					['datashare_rs2role_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_rs2rs' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleandsubid' => $this->stringType(),
					'to_roleandsubid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_rs2rs_share_roleandsubid_idx', 'share_roleandsubid'],
					['idx_datashare_rs2rs_to_roleandsubid_idx', 'to_roleandsubid'],
				],
				'primaryKeys' => [
					['datashare_rs2rs_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_rs2us' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_roleandsubid' => $this->stringType(),
					'to_userid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_rs2us_share_roleandsubid_idx', 'share_roleandsubid'],
					['datashare_rs2us_to_userid_idx', 'to_userid'],
				],
				'primaryKeys' => [
					['datashare_rs2us_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_us2grp' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_userid' => $this->integer(10),
					'to_groupid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_us2grp_share_userid_idx', 'share_userid'],
					['datashare_us2grp_to_groupid_idx', 'to_groupid'],
				],
				'primaryKeys' => [
					['datashare_us2grp_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_us2role' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_userid' => $this->integer(10),
					'to_roleid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['idx_datashare_us2role_share_userid', 'share_userid'],
					['idx_datashare_us2role_to_roleid', 'to_roleid'],
				],
				'primaryKeys' => [
					['datashare_us2role_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_us2rs' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_userid' => $this->integer(10),
					'to_roleandsubid' => $this->stringType(),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_us2rs_share_userid_idx', 'share_userid'],
					['datashare_us2rs_to_roleandsubid_idx', 'to_roleandsubid'],
				],
				'primaryKeys' => [
					['datashare_us2rs_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_datashare_us2us' => [
				'columns' => [
					'shareid' => $this->integer(10)->notNull(),
					'share_userid' => $this->integer(10),
					'to_userid' => $this->integer(10),
					'permission' => $this->integer(10),
				],
				'index' => [
					['datashare_us2us_share_userid_idx', 'share_userid'],
					['datashare_us2us_to_userid_idx', 'to_userid'],
				],
				'primaryKeys' => [
					['datashare_us2us_pk', 'shareid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_date_format' => [
				'columns' => [
					'date_formatid' => $this->primaryKey(10),
					'date_format' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_date_format_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_dayoftheweek' => [
				'columns' => [
					'dayoftheweekid' => $this->primaryKey(10),
					'dayoftheweek' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_dayoftheweek_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_def_org_field' => [
				'columns' => [
					'tabid' => $this->smallInteger(5),
					'fieldid' => $this->integer(10)->notNull(),
					'visible' => $this->integer(10),
					'readonly' => $this->integer(10),
				],
				'index' => [
					['def_org_field_tabid_fieldid_idx', ['tabid', 'fieldid']],
					['def_org_field_tabid_idx', 'tabid'],
					['def_org_field_visible_fieldid_idx', ['visible', 'fieldid']],
				],
				'primaryKeys' => [
					['def_org_field_pk', 'fieldid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_def_org_share' => [
				'columns' => [
					'ruleid' => $this->primaryKey(10),
					'tabid' => $this->smallInteger(5)->notNull(),
					'permission' => $this->integer(10),
					'editstatus' => $this->integer(10),
				],
				'index' => [
					['fk_1_vtiger_def_org_share', 'permission'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_default_record_view' => [
				'columns' => [
					'default_record_viewid' => $this->primaryKey(10),
					'default_record_view' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_default_record_view_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_defaultactivitytype' => [
				'columns' => [
					'defaultactivitytypeid' => $this->primaryKey(10),
					'defaultactivitytype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_defaultactivitytype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_defaultcv' => [
				'columns' => [
					'tabid' => $this->smallInteger(5)->notNull(),
					'defaultviewname' => $this->stringType(50)->notNull(),
					'query' => $this->text(),
				],
				'primaryKeys' => [
					['defaultcv_pk', 'tabid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_defaulteventstatus' => [
				'columns' => [
					'defaulteventstatusid' => $this->primaryKey(10),
					'defaulteventstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_defaulteventstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_duration_minutes' => [
				'columns' => [
					'minutesid' => $this->primaryKey(10),
					'duration_minutes' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_duration_minutes_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_durationhrs' => [
				'columns' => [
					'hrsid' => $this->primaryKey(10),
					'hrs' => $this->stringType(50),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_durationmins' => [
				'columns' => [
					'minsid' => $this->primaryKey(10),
					'mins' => $this->stringType(50),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_email_template_priority' => [
				'columns' => [
					'email_template_priorityid' => $this->primaryKey(10),
					'email_template_priority' => $this->smallInteger(1),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'email_template_priority' => $this->tinyInteger(1),
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_email_template_type' => [
				'columns' => [
					'email_template_typeid' => $this->primaryKey(10),
					'email_template_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_employee_education' => [
				'columns' => [
					'employee_educationid' => $this->primaryKey(10),
					'employee_education' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_employee_education_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_employee_status' => [
				'columns' => [
					'employee_statusid' => $this->primaryKey(10),
					'employee_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_employee_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_end_hour' => [
				'columns' => [
					'end_hourid' => $this->primaryKey(10),
					'end_hour' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_end_hour_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_entity_stats' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull(),
					'crmactivity' => $this->integer(8),
				],
				'primaryKeys' => [
					['entity_stats_pk', 'crmid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_entityname' => [
				'columns' => [
					'tabid' => $this->smallInteger(5)->notNull()->defaultValue(0),
					'modulename' => $this->stringType(25)->notNull(),
					'tablename' => $this->stringType(50)->notNull(),
					'fieldname' => $this->stringType(100)->notNull(),
					'entityidfield' => $this->stringType(30)->notNull(),
					'entityidcolumn' => $this->stringType(30)->notNull(),
					'searchcolumn' => $this->stringType(150)->notNull(),
					'turn_off' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'sequence' => $this->smallInteger(3)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'turn_off' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
				],
				'index' => [
					['turn_off', 'turn_off'],
				],
				'primaryKeys' => [
					['entityname_pk', 'tabid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_eventhandlers' => [
				'columns' => [
					'eventhandler_id' => $this->primaryKey(5)->unsigned(),
					'event_name' => $this->stringType(50)->notNull(),
					'handler_class' => $this->stringType(100)->notNull(),
					'is_active' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'include_modules' => $this->stringType()->notNull()->defaultValue(''),
					'exclude_modules' => $this->stringType()->notNull()->defaultValue(''),
					'priority' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(5),
					'owner_id' => $this->smallInteger(5)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'is_active' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'priority' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(5),
				],
				'index' => [
					['event_name_class', ['event_name', 'handler_class']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_eventstatus' => [
				'columns' => [
					'eventstatusid' => $this->primaryKey(10),
					'eventstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_eventstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_expectedresponse' => [
				'columns' => [
					'expectedresponseid' => $this->primaryKey(10),
					'expectedresponse' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['CampaignExpRes_UK01', 'expectedresponse', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_expectedresponse_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faq' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'faq_no' => $this->stringType(100)->notNull(),
					'product_id' => $this->integer(),
					'question' => $this->text(),
					'answer' => $this->text(),
					'category' => $this->stringType(200)->notNull(),
					'status' => $this->stringType(200)->notNull(),
				],
				'index' => [
					['faq_id_idx', 'id'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqcategories' => [
				'columns' => [
					'faqcategories_id' => $this->primaryKey(10),
					'faqcategories' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqcategories_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqcf' => [
				'columns' => [
					'faqid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['faqcf_pk', 'faqid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqcomments' => [
				'columns' => [
					'commentid' => $this->primaryKey(10),
					'faqid' => $this->integer(10),
					'comments' => $this->text(),
					'createdtime' => $this->dateTime()->notNull(),
				],
				'index' => [
					['faqcomments_faqid_idx', 'faqid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqstatus' => [
				'columns' => [
					'faqstatus_id' => $this->primaryKey(10),
					'faqstatus' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_faqstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fcorectinginvoice_formpayment' => [
				'columns' => [
					'fcorectinginvoice_formpaymentid' => $this->primaryKey(10),
					'fcorectinginvoice_formpayment' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fcorectinginvoice_status' => [
				'columns' => [
					'fcorectinginvoice_statusid' => $this->primaryKey(10),
					'fcorectinginvoice_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_feedback' => [
				'columns' => [
					'userid' => $this->integer(10),
					'dontshow' => $this->stringType(19)->defaultValue('false'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_field' => [
				'columns' => [
					'tabid' => $this->smallInteger(5)->notNull(),
					'fieldid' => $this->primaryKey(10),
					'columnname' => $this->stringType(30)->notNull(),
					'tablename' => $this->stringType(50)->notNull(),
					'generatedtype' => $this->smallInteger(3)->unsigned()->notNull()->defaultValue(0),
					'uitype' => $this->smallInteger(5)->unsigned()->notNull(),
					'fieldname' => $this->stringType(50)->notNull(),
					'fieldlabel' => $this->stringType(50)->notNull(),
					'readonly' => $this->smallInteger(1)->unsigned()->notNull(),
					'presence' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'defaultvalue' => $this->text(),
					'maximumlength' => $this->stringType(30),
					'sequence' => $this->smallInteger(5)->unsigned()->notNull(),
					'block' => $this->integer(10),
					'displaytype' => $this->smallInteger(1)->unsigned()->notNull(),
					'typeofdata' => $this->stringType(100),
					'quickcreate' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'quickcreatesequence' => $this->integer(10),
					'info_type' => $this->stringType(20),
					'masseditable' => $this->integer(10)->notNull()->defaultValue(1),
					'helpinfo' => $this->stringType(30)->defaultValue(''),
					'summaryfield' => $this->integer(10)->notNull()->defaultValue(0),
					'fieldparams' => $this->stringType()->defaultValue(''),
					'header_field' => $this->stringType(15),
					'maxlengthtext' => $this->smallInteger(3)->unsigned()->defaultValue(0),
					'maxwidthcolumn' => $this->smallInteger(3)->unsigned()->defaultValue(0),
				],
				'columns_mysql' => [
					'generatedtype' => $this->tinyInteger(3)->unsigned()->notNull()->defaultValue(0),
					'readonly' => $this->tinyInteger(1)->unsigned()->notNull(),
					'presence' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'displaytype' => $this->tinyInteger(1)->unsigned()->notNull(),
					'quickcreate' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
				],
				'index' => [
					['field_tabid_idx', 'tabid'],
					['field_fieldname_idx', 'fieldname'],
					['field_block_idx', 'block'],
					['field_displaytype_idx', 'displaytype'],
					['tabid', ['tabid', 'tablename']],
					['quickcreate', 'quickcreate'],
					['presence', 'presence'],
					['tabid_2', ['tabid', 'fieldname']],
					['tabid_3', ['tabid', 'block']],
					['field_sequence_idx', 'sequence'],
					['field_uitype_idx', 'uitype'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_field_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fieldmodulerel' => [
				'columns' => [
					'fieldid' => $this->smallInteger(5)->unsigned()->notNull(),
					'module' => $this->stringType(25)->notNull(),
					'relmodule' => $this->stringType(25)->notNull(),
					'status' => $this->stringType(10),
					'sequence' => $this->smallInteger(1)->unsigned()->defaultValue(0),
				],
				'columns_mysql' => [
					'sequence' => $this->tinyInteger(1)->unsigned()->defaultValue(0),
				],
				'index' => [
					['fieldid', 'fieldid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoice_formpayment' => [
				'columns' => [
					'finvoice_formpaymentid' => $this->primaryKey(10),
					'finvoice_formpayment' => $this->stringType(200)->notNull(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoice_paymentstatus' => [
				'columns' => [
					'finvoice_paymentstatusid' => $this->primaryKey(10),
					'finvoice_paymentstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoice_status' => [
				'columns' => [
					'finvoice_statusid' => $this->primaryKey(10),
					'finvoice_status' => $this->stringType(200)->notNull(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoice_type' => [
				'columns' => [
					'finvoice_typeid' => $this->primaryKey(10),
					'finvoice_type' => $this->stringType(200)->notNull(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoicecost_formpayment' => [
				'columns' => [
					'finvoicecost_formpaymentid' => $this->primaryKey(10),
					'finvoicecost_formpayment' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoicecost_paymentstatus' => [
				'columns' => [
					'finvoicecost_paymentstatusid' => $this->primaryKey(10),
					'finvoicecost_paymentstatus' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoicecost_status' => [
				'columns' => [
					'finvoicecost_statusid' => $this->primaryKey(10),
					'finvoicecost_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoiceproforma_formpayment' => [
				'columns' => [
					'finvoiceproforma_formpaymentid' => $this->primaryKey(10),
					'finvoiceproforma_formpayment' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_finvoiceproforma_status' => [
				'columns' => [
					'finvoiceproforma_statusid' => $this->primaryKey(10),
					'finvoiceproforma_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fixed_assets_fuel_type' => [
				'columns' => [
					'fixed_assets_fuel_typeid' => $this->primaryKey(10),
					'fixed_assets_fuel_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fixed_assets_status' => [
				'columns' => [
					'fixed_assets_statusid' => $this->primaryKey(10),
					'fixed_assets_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_fixed_assets_type' => [
				'columns' => [
					'fixed_assets_typeid' => $this->primaryKey(10),
					'fixed_assets_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
		];
		$this->foreignKey = [
			['fk_1_vtiger_account', 'vtiger_account', 'accountid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_accountaddress_ibfk_1', 'vtiger_accountaddress', 'accountaddressid', 'vtiger_account', 'accountid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_accountscf', 'vtiger_accountscf', 'accountid', 'vtiger_account', 'accountid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_activity', 'vtiger_activity', 'activityid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_activity_reminder_ibfk_1', 'vtiger_activity_reminder', 'activity_id', 'vtiger_activity', 'activityid', 'CASCADE', 'RESTRICT'],
			['vtiger_activity_reminder_popup_ibfk_1', 'vtiger_activity_reminder_popup', 'recordid', 'vtiger_activity', 'activityid', 'CASCADE', 'RESTRICT'],
			['vtiger_activity_update_dates_ibfk_1', 'vtiger_activity_update_dates', 'task_id', 'com_vtiger_workflowtasks', 'task_id', 'CASCADE', 'RESTRICT'],
			['vtiger_activity_update_dates_ibfk_2', 'vtiger_activity_update_dates', 'parent', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_activity_update_dates_ibfk_3', 'vtiger_activity_update_dates', 'activityid', 'vtiger_activity', 'activityid', 'CASCADE', 'RESTRICT'],
			['vtiger_activitycf_ibfk_1', 'vtiger_activitycf', 'activityid', 'vtiger_activity', 'activityid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_activityproductrel', 'vtiger_activityproductrel', 'productid', 'vtiger_products', 'productid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_assets', 'vtiger_assets', 'assetsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_assetscf_ibfk_1', 'vtiger_assetscf', 'assetsid', 'vtiger_assets', 'assetsid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_attachments', 'vtiger_attachments', 'attachmentsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_blocks', 'vtiger_blocks', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['vtiger_callhistory_ibfk_1', 'vtiger_callhistory', 'callhistoryid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_callhistorycf_ibfk_1', 'vtiger_callhistorycf', 'callhistoryid', 'vtiger_callhistory', 'callhistoryid', 'CASCADE', 'RESTRICT'],
			['fk_vtiger_crmentity', 'vtiger_campaign_records', 'crmid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_campaignscf', 'vtiger_campaignscf', 'campaignid', 'vtiger_campaign', 'campaignid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_contactaddress', 'vtiger_contactaddress', 'contactaddressid', 'vtiger_contactdetails', 'contactid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_contactdetails', 'vtiger_contactdetails', 'contactid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_contactscf', 'vtiger_contactscf', 'contactid', 'vtiger_contactdetails', 'contactid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_contactsubdetails', 'vtiger_contactsubdetails', 'contactsubscriptionid', 'vtiger_contactdetails', 'contactid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_customaction', 'vtiger_customaction', 'cvid', 'vtiger_customview', 'cvid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_customerdetails', 'vtiger_customerdetails', 'customerid', 'vtiger_contactdetails', 'contactid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_customview', 'vtiger_customview', 'entitytype', 'vtiger_tab', 'name', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_cvadvfilter', 'vtiger_cvadvfilter', 'cvid', 'vtiger_customview', 'cvid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_cvcolumnlist', 'vtiger_cvcolumnlist', 'cvid', 'vtiger_customview', 'cvid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_cvstdfilter', 'vtiger_cvstdfilter', 'cvid', 'vtiger_customview', 'cvid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_grp2grp', 'vtiger_datashare_grp2grp', 'to_groupid', 'vtiger_groups', 'groupid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_grp2role', 'vtiger_datashare_grp2role', 'to_roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_grp2rs', 'vtiger_datashare_grp2rs', 'to_roleandsubid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_datashare_module_rel', 'vtiger_datashare_module_rel', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_datashare_relatedmodules', 'vtiger_datashare_relatedmodules', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_role2group', 'vtiger_datashare_role2group', 'share_roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_role2role', 'vtiger_datashare_role2role', 'to_roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_role2rs', 'vtiger_datashare_role2rs', 'to_roleandsubid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_rs2grp', 'vtiger_datashare_rs2grp', 'share_roleandsubid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_rs2role', 'vtiger_datashare_rs2role', 'to_roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_datashare_rs2rs', 'vtiger_datashare_rs2rs', 'to_roleandsubid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_def_org_share', 'vtiger_def_org_share', 'permission', 'vtiger_org_share_action_mapping', 'share_action_id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_defaultcv', 'vtiger_defaultcv', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_entity_stats', 'vtiger_entity_stats', 'crmid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_entityname', 'vtiger_entityname', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_faq', 'vtiger_faq', 'id', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_faqcf', 'vtiger_faqcf', 'faqid', 'vtiger_faq', 'id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_faqcomments', 'vtiger_faqcomments', 'faqid', 'vtiger_faq', 'id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_field', 'vtiger_field', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
		];
	}

	public function data()
	{
		$this->data = [
			'vtiger_accounts_status' => [
				'columns' => ['accounts_statusid', 'accounts_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_PROMISING_CLIENT', 1, 705, 1],
					[2, 'PLL_ACTIVE_CLIENT', 1, 706, 2],
					[3, 'PLL_SPECIAL_CLIENT', 1, 707, 3],
					[4, 'PLL_LOST_CLIENT', 1, 708, 4],
					[5, 'PLL_UNOBTAINED_CLIENT', 1, 709, 5],
					[6, 'PLL_INACTIVE_CLIENT', 1, 710, 6],
					[7, 'PLL_CLOSED_CLIENT', 1, 711, 7],
					[8, 'PLL_BLACKLISTED_CLIENT', 1, 712, 8],
				]
			],
			'vtiger_accounttype' => [
				'columns' => ['accounttypeid', 'accounttype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Analyst', 1, 2, 1],
					[3, 'Competitor', 1, 3, 2],
					[4, 'Customer', 1, 4, 3],
					[5, 'Integrator', 1, 5, 4],
					[6, 'Investor', 1, 6, 5],
					[7, 'Partner', 1, 7, 6],
					[8, 'Press', 1, 8, 7],
					[9, 'Prospect', 1, 9, 8],
					[10, 'Reseller', 1, 10, 9],
					[11, 'Other', 1, 11, 10],
				]
			],
			'vtiger_accounttype_seq' => [
				'columns' => ['id'],
				'values' => [
					[11],
				]
			],
			'vtiger_actionmapping' => [
				'columns' => ['actionid', 'actionname', 'securitycheck'],
				'values' => [
					[0, 'Save', 0],
					[0, 'SavePriceBook', 1],
					[0, 'SaveVendor', 1],
					[1, 'DetailViewAjax', 1],
					[1, 'EditView', 0],
					[1, 'PriceBookEditView', 1],
					[1, 'QuickCreate', 1],
					[1, 'VendorEditView', 1],
					[2, 'Delete', 0],
					[2, 'DeletePriceBook', 1],
					[2, 'DeleteVendor', 1],
					[3, 'index', 0],
					[3, 'Popup', 1],
					[4, 'DetailView', 0],
					[4, 'PriceBookDetailView', 1],
					[4, 'VendorDetailView', 1],
					[5, 'Import', 0],
					[6, 'Export', 0],
					[7, 'CreateView', 0],
					[8, 'Merge', 0],
					[9, 'ConvertLead', 0],
					[10, 'DuplicatesHandling', 0],
					[11, 'ReceiveIncomingCalls', 0],
					[12, 'MakeOutgoingCalls', 0],
					[13, 'Print', 0],
					[14, 'CreateCustomFilter', 0],
					[15, 'DuplicateRecord', 0],
					[16, 'EditableComments', 0],
					[17, 'MassEdit', 0],
					[18, 'MassDelete', 0],
					[19, 'MassAddComment', 0],
					[20, 'MassComposeEmail', 0],
					[21, 'MassSendSMS', 0],
					[22, 'MassTransferOwnership', 0],
					[23, 'MassMoveDocuments', 0],
					[24, 'ReadRecord', 0],
					[25, 'WorkflowTrigger', 0],
					[26, 'Dashboard', 0],
					[27, 'CreateDashboardFilter', 0],
					[28, 'QuickExportToExcel', 0],
					[30, 'DetailTransferOwnership', 0],
					[31, 'ActivityPostponed', 0],
					[32, 'ActivityCancel', 0],
					[33, 'ActivityComplete', 0],
					[34, 'ExportPdf', 0],
					[35, 'PrintMail', 0],
					[36, 'RecordMapping', 0],
					[37, 'RecordMappingList', 0],
					[38, 'OpenRecord', 0],
					[39, 'CloseRecord', 0],
					[40, 'FavoriteRecords', 0],
					[41, 'WatchingRecords', 0],
					[42, 'WatchingModule', 0],
					[44, 'NotificationCreateMail', 0],
					[45, 'NotificationPreview', 0],
					[46, 'RemoveRelation', 0],
					[47, 'ReceivingMailNotifications', 0],
					[48, 'ReviewingUpdates', 0],
					[49, 'CreateDashboardChartFilter', 0],
					[50, 'ReloadRelationRecord', 0],
					[51, 'TimeLineList', 0],
				]
			],
			'vtiger_activity_view' => [
				'columns' => ['activity_viewid', 'activity_view', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Today', 0, 1],
					[2, 'This Week', 1, 1],
					[3, 'This Month', 2, 1],
					[4, 'This Year', 3, 1],
				]
			],
			'vtiger_activity_view_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_activitystatus' => [
				'columns' => ['activitystatusid', 'activitystatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_PLANNED', 0, 173, 0],
					[2, 'PLL_IN_REALIZATION', 0, 174, 1],
					[3, 'PLL_OVERDUE', 0, 175, 2],
					[4, 'PLL_POSTPONED', 0, 176, 3],
					[5, 'PLL_CANCELLED', 0, 177, 4],
					[6, 'PLL_COMPLETED', 0, 178, 5],
				]
			],
			'vtiger_activitystatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_activitytype' => [
				'columns' => ['activitytypeid', 'activitytype', 'presence', 'picklist_valueid', 'sortorderid', 'color'],
				'values' => [
					[1, 'Call', 0, 12, 0, '#80B584'],
					[2, 'Meeting', 0, 13, 1, '#FFD500'],
				]
			],
			'vtiger_activitytype_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_announcementstatus' => [
				'columns' => ['announcementstatusid', 'announcementstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 697, 1],
					[2, 'PLL_FOR_ACCEPTANCE', 1, 698, 2],
					[3, 'PLL_PUBLISHED', 0, 699, 3],
				]
			],
			'vtiger_assets_renew' => [
				'columns' => ['assets_renewid', 'assets_renew', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_PLANNED', 0, 682, 1],
					[2, 'PLL_WAITING_FOR_RENEWAL', 0, 683, 2],
					[3, 'PLL_RENEWED_VERIFICATION', 0, 684, 3],
					[4, 'PLL_NOT_RENEWED_VERIFICATION', 0, 685, 4],
					[5, 'PLL_RENEWED', 0, 686, 6],
					[6, 'PLL_NOT_RENEWED', 0, 687, 7],
					[7, 'PLL_NOT_APPLICABLE', 0, 688, 8],
					[8, 'PLL_NOT_APPLICABLE_VERIFICATION', 0, 702, 5],
				]
			],
			'vtiger_assetstatus' => [
				'columns' => ['assetstatusid', 'assetstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[3, 'PLL_DRAFT', 1, 508, 3],
					[4, 'PLL_WAITING_FOR_VERIFICATION', 1, 509, 4],
					[5, 'PLL_WAITING_FOR_ACCEPTANCE', 1, 510, 5],
					[6, 'PLL_ACCEPTED', 0, 511, 6],
					[7, 'PLL_CANCELLED', 0, 672, 7],
				]
			],
			'vtiger_assetstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[7],
				]
			],
			'vtiger_asteriskextensions' => [
				'columns' => ['userid', 'asterisk_extension', 'use_asterisk'],
				'values' => [
					[1, null, null],
				]
			],
			'vtiger_blocks' => [
				'columns' => ['blockid', 'tabid', 'blocklabel', 'sequence', 'show_title', 'visible', 'create_view', 'edit_view', 'detail_view', 'display_status', 'iscustom'],
				'values' => [
					[4, 4, 'LBL_CONTACT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[5, 4, 'LBL_CUSTOM_INFORMATION', 3, 0, 0, 0, 0, 0, 0, 0],
					[6, 4, 'LBL_CUSTOMER_PORTAL_INFORMATION', 4, 0, 0, 0, 0, 0, 0, 0],
					[7, 4, 'LBL_ADDRESS_INFORMATION', 6, 0, 0, 0, 0, 0, 0, 0],
					[8, 4, 'LBL_DESCRIPTION_INFORMATION', 8, 0, 0, 0, 0, 0, 0, 0],
					[9, 6, 'LBL_ACCOUNT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[10, 6, 'LBL_ADDRESS_MAILING_INFORMATION', 8, 0, 0, 0, 0, 0, 0, 0],
					[11, 6, 'LBL_ADDRESS_INFORMATION', 7, 0, 0, 0, 0, 0, 0, 0],
					[12, 6, 'LBL_DESCRIPTION_INFORMATION', 10, 0, 0, 0, 0, 0, 1, 0],
					[13, 7, 'LBL_LEAD_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[14, 7, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 0, 0],
					[15, 7, 'LBL_ADDRESS_INFORMATION', 5, 0, 0, 0, 0, 0, 0, 0],
					[16, 7, 'LBL_DESCRIPTION_INFORMATION', 7, 0, 0, 0, 0, 0, 0, 0],
					[17, 8, 'LBL_NOTE_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[18, 8, 'LBL_FILE_INFORMATION', 3, 1, 0, 0, 0, 0, 1, 0],
					[19, 9, 'LBL_TASK_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[20, 9, 'LBL_DESCRIPTION_INFORMATION', 2, 1, 0, 0, 0, 0, 1, 0],
					[25, 13, 'LBL_TICKET_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[26, 13, '', 3, 1, 0, 0, 0, 0, 1, 0],
					[27, 13, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[28, 13, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[29, 13, 'LBL_TICKET_RESOLUTION', 5, 0, 0, 1, 0, 0, 1, 0],
					[31, 14, 'LBL_PRODUCT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[32, 14, 'LBL_PRICING_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[33, 14, 'LBL_STOCK_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[34, 14, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[35, 14, 'LBL_IMAGE_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 0],
					[36, 14, 'LBL_DESCRIPTION_INFORMATION', 6, 0, 0, 0, 0, 0, 1, 0],
					[37, 15, 'LBL_FAQ_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[38, 15, 'LBL_COMMENT_INFORMATION', 4, 0, 0, 1, 0, 0, 1, 0],
					[39, 16, 'LBL_EVENT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[40, 16, 'LBL_REMINDER_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[41, 16, 'LBL_DESCRIPTION_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 0],
					[42, 18, 'LBL_VENDOR_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[43, 18, 'LBL_ADDRESS_MAILING_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[44, 18, 'LBL_ADDRESS_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[45, 18, 'LBL_DESCRIPTION_INFORMATION', 6, 0, 0, 0, 0, 0, 1, 0],
					[46, 19, 'LBL_PRICEBOOK_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[47, 19, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[48, 19, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[73, 4, 'LBL_IMAGE_INFORMATION', 9, 0, 0, 0, 0, 0, 0, 0],
					[74, 26, 'LBL_CAMPAIGN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[75, 26, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[76, 26, 'LBL_EXPECTATIONS_AND_ACTUALS', 3, 0, 0, 0, 0, 0, 1, 0],
					[77, 29, 'LBL_USERLOGIN_ROLE', 1, 0, 0, 0, 0, 0, 1, 0],
					[78, 29, 'LBL_CURRENCY_CONFIGURATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[79, 29, 'LBL_MORE_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[81, 26, 'LBL_DESCRIPTION_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 0],
					[82, 29, 'LBL_USER_IMAGE_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 0],
					[83, 29, 'LBL_USER_ADV_OPTIONS', 6, 0, 0, 0, 0, 0, 1, 0],
					[84, 8, 'LBL_DESCRIPTION', 2, 0, 0, 0, 0, 0, 1, 0],
					[86, 9, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[87, 9, 'LBL_RELATED_TO', 3, 0, 0, 0, 0, 0, 1, 0],
					[89, 34, 'LBL_SERVICE_CONTRACT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[90, 34, 'LBL_CUSTOM_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[91, 35, 'LBL_SERVICE_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[92, 35, 'LBL_PRICING_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[93, 35, 'LBL_CUSTOM_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[94, 35, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[95, 37, 'LBL_ASSET_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[96, 37, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[97, 37, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[98, 40, 'LBL_MODCOMMENTS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[99, 40, 'LBL_OTHER_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[100, 40, 'LBL_CUSTOM_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[101, 41, 'LBL_PROJECT_MILESTONE_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[102, 41, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[103, 41, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[104, 42, 'LBL_PROJECT_TASK_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[105, 42, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[106, 42, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[107, 43, 'LBL_PROJECT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[108, 43, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[109, 43, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[110, 45, 'LBL_SMSNOTIFIER_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[111, 45, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[112, 45, 'StatusInformation', 3, 0, 0, 0, 0, 0, 1, 0],
					[117, 16, 'LBL_RECURRENCE_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[118, 29, 'LBL_CALENDAR_SETTINGS', 2, 0, 0, 0, 0, 0, 1, 0],
					[119, 16, 'LBL_RELATED_TO', 4, 0, 0, 0, 0, 0, 1, 0],
					[128, 51, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[129, 51, 'LBL_BLOCK', 2, 0, 0, 0, 0, 0, 0, 0],
					[130, 51, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[134, 54, 'LBL_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[135, 54, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[136, 54, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 0, 0],
					[137, 54, 'LBL_ORIGN_MESSAGE', 4, 0, 0, 0, 0, 0, 0, 0],
					[138, 57, 'LBL_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[139, 57, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[140, 57, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[141, 58, 'LBL_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[142, 58, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[143, 58, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[144, 59, 'LBL_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[145, 59, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[146, 59, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[147, 60, 'LBL_OSSPASSWORD_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[148, 60, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[149, 60, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[150, 7, 'Contact Information', 2, 0, 0, 0, 0, 0, 1, 1],
					[151, 61, 'LBL_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[152, 61, 'LBL_CONTACTS', 2, 0, 0, 0, 0, 0, 1, 0],
					[153, 61, 'LBL_PERMANENT_ADDRESS', 3, 0, 0, 0, 0, 0, 1, 0],
					[154, 61, 'LBL_CORRESPONDANCE_ADDRESS', 4, 0, 0, 0, 0, 0, 1, 0],
					[178, 4, 'LBL_ADDRESS_MAILING_INFORMATION', 7, 0, 0, 0, 0, 0, 0, 1],
					[179, 18, 'LBL_ADDRESS_DELIVERY_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[181, 6, 'LBL_ADDRESS_DELIVERY_INFORMATION', 9, 0, 0, 0, 0, 0, 0, 1],
					[191, 7, 'LBL_REGISTRATION_INFO', 3, 0, 0, 0, 0, 0, 0, 1],
					[192, 37, 'BLOCK_INFORMATION_TIME', 3, 0, 0, 0, 0, 0, 1, 1],
					[194, 6, 'LBL_REGISTRATION_INFO', 5, 0, 0, 0, 0, 0, 1, 1],
					[195, 6, 'LBL_CONTACT_INFO', 2, 0, 0, 0, 0, 0, 1, 1],
					[196, 6, 'LBL_ADVANCED_BLOCK', 4, 0, 0, 0, 0, 0, 1, 1],
					[197, 4, 'LBL_CONTACT_INFO', 2, 0, 0, 0, 0, 0, 1, 1],
					[198, 6, 'LBL_FINANSIAL_SUMMARY', 3, 0, 0, 0, 0, 0, 1, 1],
					[232, 74, 'LBL_CALLHISTORY_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[233, 74, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[234, 74, 'LBL_PHONE_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[235, 74, 'LBL_RELATED_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 0],
					[236, 75, 'LBL_IDEAS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[238, 75, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[247, 78, 'LBL_HOLIDAYSENTITLEMENT_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[248, 78, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[250, 79, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[251, 79, 'LBL_PAYMENT_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[252, 80, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[253, 80, 'LBL_PAYMENT_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[254, 81, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[255, 81, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[256, 82, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[257, 82, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[258, 83, 'LBL_ANNOUNCEMENTS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[259, 83, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[260, 83, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[261, 84, 'LBL_MAIN_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[262, 84, 'LBL_BLOCK', 2, 0, 0, 0, 0, 0, 1, 0],
					[263, 84, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 0],
					[265, 85, 'LBL_QUOTESENQUIRES_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[266, 85, 'LBL_DESCRIPTION_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[267, 85, 'LBL_STATISTICS', 3, 0, 0, 0, 0, 0, 1, 1],
					[268, 85, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[269, 86, 'LBL_SSALESPROCESSES_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[270, 86, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[271, 86, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[272, 87, 'LBL_SREQUIREMENTSCARDS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[273, 87, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[274, 87, 'LBL_STATISTICS', 4, 0, 0, 0, 0, 0, 1, 1],
					[275, 87, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[276, 88, 'LBL_SCALCULATIONS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[277, 88, 'LBL_DESCRIPTION_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[278, 88, 'LBL_STATISTICS', 3, 0, 0, 0, 0, 0, 1, 1],
					[279, 88, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[280, 89, 'LBL_SQUOTES_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[281, 89, 'LBL_DESCRIPTION_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[282, 89, 'LBL_STATISTICS', 3, 0, 0, 0, 0, 0, 1, 1],
					[283, 89, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[284, 90, 'LBL_SSINGLEORDERS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[285, 90, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[286, 90, 'LBL_STATISTICS', 4, 0, 0, 0, 0, 0, 1, 1],
					[287, 90, 'LBL_CUSTOM_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 1],
					[288, 91, 'LBL_SRECURRINGORDERS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[289, 91, 'LBL_DESCRIPTION_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[290, 91, 'LBL_STATISTICS', 3, 0, 0, 0, 0, 0, 1, 1],
					[291, 91, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[292, 91, 'LBL_PERIODIC_GENERATION', 5, 0, 0, 0, 0, 0, 1, 1],
					[293, 89, 'LBL_ADDRESS_INFORMATION', 5, 0, 0, 0, 0, 0, 1, 1],
					[294, 89, 'LBL_ADDRESS_DELIVERY_INFORMATION', 6, 0, 0, 0, 0, 0, 1, 1],
					[295, 90, 'LBL_ADDRESS_INFORMATION', 6, 0, 0, 0, 0, 0, 1, 1],
					[296, 90, 'LBL_ADDRESS_DELIVERY_INFORMATION', 7, 0, 0, 0, 0, 0, 1, 1],
					[297, 91, 'LBL_ADDRESS_INFORMATION', 6, 0, 0, 0, 0, 0, 1, 1],
					[298, 91, 'LBL_ADDRESS_DELIVERY_INFORMATION', 7, 0, 0, 0, 0, 0, 1, 1],
					[299, 92, 'LBL_PARTNERS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[300, 92, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[301, 92, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[302, 92, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[303, 93, 'LBL_COMPETITION_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[304, 93, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[305, 93, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[306, 93, 'LBL_DESCRIPTION_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[307, 94, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[308, 94, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[309, 94, 'LBL_DESCRIPTION_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[310, 95, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[311, 95, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[312, 95, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[313, 95, 'LBL_ADDRESS_DELIVERY_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[314, 96, 'LBL_KNOWLEDGEBASE_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[315, 96, 'LBL_ARTICLE', 2, 0, 0, 0, 0, 0, 1, 0],
					[316, 97, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[317, 97, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[318, 97, 'LBL_DESCRIPTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 1],
					[319, 86, 'LBL_ATTENTION', 5, 0, 0, 0, 0, 0, 1, 0],
					[320, 86, 'LBL_FINANCES', 3, 0, 0, 0, 0, 0, 1, 0],
					[321, 98, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[322, 98, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[323, 99, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[324, 99, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[325, 99, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[326, 99, 'LBL_ADDRESS_DELIVERY_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[327, 100, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[328, 100, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[329, 101, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[330, 101, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[331, 102, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[332, 102, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[333, 103, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[334, 103, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[335, 98, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[336, 98, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[337, 100, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[338, 100, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[339, 101, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[340, 101, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[341, 102, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[342, 102, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[343, 97, 'LBL_ATTENTION_BLOCK', 5, 0, 0, 0, 0, 0, 0, 0],
					[344, 103, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[345, 103, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[346, 97, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[347, 104, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[348, 104, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[349, 104, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[350, 104, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[351, 105, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[352, 105, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[353, 105, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[354, 105, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[355, 106, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[356, 106, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[357, 106, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[358, 106, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[359, 96, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 1, 1],
					[360, 96, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 1, 1],
					[361, 107, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[362, 107, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[363, 107, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[364, 107, 'LBL_ADDRESS_DELIVERY_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[365, 108, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[366, 108, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[367, 108, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[368, 108, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[369, 109, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[370, 109, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[371, 109, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 0, 0],
					[372, 109, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 0, 0],
					[374, 111, 'LBL_NOTIFICATION_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[375, 111, 'LBL_NOTIFICATION_CONTENTS', 2, 0, 0, 0, 0, 0, 1, 0],
					[376, 112, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[377, 112, 'LBL_CONTENT_MAIL', 2, 0, 0, 0, 0, 0, 1, 0],
					[378, 112, 'LBL_CUSTOM_INFORMATION', 3, 0, 0, 0, 0, 0, 0, 1],
					[379, 113, 'LBL_BASIC_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[380, 113, 'LBL_VEHICLE', 2, 0, 0, 0, 0, 0, 1, 0],
					[381, 113, 'LBL_ATTENTION_BLOCK', 3, 0, 0, 0, 0, 0, 1, 1],
					[382, 113, 'LBL_DESCRIPTION_BLOCK', 4, 0, 0, 0, 0, 0, 1, 1],
					[383, 114, 'LBL_BASIC_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[384, 114, 'LBL_ADDITIONAL_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[385, 114, 'LBL_DESCRIPTION_BLOCK', 3, 0, 0, 0, 0, 0, 1, 1],
					[386, 114, 'LBL_ATTENTION_BLOCK', 4, 0, 0, 0, 0, 0, 1, 1],
					[387, 114, 'LBL_TICKET_RESOLUTION', 5, 0, 0, 0, 0, 0, 1, 1],
					[388, 115, 'LBL_BASIC_DETAILS', 1, 0, 0, 0, 0, 0, 1, 0],
					[389, 115, 'LBL_CUSTOM_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 0],
					[390, 115, 'LBL_ADDRESS_INFORMATION', 3, 0, 0, 0, 0, 0, 1, 1],
					[391, 115, 'LBL_ADDRESS_DELIVERY_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
					[392, 115, 'LBL_DESCRIPTION_BLOCK', 5, 0, 0, 0, 0, 0, 1, 1],
					[393, 115, 'LBL_ATTENTION_BLOCK', 6, 0, 0, 0, 0, 0, 1, 1],
					[394, 116, 'LBL_BASIC_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[395, 116, 'LBL_ATTENTION_BLOCK', 3, 0, 0, 0, 0, 0, 1, 0],
					[396, 116, 'LBL_DESCRIPTION_BLOCK', 4, 0, 0, 0, 0, 0, 1, 1],
					[397, 116, 'LBL_ADDITIONAL_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[398, 95, 'LBL_DESCRIPTION_BLOCK', 5, 0, 0, 0, 0, 0, 1, 1],
					[399, 95, 'LBL_ATTENTION_BLOCK', 6, 0, 0, 0, 0, 0, 1, 1],
					[400, 117, 'LBL_SCALCULATIONS_INFORMATION', 1, 0, 0, 0, 0, 0, 1, 0],
					[401, 117, 'LBL_DESCRIPTION_INFORMATION', 2, 0, 0, 0, 0, 0, 1, 1],
					[402, 117, 'LBL_STATISTICS', 3, 0, 0, 0, 0, 0, 1, 1],
					[403, 117, 'LBL_CUSTOM_INFORMATION', 4, 0, 0, 0, 0, 0, 1, 1],
				]
			],
			'vtiger_calendar_config' => [
				'columns' => ['type', 'name', 'label', 'value'],
				'values' => [
					['colors', 'Task', 'Task', '#00d4f5'],
					['reminder', 'update_event', 'LBL_UPDATE_EVENT', '0'],
					['info', 'notworkingdays', 'LBL_NOTWORKING_DAYS', null],
				]
			],
			'vtiger_callduration' => [
				'columns' => ['calldurationid', 'callduration', 'sortorderid', 'presence'],
				'values' => [
					[1, '5', 1, 1],
					[2, '10', 2, 1],
					[3, '30', 3, 1],
					[4, '60', 4, 1],
					[5, '120', 5, 1],
				]
			],
			'vtiger_callduration_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_callhistorytype' => [
				'columns' => ['callhistorytypeid', 'callhistorytype', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Outgoing missed', 1, 1],
					[2, 'Outgoing received', 2, 1],
					[3, 'Incoming', 3, 1],
					[4, 'Missed', 4, 1],
					[5, 'Voicemail', 5, 1],
				]
			],
			'vtiger_callhistorytype_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_campaignstatus' => [
				'columns' => ['campaignstatusid', 'campaignstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Planning', 1, 15, 1],
					[3, 'Active', 1, 16, 2],
					[4, 'Inactive', 1, 17, 3],
					[5, 'Completed', 1, 18, 4],
					[6, 'Cancelled', 1, 19, 5],
				]
			],
			'vtiger_campaignstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_campaigntype' => [
				'columns' => ['campaigntypeid', 'campaigntype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Conference', 1, 21, 1],
					[3, 'Webinar', 1, 22, 2],
					[4, 'Trade Show', 1, 23, 3],
					[5, 'Public Relations', 1, 24, 4],
					[6, 'Partners', 1, 25, 5],
					[7, 'Referral Program', 1, 26, 6],
					[8, 'Advertisement', 1, 27, 7],
					[9, 'Banner Ads', 1, 28, 8],
					[10, 'Direct Mail', 1, 29, 9],
					[11, 'Email', 1, 30, 10],
					[12, 'Telemarketing', 1, 31, 11],
					[13, 'Others', 1, 32, 12],
				]
			],
			'vtiger_campaigntype_seq' => [
				'columns' => ['id'],
				'values' => [
					[13],
				]
			],
			'vtiger_cmileage_logbook_status' => [
				'columns' => ['cmileage_logbook_statusid', 'cmileage_logbook_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 760, 1],
					[2, 'PLL_FOR_APPROVAL', 1, 761, 2],
					[3, 'PLL_ACCEPTED', 1, 762, 3],
					[4, 'PLL_CANCELLED', 1, 763, 4],
				]
			],
			'vtiger_contactstatus' => [
				'columns' => ['contactstatusid', 'contactstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Active', 1, 559, 1],
					[2, 'Inactive', 1, 560, 2],
				]
			],
			'vtiger_contactstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_contract_priority' => [
				'columns' => ['contract_priorityid', 'contract_priority', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Low', 1, 220, 1],
					[2, 'Normal', 1, 221, 2],
					[3, 'High', 1, 222, 3],
				]
			],
			'vtiger_contract_priority_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_contract_status' => [
				'columns' => ['contract_statusid', 'contract_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Undefined', 1, 214, 1],
					[2, 'In Planning', 1, 215, 2],
					[3, 'In Progress', 1, 216, 3],
					[4, 'On Hold', 1, 217, 4],
					[5, 'Complete', 0, 218, 5],
					[6, 'Archived', 1, 219, 6],
				]
			],
			'vtiger_contract_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_contract_type' => [
				'columns' => ['contract_typeid', 'contract_type', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Support', 1, 223, 1],
					[2, 'Services', 1, 224, 2],
					[3, 'Administrative', 1, 225, 3],
				]
			],
			'vtiger_contract_type_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_convertleadmapping' => [
				'columns' => ['cfmid', 'leadfid', 'accountfid', 'editable'],
				'values' => [
					[1, 43, 1, 0],
					[2, 49, 14, 1],
					[3, 40, 3, 1],
					[5, 44, 5, 1],
					[7, 46, 9, 0],
					[8, 48, 4, 1],
					[21, 65, 36, 1],
					[24, 41, 0, 0],
					[25, 42, 0, 1],
					[28, 47, 0, 1],
					[30, 53, 10, 1],
					[31, 51, 17, 1],
					[32, 1291, 1285, 1],
					[33, 1292, 1286, 1],
					[34, 1065, 1089, 1],
					[35, 1066, 1091, 1],
					[36, 1067, 1093, 1],
					[37, 1068, 1095, 1],
					[38, 1069, 1097, 1],
					[39, 1070, 1099, 1],
					[40, 1071, 1101, 1],
					[41, 1072, 1103, 1],
					[42, 967, 970, 1],
					[43, 969, 972, 1],
					[44, 1737, 1738, 1],
				]
			],
			'vtiger_crmentity' => [
				'columns' => ['crmid', 'smcreatorid', 'smownerid', 'shownerid', 'modifiedby', 'setype', 'description', 'attention', 'createdtime', 'modifiedtime', 'viewedtime', 'closedtime', 'status', 'version', 'presence', 'deleted', 'was_read', 'private', 'users'],
				'values' => [
					[35, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:48:32', '2017-01-11 15:10:34', null, null, null, 0, 1, 0, null, 0, null],
					[36, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:51:21', '2017-01-11 10:30:37', null, null, null, 0, 1, 0, null, 0, null],
					[37, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:53:15', '2017-01-11 10:28:40', null, null, null, 0, 1, 0, null, 0, null],
					[38, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:55:03', '2017-01-11 10:33:57', null, null, null, 0, 1, 0, null, 0, null],
					[39, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:57:26', '2017-01-11 10:59:31', '2015-10-23 08:33:00', null, null, 0, 1, 0, 0, 0, null],
					[40, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-22 23:58:38', '2017-01-11 10:45:46', '2015-10-23 08:26:04', null, null, 0, 1, 0, null, 0, null],
					[41, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 00:05:38', '2017-01-11 10:59:37', null, null, null, 0, 1, 0, null, 0, null],
					[42, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 00:06:31', '2017-01-11 11:04:07', null, null, null, 0, 1, 0, null, 0, null],
					[43, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 00:07:37', '2017-01-11 11:15:12', null, null, null, 0, 1, 0, null, 0, null],
					[44, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 21:39:36', '2017-01-11 11:18:45', null, null, null, 0, 1, 0, null, 0, null],
					[45, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 22:57:01', '2017-01-11 11:46:19', null, null, null, 0, 1, 0, null, 0, null],
					[46, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-23 23:04:19', '2017-01-11 12:12:54', null, null, null, 0, 1, 0, null, 0, null],
					[93, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-24 20:51:12', '2017-01-11 12:16:56', null, null, null, 0, 1, 0, null, 0, null],
					[94, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-24 20:52:22', '2017-01-11 12:23:54', null, null, null, 0, 1, 0, null, 0, null],
					[95, 1, 1, 0, 1, 'EmailTemplates', null, null, '2014-09-24 21:58:51', '2017-01-11 12:44:36', null, null, null, 0, 1, 0, null, 0, null],
					[103, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-01-24 11:33:24', '2017-01-11 12:50:01', '2015-02-12 08:37:31', null, null, 0, 1, 0, null, 0, null],
					[104, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-02-12 09:08:42', '2017-01-11 12:53:07', '2015-02-12 09:26:03', null, null, 0, 1, 0, 0, 0, null],
					[105, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-02-17 10:11:08', '2017-01-11 13:02:35', '2015-10-23 08:35:22', null, null, 0, 1, 0, 0, 0, null],
					[106, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-02-18 08:35:26', '2017-01-11 13:06:50', '2015-10-23 08:34:38', null, null, 0, 1, 0, 0, 0, null],
					[107, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-02-26 09:59:53', '2017-01-11 13:09:22', '2015-02-26 09:59:53', null, null, 0, 1, 0, 0, 0, null],
					[109, 1, 1, 0, 1, 'EmailTemplates', null, null, '2015-10-23 08:16:58', '2017-01-11 13:25:18', '2015-10-23 08:34:54', null, null, 0, 1, 0, 0, 0, null],
					[110, 1, 1, null, 1, 'EmailTemplates', null, null, '2016-05-24 12:40:24', '2017-01-11 13:35:37', '2016-05-24 12:40:24', null, null, 0, 1, 0, 0, 0, null],
					[111, 1, 1, null, 1, 'EmailTemplates', null, null, '2017-01-23 13:49:51', '2017-01-23 14:25:51', null, null, null, 0, 1, 0, 0, 0, ',1,'],
					[112, 1, 1, null, 1, 'EmailTemplates', null, null, '2017-06-07 00:42:02', '2017-06-07 00:42:02', null, null, null, 0, 1, 0, 0, 0, ',1,'],
				]
			],
			'vtiger_cron_task' => [
				'columns' => ['id', 'name', 'handler_file', 'frequency', 'laststart', 'lastend', 'status', 'module', 'sequence', 'description'],
				'values' => [
					[1, 'LBL_WORKFLOW', 'cron/modules/com_vtiger_workflow/com_vtiger_workflow.php', 900, null, null, 1, 'com_vtiger_workflow', 1, 'Recommended frequency for Workflow is 15 mins'],
					[2, 'LBL_ADDRESS_BOOK', 'cron/AddressBook.php', 86400, null, null, 1, 'Vtiger', 22, ''],
					[3, 'LBL_SEND_REMINDER', 'cron/SendReminder.php', 900, null, null, 1, 'Calendar', 12, 'Recommended frequency for SendReminder is 15 mins'],
					[4, 'LBL_CURRENCY_UPDATE', 'modules/Settings/CurrencyUpdate/cron/CurrencyUpdateCron.php', 86400, null, null, 1, 'CurrencyUpdate', 3, 'Recommended frequency for Currency Update is 24 hours'],
					[5, 'LBL_MAILER', 'cron/Mailer.php', 300, null, null, 1, 'Vtiger', 8, null],
					[7, 'LBL_BROWSING_HISTORY', 'cron/BrowsingHistory.php', 86400, null, null, 1, 'Vtiger', 29, null],
					[8, 'LBL_SCHEDULED_IMPORT', 'cron/modules/Import/ScheduledImport.php', 900, null, null, 1, 'Import', 4, 'Recommended frequency for MailScanner is 15 mins'],
					[9, 'LBL_SCHEDULE_REPORTS', 'cron/modules/Reports/ScheduleReports.php', 900, null, null, 1, 'Reports', 24, 'Recommended frequency for ScheduleReports is 15 mins'],
					[10, 'LBL_MAIL_SCANNER_ACTION', 'modules/OSSMailScanner/cron/action.php', 200, null, null, 1, 'OSSMailScanner', 5, ''],
					[11, 'LBL_MAIL_SCANNER_VERIFICATION', 'modules/OSSMailScanner/cron/verification.php', 100, null, null, 1, 'OSSMailScanner', 6, ''],
					[12, 'LBL_MAIL_SCANNER_BIND', 'modules/OSSMailScanner/cron/bind.php', 600, null, null, 1, 'OSSMailScanner', 7, ''],
					[13, 'LBL_BATCH_PROCESSES', 'cron/BatchProcesses.php', 600, null, null, 1, 'Vtiger', 30, null],
					[14, 'LBL_CARD_DAV', 'modules/API/cron/CardDav.php', 300, null, null, 1, 'Contacts', 9, null],
					[15, 'LBL_CAL_DAV', 'modules/API/cron/CalDav.php', 300, null, null, 1, 'Calendar', 10, null],
					[16, 'LBL_ACTIVITY_STATE', 'modules/Calendar/cron/ActivityState.php', 1800, null, null, 1, 'Calendar', 11, ''],
					[17, 'LBL_MULTI_REFERENCE_VALUE', 'cron/MultiReference.php', 900, null, null, 1, 'com_vtiger_workflow', 14, null],
					[18, 'LBL_CRMACTIVITY_DAYS', 'modules/Calendar/cron/SetCrmActivity.php', 86400, null, null, 1, 'Calendar', 15, ''],
					[19, 'LBL_ASSETS_RENEWAL', 'modules/Assets/cron/Renewal.php', 86400, null, null, 1, 'Assets', 16, ''],
					[20, 'LBL_SOLD_SERVICES_RENEWAL', 'modules/OSSSoldServices/cron/Renewal.php', 86400, null, null, 1, 'OSSSoldServices', 17, ''],
					[21, 'LBL_SEND_NOTIFICATIONS', 'modules/Notification/cron/Notifications.php', 900, null, null, 1, 'Notification', 13, ''],
					[22, 'LBL_RECORD_LABEL_UPDATER', 'cron/LabelUpdater.php', 900, null, null, 1, 'Vtiger', 18, ''],
					[23, 'LBL_PRIVILEGES_UPDATER', 'cron/PrivilegesUpdater.php', 900, null, null, 1, 'Vtiger', 19, ''],
					[24, 'LBL_UPDATER_COORDINATES', 'modules/OpenStreetMap/cron/UpdaterCoordinates.php', 60, null, null, 1, 'OpenStreetMap', 20, ''],
					[25, 'LBL_UPDATER_RECORDS_COORDINATES', 'modules/OpenStreetMap/cron/UpdaterRecordsCoordinates.php', 300, null, null, 1, 'OpenStreetMap', 21, ''],
					[26, 'LBL_MARK_RECORDS_AS_REVIEWED', 'modules/ModTracker/cron/ReviewChanges.php', 900, null, null, 1, 'ModTracker', 23, ''],
					[27, 'LBL_CACHE', 'cron/Cache.php', 86400, null, null, 1, 'Vtiger', 25, null],
					[28, 'LBL_NEVER_ENDING_RECURRING_EVENTS', 'modules/Events/cron/RecurringEvents.php', 86400, null, null, 1, 'Events', 26, null],
					[29, 'LBL_CLEAR_ATTACHMENTS_TABLE', 'cron/Attachments.php', 86400, null, null, 1, 'Vtiger', 27, null],
					[30, 'LBL_SMSNOTIFIER', 'modules/SMSNotifier/cron/SMSNotifier.php', 300, null, null, 1, 'SMSNotifier', 28, null],
					[31, 'LBK_SYSTEM_WARNINGS', 'cron/SystemWarnings.php', 86400, null, null, 1, 'Vtiger', 29, null],
					[32, 'LBL_BATCH_METHODS', 'cron/BatchMethods.php', 900, null, null, 1, 'Vtiger', 31, null],
				]
			],
			'vtiger_currencies' => [
				'columns' => ['currencyid', 'currency_name', 'currency_code', 'currency_symbol'],
				'values' => [
					[1, 'Albania, Leke', 'ALL', 'Lek'],
					[2, 'Argentina, Pesos', 'ARS', '$'],
					[3, 'Aruba, Guilders', 'AWG', 'ƒ'],
					[4, 'Australia, Dollars', 'AUD', '$'],
					[5, 'Azerbaijan, New Manats', 'AZN', 'ман'],
					[6, 'Bahamas, Dollars', 'BSD', '$'],
					[7, 'Bahrain, Dinar', 'BHD', 'BD'],
					[8, 'Barbados, Dollars', 'BBD', '$'],
					[9, 'Belarus, Rubles', 'BYR', 'p.'],
					[10, 'Belize, Dollars', 'BZD', 'BZ$'],
					[11, 'Bermuda, Dollars', 'BMD', '$'],
					[12, 'Bolivia, Bolivianos', 'BOB', '$b'],
					[13, 'China, Yuan Renminbi', 'CNY', '¥'],
					[14, 'Convertible Marka', 'BAM', 'KM'],
					[15, 'Botswana, Pulas', 'BWP', 'P'],
					[16, 'Bulgaria, Leva', 'BGN', 'лв'],
					[17, 'Brazil, Reais', 'BRL', 'R$'],
					[18, 'Great Britain Pounds', 'GBP', '£'],
					[19, 'Brunei Darussalam, Dollars', 'BND', '$'],
					[20, 'Canada, Dollars', 'CAD', '$'],
					[21, 'Cayman Islands, Dollars', 'KYD', '$'],
					[22, 'Chile, Pesos', 'CLP', '$'],
					[23, 'Colombia, Pesos', 'COP', '$'],
					[24, 'Costa Rica, Colón', 'CRC', '₡'],
					[25, 'Croatia, Kuna', 'HRK', 'kn'],
					[26, 'Cuba, Pesos', 'CUP', '₱'],
					[27, 'Czech Republic, Koruny', 'CZK', 'Kč'],
					[28, 'Cyprus, Pounds', 'CYP', '£'],
					[29, 'Denmark, Kroner', 'DKK', 'kr'],
					[30, 'Dominican Republic, Pesos', 'DOP', 'RD$'],
					[31, 'East Caribbean, Dollars', 'XCD', '$'],
					[32, 'Egypt, Pounds', 'EGP', '£'],
					[33, 'El Salvador, Colón', 'SVC', '₡'],
					[34, 'England, Pounds', 'GBP', '£'],
					[35, 'Estonia, Krooni', 'EEK', 'kr'],
					[36, 'Euro', 'EUR', '€'],
					[37, 'Falkland Islands, Pounds', 'FKP', '£'],
					[38, 'Fiji, Dollars', 'FJD', '$'],
					[39, 'Ghana, Cedis', 'GHC', '¢'],
					[40, 'Gibraltar, Pounds', 'GIP', '£'],
					[41, 'Guatemala, Quetzales', 'GTQ', 'Q'],
					[42, 'Guernsey, Pounds', 'GGP', '£'],
					[43, 'Guyana, Dollars', 'GYD', '$'],
					[44, 'Honduras, Lempiras', 'HNL', 'L'],
					[45, 'Hong Kong, Dollars', 'HKD', 'HK$'],
					[46, 'Hungary, Forint', 'HUF', 'Ft'],
					[47, 'Iceland, Krona', 'ISK', 'kr'],
					[48, 'India, Rupees', 'INR', '₹'],
					[49, 'Indonesia, Rupiahs', 'IDR', 'Rp'],
					[50, 'Iran, Rials', 'IRR', '﷼'],
					[51, 'Isle of Man, Pounds', 'IMP', '£'],
					[52, 'Israel, New Shekels', 'ILS', '₪'],
					[53, 'Jamaica, Dollars', 'JMD', 'J$'],
					[54, 'Japan, Yen', 'JPY', '¥'],
					[55, 'Jersey, Pounds', 'JEP', '£'],
					[56, 'Jordan, Dinar', 'JOD', 'JOD'],
					[57, 'Kazakhstan, Tenge', 'KZT', '〒'],
					[58, 'Kenya, Shilling', 'KES', 'KES'],
					[59, 'Korea (North), Won', 'KPW', '₩'],
					[60, 'Korea (South), Won', 'KRW', '₩'],
					[61, 'Kuwait, Dinar', 'KWD', 'KWD'],
					[62, 'Kyrgyzstan, Soms', 'KGS', 'лв'],
					[63, 'Laos, Kips', 'LAK', '₭'],
					[64, 'Latvia, Lati', 'LVL', 'Ls'],
					[65, 'Lebanon, Pounds', 'LBP', '£'],
					[66, 'Liberia, Dollars', 'LRD', '$'],
					[67, 'Switzerland Francs', 'CHF', 'CHF'],
					[68, 'Lithuania, Litai', 'LTL', 'Lt'],
					[69, 'MADAGASCAR, Malagasy Ariary', 'MGA', 'MGA'],
					[70, 'Macedonia, Denars', 'MKD', 'ден'],
					[71, 'Malaysia, Ringgits', 'MYR', 'RM'],
					[72, 'Malta, Liri', 'MTL', '₤'],
					[73, 'Mauritius, Rupees', 'MUR', '₨'],
					[74, 'Mexico, Pesos', 'MXN', '$'],
					[75, 'Mongolia, Tugriks', 'MNT', '₮'],
					[76, 'Mozambique, Meticais', 'MZN', 'MT'],
					[77, 'Namibia, Dollars', 'NAD', '$'],
					[78, 'Nepal, Rupees', 'NPR', '₨'],
					[79, 'Netherlands Antilles, Guilders', 'ANG', 'ƒ'],
					[80, 'New Zealand, Dollars', 'NZD', '$'],
					[81, 'Nicaragua, Cordobas', 'NIO', 'C$'],
					[82, 'Nigeria, Nairas', 'NGN', '₦'],
					[83, 'North Korea, Won', 'KPW', '₩'],
					[84, 'Norway, Krone', 'NOK', 'kr'],
					[85, 'Oman, Rials', 'OMR', '﷼'],
					[86, 'Pakistan, Rupees', 'PKR', '₨'],
					[87, 'Panama, Balboa', 'PAB', 'B/.'],
					[88, 'Paraguay, Guarani', 'PYG', 'Gs'],
					[89, 'Peru, Nuevos Soles', 'PEN', 'S/.'],
					[90, 'Philippines, Pesos', 'PHP', 'Php'],
					[91, 'Poland, Zlotych', 'PLN', 'zł'],
					[92, 'Qatar, Rials', 'QAR', '﷼'],
					[93, 'Romania, New Lei', 'RON', 'lei'],
					[94, 'Russia, Rubles', 'RUB', '₽'],
					[95, 'Saint Helena, Pounds', 'SHP', '£'],
					[96, 'Saudi Arabia, Riyals', 'SAR', '﷼'],
					[97, 'Serbia, Dinars', 'RSD', 'Дин.'],
					[98, 'Seychelles, Rupees', 'SCR', '₨'],
					[99, 'Singapore, Dollars', 'SGD', '$'],
					[100, 'Solomon Islands, Dollars', 'SBD', '$'],
					[101, 'Somalia, Shillings', 'SOS', 'S'],
					[102, 'South Africa, Rand', 'ZAR', 'R'],
					[103, 'South Korea, Won', 'KRW', '₩'],
					[104, 'Sri Lanka, Rupees', 'LKR', '₨'],
					[105, 'Sweden, Kronor', 'SEK', 'kr'],
					[106, 'Switzerland, Francs', 'CHF', 'CHF'],
					[107, 'Suriname, Dollars', 'SRD', '$'],
					[108, 'Syria, Pounds', 'SYP', '£'],
					[109, 'Taiwan, New Dollars', 'TWD', 'NT$'],
					[110, 'Thailand, Baht', 'THB', '฿'],
					[111, 'Trinidad and Tobago, Dollars', 'TTD', 'TT$'],
					[112, 'Turkey, New Lira', 'TRY', 'YTL'],
					[113, 'Turkey, Liras', 'TRL', '₤'],
					[114, 'Tuvalu, Dollars', 'TVD', '$'],
					[115, 'Ukraine, Hryvnia', 'UAH', '₴'],
					[116, 'United Arab Emirates, Dirham', 'AED', 'AED'],
					[117, 'United Kingdom, Pounds', 'GBP', '£'],
					[118, 'United Republic of Tanzania, Shilling', 'TZS', 'TZS'],
					[119, 'USA, Dollars', 'USD', '$'],
					[120, 'Uruguay, Pesos', 'UYU', '$U'],
					[121, 'Uzbekistan, Sums', 'UZS', 'лв'],
					[122, 'Venezuela, Bolivares Fuertes', 'VEF', 'Bs'],
					[123, 'Vietnam, Dong', 'VND', '₫'],
					[124, 'Zambia, Kwacha', 'ZMK', 'ZMK'],
					[125, 'Yemen, Rials', 'YER', '﷼'],
					[126, 'Zimbabwe Dollars', 'ZWD', 'Z$'],
					[127, 'Malawi, Kwacha', 'MWK', 'MK'],
					[128, 'Tunisian, Dinar', 'TD', 'TD'],
					[129, 'Moroccan, Dirham', 'MAD', 'DH'],
					[130, 'Iraqi Dinar', 'IQD', 'ID'],
					[131, 'Maldivian Ruffiya', 'MVR', 'MVR'],
					[132, 'Ugandan Shilling', 'UGX', 'Sh'],
					[133, 'Sudanese Pound', 'SDG', '£'],
					[134, 'CFA Franc BCEAO', 'XOF', 'CFA'],
					[135, 'CFA Franc BEAC', 'XAF', 'CFA'],
					[136, 'Haiti, Gourde', 'HTG', 'G'],
					[137, 'Libya, Dinar', 'LYD', 'LYD'],
					[138, 'CFP Franc', 'XPF', 'F'],
				]
			],
			'vtiger_currencies_seq' => [
				'columns' => ['id'],
				'values' => [
					[138],
				]
			],
			'vtiger_currency_decimal_separator' => [
				'columns' => ['currency_decimal_separatorid', 'currency_decimal_separator', 'sortorderid', 'presence'],
				'values' => [
					[1, '.', 0, 1],
					[2, ',', 1, 1],
					[3, '\'', 2, 1],
					[4, ' ', 3, 1],
					[5, '$', 4, 1],
				]
			],
			'vtiger_currency_decimal_separator_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_currency_grouping_pattern' => [
				'columns' => ['currency_grouping_patternid', 'currency_grouping_pattern', 'sortorderid', 'presence'],
				'values' => [
					[1, '123,456,789', 0, 1],
					[2, '123456789', 1, 1],
					[3, '123456,789', 2, 1],
					[4, '12,34,56,789', 3, 1],
				]
			],
			'vtiger_currency_grouping_pattern_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_currency_grouping_separator' => [
				'columns' => ['currency_grouping_separatorid', 'currency_grouping_separator', 'sortorderid', 'presence'],
				'values' => [
					[1, '.', 2, 1],
					[2, ',', 3, 1],
					[3, '\'', 4, 1],
					[4, ' ', 1, 1],
					[5, '$', 5, 1],
				]
			],
			'vtiger_currency_grouping_separator_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_currency_info' => [
				'columns' => ['id', 'currency_name', 'currency_code', 'currency_symbol', 'conversion_rate', 'currency_status', 'defaultid', 'deleted'],
				'values' => [
					[1, 'Poland, Zlotych', 'PLN', 'zł', '1.00000', 'Active', -11, 0],
				]
			],
			'vtiger_currency_symbol_placement' => [
				'columns' => ['currency_symbol_placementid', 'currency_symbol_placement', 'sortorderid', 'presence'],
				'values' => [
					[1, '$1.0', 0, 1],
					[2, '1.0$', 1, 1],
				]
			],
			'vtiger_currency_symbol_placement_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_customview' => [
				'columns' => ['cvid', 'viewname', 'setdefault', 'setmetrics', 'entitytype', 'status', 'userid', 'privileges', 'featured', 'sequence', 'presence', 'description', 'sort', 'color'],
				'values' => [
					[1, 'All', 1, 0, 'Leads', 0, 1, 0, 0, null, 0, null, '', ''],
					[4, 'All', 1, 0, 'Accounts', 0, 1, 0, 0, null, 0, null, '', ''],
					[7, 'All', 1, 0, 'Contacts', 0, 1, 0, 0, null, 0, null, '', ''],
					[13, 'All', 1, 0, 'HelpDesk', 0, 1, 0, 0, null, 0, null, '', ''],
					[19, 'All', 1, 0, 'Calendar', 0, 1, 0, 0, null, 0, null, '', ''],
					[22, 'All', 1, 0, 'Documents', 0, 1, 0, 0, null, 0, null, '', ''],
					[23, 'All', 1, 0, 'PriceBooks', 0, 1, 0, 0, null, 0, null, '', ''],
					[24, 'All', 1, 0, 'Products', 0, 1, 0, 0, null, 0, null, '', ''],
					[27, 'All', 1, 0, 'Vendors', 0, 1, 0, 0, null, 0, null, '', ''],
					[28, 'All', 1, 0, 'Faq', 0, 1, 0, 0, null, 0, null, '', ''],
					[29, 'All', 1, 0, 'Campaigns', 0, 1, 0, 0, null, 0, null, '', ''],
					[39, 'All', 1, 0, 'ServiceContracts', 0, 1, 0, 0, null, 0, null, '', ''],
					[40, 'All', 1, 0, 'Services', 0, 1, 0, 0, null, 0, null, '', ''],
					[41, 'All', 1, 0, 'Assets', 0, 1, 0, 0, null, 0, null, '', ''],
					[42, 'All', 0, 0, 'ModComments', 0, 1, 0, 0, null, 0, null, '', ''],
					[43, 'All', 1, 0, 'ProjectMilestone', 0, 1, 0, 0, null, 0, null, '', ''],
					[44, 'All', 1, 0, 'ProjectTask', 0, 1, 0, 0, null, 0, null, '', ''],
					[45, 'All', 1, 0, 'Project', 0, 1, 0, 0, null, 0, null, '', ''],
					[46, 'All', 0, 0, 'SMSNotifier', 0, 1, 0, 0, null, 0, null, '', ''],
					[49, 'All', 1, 0, 'OSSTimeControl', 0, 1, 0, 0, null, 0, null, '', ''],
					[50, 'All', 1, 0, 'OSSMailView', 0, 1, 0, 0, null, 0, null, '', ''],
					[52, 'All', 1, 0, 'OSSOutsourcedServices', 0, 1, 0, 0, null, 0, null, '', ''],
					[53, 'All', 1, 0, 'OSSSoldServices', 0, 1, 0, 0, null, 0, null, '', ''],
					[54, 'All', 1, 0, 'OutsourcedProducts', 0, 1, 0, 0, null, 0, null, '', ''],
					[55, 'All', 1, 0, 'OSSPasswords', 0, 1, 0, 0, null, 0, null, '', ''],
					[56, 'All', 1, 0, 'OSSEmployees', 0, 1, 0, 0, null, 0, null, '', ''],
					[68, 'All', 1, 0, 'CallHistory', 0, 1, 1, 0, null, 0, null, '', ''],
					[69, 'All', 1, 0, 'Ideas', 0, 1, 1, 0, null, 0, null, '', ''],
					[72, 'All', 1, 0, 'HolidaysEntitlement', 0, 1, 1, 0, null, 0, null, '', ''],
					[73, 'All', 1, 0, 'PaymentsIn', 0, 1, 1, 0, null, 0, null, '', ''],
					[74, 'All', 1, 0, 'PaymentsOut', 0, 1, 1, 0, null, 0, null, '', ''],
					[75, 'All', 1, 0, 'LettersIn', 0, 1, 1, 0, null, 0, null, '', ''],
					[76, 'All', 1, 0, 'LettersOut', 0, 1, 1, 0, null, 0, null, '', ''],
					[77, 'All', 1, 0, 'Announcements', 0, 1, 1, 0, null, 0, null, '', ''],
					[78, 'All', 1, 0, 'Reservations', 0, 1, 1, 0, null, 0, null, '', ''],
					[79, 'All', 1, 0, 'SQuoteEnquiries', 0, 1, 1, 0, null, 0, null, '', ''],
					[80, 'All', 1, 0, 'SSalesProcesses', 0, 1, 1, 0, null, 0, null, '', ''],
					[81, 'All', 1, 0, 'SRequirementsCards', 0, 1, 1, 0, null, 0, null, '', ''],
					[82, 'All', 1, 0, 'SCalculations', 0, 1, 1, 0, null, 0, null, '', ''],
					[83, 'All', 1, 0, 'SQuotes', 0, 1, 1, 0, null, 0, null, '', ''],
					[84, 'All', 1, 0, 'SSingleOrders', 0, 1, 1, 0, null, 0, null, '', ''],
					[85, 'All', 1, 0, 'SRecurringOrders', 0, 1, 1, 0, null, 0, null, '', ''],
					[86, 'All', 1, 0, 'Partners', 0, 1, 1, 0, null, 0, null, '', ''],
					[87, 'All', 1, 0, 'Competition', 0, 1, 1, 0, null, 0, null, '', ''],
					[88, 'All', 1, 0, 'FBookkeeping', 0, 1, 1, 0, null, 0, null, '', ''],
					[89, 'All', 1, 0, 'FInvoice', 0, 1, 1, 0, null, 0, null, '', ''],
					[90, 'All', 1, 0, 'KnowledgeBase', 0, 1, 1, 0, null, 0, null, '', ''],
					[91, 'All', 1, 0, 'IStorages', 0, 1, 1, 0, null, 0, null, '', ''],
					[92, 'All', 1, 0, 'IGRN', 0, 1, 1, 0, null, 0, null, '', ''],
					[93, 'All', 1, 0, 'FInvoiceProforma', 0, 1, 1, 0, null, 0, null, '', ''],
					[94, 'All', 1, 0, 'IGDN', 0, 1, 1, 0, null, 0, null, '', ''],
					[95, 'All', 1, 0, 'IIDN', 0, 1, 1, 0, null, 0, null, '', ''],
					[96, 'All', 1, 0, 'IGIN', 0, 1, 1, 0, null, 0, null, '', ''],
					[97, 'All', 1, 0, 'IPreOrder', 0, 1, 1, 0, null, 0, null, '', ''],
					[98, 'All', 1, 0, 'ISTDN', 0, 1, 1, 0, null, 0, null, '', ''],
					[99, 'All', 1, 0, 'ISTN', 0, 1, 1, 0, null, 0, null, '', ''],
					[100, 'All', 1, 0, 'ISTRN', 0, 1, 1, 0, null, 0, null, '', ''],
					[101, 'All', 1, 0, 'FCorectingInvoice', 0, 1, 1, 0, null, 0, null, '', ''],
					[102, 'All', 1, 0, 'IGRNC', 0, 1, 1, 0, null, 0, null, '', ''],
					[103, 'All', 1, 0, 'IGDNC', 0, 1, 1, 0, null, 0, null, '', ''],
					[104, 'LBL_UNREAD', 1, 0, 'Notification', 0, 1, 1, 0, 1, 0, null, null, ''],
					[105, 'All', 1, 0, 'EmailTemplates', 0, 1, 1, 0, 0, 0, '', null, ''],
					[106, 'All', 1, 0, 'CFixedAssets', 0, 1, 1, 0, 0, 0, '', '', ''],
					[107, 'All', 1, 0, 'CInternalTickets', 0, 1, 1, 0, 0, 0, '', '', ''],
					[108, 'All', 1, 0, 'FInvoiceCost', 0, 1, 1, 0, 0, 0, '', '', ''],
					[109, 'All', 1, 0, 'CMileageLogbook', 0, 1, 1, 0, 0, 0, '', '', ''],
					[110, 'All', 1, 0, 'SVendorEnquiries', 0, 1, 1, 0, 0, 0, '', '', ''],
				]
			],
			'vtiger_cvadvfilter' => [
				'columns' => ['cvid', 'columnindex', 'columnname', 'comparator', 'value', 'groupid', 'column_condition'],
				'values' => [
					[104, 0, 'u_yf_notification:notification_status:notification_status:Notification_FL_STATUS:V', 'e', 'PLL_UNREAD', 1, ''],
				]
			],
			'vtiger_cvadvfilter_grouping' => [
				'columns' => ['groupid', 'cvid', 'group_condition', 'condition_expression'],
				'values' => [
					[1, 104, 'and', ' 0 '],
					[2, 104, '', ' 0 '],
				]
			],
			'vtiger_cvcolumnlist' => [
				'columns' => ['cvid', 'columnindex', 'columnname'],
				'values' => [
					[1, 1, 'vtiger_leaddetails:firstname:firstname:Leads_First_Name:V'],
					[1, 2, 'vtiger_leaddetails:lastname:lastname:Leads_Last_Name:V'],
					[1, 3, 'vtiger_leaddetails:company:company:Leads_Company:V'],
					[1, 4, 'vtiger_leadaddress:phone:phone:Leads_Phone:V'],
					[1, 5, 'vtiger_leadsubdetails:website:website:Leads_Website:V'],
					[1, 6, 'vtiger_leaddetails:email:email:Leads_Email:V'],
					[1, 7, 'vtiger_crmentity:smownerid:assigned_user_id:Leads_Assigned_To:V'],
					[4, 1, 'vtiger_account:accountname:accountname:Accounts_Account_Name:V'],
					[4, 3, 'vtiger_account:website:website:Accounts_Website:V'],
					[4, 4, 'vtiger_account:phone:phone:Accounts_Phone:V'],
					[4, 5, 'vtiger_crmentity:smownerid:assigned_user_id:Accounts_Assigned_To:V'],
					[7, 1, 'vtiger_contactdetails:firstname:firstname:Contacts_First_Name:V'],
					[7, 2, 'vtiger_contactdetails:lastname:lastname:Contacts_Last_Name:V'],
					[7, 4, 'vtiger_contactdetails:parentid:parent_id:Contacts_Member_Of:I'],
					[7, 5, 'vtiger_contactdetails:email:email:Contacts_Email:V'],
					[7, 6, 'vtiger_contactdetails:phone:phone:Contacts_Office_Phone:V'],
					[7, 7, 'vtiger_crmentity:smownerid:assigned_user_id:Contacts_Assigned_To:V'],
					[13, 1, 'vtiger_troubletickets:title:ticket_title:HelpDesk_Title:V'],
					[13, 2, 'vtiger_troubletickets:parent_id:parent_id:HelpDesk_Related_To:V'],
					[13, 3, 'vtiger_troubletickets:status:ticketstatus:HelpDesk_Status:V'],
					[13, 4, 'vtiger_troubletickets:priority:ticketpriorities:HelpDesk_Priority:V'],
					[13, 5, 'vtiger_crmentity:smownerid:assigned_user_id:HelpDesk_Assigned_To:V'],
					[19, 1, 'vtiger_activity:activitytype:activitytype:Calendar_Type:V'],
					[19, 2, 'vtiger_activity:subject:subject:Calendar_Subject:V'],
					[19, 4, 'vtiger_activity:date_start:date_start:Calendar_Start_Date:D'],
					[19, 5, 'vtiger_activity:due_date:due_date:Calendar_End_Date:D'],
					[19, 6, 'vtiger_crmentity:smownerid:assigned_user_id:Calendar_Assigned_To:V'],
					[22, 1, 'vtiger_notes:title:notes_title:Notes_Title:V'],
					[22, 2, 'vtiger_notes:filename:filename:Notes_File:V'],
					[22, 3, 'vtiger_notes:folderid:folderid:Documents_Folder_Name:V'],
					[22, 4, 'vtiger_crmentity:smownerid:assigned_user_id:Notes_Assigned_To:V'],
					[23, 1, 'vtiger_pricebook:bookname:bookname:PriceBooks_Price_Book_Name:V'],
					[23, 2, 'vtiger_pricebook:active:active:PriceBooks_Active:V'],
					[23, 3, 'vtiger_pricebook:currency_id:currency_id:PriceBooks_Currency:V'],
					[24, 0, 'vtiger_products:productname:productname:Products_Product_Name:V'],
					[24, 1, 'vtiger_products:pscategory:pscategory:Products_Product_Category:V'],
					[24, 3, 'vtiger_products:unit_price:unit_price:Products_Unit_Price:N'],
					[24, 4, 'vtiger_products:qtyinstock:qtyinstock:Products_Qty_In_Stock:NN'],
					[24, 5, 'vtiger_products:discontinued:discontinued:Products_Product_Active:C'],
					[24, 6, 'vtiger_products:sales_start_date:sales_start_date:Products_Sales_Start_Date:D'],
					[27, 1, 'vtiger_vendor:vendorname:vendorname:Vendors_Vendor_Name:V'],
					[27, 2, 'vtiger_vendor:phone:phone:Vendors_Phone:V'],
					[27, 3, 'vtiger_vendor:email:email:Vendors_Email:V'],
					[27, 4, 'vtiger_vendor:category:category:Vendors_Category:V'],
					[27, 5, 'vtiger_crmentity:smownerid:assigned_user_id:Vendors_Assigned_To:V'],
					[28, 1, 'vtiger_faq:question:question:Faq_Question:V'],
					[28, 2, 'vtiger_faq:category:faqcategories:Faq_Category:V'],
					[28, 3, 'vtiger_faq:product_id:product_id:Faq_Product_Name:V'],
					[28, 4, 'vtiger_crmentity:createdtime:createdtime:Faq_Created_Time:DT'],
					[28, 5, 'vtiger_crmentity:modifiedtime:modifiedtime:Faq_Modified_Time:DT'],
					[29, 1, 'vtiger_campaign:campaignname:campaignname:Campaigns_Campaign_Name:V'],
					[29, 2, 'vtiger_campaign:campaigntype:campaigntype:Campaigns_Campaign_Type:N'],
					[29, 3, 'vtiger_campaign:campaignstatus:campaignstatus:Campaigns_Campaign_Status:N'],
					[29, 4, 'vtiger_campaign:expectedrevenue:expectedrevenue:Campaigns_Expected_Revenue:V'],
					[29, 5, 'vtiger_campaign:closingdate:closingdate:Campaigns_Expected_Close_Date:D'],
					[29, 6, 'vtiger_crmentity:smownerid:assigned_user_id:Campaigns_Assigned_To:V'],
					[39, 1, 'vtiger_servicecontracts:subject:subject:ServiceContracts_Subject:V'],
					[39, 2, 'vtiger_servicecontracts:sc_related_to:sc_related_to:ServiceContracts_Related_to:V'],
					[39, 3, 'vtiger_crmentity:smownerid:assigned_user_id:ServiceContracts_Assigned_To:V'],
					[39, 4, 'vtiger_servicecontracts:start_date:start_date:ServiceContracts_Start_Date:D'],
					[39, 5, 'vtiger_servicecontracts:due_date:due_date:ServiceContracts_Due_date:D'],
					[39, 7, 'vtiger_servicecontracts:progress:progress:ServiceContracts_Progress:N'],
					[39, 8, 'vtiger_servicecontracts:contract_status:contract_status:ServiceContracts_Status:V'],
					[40, 0, 'vtiger_service:servicename:servicename:Services_Service_Name:V'],
					[40, 1, 'vtiger_service:unit_price:unit_price:Services_Price:N'],
					[40, 2, 'vtiger_service:sales_start_date:sales_start_date:Services_Sales_Start_Date:D'],
					[40, 3, 'vtiger_service:sales_end_date:sales_end_date:Services_Sales_End_Date:D'],
					[40, 4, 'vtiger_service:pscategory:pscategory:Services_Service_Category:V'],
					[40, 6, 'vtiger_service:discontinued:discontinued:Services_Service_Active:V'],
					[41, 0, 'vtiger_assets:assetname:assetname:Assets_Asset_Name:V'],
					[41, 1, 'vtiger_assets:product:product:Assets_Product_Name:V'],
					[41, 2, 'vtiger_assets:parent_id:parent_id:Assets_Parent_ID:V'],
					[41, 3, 'vtiger_assets:dateinservice:dateinservice:Assets_Date_in_Service:D'],
					[41, 6, 'vtiger_assets:assetstatus:assetstatus:Assets_Status:V'],
					[42, 0, 'vtiger_modcomments:commentcontent:commentcontent:ModComments_Comment:V'],
					[42, 1, 'vtiger_modcomments:related_to:related_to:ModComments_Related_To:V'],
					[42, 2, 'vtiger_crmentity:modifiedtime:modifiedtime:ModComments_Modified_Time:DT'],
					[42, 3, 'vtiger_crmentity:smownerid:assigned_user_id:ModComments_Assigned_To:V'],
					[43, 0, 'vtiger_projectmilestone:projectmilestonename:projectmilestonename:ProjectMilestone_Project_Milestone_Name:V'],
					[43, 1, 'vtiger_projectmilestone:projectmilestonedate:projectmilestonedate:ProjectMilestone_Milestone_Date:D'],
					[43, 2, 'vtiger_crmentity:description:description:ProjectMilestone_description:V'],
					[43, 3, 'vtiger_projectmilestone:projectid:projectid:ProjectMilestone_Related_to:V'],
					[43, 4, 'vtiger_projectmilestone:projectmilestone_priority:projectmilestone_priority:ProjectMilestone_LBL_PRIORITY:V'],
					[43, 5, 'vtiger_projectmilestone:projectmilestonetype:projectmilestonetype:ProjectMilestone_Type:V'],
					[44, 2, 'vtiger_projecttask:projecttaskname:projecttaskname:ProjectTask_Project_Task_Name:V'],
					[44, 3, 'vtiger_projecttask:projectid:projectid:ProjectTask_Related_to:V'],
					[44, 4, 'vtiger_projecttask:projecttaskpriority:projecttaskpriority:ProjectTask_Priority:V'],
					[44, 5, 'vtiger_projecttask:projecttaskprogress:projecttaskprogress:ProjectTask_Progress:V'],
					[44, 6, 'vtiger_projecttask:startdate:startdate:ProjectTask_Start_Date:D'],
					[44, 7, 'vtiger_projecttask:enddate:enddate:ProjectTask_End_Date:D'],
					[44, 8, 'vtiger_crmentity:smownerid:assigned_user_id:ProjectTask_Assigned_To:V'],
					[45, 0, 'vtiger_project:projectname:projectname:Project_Project_Name:V'],
					[45, 1, 'vtiger_project:linktoaccountscontacts:linktoaccountscontacts:Project_Related_to:V'],
					[45, 2, 'vtiger_project:startdate:startdate:Project_Start_Date:D'],
					[45, 3, 'vtiger_project:targetenddate:targetenddate:Project_Target_End_Date:D'],
					[45, 4, 'vtiger_project:actualenddate:actualenddate:Project_Actual_End_Date:D'],
					[45, 5, 'vtiger_project:targetbudget:targetbudget:Project_Target_Budget:V'],
					[45, 6, 'vtiger_project:progress:progress:Project_Progress:V'],
					[45, 7, 'vtiger_project:projectstatus:projectstatus:Project_Status:V'],
					[45, 8, 'vtiger_crmentity:smownerid:assigned_user_id:Project_Assigned_To:V'],
					[46, 0, 'vtiger_smsnotifier:message:message:SMSNotifier_message:V'],
					[46, 2, 'vtiger_crmentity:smownerid:assigned_user_id:SMSNotifier_Assigned_To:V'],
					[46, 3, 'vtiger_crmentity:createdtime:createdtime:SMSNotifier_Created_Time:DT'],
					[46, 4, 'vtiger_crmentity:modifiedtime:modifiedtime:SMSNotifier_Modified_Time:DT'],
					[49, 0, 'vtiger_osstimecontrol:name:name:OSSTimeControl_name:V'],
					[49, 1, 'vtiger_osstimecontrol:date_start:date_start:OSSTimeControl_Start_Date:D'],
					[49, 2, 'vtiger_osstimecontrol:time_start:time_start:OSSTimeControl_Time_Start:T'],
					[49, 3, 'vtiger_osstimecontrol:due_date:due_date:OSSTimeControl_End_Date:D'],
					[49, 4, 'vtiger_osstimecontrol:time_end:time_end:OSSTimeControl_End_Time:T'],
					[49, 5, 'vtiger_crmentity:smownerid:assigned_user_id:OSSTimeControl_Assigned_To:V'],
					[50, 0, 'vtiger_crmentity:createdtime:createdtime:OSSMailView_Created_Time:DT'],
					[50, 1, 'vtiger_ossmailview:from_email:from_email:OSSMailView_From:V'],
					[50, 2, 'vtiger_ossmailview:subject:subject:OSSMailView_Subject:V'],
					[50, 3, 'vtiger_ossmailview:ossmailview_sendtype:ossmailview_sendtype:OSSMailView_SendType:V'],
					[50, 4, 'vtiger_ossmailview:attachments_exist:attachments_exist:OSSMailView_Attachments_Exist:C'],
					[50, 5, 'vtiger_crmentity:smownerid:assigned_user_id:OSSMailView_Assigned_To:V'],
					[52, 0, 'vtiger_ossoutsourcedservices:productname:productname:OSSOutsourcedServices_Product_Name:V'],
					[52, 1, 'vtiger_ossoutsourcedservices:osservicesstatus:osservicesstatus:OSSOutsourcedServices_LBL_osservicesstatus:V'],
					[52, 2, 'vtiger_ossoutsourcedservices:pscategory:pscategory:OSSOutsourcedServices_Category:V'],
					[52, 4, 'vtiger_ossoutsourcedservices:dateinservice:dateinservice:OSSOutsourcedServices_Date_in_Service:D'],
					[52, 5, 'vtiger_ossoutsourcedservices:parent_id:parent_id:OSSOutsourcedServices_Parent_ID:V'],
					[53, 0, 'vtiger_osssoldservices:productname:productname:OSSSoldServices_Product_Name:V'],
					[53, 1, 'vtiger_osssoldservices:serviceid:serviceid:OSSSoldServices_Services:V'],
					[53, 2, 'vtiger_osssoldservices:parent_id:parent_id:OSSSoldServices_Parent_ID:V'],
					[53, 4, 'vtiger_osssoldservices:ssservicesstatus:ssservicesstatus:OSSSoldServices_LBL_ssservicesstatus:V'],
					[53, 6, 'vtiger_osssoldservices:dateinservice:dateinservice:OSSSoldServices_Date_in_Service:D'],
					[54, 0, 'vtiger_outsourcedproducts:productname:productname:OutsourcedProducts_Product_Name:V'],
					[54, 1, 'vtiger_outsourcedproducts:oproductstatus:oproductstatus:OutsourcedProducts_Status:V'],
					[54, 2, 'vtiger_outsourcedproducts:pscategory:pscategory:OutsourcedProducts_Category:V'],
					[54, 4, 'vtiger_outsourcedproducts:parent_id:parent_id:OutsourcedProducts_Parent_ID:V'],
					[54, 6, 'vtiger_outsourcedproducts:dateinservice:dateinservice:OutsourcedProducts_Date_in_Service:D'],
					[55, 0, 'vtiger_osspasswords:osspassword_no:osspassword_no:OSSPasswords_OSSPassword_No:V'],
					[55, 1, 'vtiger_osspasswords:passwordname:passwordname:OSSPasswords_Key_name:V'],
					[55, 2, 'vtiger_osspasswords:username:username:OSSPasswords_Username:V'],
					[55, 3, 'vtiger_osspasswords:password:password:OSSPasswords_Password:V'],
					[55, 4, 'vtiger_osspasswords:link_adres:link_adres:OSSPasswords_WWW_page:V'],
					[55, 5, 'vtiger_crmentity:description:description:OSSPasswords_Description:V'],
					[56, 1, 'vtiger_ossemployees:last_name:last_name:OSSEmployees_LBL_LASTNAME:V'],
					[56, 2, 'vtiger_ossemployees:name:name:OSSEmployees_LBL_NAME:V'],
					[56, 3, 'vtiger_ossemployees:employee_status:employee_status:OSSEmployees_LBL_EMPLOYEESTATUS:V'],
					[56, 4, 'vtiger_ossemployees:employee_education:employee_education:OSSEmployees_LBL_EDUCATION:V'],
					[56, 5, 'vtiger_crmentity:smownerid:assigned_user_id:OSSEmployees_Assigned_To:V'],
					[56, 6, 'vtiger_crmentity:createdtime:createdtime:OSSEmployees_Created_Time:V'],
					[56, 7, 'vtiger_crmentity:modifiedtime:modifiedtime:OSSEmployees_Modified_Time:V'],
					[68, 0, 'vtiger_callhistory:from_number:from_number:CallHistory_LBL_FROM_NUMBER:V'],
					[68, 1, 'vtiger_callhistory:to_number:to_number:CallHistory_LBL_TO_NUMBER:V'],
					[68, 2, 'vtiger_callhistory:duration:duration:CallHistory_LBL_DURATION:I'],
					[68, 4, 'vtiger_callhistory:start_time:start_time:CallHistory_LBL_START_TIME:V'],
					[69, 0, 'vtiger_ideas:subject:subject:Ideas_LBL_SUBJECT:V'],
					[69, 1, 'vtiger_ideas:ideasstatus:ideasstatus:Ideas_LBL_STATUS:V'],
					[69, 2, 'vtiger_crmentity:smownerid:assigned_user_id:Ideas_Assigned_To:V'],
					[69, 3, 'vtiger_crmentity:createdtime:createdtime:Ideas_Created_Time:DT'],
					[69, 4, 'vtiger_crmentity:modifiedtime:modifiedtime:Ideas_Modified_Time:DT'],
					[72, 0, 'vtiger_holidaysentitlement:ossemployeesid:ossemployeesid:HolidaysEntitlement_LBL_EMPLOYEE:V'],
					[72, 1, 'vtiger_holidaysentitlement:holidaysentitlement_year:holidaysentitlement_year:HolidaysEntitlement_LBL_YEAR:V'],
					[72, 2, 'vtiger_holidaysentitlement:days:days:HolidaysEntitlement_LBL_DAYS:I'],
					[72, 3, 'vtiger_crmentity:smownerid:assigned_user_id:HolidaysEntitlement_Assigned_To:V'],
					[72, 4, 'vtiger_crmentity:createdtime:createdtime:HolidaysEntitlement_Created_Time:DT'],
					[72, 5, 'vtiger_crmentity:modifiedtime:modifiedtime:HolidaysEntitlement_Modified_Time:DT'],
					[73, 1, 'vtiger_crmentity:smownerid:assigned_user_id:PaymentsIn_Assigned_To:V'],
					[73, 2, 'vtiger_crmentity:createdtime:createdtime:PaymentsIn_Created_Time:V'],
					[73, 3, 'vtiger_crmentity:modifiedtime:modifiedtime:PaymentsIn_Modified_Time:V'],
					[73, 4, 'vtiger_paymentsin:paymentsno:paymentsno:PaymentsIn_LBL_PAYMENTSNO:V'],
					[73, 5, 'vtiger_paymentsin:paymentsname:paymentsname:PaymentsIn_LBL_PAYMENTSNAME:V'],
					[73, 6, 'vtiger_paymentsin:paymentsvalue:paymentsvalue:PaymentsIn_LBL_PAYMENTSVALUE:N'],
					[74, 1, 'vtiger_crmentity:smownerid:assigned_user_id:PaymentsOut_Assigned_To:V'],
					[74, 2, 'vtiger_crmentity:createdtime:createdtime:PaymentsOut_Created_Time:V'],
					[74, 3, 'vtiger_crmentity:modifiedtime:modifiedtime:PaymentsOut_Modified_Time:V'],
					[74, 4, 'vtiger_paymentsout:paymentsno:paymentsno:PaymentsOut_LBL_PAYMENTSNO:V'],
					[74, 5, 'vtiger_paymentsout:paymentsname:paymentsname:PaymentsOut_LBL_PAYMENTSNAME:V'],
					[74, 6, 'vtiger_paymentsout:paymentsvalue:paymentsvalue:PaymentsOut_LBL_PAYMENTSVALUE:N'],
					[75, 0, 'vtiger_lettersin:number:number:LettersIn_Number:V'],
					[75, 1, 'vtiger_lettersin:title:title:LettersIn_Title:V'],
					[75, 3, 'vtiger_crmentity:smownerid:assigned_user_id:LettersIn_Assigned_To:V'],
					[75, 4, 'vtiger_crmentity:createdtime:createdtime:LettersIn_Created_Time:V'],
					[75, 6, 'vtiger_lettersin:lin_type_ship:lin_type_ship:LettersIn_Type_of_shipping:V'],
					[75, 7, 'vtiger_lettersin:lin_type_doc:lin_type_doc:LettersIn_Type_of_document:V'],
					[76, 0, 'vtiger_lettersout:number:number:LettersOut_Number:V'],
					[76, 1, 'vtiger_lettersout:title:title:LettersOut_Title:V'],
					[76, 3, 'vtiger_crmentity:smownerid:assigned_user_id:LettersOut_Assigned_To:V'],
					[76, 4, 'vtiger_crmentity:createdtime:createdtime:LettersOut_Created_Time:V'],
					[76, 6, 'vtiger_lettersout:lout_type_ship:lout_type_ship:LettersOut_Type_of_shipping:V'],
					[76, 7, 'vtiger_lettersout:lout_type_doc:lout_type_doc:LettersOut_Type_of_document:V'],
					[77, 0, 'u_yf_announcement:subject:subject:Announcements_LBL_SUBJECT:V'],
					[77, 1, 'vtiger_crmentity:smownerid:assigned_user_id:Announcements_Assigned_To:V'],
					[77, 2, 'vtiger_crmentity:createdtime:createdtime:Announcements_Created_Time:DT'],
					[77, 3, 'vtiger_crmentity:modifiedtime:modifiedtime:Announcements_Modified_Time:DT'],
					[78, 0, 'vtiger_reservations:reservations_no:reservations_no:Reservations_LBL_NUM:V'],
					[78, 1, 'vtiger_reservations:title:title:Reservations_LBL_TITLE:V'],
					[78, 2, 'vtiger_reservations:date_start:date_start:Reservations_LBL_START_DATE:D'],
					[78, 3, 'vtiger_reservations:time_start:time_start:Reservations_LBL_TIME_START:T'],
					[78, 4, 'vtiger_reservations:due_date:due_date:Reservations_LBL_END_DATE:D'],
					[78, 5, 'vtiger_reservations:time_end:time_end:Reservations_LBL_END_TIME:T'],
					[78, 6, 'vtiger_crmentity:smownerid:assigned_user_id:Reservations_Assigned_To:V'],
					[79, 0, 'u_yf_squoteenquiries:subject:subject:SQuoteEnquiries_LBL_SUBJECT:V'],
					[79, 2, 'u_yf_squoteenquiries:squoteenquiries_status:squoteenquiries_status:SQuoteEnquiries_LBL_STATUS:V'],
					[79, 3, 'u_yf_squoteenquiries:category:category:SQuoteEnquiries_Category:V'],
					[79, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SQuoteEnquiries_Assigned_To:V'],
					[79, 5, 'vtiger_crmentity:createdtime:createdtime:SQuoteEnquiries_Created_Time:DT'],
					[80, 0, 'u_yf_ssalesprocesses:subject:subject:SSalesProcesses_LBL_SUBJECT:V'],
					[80, 1, 'u_yf_ssalesprocesses:related_to:related_to:SSalesProcesses_Accounts:V'],
					[80, 2, 'u_yf_ssalesprocesses:category:category:SSalesProcesses_Category:V'],
					[80, 3, 'vtiger_crmentity:smownerid:assigned_user_id:SSalesProcesses_Assigned_To:V'],
					[80, 4, 'vtiger_crmentity:createdtime:createdtime:SSalesProcesses_Created_Time:DT'],
					[81, 0, 'u_yf_srequirementscards:subject:subject:SRequirementsCards_LBL_SUBJECT:V'],
					[81, 1, 'u_yf_srequirementscards:salesprocessid:salesprocessid:SRequirementsCards_SINGLE_SSalesProcesses:V'],
					[81, 2, 'u_yf_srequirementscards:srequirementscards_status:srequirementscards_status:SRequirementsCards_LBL_STATUS:V'],
					[81, 3, 'u_yf_srequirementscards:category:category:SRequirementsCards_Category:V'],
					[81, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SRequirementsCards_Assigned_To:V'],
					[81, 5, 'vtiger_crmentity:createdtime:createdtime:SRequirementsCards_Created_Time:DT'],
					[82, 0, 'u_yf_scalculations:subject:subject:SCalculations_LBL_SUBJECT:V'],
					[82, 1, 'u_yf_scalculations:salesprocessid:salesprocessid:SCalculations_SINGLE_SSalesProcesses:V'],
					[82, 2, 'u_yf_scalculations:scalculations_status:scalculations_status:SCalculations_LBL_STATUS:V'],
					[82, 3, 'u_yf_scalculations:category:category:SCalculations_Category:V'],
					[82, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SCalculations_Assigned_To:V'],
					[82, 5, 'vtiger_crmentity:createdtime:createdtime:SCalculations_Created_Time:DT'],
					[83, 0, 'u_yf_squotes:subject:subject:SQuotes_LBL_SUBJECT:V'],
					[83, 1, 'u_yf_squotes:salesprocessid:salesprocessid:SQuotes_SINGLE_SSalesProcesses:V'],
					[83, 2, 'u_yf_squotes:squotes_status:squotes_status:SQuotes_LBL_STATUS:V'],
					[83, 3, 'u_yf_squotes:category:category:SQuotes_Category:V'],
					[83, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SQuotes_Assigned_To:V'],
					[83, 5, 'vtiger_crmentity:createdtime:createdtime:SQuotes_Created_Time:DT'],
					[84, 0, 'u_yf_ssingleorders:subject:subject:SSingleOrders_LBL_SUBJECT:V'],
					[84, 1, 'u_yf_ssingleorders:salesprocessid:salesprocessid:SSingleOrders_SINGLE_SSalesProcesses:V'],
					[84, 2, 'u_yf_ssingleorders:ssingleorders_status:ssingleorders_status:SSingleOrders_LBL_STATUS:V'],
					[84, 3, 'u_yf_ssingleorders:category:category:SSingleOrders_Category:V'],
					[84, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SSingleOrders_Assigned_To:V'],
					[84, 5, 'vtiger_crmentity:createdtime:createdtime:SSingleOrders_Created_Time:DT'],
					[85, 0, 'u_yf_srecurringorders:subject:subject:SRecurringOrders_LBL_SUBJECT:V'],
					[85, 1, 'u_yf_srecurringorders:salesprocessid:salesprocessid:SRecurringOrders_SINGLE_SSalesProcesses:V'],
					[85, 2, 'u_yf_srecurringorders:srecurringorders_status:srecurringorders_status:SRecurringOrders_LBL_STATUS:V'],
					[85, 3, 'u_yf_srecurringorders:category:category:SRecurringOrders_Category:V'],
					[85, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SRecurringOrders_Assigned_To:V'],
					[85, 5, 'vtiger_crmentity:createdtime:createdtime:SRecurringOrders_Created_Time:DT'],
					[86, 0, 'u_yf_partners:subject:subject:Partners_LBL_SUBJECT:V'],
					[86, 1, 'u_yf_partners:vat_id:vat_id:Partners_LBL_VAT_ID:V'],
					[86, 2, 'vtiger_crmentity:smownerid:assigned_user_id:Partners_Assigned_To:V'],
					[86, 3, 'vtiger_crmentity:createdtime:createdtime:Partners_Created_Time:DT'],
					[87, 0, 'u_yf_competition:subject:subject:Competition_LBL_SUBJECT:V'],
					[87, 1, 'u_yf_competition:vat_id:vat_id:Competition_LBL_VAT_ID:V'],
					[87, 2, 'vtiger_crmentity:smownerid:assigned_user_id:Competition_Assigned_To:V'],
					[87, 3, 'vtiger_crmentity:createdtime:createdtime:Competition_Created_Time:DT'],
					[88, 0, 'u_yf_fbookkeeping:subject:subject:FBookkeeping_FL_SUBJECT:V'],
					[88, 1, 'u_yf_fbookkeeping:number:number:FBookkeeping_FL_NUMBER:V'],
					[88, 2, 'vtiger_crmentity:createdtime:createdtime:FBookkeeping_Created_Time:DT'],
					[88, 3, 'vtiger_crmentity:smownerid:assigned_user_id:FBookkeeping_Assigned_To:V'],
					[89, 0, 'u_yf_finvoice:subject:subject:FInvoice_FL_SUBJECT:V'],
					[89, 1, 'u_yf_finvoice:number:number:FInvoice_FL_NUMBER:V'],
					[89, 2, 'u_yf_finvoice:accountid:accountid:FInvoice_FL_ACCOUNT:V'],
					[89, 3, 'vtiger_crmentity:smownerid:assigned_user_id:FInvoice_Assigned_To:V'],
					[89, 4, 'u_yf_finvoice:saledate:saledate:FInvoice_FL_SALE_DATE:D'],
					[90, 0, 'u_yf_knowledgebase:subject:subject:KnowledgeBase_FL_SUBJECT:V'],
					[90, 1, 'u_yf_knowledgebase:number:number:KnowledgeBase_FL_NUMBER:V'],
					[90, 2, 'vtiger_crmentity:createdtime:createdtime:KnowledgeBase_Created_Time:DT'],
					[90, 3, 'vtiger_crmentity:smownerid:assigned_user_id:KnowledgeBase_Assigned_To:V'],
					[91, 0, 'u_yf_istorages:subject:subject:IStorages_FL_SUBJECT:V'],
					[91, 1, 'u_yf_istorages:storage_status:storage_status:IStorages_FL_STATUS:V'],
					[91, 2, 'u_yf_istorages:storage_type:storage_type:IStorages_FL_TYPE:V'],
					[91, 3, 'u_yf_istorages:parentid:parentid:IStorages_FL_PARENT_STORAGE:V'],
					[91, 4, 'vtiger_crmentity:smownerid:assigned_user_id:IStorages_Assigned_To:V'],
					[91, 5, 'vtiger_crmentity:createdtime:createdtime:IStorages_Created_Time:DT'],
					[92, 0, 'u_yf_igrn:subject:subject:IGRN_FL_SUBJECT:V'],
					[92, 1, 'u_yf_igrn:vendorid:vendorid:IGRN_FL_VENDOR:V'],
					[92, 2, 'u_yf_igrn:number:number:IGRN_FL_NUMBER:V'],
					[92, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IGRN_Assigned_To:V'],
					[92, 4, 'u_yf_igrn:acceptance_date:acceptance_date:IGRN_FL_ACCEPTANCE_DATE:D'],
					[92, 5, 'vtiger_crmentity:createdtime:createdtime:IGRN_Created_Time:DT'],
					[93, 0, 'u_yf_finvoiceproforma:subject:subject:FInvoiceProforma_FL_SUBJECT:V'],
					[93, 1, 'u_yf_finvoiceproforma:number:number:FInvoiceProforma_FL_NUMBER:V'],
					[93, 2, 'vtiger_crmentity:createdtime:createdtime:FInvoiceProforma_Created_Time:DT'],
					[93, 3, 'vtiger_crmentity:smownerid:assigned_user_id:FInvoiceProforma_Assigned_To:V'],
					[94, 0, 'u_yf_igdn:subject:subject:IGDN_FL_SUBJECT:V'],
					[94, 1, 'u_yf_igdn:number:number:IGDN_FL_NUMBER:V'],
					[94, 2, 'u_yf_igdn:storageid:storageid:IGDN_FL_STORAGE:V'],
					[94, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IGDN_Assigned_To:V'],
					[94, 4, 'vtiger_crmentity:createdtime:createdtime:IGDN_Created_Time:DT'],
					[95, 0, 'u_yf_iidn:subject:subject:IIDN_FL_SUBJECT:V'],
					[95, 1, 'u_yf_iidn:number:number:IIDN_FL_NUMBER:V'],
					[95, 2, 'u_yf_iidn:storageid:storageid:IIDN_FL_STORAGE:V'],
					[95, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IIDN_Assigned_To:V'],
					[95, 4, 'vtiger_crmentity:createdtime:createdtime:IIDN_Created_Time:DT'],
					[96, 0, 'u_yf_igin:subject:subject:IGIN_FL_SUBJECT:V'],
					[96, 1, 'u_yf_igin:number:number:IGIN_FL_NUMBER:V'],
					[96, 2, 'u_yf_igin:storageid:storageid:IGIN_FL_STORAGE:V'],
					[96, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IGIN_Assigned_To:V'],
					[96, 4, 'vtiger_crmentity:createdtime:createdtime:IGIN_Created_Time:DT'],
					[97, 0, 'u_yf_ipreorder:subject:subject:IPreOrder_FL_SUBJECT:V'],
					[97, 1, 'u_yf_ipreorder:number:number:IPreOrder_FL_NUMBER:V'],
					[97, 2, 'u_yf_ipreorder:storageid:storageid:IPreOrder_FL_STORAGE:V'],
					[97, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IPreOrder_Assigned_To:V'],
					[97, 4, 'vtiger_crmentity:createdtime:createdtime:IPreOrder_Created_Time:DT'],
					[98, 0, 'u_yf_istdn:subject:subject:ISTDN_FL_SUBJECT:V'],
					[98, 1, 'u_yf_istdn:accountid:accountid:ISTDN_FL_ACCOUNT:V'],
					[98, 2, 'u_yf_istdn:number:number:ISTDN_FL_NUMBER:V'],
					[98, 3, 'vtiger_crmentity:smownerid:assigned_user_id:ISTDN_Assigned_To:V'],
					[98, 4, 'u_yf_istdn:acceptance_date:acceptance_date:ISTDN_FL_ACCEPTANCE_DATE:D'],
					[98, 5, 'vtiger_crmentity:createdtime:createdtime:ISTDN_Created_Time:DT'],
					[99, 0, 'u_yf_istn:subject:subject:ISTN_FL_SUBJECT:V'],
					[99, 2, 'u_yf_istn:number:number:ISTN_FL_NUMBER:V'],
					[99, 3, 'vtiger_crmentity:smownerid:assigned_user_id:ISTN_Assigned_To:V'],
					[99, 4, 'u_yf_istn:estimated_date:estimated_date:ISTN_FL_ESTIMATED_DATE:D'],
					[99, 5, 'vtiger_crmentity:createdtime:createdtime:ISTN_Created_Time:DT'],
					[100, 0, 'u_yf_istrn:subject:subject:ISTRN_FL_SUBJECT:V'],
					[100, 1, 'u_yf_istrn:vendorid:vendorid:ISTRN_FL_VENDOR:V'],
					[100, 2, 'u_yf_istrn:number:number:ISTRN_FL_NUMBER:V'],
					[100, 3, 'vtiger_crmentity:smownerid:assigned_user_id:ISTRN_Assigned_To:V'],
					[100, 4, 'u_yf_istrn:acceptance_date:acceptance_date:ISTRN_FL_ACCEPTANCE_DATE:D'],
					[100, 5, 'vtiger_crmentity:createdtime:createdtime:ISTRN_Created_Time:DT'],
					[101, 0, 'u_yf_fcorectinginvoice:subject:subject:FCorectingInvoice_FL_SUBJECT:V'],
					[101, 1, 'u_yf_fcorectinginvoice:number:number:FCorectingInvoice_FL_NUMBER:V'],
					[101, 2, 'u_yf_fcorectinginvoice:accountid:accountid:FCorectingInvoice_FL_ACCOUNT:V'],
					[101, 3, 'vtiger_crmentity:smownerid:assigned_user_id:FCorectingInvoice_Assigned_To:V'],
					[101, 4, 'u_yf_fcorectinginvoice:saledate:saledate:FCorectingInvoice_FL_SALE_DATE:D'],
					[102, 0, 'u_yf_igrnc:subject:subject:IGRNC_FL_SUBJECT:V'],
					[102, 1, 'u_yf_igrnc:vendorid:vendorid:IGRNC_FL_VENDOR:V'],
					[102, 2, 'u_yf_igrnc:number:number:IGRNC_FL_NUMBER:V'],
					[102, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IGRNC_Assigned_To:V'],
					[102, 4, 'u_yf_igrnc:acceptance_date:acceptance_date:IGRNC_FL_ACCEPTANCE_DATE:D'],
					[102, 5, 'vtiger_crmentity:createdtime:createdtime:IGRNC_Created_Time:DT'],
					[103, 0, 'u_yf_igdnc:subject:subject:IGDNC_FL_SUBJECT:V'],
					[103, 1, 'u_yf_igdnc:number:number:IGDNC_FL_NUMBER:V'],
					[103, 2, 'u_yf_igdnc:storageid:storageid:IGDNC_FL_STORAGE:V'],
					[103, 3, 'vtiger_crmentity:smownerid:assigned_user_id:IGDNC_Assigned_To:V'],
					[103, 4, 'vtiger_crmentity:createdtime:createdtime:IGDNC_Created_Time:DT'],
					[104, 0, 'u_yf_notification:title:title:Notification_FL_TITLE:V'],
					[104, 1, 'u_yf_notification:number:number:Notification_FL_NUMBER:V'],
					[104, 2, 'vtiger_crmentity:createdtime:createdtime:Notification_Created_Time:DT'],
					[104, 3, 'vtiger_crmentity:smownerid:assigned_user_id:Notification_Assigned_To:V'],
					[105, 0, 'u_yf_emailtemplates:name:name:EmailTemplates_FL_NAME:V'],
					[105, 1, 'u_yf_emailtemplates:module:module_name:EmailTemplates_FL_MODULE:V'],
					[105, 2, 'vtiger_crmentity:smownerid:assigned_user_id:EmailTemplates_Assigned_To:V'],
					[105, 3, 'u_yf_emailtemplates:email_template_type:email_template_type:EmailTemplates_FL_TYPE:V'],
					[106, 0, 'u_yf_cfixedassets:subject:subject:CFixedAssets_FL_SUBJECT:V'],
					[106, 1, 'u_yf_cfixedassets:number:number:CFixedAssets_FL_NUMBER:V'],
					[106, 2, 'vtiger_crmentity:createdtime:createdtime:CFixedAssets_Created_Time:DT'],
					[106, 3, 'vtiger_crmentity:smownerid:assigned_user_id:CFixedAssets_Assigned_To:V'],
					[107, 0, 'u_yf_cinternaltickets:subject:subject:CInternalTickets_FL_SUBJECT:V'],
					[107, 1, 'vtiger_crmentity:createdtime:createdtime:CInternalTickets_Created_Time:DT'],
					[107, 2, 'vtiger_crmentity:smownerid:assigned_user_id:CInternalTickets_Assigned_To:V'],
					[108, 0, 'u_yf_finvoicecost:subject:subject:FInvoiceCost_FL_SUBJECT:V'],
					[108, 1, 'u_yf_finvoicecost:number:number:FInvoiceCost_FL_NUMBER:V'],
					[108, 3, 'vtiger_crmentity:smownerid:assigned_user_id:FInvoiceCost_Assigned_To:V'],
					[108, 4, 'u_yf_finvoicecost:saledate:saledate:FInvoiceCost_FL_SALE_DATE:D'],
					[109, 0, 'u_yf_cmileagelogbook:subject:subject:CMileageLogbook_FL_SUBJECT:V'],
					[109, 1, 'u_yf_cmileagelogbook:number:number:CMileageLogbook_FL_NUMBER:V'],
					[109, 2, 'vtiger_crmentity:createdtime:createdtime:CMileageLogbook_Created_Time:DT'],
					[109, 3, 'vtiger_crmentity:smownerid:assigned_user_id:CMileageLogbook_Assigned_To:V'],
					[110, 0, 'u_yf_svendorenquiries:subject:subject:SVendorEnquiries_LBL_SUBJECT:V'],
					[110, 1, 'u_yf_svendorenquiries:salesprocessid:salesprocessid:SVendorEnquiries_SINGLE_SSalesProcesses:V'],
					[110, 2, 'u_yf_svendorenquiries:svendorenquiries_status:svendorenquiries_status:SVendorEnquiries_LBL_STATUS:V'],
					[110, 3, 'u_yf_svendorenquiries:category:category:SVendorEnquiries_Category:V'],
					[110, 4, 'vtiger_crmentity:smownerid:assigned_user_id:SVendorEnquiries_Assigned_To:V'],
					[110, 5, 'vtiger_crmentity:createdtime:createdtime:SVendorEnquiries_Created_Time:DT'],
				]
			],
			'vtiger_datashare_relatedmodules' => [
				'columns' => ['datashare_relatedmodule_id', 'tabid', 'relatedto_tabid'],
				'values' => [
					[2, 6, 13],
				]
			],
			'vtiger_datashare_relatedmodules_seq' => [
				'columns' => ['id'],
				'values' => [
					[9],
				]
			],
			'vtiger_date_format' => [
				'columns' => ['date_formatid', 'date_format', 'sortorderid', 'presence'],
				'values' => [
					[1, 'dd-mm-yyyy', 0, 1],
					[2, 'mm-dd-yyyy', 1, 1],
					[3, 'yyyy-mm-dd', 2, 1],
					[4, 'dd.mm.yyyy', 3, 1],
					[5, 'mm.dd.yyyy', 4, 1],
					[6, 'yyyy.mm.dd', 5, 1],
					[7, 'dd/mm/yyyy', 6, 1],
					[8, 'mm/dd/yyyy', 7, 1],
					[9, 'yyyy/mm/dd', 8, 1],
				]
			],
			'vtiger_date_format_seq' => [
				'columns' => ['id'],
				'values' => [
					[9],
				]
			],
			'vtiger_dayoftheweek' => [
				'columns' => ['dayoftheweekid', 'dayoftheweek', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Sunday', 1, 1],
					[2, 'Monday', 2, 1],
					[3, 'Tuesday', 3, 1],
					[4, 'Wednesday', 4, 1],
					[5, 'Thursday', 5, 1],
					[6, 'Friday', 6, 1],
					[7, 'Saturday', 7, 1],
				]
			],
			'vtiger_dayoftheweek_seq' => [
				'columns' => ['id'],
				'values' => [
					[7],
				]
			],
			'vtiger_def_org_field' => [
				'columns' => ['tabid', 'fieldid', 'visible', 'readonly'],
				'values' => [
					[6, 1, 0, 0],
					[6, 2, 0, 0],
					[6, 3, 0, 0],
					[6, 4, 0, 0],
					[6, 5, 0, 0],
					[6, 7, 0, 0],
					[6, 8, 0, 0],
					[6, 9, 0, 0],
					[6, 10, 0, 0],
					[6, 11, 0, 0],
					[6, 12, 0, 0],
					[6, 14, 0, 0],
					[6, 15, 0, 0],
					[6, 16, 0, 0],
					[6, 17, 0, 0],
					[6, 18, 0, 0],
					[6, 20, 0, 0],
					[6, 21, 0, 0],
					[6, 22, 0, 0],
					[6, 23, 0, 0],
					[6, 36, 0, 0],
					[7, 39, 0, 0],
					[7, 40, 0, 0],
					[7, 41, 0, 0],
					[7, 42, 0, 0],
					[7, 43, 0, 0],
					[7, 44, 0, 0],
					[7, 46, 0, 0],
					[7, 47, 0, 0],
					[7, 48, 0, 0],
					[7, 49, 0, 0],
					[7, 50, 0, 0],
					[7, 51, 0, 0],
					[7, 53, 0, 0],
					[7, 54, 0, 0],
					[7, 55, 0, 0],
					[7, 56, 0, 0],
					[7, 57, 0, 0],
					[7, 58, 0, 0],
					[7, 65, 0, 0],
					[4, 66, 0, 0],
					[4, 67, 0, 0],
					[4, 68, 0, 0],
					[4, 69, 0, 0],
					[4, 70, 0, 0],
					[4, 71, 0, 0],
					[4, 72, 0, 0],
					[4, 74, 0, 0],
					[4, 79, 0, 0],
					[4, 80, 0, 0],
					[4, 81, 0, 0],
					[4, 85, 0, 0],
					[4, 86, 0, 0],
					[4, 87, 0, 0],
					[4, 90, 0, 0],
					[4, 91, 0, 0],
					[4, 92, 0, 0],
					[4, 93, 0, 0],
					[4, 94, 0, 0],
					[4, 95, 0, 0],
					[4, 108, 0, 0],
					[4, 109, 0, 0],
					[26, 126, 0, 0],
					[26, 127, 0, 0],
					[26, 128, 0, 0],
					[26, 129, 0, 0],
					[26, 130, 0, 0],
					[26, 131, 0, 0],
					[26, 132, 0, 0],
					[26, 133, 0, 0],
					[26, 134, 0, 0],
					[26, 135, 0, 0],
					[26, 136, 0, 0],
					[26, 137, 0, 0],
					[26, 138, 0, 0],
					[26, 139, 0, 0],
					[26, 140, 0, 0],
					[26, 141, 0, 0],
					[26, 142, 0, 0],
					[26, 143, 0, 0],
					[26, 144, 0, 0],
					[26, 145, 0, 0],
					[26, 146, 0, 0],
					[26, 147, 0, 0],
					[26, 148, 0, 0],
					[26, 149, 0, 0],
					[26, 150, 0, 0],
					[13, 155, 0, 0],
					[13, 156, 0, 0],
					[13, 157, 0, 0],
					[13, 158, 0, 0],
					[13, 159, 0, 0],
					[13, 160, 0, 0],
					[13, 161, 0, 0],
					[13, 162, 0, 0],
					[13, 163, 0, 0],
					[13, 166, 0, 0],
					[13, 167, 0, 0],
					[13, 168, 0, 0],
					[13, 169, 0, 0],
					[13, 170, 0, 0],
					[13, 171, 0, 0],
					[14, 173, 0, 0],
					[14, 174, 0, 0],
					[14, 175, 0, 0],
					[14, 176, 0, 0],
					[14, 177, 0, 0],
					[14, 178, 0, 0],
					[14, 179, 0, 0],
					[14, 180, 0, 0],
					[14, 181, 0, 0],
					[14, 182, 0, 0],
					[14, 183, 0, 0],
					[14, 184, 0, 0],
					[14, 185, 0, 0],
					[14, 186, 0, 0],
					[14, 187, 0, 0],
					[14, 188, 0, 0],
					[14, 189, 0, 0],
					[14, 190, 0, 0],
					[14, 191, 0, 0],
					[14, 192, 0, 0],
					[14, 193, 0, 0],
					[14, 194, 0, 0],
					[14, 196, 0, 0],
					[14, 197, 0, 0],
					[14, 198, 0, 0],
					[14, 199, 0, 0],
					[14, 200, 0, 0],
					[14, 201, 0, 0],
					[14, 202, 0, 0],
					[14, 203, 0, 0],
					[8, 204, 0, 0],
					[8, 205, 0, 0],
					[8, 206, 0, 0],
					[8, 207, 0, 0],
					[8, 208, 0, 0],
					[8, 209, 0, 0],
					[8, 210, 0, 0],
					[8, 211, 0, 0],
					[8, 212, 0, 0],
					[8, 213, 0, 0],
					[8, 214, 0, 0],
					[8, 215, 0, 0],
					[8, 216, 0, 0],
					[8, 217, 0, 0],
					[8, 218, 0, 0],
					[9, 231, 0, 0],
					[9, 232, 0, 0],
					[9, 233, 0, 0],
					[9, 234, 0, 0],
					[9, 235, 0, 0],
					[9, 236, 0, 0],
					[9, 237, 0, 0],
					[9, 238, 0, 0],
					[9, 239, 0, 0],
					[9, 241, 0, 0],
					[9, 242, 0, 0],
					[9, 243, 0, 0],
					[9, 244, 0, 0],
					[9, 245, 0, 0],
					[9, 246, 0, 0],
					[9, 247, 0, 0],
					[9, 248, 0, 0],
					[9, 249, 0, 0],
					[9, 250, 0, 0],
					[9, 251, 0, 0],
					[9, 253, 0, 0],
					[9, 254, 0, 0],
					[16, 255, 0, 0],
					[16, 256, 0, 0],
					[16, 257, 0, 0],
					[16, 258, 0, 0],
					[16, 259, 0, 0],
					[16, 260, 0, 0],
					[16, 262, 0, 0],
					[16, 263, 0, 0],
					[16, 264, 0, 0],
					[16, 265, 0, 0],
					[16, 266, 0, 0],
					[16, 267, 0, 0],
					[16, 268, 0, 0],
					[16, 269, 0, 0],
					[16, 270, 0, 0],
					[16, 271, 0, 0],
					[16, 272, 0, 0],
					[16, 273, 0, 0],
					[16, 274, 0, 0],
					[16, 275, 0, 0],
					[16, 276, 0, 0],
					[16, 277, 0, 0],
					[15, 278, 0, 0],
					[15, 279, 0, 0],
					[15, 280, 0, 0],
					[15, 281, 0, 0],
					[15, 282, 0, 0],
					[15, 283, 0, 0],
					[15, 284, 0, 0],
					[15, 285, 0, 0],
					[15, 286, 0, 0],
					[15, 287, 0, 0],
					[18, 288, 0, 0],
					[18, 289, 0, 0],
					[18, 290, 0, 0],
					[18, 291, 0, 0],
					[18, 292, 0, 0],
					[18, 293, 0, 0],
					[18, 294, 0, 0],
					[18, 295, 0, 0],
					[18, 296, 0, 0],
					[18, 297, 0, 0],
					[18, 304, 0, 0],
					[19, 305, 0, 0],
					[19, 306, 0, 0],
					[19, 307, 0, 0],
					[19, 308, 0, 0],
					[19, 309, 0, 0],
					[19, 310, 0, 0],
					[19, 311, 0, 0],
					[19, 312, 0, 0],
					[29, 474, 0, 0],
					[29, 475, 0, 0],
					[29, 476, 0, 0],
					[29, 477, 0, 0],
					[29, 478, 0, 0],
					[29, 479, 0, 0],
					[29, 480, 0, 0],
					[29, 481, 0, 0],
					[29, 482, 0, 0],
					[29, 483, 0, 0],
					[29, 484, 0, 0],
					[29, 485, 0, 0],
					[29, 487, 0, 0],
					[29, 492, 0, 0],
					[29, 498, 0, 0],
					[29, 500, 0, 0],
					[29, 506, 0, 0],
					[29, 507, 0, 0],
					[29, 508, 0, 0],
					[29, 509, 0, 0],
					[29, 510, 0, 0],
					[29, 511, 0, 0],
					[29, 512, 0, 0],
					[29, 513, 0, 0],
					[29, 514, 0, 0],
					[29, 515, 0, 0],
					[29, 516, 0, 0],
					[29, 517, 0, 0],
					[33, 524, 0, 0],
					[33, 525, 0, 0],
					[33, 526, 0, 0],
					[33, 527, 0, 0],
					[33, 528, 0, 0],
					[33, 529, 0, 0],
					[33, 530, 0, 0],
					[33, 531, 0, 0],
					[33, 532, 0, 0],
					[33, 533, 0, 0],
					[33, 534, 0, 0],
					[33, 535, 0, 0],
					[33, 536, 0, 0],
					[33, 537, 0, 0],
					[33, 538, 0, 0],
					[33, 539, 0, 0],
					[29, 540, 0, 0],
					[34, 541, 0, 0],
					[34, 542, 0, 0],
					[34, 543, 0, 0],
					[34, 544, 0, 0],
					[34, 545, 0, 0],
					[34, 546, 0, 0],
					[34, 547, 0, 0],
					[34, 548, 0, 0],
					[34, 549, 0, 0],
					[34, 550, 0, 0],
					[34, 551, 0, 0],
					[34, 552, 0, 0],
					[34, 553, 0, 0],
					[34, 554, 0, 0],
					[34, 555, 0, 0],
					[34, 556, 0, 0],
					[34, 557, 0, 0],
					[34, 558, 0, 0],
					[34, 559, 0, 0],
					[35, 560, 0, 0],
					[35, 561, 0, 0],
					[35, 562, 0, 0],
					[35, 563, 0, 0],
					[35, 564, 0, 0],
					[35, 565, 0, 0],
					[35, 566, 0, 0],
					[35, 567, 0, 0],
					[35, 568, 0, 0],
					[35, 569, 0, 0],
					[35, 570, 0, 0],
					[35, 571, 0, 0],
					[35, 572, 0, 0],
					[35, 573, 0, 0],
					[35, 574, 0, 0],
					[35, 575, 0, 0],
					[35, 577, 0, 0],
					[35, 578, 0, 0],
					[37, 579, 0, 0],
					[37, 580, 0, 0],
					[37, 581, 0, 0],
					[37, 582, 0, 0],
					[37, 583, 0, 0],
					[37, 584, 0, 0],
					[37, 589, 0, 0],
					[37, 590, 0, 0],
					[37, 593, 0, 0],
					[37, 594, 0, 0],
					[37, 595, 0, 0],
					[40, 597, 0, 0],
					[40, 598, 0, 0],
					[40, 599, 0, 0],
					[40, 600, 0, 0],
					[40, 601, 0, 0],
					[40, 602, 0, 0],
					[40, 603, 0, 0],
					[41, 604, 0, 0],
					[41, 605, 0, 0],
					[41, 606, 0, 0],
					[41, 607, 0, 0],
					[41, 608, 0, 0],
					[41, 609, 0, 0],
					[41, 610, 0, 0],
					[41, 611, 0, 0],
					[41, 612, 0, 0],
					[41, 613, 0, 0],
					[42, 614, 0, 0],
					[42, 615, 0, 0],
					[42, 616, 0, 0],
					[42, 617, 0, 0],
					[42, 618, 0, 0],
					[42, 619, 0, 0],
					[42, 620, 0, 0],
					[42, 621, 0, 0],
					[42, 623, 0, 0],
					[42, 624, 0, 0],
					[42, 625, 0, 0],
					[42, 626, 0, 0],
					[42, 627, 0, 0],
					[42, 628, 0, 0],
					[43, 629, 0, 0],
					[43, 630, 0, 0],
					[43, 631, 0, 0],
					[43, 632, 0, 0],
					[43, 633, 0, 0],
					[43, 634, 0, 0],
					[43, 635, 0, 0],
					[43, 636, 0, 0],
					[43, 637, 0, 0],
					[43, 638, 0, 0],
					[43, 639, 0, 0],
					[43, 640, 0, 0],
					[43, 641, 0, 0],
					[43, 642, 0, 0],
					[43, 643, 0, 0],
					[43, 644, 0, 0],
					[43, 645, 0, 0],
					[45, 646, 0, 0],
					[45, 647, 0, 0],
					[45, 648, 0, 0],
					[45, 649, 0, 0],
					[45, 650, 0, 0],
					[29, 652, 0, 0],
					[29, 689, 0, 0],
					[42, 690, 0, 0],
					[40, 691, 0, 0],
					[29, 692, 0, 0],
					[29, 693, 0, 0],
					[29, 694, 0, 0],
					[29, 699, 0, 0],
					[6, 700, 0, 0],
					[4, 701, 0, 0],
					[29, 703, 0, 0],
					[18, 708, 0, 0],
					[40, 710, 0, 0],
					[40, 711, 0, 0],
					[29, 716, 0, 0],
					[29, 719, 0, 0],
					[13, 720, 0, 0],
					[29, 721, 0, 0],
					[29, 722, 0, 0],
					[4, 725, 0, 0],
					[6, 726, 0, 0],
					[7, 727, 0, 0],
					[9, 729, 0, 0],
					[13, 730, 0, 0],
					[18, 734, 0, 0],
					[33, 741, 0, 0],
					[34, 742, 0, 0],
					[35, 743, 0, 0],
					[37, 744, 0, 0],
					[40, 745, 0, 0],
					[41, 746, 0, 0],
					[42, 747, 0, 0],
					[43, 748, 0, 0],
					[45, 749, 0, 0],
					[51, 780, 0, 0],
					[51, 781, 0, 0],
					[51, 782, 0, 0],
					[51, 783, 0, 0],
					[51, 784, 0, 0],
					[51, 785, 0, 0],
					[51, 786, 0, 0],
					[51, 787, 0, 0],
					[51, 788, 0, 0],
					[51, 791, 0, 0],
					[51, 792, 0, 0],
					[13, 814, 0, 0],
					[42, 816, 0, 0],
					[37, 818, 0, 0],
					[34, 820, 0, 0],
					[43, 826, 0, 0],
					[54, 842, 0, 0],
					[54, 843, 0, 0],
					[54, 844, 0, 0],
					[54, 845, 0, 0],
					[54, 846, 0, 0],
					[54, 847, 0, 0],
					[54, 848, 0, 0],
					[54, 849, 0, 0],
					[54, 850, 0, 0],
					[54, 851, 0, 0],
					[54, 852, 0, 0],
					[54, 853, 0, 0],
					[54, 854, 0, 0],
					[54, 855, 0, 0],
					[54, 856, 0, 0],
					[54, 857, 0, 0],
					[54, 858, 0, 0],
					[54, 859, 0, 0],
					[54, 860, 0, 0],
					[54, 861, 0, 0],
					[54, 862, 0, 0],
					[54, 863, 0, 0],
					[54, 864, 0, 0],
					[8, 865, 0, 0],
					[57, 866, 0, 0],
					[57, 867, 0, 0],
					[57, 868, 0, 0],
					[57, 870, 0, 0],
					[57, 871, 0, 0],
					[57, 872, 0, 0],
					[57, 873, 0, 0],
					[57, 874, 0, 0],
					[57, 875, 0, 0],
					[57, 876, 0, 0],
					[57, 877, 0, 0],
					[57, 878, 0, 0],
					[58, 879, 0, 0],
					[58, 880, 0, 0],
					[58, 881, 0, 0],
					[58, 883, 0, 0],
					[58, 884, 0, 0],
					[58, 885, 0, 0],
					[58, 886, 0, 0],
					[58, 887, 0, 0],
					[58, 888, 0, 0],
					[58, 889, 0, 0],
					[58, 890, 0, 0],
					[58, 891, 0, 0],
					[59, 892, 0, 0],
					[59, 893, 0, 0],
					[59, 894, 0, 0],
					[59, 895, 0, 0],
					[59, 896, 0, 0],
					[59, 898, 0, 0],
					[59, 899, 0, 0],
					[59, 900, 0, 0],
					[59, 901, 0, 0],
					[59, 902, 0, 0],
					[59, 903, 0, 0],
					[59, 904, 0, 0],
					[59, 905, 0, 0],
					[60, 927, 0, 0],
					[60, 928, 0, 0],
					[60, 929, 0, 0],
					[60, 930, 0, 0],
					[60, 931, 0, 0],
					[60, 932, 0, 0],
					[60, 933, 0, 0],
					[60, 934, 0, 0],
					[60, 935, 0, 0],
					[60, 936, 0, 0],
					[7, 938, 0, 0],
					[7, 940, 0, 0],
					[61, 941, 0, 0],
					[61, 942, 0, 0],
					[61, 943, 0, 0],
					[61, 944, 0, 0],
					[61, 945, 0, 0],
					[61, 946, 0, 0],
					[61, 947, 0, 0],
					[61, 948, 0, 0],
					[61, 949, 0, 0],
					[61, 950, 0, 0],
					[61, 951, 0, 0],
					[61, 952, 0, 0],
					[61, 953, 0, 0],
					[61, 954, 0, 0],
					[61, 955, 0, 0],
					[61, 956, 0, 0],
					[61, 957, 0, 0],
					[61, 958, 0, 0],
					[61, 959, 0, 0],
					[61, 960, 0, 0],
					[61, 961, 0, 0],
					[61, 962, 0, 0],
					[61, 963, 0, 0],
					[61, 964, 0, 0],
					[61, 965, 0, 0],
					[61, 966, 0, 0],
					[7, 967, 0, 0],
					[7, 968, 0, 0],
					[7, 969, 0, 0],
					[6, 970, 0, 0],
					[6, 971, 0, 0],
					[6, 972, 0, 0],
					[18, 973, 0, 0],
					[18, 974, 0, 0],
					[18, 975, 0, 0],
					[43, 1044, 0, 0],
					[29, 1045, 0, 0],
					[13, 1049, 0, 0],
					[7, 1065, 0, 0],
					[7, 1066, 0, 0],
					[7, 1067, 0, 0],
					[7, 1068, 0, 0],
					[7, 1069, 0, 0],
					[7, 1070, 0, 0],
					[7, 1071, 0, 0],
					[7, 1072, 0, 0],
					[4, 1073, 0, 0],
					[4, 1074, 0, 0],
					[4, 1075, 0, 0],
					[4, 1076, 0, 0],
					[4, 1077, 0, 0],
					[4, 1078, 0, 0],
					[4, 1079, 0, 0],
					[4, 1080, 0, 0],
					[4, 1081, 0, 0],
					[4, 1082, 0, 0],
					[4, 1083, 0, 0],
					[4, 1084, 0, 0],
					[4, 1085, 0, 0],
					[4, 1086, 0, 0],
					[4, 1087, 0, 0],
					[4, 1088, 0, 0],
					[6, 1089, 0, 0],
					[6, 1090, 0, 0],
					[6, 1091, 0, 0],
					[6, 1092, 0, 0],
					[6, 1093, 0, 0],
					[6, 1094, 0, 0],
					[6, 1095, 0, 0],
					[6, 1096, 0, 0],
					[6, 1097, 0, 0],
					[6, 1098, 0, 0],
					[6, 1099, 0, 0],
					[6, 1100, 0, 0],
					[6, 1101, 0, 0],
					[6, 1102, 0, 0],
					[6, 1103, 0, 0],
					[6, 1104, 0, 0],
					[18, 1105, 0, 0],
					[18, 1106, 0, 0],
					[18, 1107, 0, 0],
					[18, 1108, 0, 0],
					[18, 1109, 0, 0],
					[18, 1110, 0, 0],
					[18, 1111, 0, 0],
					[18, 1112, 0, 0],
					[18, 1113, 0, 0],
					[18, 1114, 0, 0],
					[18, 1115, 0, 0],
					[18, 1116, 0, 0],
					[18, 1117, 0, 0],
					[18, 1118, 0, 0],
					[18, 1119, 0, 0],
					[18, 1120, 0, 0],
					[18, 1121, 0, 0],
					[18, 1122, 0, 0],
					[18, 1123, 0, 0],
					[18, 1124, 0, 0],
					[18, 1125, 0, 0],
					[18, 1126, 0, 0],
					[18, 1127, 0, 0],
					[18, 1128, 0, 0],
					[6, 1160, 0, 0],
					[6, 1161, 0, 0],
					[6, 1162, 0, 0],
					[6, 1163, 0, 0],
					[6, 1164, 0, 0],
					[6, 1165, 0, 0],
					[6, 1166, 0, 0],
					[6, 1167, 0, 0],
					[7, 1276, 0, 0],
					[6, 1277, 0, 0],
					[4, 1278, 0, 0],
					[18, 1279, 0, 0],
					[6, 1285, 0, 0],
					[6, 1286, 0, 0],
					[6, 1287, 0, 0],
					[6, 1288, 0, 0],
					[6, 1289, 0, 0],
					[6, 1290, 0, 0],
					[7, 1291, 0, 0],
					[7, 1292, 0, 0],
					[4, 1293, 0, 0],
					[4, 1294, 0, 0],
					[4, 1295, 0, 0],
					[4, 1296, 0, 0],
					[37, 1314, 0, 0],
					[59, 1315, 0, 0],
					[57, 1316, 0, 0],
					[58, 1317, 0, 0],
					[42, 1318, 0, 0],
					[42, 1319, 0, 0],
					[42, 1320, 0, 0],
					[7, 1321, 0, 0],
					[29, 1322, 0, 0],
					[37, 1323, 0, 0],
					[37, 1324, 0, 0],
					[51, 1326, 0, 0],
					[51, 1327, 0, 0],
					[7, 1329, 0, 0],
					[6, 1330, 0, 0],
					[6, 1331, 0, 0],
					[4, 1332, 0, 0],
					[18, 1333, 0, 0],
					[43, 1340, 0, 0],
					[13, 1341, 0, 0],
					[58, 1344, 0, 0],
					[4, 1368, 0, 0],
					[6, 1369, 0, 0],
					[6, 1375, 0, 0],
					[26, 1378, 0, 0],
					[43, 1382, 0, 0],
					[13, 1384, 0, 0],
					[4, 1386, 0, 0],
					[4, 1391, 0, 0],
					[14, 1393, 0, 0],
					[35, 1395, 0, 0],
					[7, 1397, 0, 0],
					[8, 1399, 0, 0],
					[9, 1401, 0, 0],
					[15, 1403, 0, 0],
					[16, 1405, 0, 0],
					[18, 1407, 0, 0],
					[19, 1409, 0, 0],
					[34, 1419, 0, 0],
					[37, 1421, 0, 0],
					[41, 1423, 0, 0],
					[42, 1425, 0, 0],
					[51, 1431, 0, 0],
					[54, 1433, 0, 0],
					[57, 1435, 0, 0],
					[58, 1437, 0, 0],
					[59, 1439, 0, 0],
					[60, 1441, 0, 0],
					[61, 1443, 0, 0],
					[4, 1449, 0, 0],
					[6, 1450, 0, 0],
					[7, 1451, 0, 0],
					[8, 1452, 0, 0],
					[9, 1453, 0, 0],
					[13, 1454, 0, 0],
					[14, 1455, 0, 0],
					[15, 1456, 0, 0],
					[16, 1457, 0, 0],
					[18, 1458, 0, 0],
					[19, 1459, 0, 0],
					[26, 1464, 0, 0],
					[34, 1465, 0, 0],
					[35, 1466, 0, 0],
					[37, 1467, 0, 0],
					[41, 1468, 0, 0],
					[42, 1469, 0, 0],
					[43, 1470, 0, 0],
					[51, 1473, 0, 0],
					[54, 1474, 0, 0],
					[57, 1475, 0, 0],
					[58, 1476, 0, 0],
					[59, 1477, 0, 0],
					[60, 1478, 0, 0],
					[61, 1479, 0, 0],
					[13, 1482, 0, 0],
					[13, 1483, 0, 0],
					[37, 1484, 0, 0],
					[58, 1485, 0, 0],
					[7, 1486, 0, 0],
					[4, 1487, 0, 0],
					[6, 1488, 0, 0],
					[18, 1489, 0, 0],
					[4, 1494, 0, 0],
					[6, 1495, 0, 0],
					[18, 1496, 0, 0],
					[18, 1501, 0, 0],
					[6, 1502, 0, 0],
					[4, 1503, 0, 0],
					[74, 1504, 0, 0],
					[74, 1505, 0, 0],
					[74, 1506, 0, 0],
					[74, 1507, 0, 0],
					[74, 1508, 0, 0],
					[74, 1509, 0, 0],
					[74, 1511, 0, 0],
					[74, 1512, 0, 0],
					[74, 1513, 0, 0],
					[74, 1514, 0, 0],
					[74, 1515, 0, 0],
					[74, 1516, 0, 0],
					[74, 1517, 0, 0],
					[74, 1518, 0, 0],
					[74, 1519, 0, 0],
					[74, 1520, 0, 0],
					[74, 1521, 0, 0],
					[74, 1522, 0, 0],
					[4, 1524, 0, 0],
					[6, 1525, 0, 0],
					[7, 1526, 0, 0],
					[8, 1527, 0, 0],
					[9, 1528, 0, 0],
					[13, 1529, 0, 0],
					[14, 1530, 0, 0],
					[15, 1531, 0, 0],
					[18, 1532, 0, 0],
					[19, 1533, 0, 0],
					[26, 1538, 0, 0],
					[34, 1539, 0, 0],
					[35, 1540, 0, 0],
					[37, 1541, 0, 0],
					[41, 1542, 0, 0],
					[42, 1543, 0, 0],
					[43, 1544, 0, 0],
					[51, 1547, 0, 0],
					[54, 1548, 0, 0],
					[57, 1549, 0, 0],
					[58, 1550, 0, 0],
					[59, 1551, 0, 0],
					[60, 1552, 0, 0],
					[61, 1553, 0, 0],
					[74, 1556, 0, 0],
					[75, 1557, 0, 0],
					[75, 1558, 0, 0],
					[75, 1559, 0, 0],
					[75, 1560, 0, 0],
					[75, 1561, 0, 0],
					[75, 1562, 0, 0],
					[75, 1563, 0, 0],
					[75, 1564, 0, 0],
					[75, 1566, 0, 0],
					[75, 1567, 0, 0],
					[75, 1568, 0, 0],
					[75, 1569, 0, 0],
					[51, 1600, 0, 0],
					[9, 1603, 0, 0],
					[16, 1604, 0, 0],
					[78, 1605, 0, 0],
					[78, 1606, 0, 0],
					[78, 1607, 0, 0],
					[78, 1608, 0, 0],
					[78, 1609, 0, 0],
					[78, 1610, 0, 0],
					[78, 1611, 0, 0],
					[78, 1612, 0, 0],
					[78, 1613, 0, 0],
					[78, 1615, 0, 0],
					[79, 1616, 0, 0],
					[79, 1617, 0, 0],
					[79, 1618, 0, 0],
					[79, 1619, 0, 0],
					[79, 1620, 0, 0],
					[79, 1621, 0, 0],
					[79, 1622, 0, 0],
					[79, 1625, 0, 0],
					[79, 1626, 0, 0],
					[79, 1627, 0, 0],
					[79, 1628, 0, 0],
					[6, 1630, 0, 0],
					[80, 1632, 0, 0],
					[80, 1633, 0, 0],
					[80, 1634, 0, 0],
					[80, 1635, 0, 0],
					[80, 1636, 0, 0],
					[80, 1637, 0, 0],
					[80, 1638, 0, 0],
					[80, 1640, 0, 0],
					[80, 1641, 0, 0],
					[80, 1642, 0, 0],
					[80, 1643, 0, 0],
					[80, 1644, 0, 0],
					[81, 1645, 0, 0],
					[81, 1646, 0, 0],
					[81, 1647, 0, 0],
					[81, 1648, 0, 0],
					[81, 1649, 0, 0],
					[81, 1650, 0, 0],
					[81, 1651, 0, 0],
					[81, 1652, 0, 0],
					[81, 1653, 0, 0],
					[81, 1655, 0, 0],
					[81, 1656, 0, 0],
					[81, 1657, 0, 0],
					[81, 1658, 0, 0],
					[81, 1659, 0, 0],
					[81, 1660, 0, 0],
					[81, 1661, 0, 0],
					[81, 1666, 0, 0],
					[82, 1667, 0, 0],
					[82, 1668, 0, 0],
					[82, 1669, 0, 0],
					[82, 1670, 0, 0],
					[82, 1671, 0, 0],
					[82, 1672, 0, 0],
					[82, 1673, 0, 0],
					[82, 1674, 0, 0],
					[82, 1675, 0, 0],
					[82, 1677, 0, 0],
					[82, 1678, 0, 0],
					[82, 1679, 0, 0],
					[82, 1680, 0, 0],
					[82, 1681, 0, 0],
					[82, 1682, 0, 0],
					[82, 1683, 0, 0],
					[82, 1688, 0, 0],
					[18, 1689, 0, 0],
					[18, 1690, 0, 0],
					[18, 1691, 0, 0],
					[18, 1692, 0, 0],
					[18, 1693, 0, 0],
					[18, 1694, 0, 0],
					[83, 1708, 0, 0],
					[83, 1709, 0, 0],
					[83, 1710, 0, 0],
					[83, 1711, 0, 0],
					[83, 1712, 0, 0],
					[83, 1713, 0, 0],
					[16, 1714, 0, 0],
					[9, 1715, 0, 0],
					[84, 1716, 0, 0],
					[84, 1717, 0, 0],
					[84, 1718, 0, 0],
					[84, 1719, 0, 0],
					[84, 1720, 0, 0],
					[84, 1721, 0, 0],
					[84, 1722, 0, 0],
					[84, 1723, 0, 0],
					[84, 1724, 0, 0],
					[84, 1725, 0, 0],
					[84, 1726, 0, 0],
					[84, 1727, 0, 0],
					[84, 1728, 0, 0],
					[84, 1729, 0, 0],
					[84, 1730, 0, 0],
					[84, 1731, 0, 0],
					[84, 1732, 0, 0],
					[84, 1733, 0, 0],
					[84, 1735, 0, 0],
					[7, 1736, 0, 0],
					[7, 1737, 0, 0],
					[6, 1738, 0, 0],
					[29, 1740, 0, 0],
					[41, 1741, 0, 0],
					[42, 1742, 0, 0],
					[41, 1743, 0, 0],
					[4, 1744, 0, 0],
					[54, 1745, 0, 0],
					[4, 1746, 0, 0],
					[6, 1747, 0, 0],
					[6, 1756, 0, 0],
					[6, 1757, 0, 0],
					[6, 1758, 0, 0],
					[14, 1759, 0, 0],
					[16, 1761, 0, 0],
					[9, 1762, 0, 0],
					[15, 1763, 0, 0],
					[19, 1764, 0, 0],
					[85, 1765, 0, 0],
					[85, 1766, 0, 0],
					[85, 1767, 0, 0],
					[85, 1768, 0, 0],
					[85, 1769, 0, 0],
					[85, 1770, 0, 0],
					[85, 1771, 0, 0],
					[85, 1772, 0, 0],
					[85, 1773, 0, 0],
					[85, 1774, 0, 0],
					[85, 1775, 0, 0],
					[85, 1776, 0, 0],
					[85, 1777, 0, 0],
					[85, 1778, 0, 0],
					[86, 1779, 0, 0],
					[86, 1780, 0, 0],
					[86, 1781, 0, 0],
					[86, 1782, 0, 0],
					[86, 1783, 0, 0],
					[86, 1784, 0, 0],
					[86, 1785, 0, 0],
					[86, 1786, 0, 0],
					[86, 1787, 0, 0],
					[86, 1788, 0, 0],
					[87, 1789, 0, 0],
					[87, 1790, 0, 0],
					[87, 1791, 0, 0],
					[87, 1792, 0, 0],
					[87, 1793, 0, 0],
					[87, 1794, 0, 0],
					[87, 1795, 0, 0],
					[87, 1796, 0, 0],
					[87, 1797, 0, 0],
					[87, 1798, 0, 0],
					[87, 1799, 0, 0],
					[87, 1800, 0, 0],
					[87, 1801, 0, 0],
					[87, 1802, 0, 0],
					[87, 1803, 0, 0],
					[88, 1804, 0, 0],
					[88, 1805, 0, 0],
					[88, 1806, 0, 0],
					[88, 1807, 0, 0],
					[88, 1808, 0, 0],
					[88, 1809, 0, 0],
					[88, 1810, 0, 0],
					[88, 1811, 0, 0],
					[88, 1812, 0, 0],
					[88, 1813, 0, 0],
					[88, 1814, 0, 0],
					[88, 1815, 0, 0],
					[88, 1816, 0, 0],
					[88, 1817, 0, 0],
					[88, 1818, 0, 0],
					[89, 1819, 0, 0],
					[89, 1820, 0, 0],
					[89, 1821, 0, 0],
					[89, 1822, 0, 0],
					[89, 1823, 0, 0],
					[89, 1824, 0, 0],
					[89, 1825, 0, 0],
					[89, 1826, 0, 0],
					[89, 1827, 0, 0],
					[89, 1828, 0, 0],
					[89, 1829, 0, 0],
					[89, 1830, 0, 0],
					[89, 1831, 0, 0],
					[89, 1832, 0, 0],
					[89, 1833, 0, 0],
					[90, 1834, 0, 0],
					[90, 1835, 0, 0],
					[90, 1836, 0, 0],
					[90, 1837, 0, 0],
					[90, 1838, 0, 0],
					[90, 1839, 0, 0],
					[90, 1840, 0, 0],
					[90, 1841, 0, 0],
					[90, 1842, 0, 0],
					[90, 1843, 0, 0],
					[90, 1844, 0, 0],
					[90, 1845, 0, 0],
					[90, 1846, 0, 0],
					[90, 1847, 0, 0],
					[90, 1848, 0, 0],
					[90, 1849, 0, 0],
					[90, 1850, 0, 0],
					[90, 1851, 0, 0],
					[91, 1852, 0, 0],
					[91, 1853, 0, 0],
					[91, 1854, 0, 0],
					[91, 1855, 0, 0],
					[91, 1856, 0, 0],
					[91, 1857, 0, 0],
					[91, 1858, 0, 0],
					[91, 1859, 0, 0],
					[91, 1860, 0, 0],
					[91, 1861, 0, 0],
					[91, 1862, 0, 0],
					[91, 1863, 0, 0],
					[91, 1864, 0, 0],
					[91, 1865, 0, 0],
					[91, 1866, 0, 0],
					[91, 1867, 0, 0],
					[91, 1868, 0, 0],
					[91, 1869, 0, 0],
					[91, 1870, 0, 0],
					[91, 1871, 0, 0],
					[91, 1872, 0, 0],
					[91, 1873, 0, 0],
					[91, 1874, 0, 0],
					[91, 1875, 0, 0],
					[89, 1876, 0, 0],
					[89, 1877, 0, 0],
					[89, 1878, 0, 0],
					[89, 1879, 0, 0],
					[89, 1880, 0, 0],
					[89, 1881, 0, 0],
					[89, 1882, 0, 0],
					[89, 1883, 0, 0],
					[89, 1884, 0, 0],
					[89, 1885, 0, 0],
					[89, 1886, 0, 0],
					[89, 1887, 0, 0],
					[89, 1888, 0, 0],
					[89, 1889, 0, 0],
					[89, 1890, 0, 0],
					[89, 1891, 0, 0],
					[89, 1892, 0, 0],
					[89, 1893, 0, 0],
					[89, 1894, 0, 0],
					[89, 1895, 0, 0],
					[89, 1896, 0, 0],
					[89, 1897, 0, 0],
					[90, 1898, 0, 0],
					[90, 1899, 0, 0],
					[90, 1900, 0, 0],
					[90, 1901, 0, 0],
					[90, 1902, 0, 0],
					[90, 1903, 0, 0],
					[90, 1904, 0, 0],
					[90, 1905, 0, 0],
					[90, 1906, 0, 0],
					[90, 1907, 0, 0],
					[90, 1908, 0, 0],
					[90, 1909, 0, 0],
					[90, 1910, 0, 0],
					[90, 1911, 0, 0],
					[90, 1912, 0, 0],
					[90, 1913, 0, 0],
					[90, 1914, 0, 0],
					[90, 1915, 0, 0],
					[90, 1916, 0, 0],
					[90, 1917, 0, 0],
					[90, 1918, 0, 0],
					[90, 1919, 0, 0],
					[91, 1920, 0, 0],
					[91, 1921, 0, 0],
					[91, 1922, 0, 0],
					[91, 1923, 0, 0],
					[91, 1924, 0, 0],
					[91, 1925, 0, 0],
					[91, 1926, 0, 0],
					[91, 1927, 0, 0],
					[91, 1928, 0, 0],
					[91, 1929, 0, 0],
					[91, 1930, 0, 0],
					[91, 1931, 0, 0],
					[91, 1932, 0, 0],
					[91, 1933, 0, 0],
					[91, 1934, 0, 0],
					[91, 1935, 0, 0],
					[91, 1936, 0, 0],
					[91, 1937, 0, 0],
					[91, 1938, 0, 0],
					[91, 1939, 0, 0],
					[91, 1940, 0, 0],
					[91, 1941, 0, 0],
					[89, 1942, 0, 0],
					[90, 1943, 0, 0],
					[91, 1944, 0, 0],
					[6, 1945, 0, 0],
					[6, 1946, 0, 0],
					[37, 1947, 0, 0],
					[92, 1948, 0, 0],
					[92, 1949, 0, 0],
					[92, 1950, 0, 0],
					[92, 1951, 0, 0],
					[92, 1952, 0, 0],
					[92, 1953, 0, 0],
					[92, 1954, 0, 0],
					[92, 1955, 0, 0],
					[92, 1956, 0, 0],
					[92, 1957, 0, 0],
					[92, 1958, 0, 0],
					[92, 1959, 0, 0],
					[92, 1960, 0, 0],
					[92, 1961, 0, 0],
					[92, 1962, 0, 0],
					[92, 1963, 0, 0],
					[92, 1964, 0, 0],
					[92, 1965, 0, 0],
					[92, 1966, 0, 0],
					[92, 1967, 0, 0],
					[93, 1968, 0, 0],
					[93, 1969, 0, 0],
					[93, 1970, 0, 0],
					[93, 1971, 0, 0],
					[93, 1972, 0, 0],
					[93, 1973, 0, 0],
					[93, 1974, 0, 0],
					[93, 1975, 0, 0],
					[93, 1976, 0, 0],
					[93, 1977, 0, 0],
					[93, 1978, 0, 0],
					[93, 1979, 0, 0],
					[93, 1980, 0, 0],
					[93, 1981, 0, 0],
					[93, 1982, 0, 0],
					[93, 1983, 0, 0],
					[93, 1984, 0, 0],
					[93, 1985, 0, 0],
					[93, 1986, 0, 0],
					[93, 1987, 0, 0],
					[59, 1988, 0, 0],
					[37, 1989, 0, 0],
					[57, 1990, 0, 0],
					[58, 1991, 0, 0],
					[9, 1992, 0, 0],
					[16, 1993, 0, 0],
					[51, 1994, 0, 0],
					[51, 1995, 0, 0],
					[51, 1996, 0, 0],
					[4, 1997, 0, 0],
					[7, 1998, 0, 0],
					[18, 1999, 0, 0],
					[92, 2000, 0, 0],
					[93, 2001, 0, 0],
					[61, 2002, 0, 0],
					[86, 2003, 0, 0],
					[26, 2004, 0, 0],
					[41, 2005, 0, 0],
					[85, 2006, 0, 0],
					[87, 2007, 0, 0],
					[88, 2008, 0, 0],
					[89, 2009, 0, 0],
					[90, 2010, 0, 0],
					[91, 2011, 0, 0],
					[92, 2012, 0, 0],
					[93, 2013, 0, 0],
					[4, 2014, 0, 0],
					[6, 2015, 0, 0],
					[7, 2016, 0, 0],
					[13, 2017, 0, 0],
					[18, 2018, 0, 0],
					[26, 2019, 0, 0],
					[34, 2020, 0, 0],
					[43, 2021, 0, 0],
					[61, 2022, 0, 0],
					[85, 2023, 0, 0],
					[86, 2024, 0, 0],
					[87, 2025, 0, 0],
					[88, 2026, 0, 0],
					[89, 2027, 0, 0],
					[90, 2028, 0, 0],
					[91, 2029, 0, 0],
					[92, 2030, 0, 0],
					[93, 2031, 0, 0],
					[94, 2032, 0, 0],
					[94, 2033, 0, 0],
					[94, 2034, 0, 0],
					[94, 2035, 0, 0],
					[94, 2036, 0, 0],
					[94, 2037, 0, 0],
					[94, 2038, 0, 0],
					[94, 2039, 0, 0],
					[95, 2040, 0, 0],
					[95, 2041, 0, 0],
					[95, 2042, 0, 0],
					[95, 2043, 0, 0],
					[95, 2044, 0, 0],
					[95, 2046, 0, 0],
					[95, 2047, 0, 0],
					[95, 2048, 0, 0],
					[95, 2049, 0, 0],
					[95, 2050, 0, 0],
					[95, 2051, 0, 0],
					[95, 2052, 0, 0],
					[95, 2053, 0, 0],
					[95, 2054, 0, 0],
					[95, 2055, 0, 0],
					[95, 2056, 0, 0],
					[95, 2057, 0, 0],
					[95, 2058, 0, 0],
					[95, 2059, 0, 0],
					[95, 2060, 0, 0],
					[95, 2061, 0, 0],
					[95, 2062, 0, 0],
					[95, 2063, 0, 0],
					[95, 2064, 0, 0],
					[95, 2065, 0, 0],
					[95, 2066, 0, 0],
					[95, 2067, 0, 0],
					[95, 2068, 0, 0],
					[95, 2069, 0, 0],
					[95, 2070, 0, 0],
					[95, 2071, 0, 0],
					[95, 2072, 0, 0],
					[95, 2073, 0, 0],
					[95, 2074, 0, 0],
					[96, 2075, 0, 0],
					[96, 2076, 0, 0],
					[96, 2077, 0, 0],
					[96, 2078, 0, 0],
					[96, 2079, 0, 0],
					[96, 2080, 0, 0],
					[96, 2081, 0, 0],
					[96, 2082, 0, 0],
					[97, 2083, 0, 0],
					[97, 2084, 0, 0],
					[97, 2085, 0, 0],
					[97, 2086, 0, 0],
					[97, 2087, 0, 0],
					[97, 2088, 0, 0],
					[97, 2089, 0, 0],
					[97, 2090, 0, 0],
					[97, 2091, 0, 0],
					[97, 2092, 0, 0],
					[97, 2093, 0, 0],
					[86, 2094, 0, 0],
					[86, 2095, 0, 0],
					[86, 2096, 0, 0],
					[86, 2097, 0, 0],
					[86, 2098, 0, 0],
					[86, 2099, 0, 0],
					[86, 2100, 0, 0],
					[86, 2101, 0, 0],
					[86, 2102, 0, 0],
					[88, 2103, 0, 0],
					[88, 2104, 0, 0],
					[88, 2105, 0, 0],
					[89, 2106, 0, 0],
					[89, 2107, 0, 0],
					[89, 2108, 0, 0],
					[89, 2109, 0, 0],
					[89, 2110, 0, 0],
					[90, 2111, 0, 0],
					[90, 2112, 0, 0],
					[90, 2113, 0, 0],
					[90, 2114, 0, 0],
					[90, 2115, 0, 0],
					[98, 2116, 0, 0],
					[98, 2117, 0, 0],
					[98, 2118, 0, 0],
					[98, 2119, 0, 0],
					[98, 2120, 0, 0],
					[98, 2121, 0, 0],
					[99, 2122, 0, 0],
					[99, 2123, 0, 0],
					[99, 2124, 0, 0],
					[99, 2125, 0, 0],
					[99, 2126, 0, 0],
					[99, 2127, 0, 0],
					[99, 2128, 0, 0],
					[99, 2129, 0, 0],
					[99, 2130, 0, 0],
					[99, 2131, 0, 0],
					[99, 2132, 0, 0],
					[99, 2133, 0, 0],
					[99, 2134, 0, 0],
					[99, 2135, 0, 0],
					[99, 2136, 0, 0],
					[99, 2137, 0, 0],
					[99, 2138, 0, 0],
					[99, 2139, 0, 0],
					[99, 2140, 0, 0],
					[99, 2141, 0, 0],
					[99, 2142, 0, 0],
					[99, 2143, 0, 0],
					[99, 2144, 0, 0],
					[99, 2145, 0, 0],
					[99, 2146, 0, 0],
					[99, 2147, 0, 0],
					[99, 2148, 0, 0],
					[99, 2149, 0, 0],
					[99, 2150, 0, 0],
					[99, 2151, 0, 0],
					[99, 2152, 0, 0],
					[99, 2153, 0, 0],
					[99, 2154, 0, 0],
					[99, 2155, 0, 0],
					[100, 2156, 0, 0],
					[100, 2157, 0, 0],
					[100, 2158, 0, 0],
					[100, 2159, 0, 0],
					[100, 2160, 0, 0],
					[100, 2161, 0, 0],
					[100, 2162, 0, 0],
					[101, 2163, 0, 0],
					[101, 2164, 0, 0],
					[101, 2165, 0, 0],
					[101, 2166, 0, 0],
					[101, 2167, 0, 0],
					[101, 2168, 0, 0],
					[101, 2169, 0, 0],
					[98, 2170, 0, 0],
					[102, 2171, 0, 0],
					[102, 2172, 0, 0],
					[102, 2173, 0, 0],
					[102, 2174, 0, 0],
					[102, 2175, 0, 0],
					[102, 2176, 0, 0],
					[102, 2177, 0, 0],
					[98, 2178, 0, 0],
					[100, 2179, 0, 0],
					[101, 2180, 0, 0],
					[102, 2181, 0, 0],
					[103, 2182, 0, 0],
					[103, 2183, 0, 0],
					[103, 2184, 0, 0],
					[103, 2185, 0, 0],
					[103, 2186, 0, 0],
					[103, 2187, 0, 0],
					[103, 2188, 0, 0],
					[98, 2189, 0, 0],
					[98, 2190, 0, 0],
					[98, 2191, 0, 0],
					[98, 2192, 0, 0],
					[100, 2193, 0, 0],
					[100, 2194, 0, 0],
					[100, 2195, 0, 0],
					[100, 2196, 0, 0],
					[101, 2197, 0, 0],
					[101, 2198, 0, 0],
					[101, 2199, 0, 0],
					[102, 2200, 0, 0],
					[102, 2201, 0, 0],
					[102, 2202, 0, 0],
					[103, 2203, 0, 0],
					[103, 2204, 0, 0],
					[103, 2205, 0, 0],
					[103, 2206, 0, 0],
					[103, 2207, 0, 0],
					[14, 2208, 0, 0],
					[98, 2209, 0, 0],
					[96, 2210, 0, 0],
					[97, 2211, 0, 0],
					[97, 2212, 0, 0],
					[97, 2213, 0, 0],
					[97, 2214, 0, 0],
					[97, 2215, 0, 0],
					[97, 2216, 0, 0],
					[97, 2217, 0, 0],
					[97, 2218, 0, 0],
					[97, 2219, 0, 0],
					[97, 2220, 0, 0],
					[97, 2221, 0, 0],
					[104, 2222, 0, 0],
					[104, 2223, 0, 0],
					[104, 2224, 0, 0],
					[104, 2225, 0, 0],
					[104, 2226, 0, 0],
					[104, 2227, 0, 0],
					[104, 2228, 0, 0],
					[104, 2229, 0, 0],
					[104, 2230, 0, 0],
					[104, 2231, 0, 0],
					[104, 2232, 0, 0],
					[104, 2233, 0, 0],
					[104, 2234, 0, 0],
					[105, 2235, 0, 0],
					[105, 2236, 0, 0],
					[105, 2237, 0, 0],
					[105, 2238, 0, 0],
					[105, 2239, 0, 0],
					[105, 2240, 0, 0],
					[105, 2241, 0, 0],
					[105, 2242, 0, 0],
					[105, 2243, 0, 0],
					[105, 2244, 0, 0],
					[105, 2245, 0, 0],
					[106, 2246, 0, 0],
					[106, 2247, 0, 0],
					[106, 2248, 0, 0],
					[106, 2249, 0, 0],
					[106, 2250, 0, 0],
					[106, 2251, 0, 0],
					[106, 2252, 0, 0],
					[106, 2253, 0, 0],
					[106, 2254, 0, 0],
					[106, 2255, 0, 0],
					[106, 2256, 0, 0],
					[106, 2257, 0, 0],
					[106, 2258, 0, 0],
					[104, 2259, 0, 0],
					[106, 2260, 0, 0],
					[104, 2261, 0, 0],
					[106, 2262, 0, 0],
					[96, 2263, 0, 0],
					[96, 2264, 0, 0],
					[96, 2265, 0, 0],
					[96, 2266, 0, 0],
					[58, 2267, 0, 0],
					[37, 2268, 0, 0],
					[14, 2269, 0, 0],
					[107, 2270, 0, 0],
					[107, 2271, 0, 0],
					[107, 2272, 0, 0],
					[107, 2273, 0, 0],
					[107, 2274, 0, 0],
					[107, 2275, 0, 0],
					[107, 2276, 0, 0],
					[107, 2277, 0, 0],
					[107, 2278, 0, 0],
					[107, 2279, 0, 0],
					[107, 2280, 0, 0],
					[107, 2281, 0, 0],
					[107, 2282, 0, 0],
					[107, 2283, 0, 0],
					[107, 2284, 0, 0],
					[107, 2285, 0, 0],
					[107, 2286, 0, 0],
					[107, 2287, 0, 0],
					[107, 2288, 0, 0],
					[107, 2289, 0, 0],
					[107, 2290, 0, 0],
					[107, 2291, 0, 0],
					[107, 2292, 0, 0],
					[107, 2293, 0, 0],
					[107, 2294, 0, 0],
					[107, 2295, 0, 0],
					[107, 2296, 0, 0],
					[107, 2297, 0, 0],
					[107, 2298, 0, 0],
					[107, 2299, 0, 0],
					[107, 2300, 0, 0],
					[107, 2301, 0, 0],
					[107, 2302, 0, 0],
					[107, 2303, 0, 0],
					[107, 2304, 0, 0],
					[108, 2305, 0, 0],
					[108, 2306, 0, 0],
					[108, 2307, 0, 0],
					[108, 2308, 0, 0],
					[108, 2309, 0, 0],
					[108, 2310, 0, 0],
					[108, 2311, 0, 0],
					[108, 2312, 0, 0],
					[108, 2313, 0, 0],
					[108, 2314, 0, 0],
					[108, 2315, 0, 0],
					[108, 2316, 0, 0],
					[108, 2317, 0, 0],
					[108, 2318, 0, 0],
					[109, 2319, 0, 0],
					[109, 2320, 0, 0],
					[109, 2321, 0, 0],
					[109, 2322, 0, 0],
					[109, 2323, 0, 0],
					[109, 2324, 0, 0],
					[109, 2325, 0, 0],
					[109, 2326, 0, 0],
					[109, 2327, 0, 0],
					[109, 2328, 0, 0],
					[109, 2329, 0, 0],
					[109, 2330, 0, 0],
					[109, 2331, 0, 0],
					[13, 2332, 0, 0],
					[13, 2333, 0, 0],
					[14, 2334, 0, 0],
					[35, 2335, 0, 0],
					[37, 2336, 0, 0],
					[58, 2337, 0, 0],
					[90, 2344, 0, 0],
					[100, 2345, 0, 0],
					[6, 2346, 0, 0],
					[13, 2347, 0, 0],
					[13, 2348, 0, 0],
					[4, 2349, 0, 0],
					[6, 2350, 0, 0],
					[7, 2351, 0, 0],
					[92, 2352, 0, 0],
					[18, 2353, 0, 0],
					[93, 2354, 0, 0],
					[83, 2355, 0, 0],
					[83, 2356, 0, 0],
					[14, 2357, 0, 0],
					[89, 2358, 0, 0],
					[92, 2381, 0, 0],
					[61, 2382, 0, 0],
					[61, 2383, 0, 0],
					[61, 2384, 0, 0],
					[86, 2385, 0, 0],
					[43, 2386, 0, 0],
					[111, 2387, 0, 0],
					[111, 2388, 0, 0],
					[111, 2389, 0, 0],
					[111, 2390, 0, 0],
					[111, 2391, 0, 0],
					[111, 2392, 0, 0],
					[111, 2393, 0, 0],
					[111, 2395, 0, 0],
					[111, 2396, 0, 0],
					[111, 2397, 0, 0],
					[111, 2398, 0, 0],
					[111, 2399, 0, 0],
					[111, 2400, 0, 0],
					[6, 2401, 0, 0],
					[95, 2402, 0, 0],
					[95, 2403, 0, 0],
					[86, 2404, 0, 0],
					[6, 2405, 0, 0],
					[4, 2406, 0, 0],
					[7, 2407, 0, 0],
					[18, 2408, 0, 0],
					[14, 2409, 0, 0],
					[86, 2410, 0, 0],
					[97, 2411, 0, 0],
					[26, 2412, 0, 0],
					[16, 2413, 0, 0],
					[13, 2414, 0, 0],
					[105, 2415, 0, 0],
					[35, 2416, 0, 0],
					[106, 2417, 0, 0],
					[98, 2418, 0, 0],
					[54, 2419, 0, 0],
					[108, 2420, 0, 0],
					[100, 2421, 0, 0],
					[109, 2422, 0, 0],
					[9, 2423, 0, 0],
					[43, 2424, 0, 0],
					[101, 2425, 0, 0],
					[102, 2426, 0, 0],
					[37, 2427, 0, 0],
					[61, 2428, 0, 0],
					[103, 2429, 0, 0],
					[15, 2430, 0, 0],
					[74, 2431, 0, 0],
					[104, 2432, 0, 0],
					[34, 2433, 0, 0],
					[59, 2434, 0, 0],
					[51, 2435, 0, 0],
					[19, 2436, 0, 0],
					[41, 2437, 0, 0],
					[8, 2438, 0, 0],
					[58, 2439, 0, 0],
					[84, 2440, 0, 0],
					[60, 2441, 0, 0],
					[57, 2442, 0, 0],
					[83, 2443, 0, 0],
					[42, 2444, 0, 0],
					[75, 2445, 0, 0],
					[78, 2446, 0, 0],
					[79, 2447, 0, 0],
					[80, 2448, 0, 0],
					[81, 2449, 0, 0],
					[107, 2450, 0, 0],
					[82, 2451, 0, 0],
					[99, 2452, 0, 0],
					[94, 2453, 0, 0],
					[95, 2454, 0, 0],
					[96, 2455, 0, 0],
					[111, 2456, 0, 0],
					[85, 2458, 0, 0],
					[87, 2459, 0, 0],
					[89, 2460, 0, 0],
					[91, 2461, 0, 0],
					[93, 2462, 0, 0],
					[88, 2463, 0, 0],
					[90, 2464, 0, 0],
					[92, 2465, 0, 0],
					[33, 2466, 0, 0],
					[35, 2467, 0, 0],
					[29, 2468, 0, 0],
					[29, 2469, 0, 0],
					[29, 2470, 0, 0],
					[112, 2471, 0, 0],
					[112, 2472, 0, 0],
					[112, 2473, 0, 0],
					[112, 2474, 0, 0],
					[112, 2475, 0, 0],
					[112, 2476, 0, 0],
					[112, 2477, 0, 0],
					[112, 2478, 0, 0],
					[112, 2479, 0, 0],
					[112, 2480, 0, 0],
					[112, 2481, 0, 0],
					[112, 2482, 0, 0],
					[112, 2483, 0, 0],
					[86, 2484, 0, 0],
					[113, 2485, 0, 0],
					[113, 2486, 0, 0],
					[113, 2487, 0, 0],
					[113, 2488, 0, 0],
					[113, 2489, 0, 0],
					[113, 2490, 0, 0],
					[113, 2491, 0, 0],
					[113, 2492, 0, 0],
					[113, 2493, 0, 0],
					[113, 2494, 0, 0],
					[113, 2495, 0, 0],
					[113, 2496, 0, 0],
					[113, 2497, 0, 0],
					[113, 2498, 0, 0],
					[113, 2499, 0, 0],
					[113, 2500, 0, 0],
					[113, 2501, 0, 0],
					[113, 2502, 0, 0],
					[113, 2503, 0, 0],
					[113, 2504, 0, 0],
					[113, 2505, 0, 0],
					[113, 2506, 0, 0],
					[113, 2507, 0, 0],
					[113, 2508, 0, 0],
					[114, 2509, 0, 0],
					[114, 2510, 0, 0],
					[114, 2511, 0, 0],
					[114, 2512, 0, 0],
					[114, 2513, 0, 0],
					[114, 2514, 0, 0],
					[114, 2515, 0, 0],
					[114, 2516, 0, 0],
					[114, 2517, 0, 0],
					[114, 2518, 0, 0],
					[114, 2519, 0, 0],
					[115, 2520, 0, 0],
					[115, 2521, 0, 0],
					[115, 2522, 0, 0],
					[115, 2523, 0, 0],
					[115, 2524, 0, 0],
					[115, 2525, 0, 0],
					[115, 2526, 0, 0],
					[115, 2527, 0, 0],
					[115, 2528, 0, 0],
					[115, 2529, 0, 0],
					[115, 2530, 0, 0],
					[115, 2531, 0, 0],
					[115, 2532, 0, 0],
					[115, 2533, 0, 0],
					[115, 2534, 0, 0],
					[115, 2535, 0, 0],
					[115, 2536, 0, 0],
					[115, 2537, 0, 0],
					[115, 2538, 0, 0],
					[115, 2539, 0, 0],
					[115, 2540, 0, 0],
					[115, 2541, 0, 0],
					[115, 2542, 0, 0],
					[115, 2543, 0, 0],
					[115, 2544, 0, 0],
					[115, 2545, 0, 0],
					[115, 2546, 0, 0],
					[115, 2547, 0, 0],
					[115, 2548, 0, 0],
					[115, 2549, 0, 0],
					[115, 2550, 0, 0],
					[115, 2551, 0, 0],
					[115, 2552, 0, 0],
					[115, 2553, 0, 0],
					[115, 2554, 0, 0],
					[115, 2555, 0, 0],
					[115, 2556, 0, 0],
					[115, 2557, 0, 0],
					[115, 2558, 0, 0],
					[116, 2559, 0, 0],
					[116, 2560, 0, 0],
					[116, 2561, 0, 0],
					[116, 2562, 0, 0],
					[116, 2563, 0, 0],
					[116, 2564, 0, 0],
					[116, 2565, 0, 0],
					[116, 2566, 0, 0],
					[116, 2567, 0, 0],
					[116, 2568, 0, 0],
					[116, 2569, 0, 0],
					[95, 2570, 0, 0],
					[95, 2571, 0, 0],
					[95, 2572, 0, 0],
					[95, 2573, 0, 0],
					[113, 2574, 0, 0],
					[113, 2575, 0, 0],
					[113, 2576, 0, 0],
					[117, 2577, 0, 0],
					[117, 2578, 0, 0],
					[117, 2579, 0, 0],
					[117, 2580, 0, 0],
					[117, 2581, 0, 0],
					[117, 2582, 0, 0],
					[117, 2583, 0, 0],
					[117, 2584, 0, 0],
					[117, 2585, 0, 0],
					[117, 2586, 0, 0],
					[117, 2587, 0, 0],
					[117, 2588, 0, 0],
					[117, 2589, 0, 0],
					[117, 2590, 0, 0],
					[117, 2591, 0, 0],
					[117, 2592, 0, 0],
					[117, 2593, 0, 0],
					[117, 2594, 0, 0],
					[117, 2595, 0, 0],
					[117, 2596, 0, 0],
					[117, 2597, 0, 0],
					[117, 2598, 0, 0],
					[16, 2600, 0, 0],
					[16, 2601, 0, 0],
					[9, 2602, 0, 0],
					[9, 2603, 0, 0],
					[43, 2604, 0, 0],
					[83, 2605, 0, 0],
					[45, 2606, 0, 0],
				]
			],
			'vtiger_def_org_share' => [
				'columns' => ['ruleid', 'tabid', 'permission', 'editstatus'],
				'values' => [
					[2, 4, 2, 2],
					[3, 6, 2, 0],
					[4, 7, 2, 0],
					[5, 9, 3, 0],
					[6, 13, 2, 0],
					[7, 16, 3, 2],
					[12, 26, 2, 0],
					[13, 8, 2, 0],
					[14, 14, 2, 0],
					[15, 33, 3, 0],
					[16, 34, 2, 0],
					[17, 35, 2, 0],
					[18, 37, 2, 0],
					[19, 40, 2, 0],
					[20, 41, 2, 0],
					[21, 42, 2, 0],
					[22, 43, 2, 0],
					[23, 45, 2, 0],
					[24, 18, 2, 0],
					[28, 51, 2, 0],
					[29, 54, 3, 0],
					[30, 57, 2, 0],
					[31, 58, 2, 0],
					[32, 59, 2, 0],
					[33, 60, 2, 0],
					[34, 61, 2, 0],
					[45, 74, 2, 0],
					[46, 75, 2, 0],
					[49, 78, 2, 0],
					[50, 79, 2, 0],
					[51, 80, 2, 0],
					[52, 81, 2, 0],
					[53, 82, 2, 0],
					[54, 83, 2, 0],
					[55, 84, 2, 0],
					[56, 15, 2, 0],
					[57, 19, 2, 0],
					[58, 85, 2, 0],
					[59, 86, 2, 0],
					[60, 87, 2, 0],
					[61, 88, 2, 0],
					[62, 89, 2, 0],
					[63, 90, 2, 0],
					[64, 91, 2, 0],
					[65, 92, 2, 0],
					[66, 93, 2, 0],
					[67, 94, 2, 0],
					[68, 95, 2, 0],
					[69, 96, 2, 0],
					[70, 97, 2, 0],
					[71, 98, 2, 0],
					[72, 99, 2, 0],
					[73, 100, 2, 0],
					[74, 101, 2, 0],
					[75, 102, 2, 0],
					[76, 103, 2, 0],
					[77, 104, 2, 0],
					[78, 105, 2, 0],
					[79, 106, 2, 0],
					[80, 107, 2, 0],
					[81, 108, 2, 0],
					[82, 109, 2, 0],
					[83, 111, 2, 0],
					[84, 112, 2, 0],
					[85, 113, 2, 0],
					[86, 114, 2, 0],
					[87, 115, 2, 0],
					[88, 116, 2, 0],
					[89, 117, 2, 0],
				]
			],
			'vtiger_default_record_view' => [
				'columns' => ['default_record_viewid', 'default_record_view', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Summary', 1, 1],
					[2, 'Detail', 2, 1],
				]
			],
			'vtiger_default_record_view_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_defaultactivitytype' => [
				'columns' => ['defaultactivitytypeid', 'defaultactivitytype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Call', 1, 325, 1],
					[2, 'Meeting', 1, 326, 2],
				]
			],
			'vtiger_defaultactivitytype_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_defaulteventstatus' => [
				'columns' => ['defaulteventstatusid', 'defaulteventstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Held', 1, 323, 2],
					[2, 'Not Held', 1, 324, 3],
				]
			],
			'vtiger_defaulteventstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_duration_minutes' => [
				'columns' => ['minutesid', 'duration_minutes', 'sortorderid', 'presence'],
				'values' => [
					[1, '00', 0, 1],
					[2, '15', 1, 1],
					[3, '30', 2, 1],
					[4, '45', 3, 1],
				]
			],
			'vtiger_duration_minutes_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_email_template_priority' => [
				'columns' => ['email_template_priorityid', 'email_template_priority', 'presence', 'sortorderid'],
				'values' => [
					[1, 1, 1, 1],
					[2, 2, 1, 2],
					[3, 3, 1, 3],
					[4, 4, 1, 4],
					[5, 5, 1, 5],
					[6, 6, 1, 6],
					[7, 7, 1, 7],
					[8, 8, 1, 8],
					[9, 9, 1, 9],
				]
			],
			'vtiger_email_template_type' => [
				'columns' => ['email_template_typeid', 'email_template_type', 'presence', 'sortorderid'],
				'values' => [
					[1, 'PLL_MAIL', 1, 1],
					[2, 'PLL_RECORD', 1, 2],
				]
			],
			'vtiger_employee_education' => [
				'columns' => ['employee_educationid', 'employee_education', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Primary', 1, 425, 1],
					[2, 'Secondary', 1, 426, 2],
					[3, 'Higher', 1, 427, 3],
				]
			],
			'vtiger_employee_education_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_employee_status' => [
				'columns' => ['employee_statusid', 'employee_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Recruiting', 1, 422, 1],
					[2, 'Employee', 1, 423, 2],
					[3, 'Inactive', 1, 424, 3],
				]
			],
			'vtiger_employee_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_end_hour' => [
				'columns' => ['end_hourid', 'end_hour', 'sortorderid', 'presence'],
				'values' => [
					[1, '00:00', 1, 1],
					[2, '01:00', 2, 1],
					[3, '02:00', 3, 1],
					[4, '03:00', 4, 1],
					[5, '04:00', 5, 1],
					[6, '05:00', 6, 1],
					[7, '06:00', 7, 1],
					[8, '07:00', 8, 1],
					[9, '08:00', 9, 1],
					[10, '09:00', 10, 1],
					[11, '10:00', 11, 1],
					[12, '11:00', 12, 1],
					[13, '12:00', 13, 1],
					[14, '13:00', 14, 1],
					[15, '14:00', 15, 1],
					[16, '15:00', 16, 1],
					[17, '16:00', 17, 1],
					[18, '17:00', 18, 1],
					[19, '18:00', 19, 1],
					[20, '19:00', 20, 1],
					[21, '20:00', 21, 1],
					[22, '21:00', 22, 1],
					[23, '22:00', 23, 1],
					[24, '23:00', 24, 1],
				]
			],
			'vtiger_end_hour_seq' => [
				'columns' => ['id'],
				'values' => [
					[1],
				]
			],
			'vtiger_entityname' => [
				'columns' => ['tabid', 'modulename', 'tablename', 'fieldname', 'entityidfield', 'entityidcolumn', 'searchcolumn', 'turn_off', 'sequence'],
				'values' => [
					[4, 'Contacts', 'vtiger_contactdetails', 'firstname,lastname', 'contactid', 'contact_id', 'firstname,lastname', 1, 0],
					[6, 'Accounts', 'vtiger_account', 'accountname', 'accountid', 'account_id', 'accountname', 1, 0],
					[7, 'Leads', 'vtiger_leaddetails', 'company', 'leadid', 'leadid', 'company', 1, 0],
					[8, 'Documents', 'vtiger_notes', 'title', 'notesid', 'notesid', 'title', 1, 0],
					[9, 'Calendar', 'vtiger_activity', 'subject', 'activityid', 'activityid', 'subject', 1, 0],
					[13, 'HelpDesk', 'vtiger_troubletickets', 'title', 'ticketid', 'ticketid', 'title', 1, 0],
					[14, 'Products', 'vtiger_products', 'productname', 'productid', 'product_id', 'productname,ean', 1, 0],
					[15, 'Faq', 'vtiger_faq', 'question', 'id', 'id', 'question', 1, 0],
					[16, 'Events', 'vtiger_activity', 'subject', 'activityid', 'activityid', 'subject', 1, 0],
					[18, 'Vendors', 'vtiger_vendor', 'vendorname', 'vendorid', 'vendor_id', 'vendorname', 1, 0],
					[19, 'PriceBooks', 'vtiger_pricebook', 'bookname', 'pricebookid', 'pricebookid', 'bookname', 1, 0],
					[26, 'Campaigns', 'vtiger_campaign', 'campaignname', 'campaignid', 'campaignid', 'campaignname', 1, 0],
					[29, 'Users', 'vtiger_users', 'last_name,first_name', 'id', 'id', 'last_name,first_name', 1, 0],
					[34, 'ServiceContracts', 'vtiger_servicecontracts', 'subject', 'servicecontractsid', 'servicecontractsid', 'subject', 1, 0],
					[35, 'Services', 'vtiger_service', 'servicename', 'serviceid', 'serviceid', 'servicename', 1, 0],
					[37, 'Assets', 'vtiger_assets', 'assetname', 'assetsid', 'assetsid', 'assetname', 1, 0],
					[40, 'ModComments', 'vtiger_modcomments', 'commentcontent', 'modcommentsid', 'modcommentsid', 'commentcontent', 1, 0],
					[41, 'ProjectMilestone', 'vtiger_projectmilestone', 'projectmilestonename', 'projectmilestoneid', 'projectmilestoneid', 'projectmilestonename', 1, 0],
					[42, 'ProjectTask', 'vtiger_projecttask', 'projecttaskname', 'projecttaskid', 'projecttaskid', 'projecttaskname', 1, 0],
					[43, 'Project', 'vtiger_project', 'projectname', 'projectid', 'projectid', 'projectname', 1, 0],
					[45, 'SMSNotifier', 'vtiger_smsnotifier', 'message', 'smsnotifierid', 'smsnotifierid', 'message', 1, 0],
					[51, 'OSSTimeControl', 'vtiger_osstimecontrol', 'name', 'osstimecontrolid', 'osstimecontrolid', 'name', 1, 0],
					[54, 'OSSMailView', 'vtiger_ossmailview', 'subject', 'ossmailviewid', 'ossmailviewid', 'subject', 1, 0],
					[57, 'OSSOutsourcedServices', 'vtiger_ossoutsourcedservices', 'productname', 'ossoutsourcedservicesid', 'ossoutsourcedservicesid', 'productname', 1, 0],
					[58, 'OSSSoldServices', 'vtiger_osssoldservices', 'productname', 'osssoldservicesid', 'osssoldservicesid', 'productname', 1, 0],
					[59, 'OutsourcedProducts', 'vtiger_outsourcedproducts', 'productname', 'outsourcedproductsid', 'outsourcedproductsid', 'productname', 1, 0],
					[60, 'OSSPasswords', 'vtiger_osspasswords', 'passwordname', 'osspasswordsid', 'osspasswordsid', 'passwordname', 1, 0],
					[61, 'OSSEmployees', 'vtiger_ossemployees', 'last_name', 'ossemployeesid', 'ossemployeesid', 'last_name', 1, 0],
					[74, 'CallHistory', 'vtiger_callhistory', 'to_number', 'callhistoryid', 'callhistoryid', 'to_number', 1, 0],
					[75, 'Ideas', 'vtiger_ideas', 'subject', 'ideasid', 'ideasid', 'subject', 1, 0],
					[78, 'HolidaysEntitlement', 'vtiger_holidaysentitlement', 'ossemployeesid,days', 'holidaysentitlementid', 'holidaysentitlementid', 'ossemployeesid', 1, 0],
					[79, 'PaymentsIn', 'vtiger_paymentsin', 'paymentsname', 'paymentsinid', 'paymentsinid', 'paymentsno,paymentsname', 1, 0],
					[80, 'PaymentsOut', 'vtiger_paymentsout', 'paymentsname', 'paymentsoutid', 'paymentsoutid', 'paymentsno,paymentsname', 1, 0],
					[81, 'LettersIn', 'vtiger_lettersin', 'title', 'lettersinid', 'lettersinid', 'title', 1, 0],
					[82, 'LettersOut', 'vtiger_lettersout', 'title', 'lettersoutid', 'lettersoutid', 'title', 1, 0],
					[83, 'Announcements', 'u_yf_announcement', 'subject', 'announcementid', 'announcementid', 'subject', 1, 0],
					[84, 'Reservations', 'vtiger_reservations', 'title', 'reservationsid', 'reservationsid', 'title', 1, 0],
					[85, 'SQuoteEnquiries', 'u_yf_squoteenquiries', 'subject', 'squoteenquiriesid', 'squoteenquiriesid', 'subject', 1, 0],
					[86, 'SSalesProcesses', 'u_yf_ssalesprocesses', 'subject', 'ssalesprocessesid', 'ssalesprocessesid', 'subject', 1, 0],
					[87, 'SRequirementsCards', 'u_yf_srequirementscards', 'subject', 'srequirementscardsid', 'srequirementscardsid', 'subject', 1, 0],
					[88, 'SCalculations', 'u_yf_scalculations', 'subject', 'scalculationsid', 'scalculationsid', 'subject', 1, 0],
					[89, 'SQuotes', 'u_yf_squotes', 'subject', 'squotesid', 'squotesid', 'subject', 1, 0],
					[90, 'SSingleOrders', 'u_yf_ssingleorders', 'subject', 'ssingleordersid', 'ssingleordersid', 'subject', 1, 0],
					[91, 'SRecurringOrders', 'u_yf_srecurringorders', 'subject', 'srecurringordersid', 'srecurringordersid', 'subject', 1, 0],
					[92, 'Partners', 'u_yf_partners', 'subject', 'partnersid', 'partnersid', 'subject', 1, 0],
					[93, 'Competition', 'u_yf_competition', 'subject', 'competitionid', 'competitionid', 'subject', 1, 0],
					[94, 'FBookkeeping', 'u_yf_fbookkeeping', 'subject', 'fbookkeepingid', 'fbookkeepingid', 'subject', 1, 0],
					[95, 'FInvoice', 'u_yf_finvoice', 'subject', 'finvoiceid', 'finvoiceid', 'subject', 1, 0],
					[96, 'KnowledgeBase', 'u_yf_knowledgebase', 'subject', 'knowledgebaseid', 'knowledgebaseid', 'subject', 1, 0],
					[97, 'IStorages', 'u_yf_istorages', 'subject', 'istorageid', 'istorageid', 'subject', 1, 0],
					[98, 'IGRN', 'u_yf_igrn', 'subject', 'igrnid', 'igrnid', 'subject', 1, 0],
					[99, 'FInvoiceProforma', 'u_yf_finvoiceproforma', 'subject', 'finvoiceproformaid', 'finvoiceproformaid', 'subject', 1, 0],
					[100, 'IGDN', 'u_yf_igdn', 'subject', 'igdnid', 'igdnid', 'subject', 1, 0],
					[101, 'IIDN', 'u_yf_iidn', 'subject', 'iidnid', 'iidnid', 'subject', 1, 0],
					[102, 'IGIN', 'u_yf_igin', 'subject', 'iginid', 'iginid', 'subject', 1, 0],
					[103, 'IPreOrder', 'u_yf_ipreorder', 'subject', 'ipreorderid', 'ipreorderid', 'subject', 1, 0],
					[104, 'ISTDN', 'u_yf_istdn', 'subject', 'istdnid', 'istdnid', 'subject', 1, 0],
					[105, 'ISTN', 'u_yf_istn', 'subject', 'istnid', 'istnid', 'subject', 1, 0],
					[106, 'ISTRN', 'u_yf_istrn', 'subject', 'istrnid', 'istrnid', 'subject', 1, 0],
					[107, 'FCorectingInvoice', 'u_yf_fcorectinginvoice', 'subject', 'fcorectinginvoiceid', 'fcorectinginvoiceid', 'subject', 1, 0],
					[108, 'IGRNC', 'u_yf_igrnc', 'subject', 'igrncid', 'igrncid', 'subject', 1, 0],
					[109, 'IGDNC', 'u_yf_igdnc', 'subject', 'igdncid', 'igdncid', 'subject', 1, 0],
					[111, 'Notification', 'u_yf_notification', 'title', 'notificationid', 'notificationid', 'title', 1, 0],
					[112, 'EmailTemplates', 'u_yf_emailtemplates', 'name', 'emailtemplatesid', 'emailtemplatesid', 'name', 1, 0],
					[113, 'CFixedAssets', 'u_yf_cfixedassets', 'subject', 'cfixedassetsid', 'cfixedassetsid', 'subject', 1, 0],
					[114, 'CInternalTickets', 'u_yf_cinternaltickets', 'subject', 'cinternalticketsid', 'cinternalticketsid', 'subject', 1, 0],
					[115, 'FInvoiceCost', 'u_yf_finvoicecost', 'subject', 'finvoicecostid', 'finvoicecostid', 'subject', 1, 0],
					[116, 'CMileageLogbook', 'u_yf_cmileagelogbook', 'subject', 'cmileagelogbookid', 'cmileagelogbookid', 'subject', 1, 0],
					[117, 'SVendorEnquiries', 'u_yf_svendorenquiries', 'subject', 'svendorenquiriesid', 'svendorenquiriesid', 'subject', 1, 0],
				]
			],
			'vtiger_eventhandlers' => [
				'columns' => ['eventhandler_id', 'event_name', 'handler_class', 'is_active', 'include_modules', 'exclude_modules', 'priority', 'owner_id'],
				'values' => [
					[3, 'EntityAfterSave', 'Vtiger_Workflow_Handler', 1, '', '', 5, 0],
					[4, 'EntityAfterRestore', 'Vtiger_Workflow_Handler', 1, '', '', 5, 0],
					[8, 'EntityChangeState', 'ModTracker_ModTrackerHandler_Handler', 1, '', '', 5, 32],
					[16, 'EntityAfterSave', 'ServiceContracts_ServiceContractsHandler_Handler', 1, 'HelpDesk,ServiceContracts', '', 5, 34],
					[18, 'EntityAfterSave', 'Vtiger_RecordLabelUpdater_Handler', 1, '', '', 5, 0],
					[19, 'DetailViewBefore', 'ModTracker_ModTrackerHandler_Handler', 1, '', '', 5, 32],
					[22, 'PicklistAfterRename', 'Settings_Picklist_PickListHandler_Handler', 1, '', '', 5, 0],
					[23, 'PicklistAfterDelete', 'Settings_Picklist_PickListHandler_Handler', 1, '', '', 5, 0],
					[24, 'EntityAfterSave', 'OSSPasswords_Secure_Handler', 1, 'OSSPasswords', '', 3, 60],
					[25, 'EntityAfterSave', 'OSSTimeControl_TimeControl_Handler', 1, 'OSSTimeControl', '', 3, 51],
					[29, 'EntityAfterSave', 'Vtiger_SharingPrivileges_Handler', 1, '', '', 3, 0],
					[31, 'EntityAfterSave', 'HelpDesk_TicketRangeTime_Handler', 1, 'HelpDesk', '', 3, 13],
					[32, 'EntityAfterUnLink', 'OSSTimeControl_TimeControl_Handler', 1, 'OSSTimeControl', '', 5, 51],
					[33, 'EntityAfterDelete', 'OSSTimeControl_TimeControl_Handler', 1, 'OSSTimeControl', '', 3, 51],
					[34, 'EntityAfterSave', 'API_CardDAV_Handler', 1, 'Contacts,OSSEmployees', '', 3, 0],
					[35, 'EntityAfterSave', 'API_CalDAV_Handler', 1, 'Events,Calendar', '', 3, 0],
					[36, 'EntityAfterLink', 'HelpDesk_TicketRangeTime_Handler', 1, 'HelpDesk', '', 5, 13],
					[38, 'EntityAfterSave', 'ProjectTask_ProjectTaskHandler_Handler', 1, 'ProjectTask', '', 3, 42],
					[39, 'EntityAfterDelete', 'ProjectTask_ProjectTaskHandler_Handler', 1, 'ProjectTask', '', 5, 42],
					[40, 'EntityAfterRestore', 'ProjectTask_ProjectTaskHandler_Handler', 1, 'ProjectTask', '', 5, 42],
					[41, 'EntityAfterLink', 'ModTracker_ModTrackerHandler_Handler', 1, '', '', 5, 32],
					[42, 'EntityAfterUnLink', 'ModTracker_ModTrackerHandler_Handler', 1, '', '', 5, 32],
					[44, 'EntityBeforeSave', 'Calendar_CalendarHandler_Handler', 1, 'Calendar,Events,Activity', '', 5, 9],
					[45, 'EntityAfterLink', 'Vtiger_MultiReferenceUpdater_Handler', 1, '', '', 5, 0],
					[46, 'EntityAfterUnLink', 'Vtiger_MultiReferenceUpdater_Handler', 1, '', '', 5, 0],
					[48, 'EntityAfterRestore', 'OSSTimeControl_TimeControl_Handler', 1, 'OSSTimeControl', '', 5, 51],
					[49, 'EntityAfterUnLink', 'Calendar_CalendarHandler_Handler', 1, 'Calendar,Events,Activity', '', 5, 9],
					[50, 'EntityAfterSave', 'Calendar_CalendarHandler_Handler', 1, 'Calendar,Events,Activity', '', 3, 9],
					[51, 'EntityAfterRestore', 'Calendar_CalendarHandler_Handler', 1, 'Calendar,Events,Activity', '', 5, 9],
					[52, 'EntityAfterSave', 'IStorages_RecalculateStockHandler_Handler', 1, 'IGRN,IIDN,IGDN,IGIN,IPreOrder,ISTDN,ISTRN,IGRNC,IGDNC', '', 3, 97],
					[53, 'EntityAfterSave', 'Accounts_SaveChanges_Handler', 1, 'Accounts', '', 5, 6],
					[54, 'EntityAfterSave', 'OpenStreetMap_OpenStreetMapHandler_Handler', 1, 'Accounts,Leads,Partners,Vendors,Competition,Contacts', '', 3, 110],
					[55, 'EntitySystemAfterCreate', 'Vtiger_AutomaticAssignment_Handler', 0, '', '', 5, 0],
					[56, 'EntityAfterSave', 'ModTracker_ModTrackerHandler_Handler', 1, '', '', 5, 32],
					[57, 'EntityAfterSave', 'Vtiger_MultiReferenceUpdater_Handler', 1, '', '', 5, 0],
					[58, 'EntityAfterSave', 'Vtiger_Attachments_Handler', 1, '', '', 5, 0],
				]
			],
			'vtiger_eventstatus' => [
				'columns' => ['eventstatusid', 'eventstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Held', 0, 39, 1],
					[2, 'Not Held', 0, 40, 2],
				]
			],
			'vtiger_eventstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_expectedresponse' => [
				'columns' => ['expectedresponseid', 'expectedresponse', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Excellent', 1, 42, 1],
					[3, 'Good', 1, 43, 2],
					[4, 'Average', 1, 44, 3],
					[5, 'Poor', 1, 45, 4],
				]
			],
			'vtiger_expectedresponse_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_faqcategories' => [
				'columns' => ['faqcategories_id', 'faqcategories', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'General', 1, 46, 0],
				]
			],
			'vtiger_faqcategories_seq' => [
				'columns' => ['id'],
				'values' => [
					[1],
				]
			],
			'vtiger_faqstatus' => [
				'columns' => ['faqstatus_id', 'faqstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Draft', 0, 47, 0],
					[2, 'Reviewed', 0, 48, 1],
					[3, 'Published', 0, 49, 2],
					[4, 'Obsolete', 0, 50, 3],
				]
			],
			'vtiger_faqstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_fcorectinginvoice_formpayment' => [
				'columns' => ['fcorectinginvoice_formpaymentid', 'fcorectinginvoice_formpayment', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_TRANSFER', 1, 691, 1],
					[2, 'PLL_CASH', 1, 693, 2],
				]
			],
			'vtiger_fcorectinginvoice_status' => [
				'columns' => ['fcorectinginvoice_statusid', 'fcorectinginvoice_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'None', 1, 692, 1],
				]
			],
			'vtiger_field' => [
				'columns' => ['tabid', 'fieldid', 'columnname', 'tablename', 'generatedtype', 'uitype', 'fieldname', 'fieldlabel', 'readonly', 'presence', 'defaultvalue', 'maximumlength', 'sequence', 'block', 'displaytype', 'typeofdata', 'quickcreate', 'quickcreatesequence', 'info_type', 'masseditable', 'helpinfo', 'summaryfield', 'fieldparams', 'header_field', 'maxlengthtext', 'maxwidthcolumn'],
				'values' => [
					[6, 1, 'accountname', 'vtiger_account', 1, 2, 'accountname', 'Account Name', 1, 0, '', 100, 1, 9, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[6, 2, 'account_no', 'vtiger_account', 1, 4, 'account_no', 'Account No', 1, 0, '', 100, 2, 9, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 3, 'phone', 'vtiger_account', 1, 11, 'phone', 'Phone', 1, 2, '', 100, 2, 195, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[6, 4, 'website', 'vtiger_account', 1, 17, 'website', 'Website', 1, 2, '', 100, 5, 195, 1, 'V~O', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[6, 5, 'fax', 'vtiger_account', 1, 11, 'fax', 'Fax', 1, 2, '', 100, 6, 195, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 7, 'otherphone', 'vtiger_account', 1, 11, 'otherphone', 'Other Phone', 1, 2, '', 100, 4, 195, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 8, 'parentid', 'vtiger_account', 1, 51, 'account_id', 'Member Of', 1, 2, '', 100, 3, 9, 1, 'I~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 9, 'email1', 'vtiger_account', 1, 13, 'email1', 'Email', 1, 2, '', 100, 1, 195, 1, 'E~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[6, 10, 'employees', 'vtiger_account', 1, 7, 'employees', 'Employees', 1, 2, '', 100, 4, 196, 1, 'I~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 11, 'email2', 'vtiger_account', 1, 13, 'email2', 'Other Email', 1, 2, '', 100, 3, 195, 1, 'E~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 12, 'ownership', 'vtiger_account', 1, 1, 'ownership', 'Ownership', 1, 2, '', 100, 1, 194, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 14, 'industry', 'vtiger_account', 1, 15, 'industry', 'industry', 1, 2, '', 100, 4, 9, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 15, 'siccode', 'vtiger_account', 1, 1, 'siccode', 'SIC Code', 1, 2, '', 100, 5, 194, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 16, 'account_type', 'vtiger_account', 1, 15, 'accounttype', 'Type', 1, 2, '', 100, 5, 9, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 17, 'annualrevenue', 'vtiger_account', 1, 71, 'annual_revenue', 'Annual Revenue', 1, 2, '', 100, 3, 194, 1, 'N~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 18, 'emailoptout', 'vtiger_account', 1, 56, 'emailoptout', 'Approval for email', 1, 2, '', 100, 7, 195, 1, 'C~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[6, 20, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 6, 9, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[6, 21, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 7, 196, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 22, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 8, 196, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 23, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 23, 9, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 36, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 12, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 39, 'lead_no', 'vtiger_leaddetails', 1, 4, 'lead_no', 'Lead No', 1, 0, '', 100, 6, 14, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 40, 'phone', 'vtiger_leadaddress', 1, 11, 'phone', 'Phone', 1, 2, '', 100, 1, 150, 1, 'V~O', 2, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 41, 'lastname', 'vtiger_leaddetails', 1, 2, 'lastname', 'Short name', 1, 1, '', 100, 2, 13, 1, 'V~O', 0, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 42, 'mobile', 'vtiger_leadaddress', 1, 11, 'mobile', 'Mobile', 1, 2, '', 100, 3, 150, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 43, 'company', 'vtiger_leaddetails', 1, 2, 'company', 'Company', 1, 2, '', 100, 1, 13, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 44, 'fax', 'vtiger_leadaddress', 1, 11, 'fax', 'Fax', 1, 2, '', 100, 5, 150, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 46, 'email', 'vtiger_leaddetails', 1, 13, 'email', 'Email', 1, 2, '', 100, 2, 150, 1, 'E~O', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 47, 'leadsource', 'vtiger_leaddetails', 1, 15, 'leadsource', 'Lead Source', 1, 2, '', 100, 2, 14, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 48, 'website', 'vtiger_leadsubdetails', 1, 17, 'website', 'Website', 1, 2, '', 100, 6, 150, 1, 'V~O', 1, null, 'ADV', 1, '', 1, '', null, 0, 0],
					[7, 49, 'industry', 'vtiger_leaddetails', 1, 15, 'industry', 'Industry', 1, 2, '', 100, 6, 13, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[7, 50, 'leadstatus', 'vtiger_leaddetails', 1, 15, 'leadstatus', 'Lead Status', 1, 2, '', 100, 2, 13, 1, 'V~M', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 51, 'annualrevenue', 'vtiger_leaddetails', 1, 71, 'annualrevenue', 'Annual Revenue', 1, 2, '', 100, 4, 191, 1, 'N~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[7, 53, 'noofemployees', 'vtiger_leaddetails', 1, 1, 'noofemployees', 'No Of Employees', 1, 2, '', 100, 8, 13, 1, 'I~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[7, 54, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 3, 13, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[7, 55, 'secondaryemail', 'vtiger_leaddetails', 1, 13, 'secondaryemail', 'Secondary Email', 1, 2, '', 100, 4, 150, 1, 'E~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[7, 56, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 8, 14, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 57, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 5, 14, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 58, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 23, 13, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 65, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 16, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 66, 'salutation', 'vtiger_contactdetails', 1, 55, 'salutationtype', 'Salutation', 1, 0, '', 100, 1, 4, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 67, 'firstname', 'vtiger_contactdetails', 1, 55, 'firstname', 'First Name', 1, 0, '', 100, 1, 4, 1, 'V~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 68, 'contact_no', 'vtiger_contactdetails', 1, 4, 'contact_no', 'Contact Id', 1, 0, '', 100, 2, 4, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[4, 69, 'phone', 'vtiger_contactdetails', 1, 11, 'phone', 'Office Phone', 1, 2, '', 100, 1, 197, 1, 'V~O', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 70, 'lastname', 'vtiger_contactdetails', 1, 255, 'lastname', 'Last Name', 1, 0, '', 100, 3, 4, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 71, 'mobile', 'vtiger_contactdetails', 1, 11, 'mobile', 'Mobile', 1, 2, '', 100, 3, 197, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 72, 'parentid', 'vtiger_contactdetails', 1, 10, 'parent_id', 'Member Of', 1, 2, '', 100, 4, 4, 1, 'I~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 74, 'leadsource', 'vtiger_contactsubdetails', 1, 15, 'leadsource', 'Lead Source', 1, 2, '', 100, 5, 4, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 79, 'birthday', 'vtiger_contactsubdetails', 1, 5, 'birthday', 'Birthdate', 1, 2, '', 100, 1, 5, 1, 'D~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[4, 80, 'email', 'vtiger_contactdetails', 1, 13, 'email', 'Email', 1, 2, '', 100, 2, 197, 1, 'E~O', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 81, 'reportsto', 'vtiger_contactdetails', 1, 57, 'contact_id', 'Reports To', 1, 2, '', 100, 2, 5, 1, 'V~O', 1, null, 'ADV', 0, '', 0, '', null, 0, 0],
					[4, 85, 'donotcall', 'vtiger_contactdetails', 1, 56, 'donotcall', 'Approval for phone calls', 1, 2, '', 100, 5, 197, 1, 'C~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[4, 86, 'emailoptout', 'vtiger_contactdetails', 1, 56, 'emailoptout', 'Approval for email', 1, 2, '', 100, 6, 197, 1, 'C~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[4, 87, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 4, 1, 'V~M', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 90, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 7, 5, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[4, 91, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 5, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[4, 92, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 28, 4, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[4, 93, 'portal', 'vtiger_customerdetails', 1, 56, 'portal', 'Portal User', 1, 2, '', 100, 1, 6, 1, 'C~O', 1, null, 'ADV', 0, '', 0, '', null, 0, 0],
					[4, 94, 'support_start_date', 'vtiger_customerdetails', 1, 5, 'support_start_date', 'Support Start Date', 1, 2, '', 100, 2, 6, 1, 'D~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[4, 95, 'support_end_date', 'vtiger_customerdetails', 1, 5, 'support_end_date', 'Support End Date', 1, 2, '', 100, 3, 6, 1, 'D~O~OTH~GE~support_start_date~Support Start Date', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[4, 108, 'imagename', 'vtiger_contactdetails', 1, 69, 'imagename', 'Contact Image', 1, 2, '', 100, 1, 73, 1, 'V~O', 3, null, 'ADV', 0, '', 0, '', null, 0, 0],
					[4, 109, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 8, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 126, 'campaignname', 'vtiger_campaign', 1, 2, 'campaignname', 'Campaign Name', 1, 0, '', 100, 1, 74, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 127, 'campaign_no', 'vtiger_campaign', 1, 4, 'campaign_no', 'Campaign No', 1, 0, '', 100, 2, 74, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 128, 'campaigntype', 'vtiger_campaign', 1, 15, 'campaigntype', 'Campaign Type', 1, 2, '', 100, 5, 74, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 129, 'product_id', 'vtiger_campaign', 1, 59, 'product_id', 'Product', 1, 2, '', 100, 8, 74, 1, 'I~O', 2, 5, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 130, 'campaignstatus', 'vtiger_campaign', 1, 15, 'campaignstatus', 'FL_CAMPAIGN_STATUS', 1, 2, '', 100, 6, 74, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 131, 'closingdate', 'vtiger_campaign', 1, 23, 'closingdate', 'Expected Close Date', 1, 2, '', 100, 10, 74, 1, 'D~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 132, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 3, 74, 1, 'V~M', 0, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 133, 'numsent', 'vtiger_campaign', 1, 9, 'numsent', 'Num Sent', 1, 2, '', 100, 14, 74, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 134, 'sponsor', 'vtiger_campaign', 1, 1, 'sponsor', 'Sponsor', 1, 2, '', 100, 9, 74, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 135, 'targetaudience', 'vtiger_campaign', 1, 1, 'targetaudience', 'Target Audience', 1, 2, '', 100, 7, 74, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 136, 'targetsize', 'vtiger_campaign', 1, 1, 'targetsize', 'TargetSize', 1, 2, '', 100, 12, 74, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 137, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 11, 74, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 138, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 13, 74, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 139, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 16, 74, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 140, 'expectedresponse', 'vtiger_campaign', 1, 15, 'expectedresponse', 'Expected Response', 1, 2, '', 100, 3, 76, 1, 'V~O', 2, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 141, 'expectedrevenue', 'vtiger_campaign', 1, 71, 'expectedrevenue', 'Expected Revenue', 1, 2, '', 100, 4, 76, 1, 'N~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[26, 142, 'budgetcost', 'vtiger_campaign', 1, 71, 'budgetcost', 'Budget Cost', 1, 2, '', 100, 1, 76, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 143, 'actualcost', 'vtiger_campaign', 1, 71, 'actualcost', 'Actual Cost', 1, 2, '', 100, 2, 76, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 144, 'expectedresponsecount', 'vtiger_campaign', 1, 1, 'expectedresponsecount', 'Expected Response Count', 1, 2, '', 100, 7, 76, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 145, 'expectedsalescount', 'vtiger_campaign', 1, 1, 'expectedsalescount', 'Expected Sales Count', 1, 2, '', 100, 5, 76, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 146, 'expectedroi', 'vtiger_campaign', 1, 71, 'expectedroi', 'Expected ROI', 1, 2, '', 100, 9, 76, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 147, 'actualresponsecount', 'vtiger_campaign', 1, 1, 'actualresponsecount', 'Actual Response Count', 1, 2, '', 100, 8, 76, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 148, 'actualsalescount', 'vtiger_campaign', 1, 1, 'actualsalescount', 'Actual Sales Count', 1, 2, '', 100, 6, 76, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 149, 'actualroi', 'vtiger_campaign', 1, 71, 'actualroi', 'Actual ROI', 1, 2, '', 100, 10, 76, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 150, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 81, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 155, 'ticket_no', 'vtiger_troubletickets', 1, 4, 'ticket_no', 'Ticket No', 1, 0, '', 100, 1, 27, 1, 'V~O', 3, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[13, 156, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 5, 25, 1, 'V~M', 0, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 157, 'parent_id', 'vtiger_troubletickets', 1, 10, 'parent_id', 'Related To', 1, 2, '', 100, 8, 25, 1, 'I~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 158, 'priority', 'vtiger_troubletickets', 1, 15, 'ticketpriorities', 'Priority', 1, 2, 'Low', 100, 7, 25, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 159, 'product_id', 'vtiger_troubletickets', 1, 10, 'product_id', 'Product Name', 1, 2, '', 100, 4, 25, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 160, 'severity', 'vtiger_troubletickets', 1, 15, 'ticketseverities', 'Severity', 1, 2, '', 100, 9, 25, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 161, 'status', 'vtiger_troubletickets', 1, 15, 'ticketstatus', 'Status', 1, 2, 'Open', 100, 3, 25, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 162, 'category', 'vtiger_troubletickets', 1, 302, 'ticketcategories', 'Category', 1, 2, '', 100, 10, 25, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '17', null, 0, 0],
					[13, 163, 'update_log', 'vtiger_troubletickets', 1, 19, 'update_log', 'Update History', 1, 0, '', 100, 12, 25, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 166, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 2, 27, 2, 'DT~O', 3, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[13, 167, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 4, 27, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 168, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 17, 25, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 169, 'title', 'vtiger_troubletickets', 1, 22, 'ticket_title', 'Title', 1, 0, '', 100, 1, 25, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 170, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 28, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[13, 171, 'solution', 'vtiger_troubletickets', 1, 300, 'solution', 'Solution', 1, 0, '', 100, 1, 29, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 173, 'productname', 'vtiger_products', 1, 2, 'productname', 'Product Name', 1, 0, '', 100, 1, 31, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[14, 174, 'product_no', 'vtiger_products', 1, 4, 'product_no', 'Product No', 1, 0, '', 100, 2, 31, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 175, 'productcode', 'vtiger_products', 1, 1, 'productcode', 'Part Number', 1, 2, '', 100, 4, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 176, 'discontinued', 'vtiger_products', 1, 56, 'discontinued', 'Product Active', 1, 2, '', 100, 3, 31, 1, 'V~O', 2, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 177, 'manufacturer', 'vtiger_products', 1, 15, 'manufacturer', 'Manufacturer', 1, 2, '', 100, 6, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 178, 'pscategory', 'vtiger_products', 1, 302, 'pscategory', 'Product Category', 1, 2, '', 100, 6, 31, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '2', null, 0, 0],
					[14, 179, 'sales_start_date', 'vtiger_products', 1, 5, 'sales_start_date', 'Sales Start Date', 1, 2, '', 100, 5, 31, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 180, 'sales_end_date', 'vtiger_products', 1, 5, 'sales_end_date', 'Sales End Date', 1, 2, null, 100, 8, 31, 1, 'D~O~OTH~GE~sales_start_date~Sales Start Date', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 181, 'start_date', 'vtiger_products', 1, 5, 'start_date', 'Support Start Date', 1, 2, '', 100, 7, 31, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 182, 'expiry_date', 'vtiger_products', 1, 5, 'expiry_date', 'Support Expiry Date', 1, 2, '', 100, 10, 31, 1, 'D~O~OTH~GE~start_date~Start Date', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 183, 'website', 'vtiger_products', 1, 17, 'website', 'Website', 1, 2, '', 100, 14, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 184, 'vendor_id', 'vtiger_products', 1, 75, 'vendor_id', 'Vendor Name', 1, 2, '', 100, 13, 31, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 185, 'mfr_part_no', 'vtiger_products', 1, 1, 'mfr_part_no', 'Mfr PartNo', 1, 2, '', 100, 16, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 186, 'vendor_part_no', 'vtiger_products', 1, 1, 'vendor_part_no', 'Vendor PartNo', 1, 2, '', 100, 15, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 187, 'serialno', 'vtiger_products', 1, 1, 'serial_no', 'Serial No', 1, 2, '', 100, 18, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 188, 'productsheet', 'vtiger_products', 1, 1, 'productsheet', 'Product Sheet', 1, 2, '', 100, 17, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 189, 'glacct', 'vtiger_products', 1, 15, 'glacct', 'GL Account', 1, 2, '', 100, 20, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 190, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 19, 31, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 191, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 21, 31, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 192, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 22, 31, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 193, 'unit_price', 'vtiger_products', 1, 72, 'unit_price', 'Unit Price', 1, 0, '', 100, 1, 32, 1, 'N~O', 2, 3, 'BAS', 0, '', 1, '', null, 0, 0],
					[14, 194, 'commissionrate', 'vtiger_products', 1, 9, 'commissionrate', 'Commission Rate', 1, 2, '', 100, 2, 32, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 196, 'usageunit', 'vtiger_products', 1, 15, 'usageunit', 'Usage Unit', 1, 2, '', 100, 1, 33, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[14, 197, 'qty_per_unit', 'vtiger_products', 1, 1, 'qty_per_unit', 'Qty/Unit', 1, 2, '', 100, 2, 33, 1, 'N~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[14, 198, 'qtyinstock', 'vtiger_products', 1, 1, 'qtyinstock', 'Qty In Stock', 1, 2, '', 100, 3, 33, 10, 'NN~O', 0, 7, 'ADV', 1, '', 1, '', null, 0, 0],
					[14, 199, 'reorderlevel', 'vtiger_products', 1, 1, 'reorderlevel', 'Reorder Level', 1, 2, '', 100, 4, 33, 1, 'I~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[14, 200, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Handler', 1, 2, '', 100, 5, 33, 1, 'V~M', 2, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 201, 'qtyindemand', 'vtiger_products', 1, 1, 'qtyindemand', 'Qty In Demand', 1, 2, '', 100, 6, 33, 1, 'I~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[14, 202, 'imagename', 'vtiger_products', 1, 69, 'imagename', 'Product Image', 1, 2, '', 100, 1, 35, 1, 'V~O', 3, null, 'ADV', 0, '', 0, '', null, 0, 0],
					[14, 203, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 36, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 204, 'title', 'vtiger_notes', 1, 2, 'notes_title', 'Title', 1, 0, '', 100, 1, 17, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[8, 205, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 17, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 206, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 8, 17, 2, 'DT~O', 3, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[8, 207, 'filename', 'vtiger_notes', 1, 28, 'filename', 'File Name', 1, 2, '', 100, 3, 18, 1, 'V~O', 0, 2, 'BAS', 0, '', 1, '', null, 0, 0],
					[8, 208, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 4, 17, 1, 'V~M', 0, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[8, 209, 'notecontent', 'vtiger_notes', 1, 300, 'notecontent', 'Note', 1, 2, '', 100, 1, 84, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 210, 'filetype', 'vtiger_notes', 1, 1, 'filetype', 'File Type', 1, 2, '', 100, 5, 18, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 211, 'filesize', 'vtiger_notes', 1, 1, 'filesize', 'File Size', 1, 2, '', 100, 4, 18, 2, 'I~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 212, 'filelocationtype', 'vtiger_notes', 1, 27, 'filelocationtype', 'Download Type', 1, 0, '', 100, 1, 18, 1, 'V~O', 0, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 213, 'fileversion', 'vtiger_notes', 1, 1, 'fileversion', 'Version', 1, 2, '', 100, 6, 18, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 214, 'filestatus', 'vtiger_notes', 1, 56, 'filestatus', 'Active', 1, 2, '1', 100, 2, 18, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 215, 'filedownloadcount', 'vtiger_notes', 1, 1, 'filedownloadcount', 'Download Count', 1, 2, '', 100, 7, 18, 2, 'I~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 216, 'folderid', 'vtiger_notes', 1, 302, 'folderid', 'Folder Name', 1, 2, 'T1', 100, 2, 17, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '1', null, 0, 0],
					[8, 217, 'note_no', 'vtiger_notes', 1, 4, 'note_no', 'Document No', 1, 0, '', 100, 3, 17, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 218, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 12, 17, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[9, 231, 'subject', 'vtiger_activity', 1, 2, 'subject', 'Subject', 1, 0, '', 100, 1, 19, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 232, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 2, 19, 1, 'V~M', 0, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 233, 'date_start', 'vtiger_activity', 1, 6, 'date_start', 'Start Date & Time', 1, 0, '', 100, 3, 19, 1, 'DT~M~time_start', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 234, 'time_start', 'vtiger_activity', 1, 2, 'time_start', 'Time Start', 1, 0, '', 100, 4, 19, 3, 'T~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 235, 'time_end', 'vtiger_activity', 1, 2, 'time_end', 'End Time', 1, 0, '', 100, 4, 19, 3, 'T~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 236, 'due_date', 'vtiger_activity', 1, 23, 'due_date', 'Due Date', 1, 0, '', 100, 5, 19, 1, 'D~M~OTH~GE~date_start~Start Date & Time', 1, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 237, 'process', 'vtiger_activity', 1, 66, 'process', 'FL_PROCESS', 1, 0, '', 100, 2, 87, 1, 'I~O', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 238, 'link', 'vtiger_activity', 1, 67, 'link', 'FL_RELATION', 1, 0, '', 100, 1, 87, 1, 'I~O', 2, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 239, 'status', 'vtiger_activity', 1, 15, 'activitystatus', 'Status', 1, 0, 'PLL_PLANNED', 100, 9, 19, 10, 'V~O', 1, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 241, 'priority', 'vtiger_activity', 1, 15, 'taskpriority', 'Priority', 1, 0, 'Low', 100, 10, 19, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 242, 'sendnotification', 'vtiger_activity', 1, 56, 'sendnotification', 'Send Notification', 1, 0, '', 100, 11, 19, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 243, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 14, 19, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[9, 244, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 13, 19, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[9, 245, 'activitytype', 'vtiger_activity', 1, 15, 'activitytype', 'Activity Type', 1, 0, '', 100, 16, 19, 3, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[9, 246, 'visibility', 'vtiger_activity', 1, 16, 'visibility', 'Visibility', 1, 0, '', 100, 17, 19, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 247, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 0, '', 100, 1, 20, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 248, 'duration_hours', 'vtiger_activity', 1, 63, 'duration_hours', 'Duration', 1, 0, '', 100, 17, 19, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 249, 'duration_minutes', 'vtiger_activity', 1, 16, 'duration_minutes', 'Duration Minutes', 1, 0, '', 100, 18, 19, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 250, 'location', 'vtiger_activity', 1, 1, 'location', 'Location', 1, 0, '', 100, 19, 19, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 251, 'reminder_time', 'vtiger_activity_reminder', 1, 30, 'reminder_time', 'Send Reminder', 1, 0, '', 100, 1, 19, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 253, 'notime', 'vtiger_activity', 1, 56, 'notime', 'No Time', 1, 0, '', 100, 20, 19, 3, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 254, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 22, 19, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 255, 'subject', 'vtiger_activity', 1, 2, 'subject', 'Subject', 1, 0, '', 100, 1, 39, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 256, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 2, 39, 1, 'V~M', 0, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 257, 'date_start', 'vtiger_activity', 1, 6, 'date_start', 'Start Date & Time', 1, 0, '', 100, 3, 39, 1, 'DT~M~time_start', 0, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 258, 'time_start', 'vtiger_activity', 1, 2, 'time_start', 'Time Start', 1, 0, '', 100, 4, 39, 3, 'T~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 259, 'due_date', 'vtiger_activity', 1, 23, 'due_date', 'End Date', 1, 0, '', 100, 5, 39, 1, 'D~M~OTH~GE~date_start~Start Date & Time', 0, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 260, 'time_end', 'vtiger_activity', 1, 2, 'time_end', 'End Time', 1, 0, '', 100, 5, 39, 3, 'T~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 262, 'duration_hours', 'vtiger_activity', 1, 63, 'duration_hours', 'Duration', 1, 0, '', 100, 7, 39, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 263, 'duration_minutes', 'vtiger_activity', 1, 16, 'duration_minutes', 'Duration Minutes', 1, 0, '', 100, 8, 39, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 264, 'process', 'vtiger_activity', 1, 66, 'process', 'FL_PROCESS', 1, 0, '', 100, 2, 119, 1, 'I~O', 2, 10, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 265, 'status', 'vtiger_activity', 1, 15, 'activitystatus', 'Status', 1, 0, 'PLL_PLANNED', 100, 7, 39, 10, 'V~O', 0, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 266, 'sendnotification', 'vtiger_activity', 1, 56, 'sendnotification', 'Send Notification', 1, 0, '', 100, 11, 39, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 267, 'activitytype', 'vtiger_activity', 1, 15, 'activitytype', 'Activity Type', 1, 0, '', 100, 9, 39, 1, 'V~M', 0, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 268, 'location', 'vtiger_activity', 1, 1, 'location', 'Location', 1, 0, '', 100, 13, 39, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 269, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 11, 39, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 270, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 15, 39, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 271, 'priority', 'vtiger_activity', 1, 15, 'taskpriority', 'Priority', 1, 0, 'Low', 100, 13, 39, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 272, 'notime', 'vtiger_activity', 1, 56, 'notime', 'No Time', 1, 0, '', 100, 17, 39, 3, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 273, 'visibility', 'vtiger_activity', 1, 16, 'visibility', 'Visibility', 1, 0, '', 100, 18, 39, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 274, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 22, 39, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 275, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 0, '', 100, 1, 41, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 276, 'reminder_time', 'vtiger_activity_reminder', 1, 30, 'reminder_time', 'Send Reminder', 1, 0, '', 100, 1, 40, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 277, 'link', 'vtiger_activity', 1, 67, 'link', 'FL_RELATION', 1, 0, '', 100, 1, 119, 1, 'I~O', 2, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[15, 278, 'product_id', 'vtiger_faq', 1, 10, 'product_id', 'Product Name', 1, 2, '', 100, 1, 37, 1, 'I~O', 3, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[15, 279, 'faq_no', 'vtiger_faq', 1, 4, 'faq_no', 'Faq No', 1, 0, '', 100, 2, 37, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[15, 280, 'category', 'vtiger_faq', 1, 15, 'faqcategories', 'Category', 1, 2, '', 100, 4, 37, 1, 'V~O', 3, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[15, 281, 'status', 'vtiger_faq', 1, 15, 'faqstatus', 'Status', 1, 2, '', 100, 3, 37, 1, 'V~M', 3, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 282, 'question', 'vtiger_faq', 1, 20, 'question', 'Question', 1, 2, '', 100, 7, 37, 1, 'V~M', 3, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[15, 283, 'answer', 'vtiger_faq', 1, 20, 'faq_answer', 'Answer', 1, 2, '', 100, 8, 37, 1, 'V~M', 3, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 284, 'comments', 'vtiger_faqcomments', 1, 19, 'comments', 'Add Comment', 1, 1, '', 100, 1, 38, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[15, 285, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 37, 2, 'DT~O', 3, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[15, 286, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 37, 2, 'DT~O', 3, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[15, 287, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 7, 37, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 288, 'vendorname', 'vtiger_vendor', 1, 2, 'vendorname', 'Vendor Name', 1, 0, '', 100, 1, 42, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[18, 289, 'vendor_no', 'vtiger_vendor', 1, 4, 'vendor_no', 'Vendor No', 1, 0, '', 100, 2, 42, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 290, 'phone', 'vtiger_vendor', 1, 11, 'phone', 'Phone', 1, 2, '', 100, 4, 42, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[18, 291, 'email', 'vtiger_vendor', 1, 13, 'email', 'Email', 1, 2, '', 100, 3, 42, 1, 'E~O', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[18, 292, 'website', 'vtiger_vendor', 1, 17, 'website', 'Website', 1, 2, '', 100, 6, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 293, 'glacct', 'vtiger_vendor', 1, 15, 'glacct', 'GL Account', 1, 2, '', 100, 5, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 294, 'category', 'vtiger_vendor', 1, 1, 'category', 'Category', 1, 2, '', 100, 8, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[18, 295, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 7, 42, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 296, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 9, 42, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 297, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 12, 42, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 304, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 45, 1, 'V~O', 1, null, 'ADV', 1, '', 0, '', null, 0, 0],
					[19, 305, 'bookname', 'vtiger_pricebook', 1, 2, 'bookname', 'Price Book Name', 1, 0, '', 100, 1, 46, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[19, 306, 'pricebook_no', 'vtiger_pricebook', 1, 4, 'pricebook_no', 'PriceBook No', 1, 0, '', 100, 3, 46, 1, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 307, 'active', 'vtiger_pricebook', 1, 56, 'active', 'Active', 1, 2, '1', 100, 2, 46, 1, 'C~O', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[19, 308, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 4, 46, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 309, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 5, 46, 2, 'DT~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 310, 'currency_id', 'vtiger_pricebook', 1, 117, 'currency_id', 'Currency', 1, 0, '', 100, 6, 46, 1, 'V~M', 0, 3, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 311, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 7, 46, 3, 'V~O', 3, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 312, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 48, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 474, 'user_name', 'vtiger_users', 1, 106, 'user_name', 'User Name', 1, 0, '', 11, 1, 77, 4, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 475, 'is_admin', 'vtiger_users', 1, 156, 'is_admin', 'Admin', 1, 0, '', 3, 7, 77, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 476, 'user_password', 'vtiger_users', 1, 99, 'user_password', 'Password', 1, 0, '', 30, 5, 77, 4, 'P~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 477, 'confirm_password', 'vtiger_users', 1, 99, 'confirm_password', 'Confirm Password', 1, 0, '', 30, 6, 77, 4, 'P~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 478, 'first_name', 'vtiger_users', 1, 1, 'first_name', 'First Name', 1, 0, '', 30, 3, 77, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 479, 'last_name', 'vtiger_users', 1, 2, 'last_name', 'Last Name', 1, 0, '', 30, 4, 77, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 480, 'roleid', 'vtiger_user2role', 1, 98, 'roleid', 'Role', 1, 0, '', 200, 8, 77, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 481, 'email1', 'vtiger_users', 1, 104, 'email1', 'Email', 1, 0, '', 100, 2, 77, 1, 'E~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 482, 'status', 'vtiger_users', 1, 115, 'status', 'Status', 1, 0, 'Active', 100, 10, 77, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 483, 'activity_view', 'vtiger_users', 1, 16, 'activity_view', 'Default Activity View', 1, 0, 'This Month', 100, 6, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 484, 'lead_view', 'vtiger_users', 1, 16, 'lead_view', 'Default Lead View', 1, 0, '', 100, 9, 77, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 485, 'hour_format', 'vtiger_users', 1, 16, 'hour_format', 'Calendar Hour Format', 1, 0, '24', 100, 3, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 487, 'start_hour', 'vtiger_users', 1, 16, 'start_hour', 'Day starts at', 1, 0, '08:00', 100, 2, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 492, 'reports_to_id', 'vtiger_users', 1, 101, 'reports_to_id', 'Reports To', 1, 0, '', 50, 8, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 498, 'date_format', 'vtiger_users', 1, 16, 'date_format', 'Date Format', 1, 0, 'yyyy-mm-dd', 30, 1, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 500, 'description', 'vtiger_users', 1, 21, 'description', 'Documents', 1, 0, '', 250, 14, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 506, 'accesskey', 'vtiger_users', 1, 3, 'accesskey', 'Webservice Access Key', 1, 0, '', 100, 2, 83, 2, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 507, 'time_zone', 'vtiger_users', 1, 16, 'time_zone', 'Time Zone', 1, 0, '', 200, 5, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 508, 'currency_id', 'vtiger_users', 1, 117, 'currency_id', 'Currency', 1, 0, '', 100, 1, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 509, 'currency_grouping_pattern', 'vtiger_users', 1, 16, 'currency_grouping_pattern', 'Digit Grouping Pattern', 1, 0, '123,456,789', 100, 2, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 510, 'currency_decimal_separator', 'vtiger_users', 1, 16, 'currency_decimal_separator', 'Decimal Separator', 1, 0, '.', 2, 3, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 511, 'currency_grouping_separator', 'vtiger_users', 1, 16, 'currency_grouping_separator', 'Digit Grouping Separator', 1, 0, ' ', 2, 4, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 512, 'currency_symbol_placement', 'vtiger_users', 1, 16, 'currency_symbol_placement', 'Symbol Placement', 1, 0, '1.0$', 20, 5, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 513, 'imagename', 'vtiger_users', 1, 105, 'imagename', 'User Image', 1, 0, '', 250, 10, 82, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 514, 'internal_mailer', 'vtiger_users', 1, 56, 'internal_mailer', 'INTERNAL_MAIL_COMPOSER', 1, 0, '', 50, 15, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 515, 'theme', 'vtiger_users', 1, 31, 'theme', 'Theme', 1, 0, 'twilight', 100, 16, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 516, 'language', 'vtiger_users', 1, 32, 'language', 'Language', 1, 0, '', 100, 17, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 517, 'reminder_interval', 'vtiger_users', 1, 16, 'reminder_interval', 'Reminder Interval', 1, 0, '15 Minutes', 100, 9, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 540, 'phone_crm_extension', 'vtiger_users', 1, 11, 'phone_crm_extension', 'CRM Phone Extension', 1, 2, '', 100, 18, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 541, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 16, 89, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 542, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 90, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 543, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 7, 90, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 544, 'start_date', 'vtiger_servicecontracts', 1, 5, 'start_date', 'Start Date', 1, 2, '', 100, 7, 89, 1, 'D~O', 2, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 545, 'end_date', 'vtiger_servicecontracts', 1, 5, 'end_date', 'End Date', 1, 2, '', 100, 11, 89, 2, 'D~O', 3, 0, 'BAS', 0, '', 1, '', null, 0, 0],
					[34, 546, 'sc_related_to', 'vtiger_servicecontracts', 1, 10, 'sc_related_to', 'Related to', 1, 2, '', 100, 3, 89, 1, 'V~O', 2, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 547, 'tracking_unit', 'vtiger_servicecontracts', 1, 15, 'tracking_unit', 'Tracking Unit', 1, 2, '', 100, 4, 89, 1, 'V~O', 2, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 548, 'total_units', 'vtiger_servicecontracts', 1, 7, 'total_units', 'Total Units', 1, 2, '', 100, 6, 89, 1, 'V~O', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 549, 'used_units', 'vtiger_servicecontracts', 1, 7, 'used_units', 'Used Units', 1, 2, '', 100, 8, 89, 10, 'V~O', 2, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 550, 'subject', 'vtiger_servicecontracts', 1, 1, 'subject', 'Subject', 1, 0, '', 100, 1, 89, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 551, 'due_date', 'vtiger_servicecontracts', 1, 23, 'due_date', 'Due date', 1, 2, '', 100, 9, 89, 1, 'D~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 552, 'planned_duration', 'vtiger_servicecontracts', 1, 1, 'planned_duration', 'Planned Duration', 1, 2, '', 100, 13, 89, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 553, 'actual_duration', 'vtiger_servicecontracts', 1, 1, 'actual_duration', 'Actual Duration', 1, 2, '', 100, 15, 89, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 554, 'contract_status', 'vtiger_servicecontracts', 1, 15, 'contract_status', 'Status', 1, 2, '', 100, 10, 89, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 555, 'priority', 'vtiger_servicecontracts', 1, 15, 'contract_priority', 'Priority', 1, 2, 'Low', 100, 12, 89, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 556, 'contract_type', 'vtiger_servicecontracts', 1, 15, 'contract_type', 'Type', 1, 2, '', 100, 5, 89, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[34, 557, 'progress', 'vtiger_servicecontracts', 1, 9, 'progress', 'Progress', 1, 2, '', 100, 14, 89, 2, 'N~O~2~2', 3, 3, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 558, 'contract_no', 'vtiger_servicecontracts', 1, 4, 'contract_no', 'Contract No', 1, 0, '', 100, 2, 89, 1, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 559, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 17, 89, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 560, 'servicename', 'vtiger_service', 1, 2, 'servicename', 'Service Name', 1, 0, '', 100, 1, 91, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[35, 561, 'service_no', 'vtiger_service', 1, 4, 'service_no', 'Service No', 1, 2, '', 100, 2, 91, 1, 'V~O', 1, 0, 'BAS', 2, '', 0, '', null, 0, 0],
					[35, 562, 'discontinued', 'vtiger_service', 1, 56, 'discontinued', 'Service Active', 1, 2, '', 100, 4, 91, 1, 'V~O', 2, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 563, 'sales_start_date', 'vtiger_service', 1, 5, 'sales_start_date', 'Sales Start Date', 1, 2, '', 100, 9, 91, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 564, 'sales_end_date', 'vtiger_service', 1, 5, 'sales_end_date', 'Sales End Date', 1, 2, null, 100, 10, 91, 1, 'D~O~OTH~GE~sales_start_date~Sales Start Date', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[35, 565, 'start_date', 'vtiger_service', 1, 5, 'start_date', 'Support Start Date', 1, 2, '', 100, 11, 91, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 566, 'expiry_date', 'vtiger_service', 1, 5, 'expiry_date', 'Support Expiry Date', 1, 2, '', 100, 12, 91, 1, 'D~O~OTH~GE~start_date~Start Date', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 567, 'website', 'vtiger_service', 1, 17, 'website', 'Website', 1, 2, '', 100, 6, 91, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 568, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 13, 91, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 569, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 14, 91, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 570, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 16, 91, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 571, 'service_usageunit', 'vtiger_service', 1, 15, 'service_usageunit', 'Usage Unit', 1, 2, '', 100, 3, 91, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 572, 'qty_per_unit', 'vtiger_service', 1, 1, 'qty_per_unit', 'No of Units', 1, 2, '', 100, 5, 91, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 573, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Owner', 1, 2, '', 100, 8, 91, 1, 'I~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[35, 574, 'pscategory', 'vtiger_service', 1, 302, 'pscategory', 'Service Category', 1, 2, '', 100, 7, 91, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '3', null, 0, 0],
					[35, 575, 'unit_price', 'vtiger_service', 1, 72, 'unit_price', 'Price', 1, 0, '', 100, 1, 92, 1, 'N~O', 2, 2, 'BAS', 0, '', 1, '', null, 0, 0],
					[35, 577, 'commissionrate', 'vtiger_service', 1, 9, 'commissionrate', 'Commission Rate', 1, 2, '', 100, 2, 92, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 578, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 94, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 579, 'asset_no', 'vtiger_assets', 1, 4, 'asset_no', 'Asset No', 1, 0, '', 100, 1, 95, 1, 'V~O', 1, 0, 'BAS', 2, '', 0, '', null, 0, 0],
					[37, 580, 'product', 'vtiger_assets', 1, 10, 'product', 'Product Name', 1, 2, '', 100, 4, 96, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[37, 581, 'serialnumber', 'vtiger_assets', 1, 2, 'serialnumber', 'Serial Number', 1, 2, '', 100, 2, 95, 1, 'V~M', 0, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 582, 'datesold', 'vtiger_assets', 1, 5, 'datesold', 'Date Sold', 1, 2, '', 100, 3, 192, 1, 'D~M~OTH~GE~datesold~Date Sold', 0, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 583, 'dateinservice', 'vtiger_assets', 1, 5, 'dateinservice', 'Date in Service', 1, 0, '', 100, 2, 192, 1, 'D~M~OTH~GE~dateinservice~Date in Service', 0, 9, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 584, 'assetstatus', 'vtiger_assets', 1, 15, 'assetstatus', 'Status', 1, 0, '', 100, 3, 95, 1, 'V~M', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[37, 589, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 5, 96, 1, 'V~M', 2, 11, 'BAS', 1, '', 1, '', null, 0, 0],
					[37, 590, 'assetname', 'vtiger_assets', 1, 1, 'assetname', 'Asset Name', 1, 0, '', 100, 4, 95, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[37, 593, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 4, 192, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[37, 594, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 5, 192, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[37, 595, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 16, 95, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[40, 597, 'commentcontent', 'vtiger_modcomments', 1, 19, 'commentcontent', 'Comment', 1, 0, '', 100, 4, 98, 1, 'V~M', 0, 1, 'BAS', 2, '', 1, '', null, 0, 0],
					[40, 598, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 1, 99, 1, 'V~M', 0, 2, 'BAS', 2, '', 1, '', null, 0, 0],
					[40, 599, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 99, 2, 'DT~O', 0, 2, 'BAS', 0, '', 0, '', null, 0, 0],
					[40, 600, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 99, 2, 'DT~O', 0, 3, 'BAS', 0, '', 0, '', null, 0, 0],
					[40, 601, 'related_to', 'vtiger_modcomments', 1, 10, 'related_to', 'Related To', 1, 2, '', 100, 2, 99, 1, 'V~M', 2, 3, 'BAS', 2, '', 0, '', null, 0, 0],
					[40, 602, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'creator', 'Creator', 1, 2, '', 100, 4, 99, 2, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[40, 603, 'parent_comments', 'vtiger_modcomments', 1, 10, 'parent_comments', 'Related To Comments', 1, 2, '', 100, 7, 99, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 604, 'projectmilestonename', 'vtiger_projectmilestone', 1, 2, 'projectmilestonename', 'Project Milestone Name', 1, 2, '', 100, 1, 101, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[41, 605, 'projectmilestonedate', 'vtiger_projectmilestone', 1, 5, 'projectmilestonedate', 'Milestone Date', 1, 2, '', 100, 4, 101, 1, 'D~M', 0, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[41, 606, 'projectid', 'vtiger_projectmilestone', 1, 10, 'projectid', 'Related to', 1, 0, '', 100, 3, 101, 1, 'V~M', 0, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[41, 607, 'projectmilestonetype', 'vtiger_projectmilestone', 1, 15, 'projectmilestonetype', 'Type', 1, 2, '', 100, 8, 101, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[41, 608, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 5, 101, 1, 'V~M', 0, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 609, 'projectmilestone_no', 'vtiger_projectmilestone', 1, 4, 'projectmilestone_no', 'Project Milestone No', 1, 0, '', 100, 2, 101, 1, 'V~O', 3, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[41, 610, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 102, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 611, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 2, 102, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 612, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 3, 102, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[41, 613, 'description', 'vtiger_crmentity', 1, 300, 'description', 'description', 1, 2, '', 100, 1, 103, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 614, 'projecttaskname', 'vtiger_projecttask', 1, 2, 'projecttaskname', 'Project Task Name', 1, 2, '', 100, 1, 104, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 615, 'projecttasktype', 'vtiger_projecttask', 1, 15, 'projecttasktype', 'Type', 1, 2, '', 100, 4, 104, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 616, 'projecttaskpriority', 'vtiger_projecttask', 1, 15, 'projecttaskpriority', 'Priority', 1, 2, 'Low', 100, 3, 104, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 617, 'projectid', 'vtiger_projecttask', 1, 10, 'projectid', 'Related to', 1, 0, '', 100, 6, 104, 1, 'V~M', 0, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 618, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 104, 1, 'V~M', 0, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 619, 'projecttasknumber', 'vtiger_projecttask', 1, 7, 'projecttasknumber', 'Project Task Number', 1, 2, '', 100, 5, 104, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 620, 'projecttask_no', 'vtiger_projecttask', 1, 4, 'projecttask_no', 'Project Task No', 1, 0, '', 100, 2, 104, 1, 'V~O', 3, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[42, 621, 'projecttaskprogress', 'vtiger_projecttask', 1, 15, 'projecttaskprogress', 'Progress', 1, 2, '', 100, 3, 105, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 623, 'startdate', 'vtiger_projecttask', 1, 5, 'startdate', 'Start Date', 1, 2, '', 100, 5, 105, 1, 'D~M', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 624, 'enddate', 'vtiger_projecttask', 1, 5, 'enddate', 'End Date', 1, 2, '', 100, 7, 105, 1, 'D~O~OTH~GE~startdate~Start Date', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 625, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 105, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 626, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 2, 105, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 627, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 7, 105, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[42, 628, 'description', 'vtiger_crmentity', 1, 300, 'description', 'description', 1, 2, '', 100, 1, 106, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 629, 'projectname', 'vtiger_project', 1, 2, 'projectname', 'Project Name', 1, 2, '', 100, 1, 107, 1, 'V~M', 0, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 630, 'startdate', 'vtiger_project', 1, 23, 'startdate', 'Start Date', 1, 2, '', 100, 5, 107, 1, 'D~O', 0, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 631, 'targetenddate', 'vtiger_project', 1, 23, 'targetenddate', 'Target End Date', 1, 2, '', 100, 7, 107, 1, 'D~O~OTH~GE~startdate~Start Date', 0, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 632, 'actualenddate', 'vtiger_project', 1, 23, 'actualenddate', 'Actual End Date', 1, 2, '', 100, 6, 107, 1, 'D~O~OTH~GE~startdate~Start Date', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 633, 'projectstatus', 'vtiger_project', 1, 15, 'projectstatus', 'Status', 1, 2, '', 100, 9, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 634, 'projecttype', 'vtiger_project', 1, 15, 'projecttype', 'Type', 1, 2, '', 100, 8, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 635, 'linktoaccountscontacts', 'vtiger_project', 1, 10, 'linktoaccountscontacts', 'Related to', 1, 2, '', 100, 11, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 636, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 4, 107, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[43, 637, 'project_no', 'vtiger_project', 1, 4, 'project_no', 'Project No', 1, 0, '', 100, 2, 107, 1, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[43, 638, 'targetbudget', 'vtiger_project', 1, 7, 'targetbudget', 'Target Budget', 1, 2, '', 100, 1, 108, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 639, 'projecturl', 'vtiger_project', 1, 17, 'projecturl', 'Project Url', 1, 2, '', 100, 2, 108, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 640, 'projectpriority', 'vtiger_project', 1, 15, 'projectpriority', 'Priority', 1, 2, 'Low', 100, 3, 108, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 641, 'progress', 'vtiger_project', 1, 15, 'progress', 'Progress', 1, 2, '', 100, 4, 108, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 642, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 7, 108, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 643, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 6, 108, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 644, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 7, 108, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[43, 645, 'description', 'vtiger_crmentity', 1, 300, 'description', 'description', 1, 2, '', 100, 1, 109, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[45, 646, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 2, 110, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[45, 647, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 110, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[45, 648, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 110, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[45, 649, 'message', 'vtiger_smsnotifier', 1, 21, 'message', 'message', 1, 0, '', 100, 1, 110, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[45, 650, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 7, 110, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[29, 652, 'no_of_currency_decimals', 'vtiger_users', 1, 16, 'no_of_currency_decimals', 'Number Of Currency Decimals', 1, 2, '2', 100, 6, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 689, 'truncate_trailing_zeros', 'vtiger_users', 1, 56, 'truncate_trailing_zeros', 'Truncate Trailing Zeros', 1, 2, '0', 100, 7, 78, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 690, 'projecttaskstatus', 'vtiger_projecttask', 1, 15, 'projecttaskstatus', 'Status', 1, 2, 'Open', 100, 8, 104, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[40, 691, 'customer', 'vtiger_modcomments', 1, 10, 'customer', 'Customer', 1, 2, '', 100, 5, 98, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 692, 'dayoftheweek', 'vtiger_users', 1, 16, 'dayoftheweek', 'Starting Day of the week', 1, 2, 'Monday', 100, 7, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 693, 'callduration', 'vtiger_users', 1, 16, 'callduration', 'Default Call Duration', 1, 2, '30', 100, 10, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 694, 'othereventduration', 'vtiger_users', 1, 16, 'othereventduration', 'Other Event Duration', 1, 2, '30', 100, 8, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 700, 'isconvertedfromlead', 'vtiger_account', 1, 56, 'isconvertedfromlead', 'Is Converted From Lead', 1, 2, 'no', 100, 1, 196, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 701, 'isconvertedfromlead', 'vtiger_contactdetails', 1, 56, 'isconvertedfromlead', 'Is Converted From Lead', 1, 2, 'no', 100, 5, 5, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 703, 'default_record_view', 'vtiger_users', 1, 16, 'default_record_view', 'Default Record View', 1, 2, 'Summary', 100, 19, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 708, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 10, 42, 1, 'V~M', 1, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[40, 710, 'userid', 'vtiger_modcomments', 1, 10, 'userid', 'UserId', 1, 2, '', 100, 6, 98, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[40, 711, 'reasontoedit', 'vtiger_modcomments', 1, 19, 'reasontoedit', 'ReasonToEdit', 1, 2, '', 100, 7, 98, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 716, 'leftpanelhide', 'vtiger_users', 1, 56, 'leftpanelhide', 'LBL_MENU_EXPANDED_BY_DEFAULT', 1, 2, '0', 100, 20, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 719, 'rowheight', 'vtiger_users', 1, 16, 'rowheight', 'Row Height', 1, 2, 'medium', 100, 21, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 720, 'from_portal', 'vtiger_troubletickets', 1, 56, 'from_portal', 'From Portal', 1, 0, '', 100, 19, 25, 3, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 721, 'defaulteventstatus', 'vtiger_users', 1, 15, 'defaulteventstatus', 'Default Event Status', 1, 2, 'PLL_PLANNED', 100, 14, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 722, 'defaultactivitytype', 'vtiger_users', 1, 15, 'defaultactivitytype', 'Default Activity Type', 1, 2, 'Meeting', 100, 12, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 725, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 4, 5, 2, 'V~O', 3, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 726, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 3, 196, 2, 'V~O', 3, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 727, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 7, 14, 2, 'V~O', 3, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[9, 729, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 15, 19, 2, 'V~O', 3, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 730, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 3, 27, 2, 'V~O', 3, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 734, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 11, 42, 2, 'V~O', 3, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[33, 741, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 17, 88, 2, 'V~O', 3, 1, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 742, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 3, 90, 2, 'V~O', 3, 10, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 743, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 17, 91, 2, 'V~O', 3, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[37, 744, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 7, 96, 2, 'V~O', 3, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[40, 745, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 8, 98, 2, 'V~O', 3, 6, 'BAS', 0, '', 0, '', null, 0, 0],
					[41, 746, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 7, 101, 2, 'V~O', 3, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[42, 747, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 11, 104, 2, 'V~O', 3, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[43, 748, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 10, 107, 2, 'V~O', 3, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[45, 749, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'created_user_id', 'Created By', 1, 2, '', 100, 8, 110, 2, 'V~O', 3, 1, 'BAS', 0, '', 0, '', null, 0, 0],
					[51, 780, 'name', 'vtiger_osstimecontrol', 1, 1, 'name', 'name', 1, 2, '', 100, 1, 128, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 781, 'osstimecontrol_no', 'vtiger_osstimecontrol', 1, 4, 'osstimecontrol_no', 'No.', 1, 2, '', 100, 2, 128, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 782, 'osstimecontrol_status', 'vtiger_osstimecontrol', 1, 15, 'osstimecontrol_status', 'LBL_STATUS', 1, 2, 'To approval', 100, 2, 128, 1, 'V~M', 2, 2, 'BAS', 2, '', 1, '', null, 0, 0],
					[51, 783, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 128, 1, 'V~M', 1, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 784, 'date_start', 'vtiger_osstimecontrol', 1, 6, 'date_start', 'Start Date', 1, 0, '', 100, 3, 128, 1, 'D~M', 0, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 785, 'time_start', 'vtiger_osstimecontrol', 1, 14, 'time_start', 'Time Start', 1, 0, '', 100, 5, 128, 1, 'T~M', 1, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 786, 'time_end', 'vtiger_osstimecontrol', 1, 14, 'time_end', 'End Time', 1, 0, '', 100, 6, 128, 1, 'T~M~OTH~GE~time_start~Time Start', 1, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 787, 'due_date', 'vtiger_osstimecontrol', 1, 23, 'due_date', 'End Date', 1, 0, '', 100, 4, 128, 1, 'D~M~OTH~GE~date_start~Start Date', 1, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 788, 'sum_time', 'vtiger_osstimecontrol', 1, 7, 'sum_time', 'LBL_TOTAL_TIME', 1, 2, '', 100, 8, 128, 1, 'NN~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 791, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 11, 128, 2, 'DT~O', 3, 2, 'BAS', 0, '', 0, '', null, 0, 0],
					[51, 792, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 10, 128, 2, 'DT~O', 3, 2, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 814, 'sum_time', 'vtiger_troubletickets', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 10, 27, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 816, 'sum_time', 'vtiger_projecttask', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 10, 104, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 818, 'sum_time', 'vtiger_assets', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 1, 192, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 820, 'sum_time', 'vtiger_servicecontracts', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 1, 90, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 826, 'sum_time', 'vtiger_project', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 108, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 842, 'ossmailview_no', 'vtiger_ossmailview', 1, 4, 'ossmailview_no', 'number', 1, 0, '', 100, 1, 134, 1, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 843, 'from_email', 'vtiger_ossmailview', 1, 1, 'from_email', 'From', 1, 2, '', 100, 2, 134, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 844, 'to_email', 'vtiger_ossmailview', 1, 1, 'to_email', 'To', 1, 2, '', 100, 3, 134, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 845, 'subject', 'vtiger_ossmailview', 1, 1, 'subject', 'Subject', 1, 2, '', 100, 4, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 846, 'cc_email', 'vtiger_ossmailview', 1, 1, 'cc_email', 'CC', 1, 2, '', 100, 5, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 847, 'bcc_email', 'vtiger_ossmailview', 1, 1, 'bcc_email', 'BCC', 1, 2, '', 100, 6, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 848, 'ossmailview_sendtype', 'vtiger_ossmailview', 1, 15, 'ossmailview_sendtype', 'SendType', 1, 0, '', 100, 7, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 849, 'uid', 'vtiger_ossmailview', 1, 1, 'uid', 'messageid', 1, 2, '', 100, 12, 134, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 850, 'reply_to_email', 'vtiger_ossmailview', 1, 1, 'reply_to_email', 'Reply_to_email', 1, 2, '', 100, 8, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 851, 'attachments_exist', 'vtiger_ossmailview', 1, 56, 'attachments_exist', 'Attachments_Exist', 1, 2, '', 100, 9, 134, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 852, 'rc_user', 'vtiger_ossmailview', 1, 1, 'rc_user', 'RC user', 1, 2, '', 100, 15, 134, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 853, 'type', 'vtiger_ossmailview', 1, 1, 'type', 'Type', 1, 2, '', 100, 16, 134, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 854, 'from_id', 'vtiger_ossmailview', 1, 1, 'from_id', 'FromID', 1, 2, '', 100, 17, 134, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 855, 'to_id', 'vtiger_ossmailview', 1, 1, 'to_id', 'ToID', 1, 2, '', 100, 18, 134, 3, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 856, 'id', 'vtiger_ossmailview', 1, 1, 'mid', 'Id', 1, 2, '', 100, 19, 134, 3, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 857, 'mbox', 'vtiger_ossmailview', 1, 1, 'mbox', 'mbox', 1, 2, '', 100, 20, 134, 3, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 858, 'verify', 'vtiger_ossmailview', 1, 56, 'verify', 'LBL_VERIFY', 1, 2, '0', 100, 10, 134, 2, 'C~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 859, 'rel_mod', 'vtiger_ossmailview', 1, 33, 'rel_mod', 'LBL_REL_MOD', 1, 2, '', 100, 11, 134, 2, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 860, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 3, 135, 1, 'V~M', 0, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 861, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 1, 135, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 862, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 2, 135, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 863, 'content', 'vtiger_ossmailview', 1, 19, 'content', 'Content', 1, 2, '', 100, 1, 136, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 864, 'orginal_mail', 'vtiger_ossmailview', 1, 21, 'orginal_mail', 'LBL_ORGINAL_MAIL_CONTENT', 1, 2, '', 100, 1, 137, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 865, 'ossdc_status', 'vtiger_notes', 1, 15, 'ossdc_status', 'ossdc_status', 1, 2, '', 100, 7, 17, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 866, 'osservicesstatus', 'vtiger_ossoutsourcedservices', 1, 15, 'osservicesstatus', 'LBL_osservicesstatus', 1, 2, '', 100, 3, 138, 1, 'V~O', 2, 3, 'BAS', 2, '', 1, '', null, 0, 0],
					[57, 867, 'productname', 'vtiger_ossoutsourcedservices', 1, 2, 'productname', 'Product Name', 1, 2, '', 250, 1, 138, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[57, 868, 'pscategory', 'vtiger_ossoutsourcedservices', 1, 302, 'pscategory', 'Category', 1, 2, '', 250, 2, 138, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '4', null, 0, 0],
					[57, 870, 'datesold', 'vtiger_ossoutsourcedservices', 1, 5, 'datesold', 'Date Sold', 1, 2, '', 100, 5, 138, 1, 'D~O', 2, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[57, 871, 'dateinservice', 'vtiger_ossoutsourcedservices', 1, 5, 'dateinservice', 'Date in Service', 1, 2, '', 100, 6, 138, 1, 'D~O', 2, 10, 'BAS', 1, '', 1, '', null, 0, 0],
					[57, 872, 'wherebought', 'vtiger_ossoutsourcedservices', 1, 1, 'wherebought', 'Where bought', 1, 2, '', 250, 8, 138, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 873, 'ossoutsourcedservices_no', 'vtiger_ossoutsourcedservices', 1, 4, 'ossoutsourcedservices_no', 'Number', 1, 2, '', 100, 2, 139, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 874, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 1, 139, 1, 'V~M', 1, 8, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 875, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 4, 139, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[57, 876, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 5, 139, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[57, 877, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 4, 139, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[57, 878, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Notes', 1, 2, '', 100, 1, 140, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 879, 'productname', 'vtiger_osssoldservices', 1, 2, 'productname', 'Product Name', 1, 2, '', 100, 1, 141, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[58, 880, 'ssservicesstatus', 'vtiger_osssoldservices', 1, 15, 'ssservicesstatus', 'LBL_ssservicesstatus', 1, 0, '', 100, 3, 141, 1, 'V~M', 2, 7, 'BAS', 2, '', 1, '', null, 0, 0],
					[58, 881, 'pscategory', 'vtiger_osssoldservices', 1, 302, 'pscategory', 'Category', 1, 2, '', 100, 4, 141, 1, 'V~O', 2, 9, 'BAS', 1, '', 1, '5', null, 0, 0],
					[58, 883, 'datesold', 'vtiger_osssoldservices', 1, 5, 'datesold', 'Date Sold', 1, 2, '', 100, 8, 141, 1, 'D~M', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[58, 884, 'dateinservice', 'vtiger_osssoldservices', 1, 5, 'dateinservice', 'Date in Service', 1, 2, '', 100, 5, 141, 1, 'D~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[58, 885, 'invoice', 'vtiger_osssoldservices', 1, 1, 'invoice', 'Invoice', 1, 2, '', 100, 7, 141, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 886, 'osssoldservices_no', 'vtiger_osssoldservices', 1, 4, 'osssoldservices_no', 'Number', 1, 2, '', 100, 1, 142, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 887, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 5, 142, 1, 'V~M', 1, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 888, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 8, 142, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[58, 889, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 9, 142, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[58, 890, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 4, 142, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[58, 891, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 143, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 892, 'productname', 'vtiger_outsourcedproducts', 1, 1, 'productname', 'Product Name', 1, 2, '', 100, 1, 144, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[59, 893, 'oproductstatus', 'vtiger_outsourcedproducts', 1, 15, 'oproductstatus', 'Status', 1, 2, '', 100, 2, 144, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[59, 894, 'datesold', 'vtiger_outsourcedproducts', 1, 5, 'datesold', 'Date Sold', 1, 2, '', 100, 3, 144, 1, 'D~O', 2, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[59, 895, 'dateinservice', 'vtiger_outsourcedproducts', 1, 5, 'dateinservice', 'Date in Service', 1, 2, '', 100, 5, 144, 1, 'D~O', 2, 10, 'BAS', 1, '', 1, '', null, 0, 0],
					[59, 896, 'pscategory', 'vtiger_outsourcedproducts', 1, 302, 'pscategory', 'Category', 1, 2, '', 100, 17, 144, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '6', null, 0, 0],
					[59, 898, 'wherebought', 'vtiger_outsourcedproducts', 1, 1, 'wherebought', 'Where bought', 1, 2, '', 100, 22, 144, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 899, 'prodcount', 'vtiger_outsourcedproducts', 1, 7, 'prodcount', 'Quantity', 1, 2, '', 100, 23, 144, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 900, 'asset_no', 'vtiger_outsourcedproducts', 1, 4, 'asset_no', 'Number', 1, 2, '', 100, 1, 145, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[59, 901, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 2, 145, 1, 'V~M', 1, 8, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 902, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 6, 145, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[59, 903, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 3, 145, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[59, 904, 'modifiedby', 'vtiger_crmentity', 1, 52, 'modifiedby', 'Last Modified By', 1, 0, '', 100, 4, 145, 3, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[59, 905, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 146, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 927, 'osspassword_no', 'vtiger_osspasswords', 1, 4, 'osspassword_no', 'OSSPassword No', 1, 0, '', 100, 1, 147, 1, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[60, 928, 'passwordname', 'vtiger_osspasswords', 1, 1, 'passwordname', 'Key name', 0, 2, '', 100, 2, 147, 1, 'V~M', 0, 1, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 929, 'linkto', 'vtiger_osspasswords', 1, 67, 'linkto', 'Related to', 1, 2, '', 100, 3, 147, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 930, 'username', 'vtiger_osspasswords', 1, 1, 'username', 'Username', 0, 2, '', 100, 4, 147, 1, 'V~O', 0, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 931, 'password', 'vtiger_osspasswords', 1, 1, 'password', 'Password', 0, 2, '', 100, 5, 147, 1, 'V~O', 0, 5, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 932, 'link_adres', 'vtiger_osspasswords', 1, 17, 'link_adres', 'WWW page', 0, 2, '', 255, 6, 147, 1, 'V~O~LE~255', 0, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 933, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 147, 1, 'V~M', 0, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 934, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 8, 147, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[60, 935, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 11, 147, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[60, 936, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 10, 149, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 938, 'noapprovalcalls', 'vtiger_leaddetails', 1, 56, 'noapprovalcalls', 'Approval for phone calls', 1, 2, '', 100, 7, 150, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 940, 'noapprovalemails', 'vtiger_leaddetails', 1, 56, 'noapprovalemails', 'Approval for email', 1, 2, '', 100, 8, 150, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 941, 'ossemployees_no', 'vtiger_ossemployees', 1, 4, 'ossemployees_no', 'No.', 1, 2, '', 100, 1, 151, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[61, 942, 'employee_status', 'vtiger_ossemployees', 1, 15, 'employee_status', 'LBL_EMPLOYEESTATUS', 1, 2, '', 100, 1, 151, 1, 'V~O', 0, 4, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 943, 'parentid', 'vtiger_ossemployees', 1, 10, 'parentid', 'LBL_Member_Of', 1, 2, '', 100, 2, 151, 1, 'I~O', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 944, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 9, 151, 1, 'V~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 945, 'name', 'vtiger_ossemployees', 1, 1, 'name', 'LBL_NAME', 0, 2, '', 100, 3, 151, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 946, 'last_name', 'vtiger_ossemployees', 1, 1, 'last_name', 'LBL_LASTNAME', 0, 2, '', 100, 4, 151, 1, 'V~M', 1, 2, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 947, 'pesel', 'vtiger_ossemployees', 1, 1, 'pesel', 'LBL_PESEL', 0, 2, '', 100, 5, 151, 1, 'V~O', 1, 17, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 948, 'id_card', 'vtiger_ossemployees', 1, 1, 'id_card', 'LBL_IDCARD', 0, 2, '', 100, 6, 151, 1, 'V~O', 1, 18, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 949, 'employee_education', 'vtiger_ossemployees', 1, 15, 'employee_education', 'LBL_EDUCATION', 1, 2, '', 100, 7, 151, 1, 'V~O', 1, 5, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 950, 'birth_date', 'vtiger_ossemployees', 1, 5, 'birth_date', 'LBL_BIRTHDATE', 1, 2, '', 100, 8, 151, 1, 'D~O', 2, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 951, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 10, 151, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 952, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 13, 151, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 953, 'business_phone', 'vtiger_ossemployees', 1, 11, 'business_phone', 'LBL_BUSINESSPHONE', 0, 2, '', 100, 1, 152, 1, 'V~O', 1, 19, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 954, 'private_phone', 'vtiger_ossemployees', 1, 11, 'private_phone', 'LBL_PRIVATEPHONE', 0, 2, '', 100, 3, 152, 1, 'V~O', 1, 16, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 955, 'business_mail', 'vtiger_ossemployees', 1, 13, 'business_mail', 'LBL_BUSINESSMAIL', 0, 2, '', 100, 2, 152, 1, 'E~O', 1, 21, 'BAS', 0, '', 1, '', null, 0, 0],
					[61, 956, 'private_mail', 'vtiger_ossemployees', 1, 13, 'private_mail', 'LBL_PRIVATEMAIL', 0, 2, '', 100, 4, 152, 1, 'E~O', 1, 20, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 957, 'street', 'vtiger_ossemployees', 1, 1, 'street', 'LBL_STREET', 0, 2, '', 100, 1, 153, 1, 'V~O', 1, 22, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 958, 'code', 'vtiger_ossemployees', 1, 1, 'code', 'LBL_CODE', 0, 2, '', 100, 2, 153, 1, 'V~O', 1, 13, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 959, 'city', 'vtiger_ossemployees', 1, 1, 'city', 'LBL_CITY', 0, 2, '', 100, 3, 153, 1, 'V~O', 1, 9, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 960, 'state', 'vtiger_ossemployees', 1, 1, 'state', 'LBL_STATE', 0, 2, '', 100, 4, 153, 1, 'V~O', 1, 8, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 961, 'country', 'vtiger_ossemployees', 1, 1, 'country', 'LBL_COUNTRY', 0, 2, '', 100, 5, 153, 1, 'V~O', 1, 10, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 962, 'ship_street', 'vtiger_ossemployees', 1, 1, 'ship_street', 'LBL_SHIPSTREET', 0, 2, '', 100, 1, 154, 1, 'V~O', 1, 11, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 963, 'ship_code', 'vtiger_ossemployees', 1, 1, 'ship_code', 'LBL_SHIPCODE', 0, 2, '', 100, 2, 154, 1, 'V~O', 1, 15, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 964, 'ship_city', 'vtiger_ossemployees', 1, 1, 'ship_city', 'LBL_SHIPCITY', 0, 2, '', 100, 3, 154, 1, 'V~O', 1, 12, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 965, 'ship_state', 'vtiger_ossemployees', 1, 1, 'ship_state', 'LBL_SHIPSTATE', 0, 2, '', 100, 4, 154, 1, 'V~O', 1, 14, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 966, 'ship_country', 'vtiger_ossemployees', 1, 1, 'ship_country', 'LBL_SHIPCOUNTRY', 0, 2, '', 100, 5, 154, 1, 'V~O', 1, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 967, 'vat_id', 'vtiger_leaddetails', 1, 1, 'vat_id', 'Vat ID', 1, 2, '', 100, 5, 13, 1, 'V~O', 2, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 968, 'registration_number_1', 'vtiger_leaddetails', 1, 1, 'registration_number_1', 'Registration number 1', 1, 2, '', 100, 3, 191, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 969, 'registration_number_2', 'vtiger_leaddetails', 1, 1, 'registration_number_2', 'Registration number 2', 1, 2, '', 100, 2, 191, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 970, 'vat_id', 'vtiger_account', 1, 1, 'vat_id', 'Vat ID', 1, 2, '', 100, 2, 194, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 971, 'registration_number_1', 'vtiger_account', 1, 1, 'registration_number_1', 'Registration number 1', 1, 2, '', 100, 6, 194, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 972, 'registration_number_2', 'vtiger_account', 1, 1, 'registration_number_2', 'Registration number 2', 1, 2, '', 100, 4, 194, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 973, 'vat_id', 'vtiger_vendor', 1, 1, 'vat_id', 'Vat ID', 1, 2, '', 100, 14, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 974, 'registration_number_1', 'vtiger_vendor', 1, 1, 'registration_number_1', 'Registration number 1', 1, 2, '', 100, 13, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 975, 'registration_number_2', 'vtiger_vendor', 1, 1, 'registration_number_2', 'Registration number 2', 1, 2, '', 100, 16, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 1044, 'servicecontractsid', 'vtiger_project', 1, 10, 'servicecontractsid', 'ServiceContracts', 1, 2, '', 100, 13, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 1045, 'is_owner', 'vtiger_users', 1, 1, 'is_owner', 'Account Owner', 0, 2, '0', 100, 12, 77, 5, 'V~O', 0, 1, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 1049, 'servicecontractsid', 'vtiger_troubletickets', 1, 10, 'servicecontractsid', 'FL_SERVICE_CONTRACTS', 1, 2, '', 100, 6, 25, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1065, 'addresslevel1a', 'vtiger_leadaddress', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 9, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1066, 'addresslevel2a', 'vtiger_leadaddress', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 7, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1067, 'addresslevel3a', 'vtiger_leadaddress', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 10, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1068, 'addresslevel4a', 'vtiger_leadaddress', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1069, 'addresslevel5a', 'vtiger_leadaddress', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 3, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1070, 'addresslevel6a', 'vtiger_leadaddress', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1071, 'addresslevel7a', 'vtiger_leadaddress', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1072, 'addresslevel8a', 'vtiger_leadaddress', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1073, 'addresslevel1a', 'vtiger_contactaddress', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1074, 'addresslevel1b', 'vtiger_contactaddress', 1, 1, 'addresslevel1b', 'AddressLevel1', 1, 2, '', 100, 10, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1075, 'addresslevel2a', 'vtiger_contactaddress', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1076, 'addresslevel2b', 'vtiger_contactaddress', 1, 1, 'addresslevel2b', 'AddressLevel2', 1, 2, '', 100, 9, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1077, 'addresslevel3a', 'vtiger_contactaddress', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1078, 'addresslevel3b', 'vtiger_contactaddress', 1, 1, 'addresslevel3b', 'AddressLevel3', 1, 2, '', 100, 8, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1079, 'addresslevel4a', 'vtiger_contactaddress', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1080, 'addresslevel4b', 'vtiger_contactaddress', 1, 1, 'addresslevel4b', 'AddressLevel4', 1, 2, '', 100, 7, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1081, 'addresslevel5a', 'vtiger_contactaddress', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1082, 'addresslevel5b', 'vtiger_contactaddress', 1, 1, 'addresslevel5b', 'AddressLevel5', 1, 2, '', 100, 6, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1083, 'addresslevel6a', 'vtiger_contactaddress', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1084, 'addresslevel6b', 'vtiger_contactaddress', 1, 1, 'addresslevel6b', 'AddressLevel6', 1, 2, '', 100, 5, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1085, 'addresslevel7a', 'vtiger_contactaddress', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1086, 'addresslevel7b', 'vtiger_contactaddress', 1, 1, 'addresslevel7b', 'AddressLevel7', 1, 2, '', 100, 4, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1087, 'addresslevel8a', 'vtiger_contactaddress', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1088, 'addresslevel8b', 'vtiger_contactaddress', 1, 1, 'addresslevel8b', 'AddressLevel8', 1, 2, '', 100, 1, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1089, 'addresslevel1a', 'vtiger_accountaddress', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1090, 'addresslevel1b', 'vtiger_accountaddress', 1, 1, 'addresslevel1b', 'AddressLevel1', 1, 2, '', 100, 10, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1091, 'addresslevel2a', 'vtiger_accountaddress', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1092, 'addresslevel2b', 'vtiger_accountaddress', 1, 1, 'addresslevel2b', 'AddressLevel2', 1, 2, '', 100, 9, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1093, 'addresslevel3a', 'vtiger_accountaddress', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1094, 'addresslevel3b', 'vtiger_accountaddress', 1, 1, 'addresslevel3b', 'AddressLevel3', 1, 2, '', 100, 8, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1095, 'addresslevel4a', 'vtiger_accountaddress', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1096, 'addresslevel4b', 'vtiger_accountaddress', 1, 1, 'addresslevel4b', 'AddressLevel4', 1, 2, '', 100, 7, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1097, 'addresslevel5a', 'vtiger_accountaddress', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1098, 'addresslevel5b', 'vtiger_accountaddress', 1, 1, 'addresslevel5b', 'AddressLevel5', 1, 2, '', 100, 6, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1099, 'addresslevel6a', 'vtiger_accountaddress', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1100, 'addresslevel6b', 'vtiger_accountaddress', 1, 1, 'addresslevel6b', 'AddressLevel6', 1, 2, '', 100, 5, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1101, 'addresslevel7a', 'vtiger_accountaddress', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1102, 'addresslevel7b', 'vtiger_accountaddress', 1, 1, 'addresslevel7b', 'AddressLevel7', 1, 2, '', 100, 4, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1103, 'addresslevel8a', 'vtiger_accountaddress', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1104, 'addresslevel8b', 'vtiger_accountaddress', 1, 1, 'addresslevel8b', 'AddressLevel8', 1, 2, '', 100, 3, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1105, 'addresslevel1a', 'vtiger_vendoraddress', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 8, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1106, 'addresslevel1b', 'vtiger_vendoraddress', 1, 1, 'addresslevel1b', 'AddressLevel1', 1, 2, '', 100, 8, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1107, 'addresslevel1c', 'vtiger_vendoraddress', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 8, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1108, 'addresslevel2a', 'vtiger_vendoraddress', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 7, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1109, 'addresslevel2b', 'vtiger_vendoraddress', 1, 1, 'addresslevel2b', 'AddressLevel2', 1, 2, '', 100, 7, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1110, 'addresslevel2c', 'vtiger_vendoraddress', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 7, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1111, 'addresslevel3a', 'vtiger_vendoraddress', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 6, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1112, 'addresslevel3b', 'vtiger_vendoraddress', 1, 1, 'addresslevel3b', 'AddressLevel3', 1, 2, '', 100, 6, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1113, 'addresslevel3c', 'vtiger_vendoraddress', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 6, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1114, 'addresslevel4a', 'vtiger_vendoraddress', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 5, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1115, 'addresslevel4b', 'vtiger_vendoraddress', 1, 1, 'addresslevel4b', 'AddressLevel4', 1, 2, '', 100, 5, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1116, 'addresslevel4c', 'vtiger_vendoraddress', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 5, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1117, 'addresslevel5a', 'vtiger_vendoraddress', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 4, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1118, 'addresslevel5b', 'vtiger_vendoraddress', 1, 1, 'addresslevel5b', 'AddressLevel5', 1, 2, '', 100, 4, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1119, 'addresslevel5c', 'vtiger_vendoraddress', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 4, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1120, 'addresslevel6a', 'vtiger_vendoraddress', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 3, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1121, 'addresslevel6b', 'vtiger_vendoraddress', 1, 1, 'addresslevel6b', 'AddressLevel6', 1, 2, '', 100, 3, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1122, 'addresslevel6c', 'vtiger_vendoraddress', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 3, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1123, 'addresslevel7a', 'vtiger_vendoraddress', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 2, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1124, 'addresslevel7b', 'vtiger_vendoraddress', 1, 1, 'addresslevel7b', 'AddressLevel7', 1, 2, '', 100, 2, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1125, 'addresslevel7c', 'vtiger_vendoraddress', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 2, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1126, 'addresslevel8a', 'vtiger_vendoraddress', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1127, 'addresslevel8b', 'vtiger_vendoraddress', 1, 1, 'addresslevel8b', 'AddressLevel8', 1, 2, '', 100, 1, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1128, 'addresslevel8c', 'vtiger_vendoraddress', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 1, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1160, 'addresslevel1c', 'vtiger_accountaddress', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 10, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1161, 'addresslevel2c', 'vtiger_accountaddress', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 9, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1162, 'addresslevel3c', 'vtiger_accountaddress', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 8, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1163, 'addresslevel4c', 'vtiger_accountaddress', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 7, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1164, 'addresslevel5c', 'vtiger_accountaddress', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 6, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1165, 'addresslevel6c', 'vtiger_accountaddress', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 5, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1166, 'addresslevel7c', 'vtiger_accountaddress', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 4, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1167, 'addresslevel8c', 'vtiger_accountaddress', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 3, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1276, 'verification', 'vtiger_leaddetails', 1, 33, 'verification', 'Verification data', 1, 2, '', 100, 4, 14, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1277, 'verification', 'vtiger_account', 1, 33, 'verification', 'Verification data', 1, 2, '', 100, 2, 196, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1278, 'verification', 'vtiger_contactdetails', 1, 33, 'verification', 'Verification data', 1, 2, '', 100, 3, 5, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1279, 'verification', 'vtiger_vendor', 1, 33, 'verification', 'Verification data', 1, 2, '', 100, 15, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1285, 'buildingnumbera', 'vtiger_accountaddress', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1286, 'localnumbera', 'vtiger_accountaddress', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1287, 'buildingnumberb', 'vtiger_accountaddress', 1, 1, 'buildingnumberb', 'Building number', 1, 2, '', 100, 1, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1288, 'localnumberb', 'vtiger_accountaddress', 1, 1, 'localnumberb', 'Local number', 1, 2, '', 100, 2, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1289, 'buildingnumberc', 'vtiger_accountaddress', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 1, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1290, 'localnumberc', 'vtiger_accountaddress', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1291, 'buildingnumbera', 'vtiger_leadaddress', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 2, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1292, 'localnumbera', 'vtiger_leadaddress', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 4, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1293, 'buildingnumbera', 'vtiger_contactaddress', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1294, 'localnumbera', 'vtiger_contactaddress', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1295, 'buildingnumberb', 'vtiger_contactaddress', 1, 1, 'buildingnumberb', 'Building number', 1, 2, '', 100, 3, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1296, 'localnumberb', 'vtiger_contactaddress', 1, 1, 'localnumberb', 'Local number', 1, 2, '', 100, 2, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1314, 'parent_id', 'vtiger_assets', 1, 10, 'parent_id', 'Parent ID', 1, 2, '', 100, 1, 96, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[59, 1315, 'parent_id', 'vtiger_outsourcedproducts', 1, 10, 'parent_id', 'Parent ID', 1, 2, '', 100, 29, 144, 1, 'V~M', 2, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 1316, 'parent_id', 'vtiger_ossoutsourcedservices', 1, 10, 'parent_id', 'Parent ID', 1, 2, '', 100, 14, 138, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[58, 1317, 'parent_id', 'vtiger_osssoldservices', 1, 10, 'parent_id', 'Parent ID', 1, 2, '', 100, 1, 142, 1, 'V~M', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[42, 1318, 'parentid', 'vtiger_projecttask', 1, 10, 'parentid', 'Parent ID', 1, 2, '', 100, 13, 104, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1319, 'projectmilestoneid', 'vtiger_projecttask', 1, 10, 'projectmilestoneid', 'ProjectMilestone', 1, 2, '', 100, 12, 104, 1, 'V~M', 2, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1320, 'targetenddate', 'vtiger_projecttask', 1, 5, 'targetenddate', 'Target End Date', 1, 2, '', 100, 6, 105, 1, 'D~M', 2, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1321, 'subindustry', 'vtiger_leaddetails', 1, 16, 'subindustry', 'Sub industry', 1, 2, '', 100, 7, 13, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 1322, 'end_hour', 'vtiger_users', 1, 16, 'end_hour', 'Day ends at', 1, 2, '16:00', 100, 4, 118, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1323, 'description', 'vtiger_crmentity', 1, 300, 'description', 'description', 1, 2, '', 100, 1, 97, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1324, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 97, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1326, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 130, 1, 'V~O', 2, 9, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1327, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 130, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1329, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 16, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1330, 'no_approval', 'vtiger_account', 1, 56, 'no_approval', 'Approval for phone calls', 1, 2, '', 100, 8, 195, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1331, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 12, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1332, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 8, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1333, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 45, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 1340, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 109, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1341, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 28, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1344, 'serviceid', 'vtiger_osssoldservices', 1, 10, 'serviceid', 'Services', 1, 2, '', 100, 4, 142, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[4, 1368, 'secondary_email', 'vtiger_contactdetails', 1, 13, 'secondary_email', 'Secondary Email', 1, 2, '', 100, 4, 197, 1, 'E~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1369, 'balance', 'vtiger_account', 1, 71, 'balance', 'Balance', 1, 2, '', 100, 2, 198, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1375, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 8, 9, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 1378, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 74, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 1382, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 3, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1384, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 27, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1386, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 8, 4, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1391, 'notifilanguage', 'vtiger_contactdetails', 1, 32, 'notifilanguage', 'LBL_LANGUAGE_NOTIFICATIONS', 1, 2, '', 100, 4, 6, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 1393, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 25, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 1395, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 20, 91, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1397, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 13, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 1399, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 17, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1401, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 19, 1, 'V~O', 2, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 1403, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 11, 37, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 1405, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 39, 1, 'V~O', 2, 8, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1407, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 12, 42, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[19, 1409, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 46, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 1419, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 17, 89, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1421, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 95, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 1423, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 101, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1425, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 104, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1431, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 128, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 1433, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 13, 134, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 1435, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 3, 139, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1437, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 142, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 1439, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 145, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 1441, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 147, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 1443, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 11, 151, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1449, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 8, 5, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1450, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 9, 196, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1451, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 9, 14, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 1452, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 10, 17, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1453, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 24, 19, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1454, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 6, 27, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[14, 1455, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 26, 31, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 1456, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 9, 37, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 1457, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 15, 39, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1458, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 18, 42, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[19, 1459, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 7, 46, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 1464, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 16, 74, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 1465, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 6, 90, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 1466, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 21, 91, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1467, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 6, 192, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 1468, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 4, 102, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1469, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 8, 105, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 1470, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 9, 108, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1473, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 13, 128, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 1474, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 4, 135, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 1475, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 6, 139, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1476, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 11, 142, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 1477, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 5, 145, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 1478, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 10, 147, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 1479, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 12, 151, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1482, 'pssold_id', 'vtiger_troubletickets', 1, 10, 'pssold_id', 'P&S Sold', 1, 2, '', 100, 2, 25, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1483, 'ordertime', 'vtiger_troubletickets', 1, 7, 'ordertime', 'LBL_ORDER_TIME', 1, 2, '', 100, 9, 27, 1, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1484, 'ordertime', 'vtiger_assets', 1, 7, 'ordertime', 'LBL_ORDER_TIME', 1, 2, '', 100, 7, 192, 1, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1485, 'ordertime', 'vtiger_osssoldservices', 1, 7, 'ordertime', 'LBL_ORDER_TIME', 1, 2, '', 100, 10, 142, 1, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1486, 'poboxa', 'vtiger_leadaddress', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 15, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1487, 'poboxa', 'vtiger_contactaddress', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 7, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1488, 'poboxa', 'vtiger_accountaddress', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 11, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1489, 'poboxa', 'vtiger_vendoraddress', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 9, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1494, 'poboxb', 'vtiger_contactaddress', 1, 1, 'poboxb', 'Po Box', 1, 2, '', 100, 11, 178, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1495, 'poboxb', 'vtiger_accountaddress', 1, 1, 'poboxb', 'Po Box', 1, 2, '', 100, 11, 10, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1496, 'poboxb', 'vtiger_vendoraddress', 1, 1, 'poboxb', 'Po Box', 1, 2, '', 100, 9, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1501, 'poboxc', 'vtiger_vendoraddress', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 9, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1502, 'poboxc', 'vtiger_accountaddress', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 11, 181, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1503, 'contactstatus', 'vtiger_contactdetails', 1, 15, 'contactstatus', 'Status', 1, 2, '', 100, 7, 4, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1504, 'callhistorytype', 'vtiger_callhistory', 1, 16, 'callhistorytype', 'LBL_TYPE', 1, 2, '', 100, 2, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1505, 'from_number', 'vtiger_callhistory', 1, 11, 'from_number', 'LBL_FROM_NUMBER', 1, 2, '', 100, 1, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1506, 'to_number', 'vtiger_callhistory', 1, 11, 'to_number', 'LBL_TO_NUMBER', 1, 2, '', 100, 3, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1507, 'location', 'vtiger_callhistory', 1, 1, 'location', 'LBL_LOCATION', 1, 2, '', 100, 9, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1508, 'phonecallid', 'vtiger_callhistory', 1, 1, 'phonecallid', 'LBL_PHONE_CALL_ID', 1, 2, '', 100, 8, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1509, 'duration', 'vtiger_callhistory', 1, 7, 'duration', 'LBL_DURATION', 1, 2, '', 100, 7, 232, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1511, 'start_time', 'vtiger_callhistory', 1, 70, 'start_time', 'LBL_START_TIME', 1, 2, '', 100, 5, 232, 1, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[74, 1512, 'end_time', 'vtiger_callhistory', 1, 70, 'end_time', 'LBL_END_TIME', 1, 2, '', 100, 6, 232, 1, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[74, 1513, 'country', 'vtiger_callhistory', 1, 1, 'country', 'LBL_COUNTRY_CODE', 1, 2, '', 100, 10, 232, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1514, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 233, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1515, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 233, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[74, 1516, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 2, 233, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[74, 1517, 'imei', 'vtiger_callhistory', 1, 1, 'imei', 'LBL_IMEI', 1, 2, '', 100, 1, 234, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1518, 'ipaddress', 'vtiger_callhistory', 1, 1, 'ipAddress', 'LBL_IPADDRESS', 1, 2, '', 100, 4, 234, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1519, 'simserial', 'vtiger_callhistory', 1, 1, 'simSerial', 'LBL_SIMSERIAL', 1, 2, '', 100, 3, 234, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1520, 'subscriberid', 'vtiger_callhistory', 1, 1, 'subscriberId', 'LBL_SUBSCRIBERID', 1, 2, '', 100, 2, 234, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1521, 'destination', 'vtiger_callhistory', 1, 10, 'destination', 'LBL_DESTINATION', 1, 2, '', 100, 1, 235, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1522, 'source', 'vtiger_callhistory', 1, 10, 'source', 'LBL_SOURCE', 1, 2, '', 100, 2, 235, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1524, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 10, 4, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1525, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 7, 9, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1526, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 10, 13, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[8, 1527, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 9, 17, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1528, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 17, 19, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 1529, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 8, 27, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 1530, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 27, 31, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 1531, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 10, 37, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1532, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 17, 42, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[19, 1533, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 8, 46, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 1538, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 15, 74, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[34, 1539, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 4, 90, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 1540, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 22, 91, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1541, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 6, 95, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 1542, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 10, 101, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1543, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 14, 104, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 1544, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 12, 107, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1547, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 12, 128, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 1548, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 12, 134, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 1549, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 15, 138, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1550, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 9, 141, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 1551, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 30, 144, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[60, 1552, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 12, 147, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 1553, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 14, 151, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[74, 1556, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 11, 232, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1557, 'subject', 'vtiger_ideas', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 236, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1558, 'ideas_no', 'vtiger_ideas', 1, 4, 'ideas_no', 'LBL_NO', 1, 2, '', 100, 2, 236, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[75, 1559, 'ideasstatus', 'vtiger_ideas', 1, 15, 'ideasstatus', 'LBL_STATUS', 1, 2, '', 100, 2, 236, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[75, 1560, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 236, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1561, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 4, 236, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1562, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 7, 236, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1563, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 6, 236, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1564, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 8, '', 100, 14, 236, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1566, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 236, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1567, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 3, 238, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1568, 'extent_description', 'vtiger_ideas', 1, 300, 'extent_description', 'LBL_EXTENT_DESCRIPTION', 1, 2, '', 100, 2, 238, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[75, 1569, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 238, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1600, 'timecontrol_type', 'vtiger_osstimecontrol', 1, 16, 'timecontrol_type', 'Type', 1, 2, 'PLL_WORKING_TIME', 100, 14, 128, 1, 'V~M', 2, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1603, 'allday', 'vtiger_activity', 1, 56, 'allday', 'All day', 1, 2, '', 100, 26, 19, 1, 'C~O', 0, 5, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 1604, 'allday', 'vtiger_activity', 1, 56, 'allday', 'All day', 1, 2, '', 100, 24, 39, 1, 'C~O', 0, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1605, 'holidaysentitlement_year', 'vtiger_holidaysentitlement', 1, 15, 'holidaysentitlement_year', 'LBL_YEAR', 1, 2, '', 100, 1, 247, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[78, 1606, 'days', 'vtiger_holidaysentitlement', 1, 1, 'days', 'LBL_DAYS', 1, 2, '', 100, 2, 247, 1, 'I~M', 2, 3, 'BAS', 2, '', 1, '', null, 0, 0],
					[78, 1607, 'ossemployeesid', 'vtiger_holidaysentitlement', 1, 10, 'ossemployeesid', 'LBL_EMPLOYEE', 1, 2, '', 100, 3, 247, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[78, 1608, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 1, 248, 1, 'V~M', 1, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1609, 'holidaysentitlement_no', 'vtiger_holidaysentitlement', 1, 4, 'holidaysentitlement_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 248, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[78, 1610, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 248, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1611, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 5, 248, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1612, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 4, 248, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1613, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 8, '', 100, 6, 248, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[78, 1615, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 3, 248, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[79, 1616, 'paymentsno', 'vtiger_paymentsin', 1, 4, 'paymentsno', 'LBL_PAYMENTSNO', 1, 2, '', 100, 1, 250, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[79, 1617, 'paymentsname', 'vtiger_paymentsin', 1, 1, 'paymentsname', 'LBL_PAYMENTSNAME', 1, 2, '', 100, 2, 250, 1, 'V~O', 0, 3, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1618, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 250, 1, 'V~M', 1, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[79, 1619, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 4, 250, 2, 'DT~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1620, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 5, 250, 2, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1621, 'paymentsin_status', 'vtiger_paymentsin', 1, 15, 'paymentsin_status', 'LBL_PAYMENTSSTATUS', 1, 2, 'Created', 100, 6, 250, 1, 'V~O', 0, 8, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1622, 'relatedid', 'vtiger_paymentsin', 1, 10, 'relatedid', 'LBL_RELATEDID', 1, 2, '', 100, 7, 250, 1, 'V~O', 0, 7, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1625, 'paymentsvalue', 'vtiger_paymentsin', 1, 71, 'paymentsvalue', 'LBL_PAYMENTSVALUE', 1, 2, '', 100, 1, 251, 1, 'N~M', 1, 1, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1626, 'paymentscurrency', 'vtiger_paymentsin', 1, 117, 'paymentscurrency', 'LBL_PAYMENTSCURRENCY', 1, 2, '', 100, 2, 251, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1627, 'paymentstitle', 'vtiger_paymentsin', 1, 22, 'paymentstitle', 'LBL_PAYMENTSTITLE', 1, 2, '', 100, 4, 251, 1, 'V~O', 0, 5, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 1628, 'bank_account', 'vtiger_paymentsin', 1, 1, 'bank_account', 'LBL_BANK_ACCOUNT', 1, 2, '', 100, 3, 251, 1, 'V~O', 0, 4, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 1630, 'payment_balance', 'vtiger_account', 1, 7, 'payment_balance', 'Payment balance', 1, 2, '', 100, 6, 198, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[80, 1632, 'paymentsno', 'vtiger_paymentsout', 1, 4, 'paymentsno', 'LBL_PAYMENTSNO', 1, 2, '', 100, 1, 252, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[80, 1633, 'paymentsname', 'vtiger_paymentsout', 1, 1, 'paymentsname', 'LBL_PAYMENTSNAME', 1, 2, '', 100, 2, 252, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1634, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 252, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[80, 1635, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 4, 252, 2, 'DT~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1636, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 5, 252, 2, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1637, 'paymentsout_status', 'vtiger_paymentsout', 1, 15, 'paymentsout_status', 'LBL_PAYMENTSSTATUS', 1, 2, 'Created', 100, 6, 252, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1638, 'relatedid', 'vtiger_paymentsout', 1, 10, 'relatedid', 'LBL_RELATEDID', 1, 2, '', 100, 7, 252, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1640, 'parentid', 'vtiger_paymentsout', 1, 10, 'parentid', 'LBL_PARENTID', 1, 2, '', 100, 8, 252, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1641, 'paymentsvalue', 'vtiger_paymentsout', 1, 71, 'paymentsvalue', 'LBL_PAYMENTSVALUE', 1, 2, '', 100, 1, 253, 1, 'N~M', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1642, 'paymentscurrency', 'vtiger_paymentsout', 1, 117, 'paymentscurrency', 'LBL_PAYMENTSCURRENCY', 1, 2, '', 100, 2, 253, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1643, 'paymentstitle', 'vtiger_paymentsout', 1, 1, 'paymentstitle', 'LBL_PAYMENTSTITLE', 1, 2, '', 100, 4, 253, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 1644, 'bank_account', 'vtiger_paymentsout', 1, 1, 'bank_account', 'LBL_BANK_ACCOUNT', 1, 2, '', 100, 3, 253, 1, 'V~O', 0, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[81, 1645, 'number', 'vtiger_lettersin', 1, 4, 'number', 'Number', 1, 2, '', 100, 1, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1646, 'title', 'vtiger_lettersin', 1, 1, 'title', 'Title', 1, 2, '', 250, 2, 254, 1, 'V~M', 2, 1, 'BAS', 2, '', 1, '', null, 0, 0],
					[81, 1647, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 4, 254, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1648, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 254, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[81, 1649, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 254, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[81, 1650, 'date_adoption', 'vtiger_lettersin', 1, 5, 'date_adoption', 'Date of adoption', 1, 2, '', 100, 7, 254, 1, 'D~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1651, 'lin_type_ship', 'vtiger_lettersin', 1, 16, 'lin_type_ship', 'Type of shipping', 1, 2, '', 100, 8, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1652, 'lin_type_doc', 'vtiger_lettersin', 1, 33, 'lin_type_doc', 'Type of document', 1, 2, '', 100, 9, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1653, 'lin_status', 'vtiger_lettersin', 1, 15, 'lin_status', 'Status', 1, 2, '', 100, 10, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1655, 'relatedid', 'vtiger_lettersin', 1, 10, 'relatedid', 'Related', 1, 2, '', 100, 12, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1656, 'person_receiving', 'vtiger_lettersin', 1, 53, 'person_receiving', 'Person receiving', 1, 2, '', 100, 13, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1657, 'deadline_reply', 'vtiger_lettersin', 1, 5, 'deadline_reply', 'Deadline for reply', 1, 2, '', 100, 14, 254, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1658, 'cocument_no', 'vtiger_lettersin', 1, 1, 'cocument_no', 'Document no', 1, 2, '', 100, 15, 254, 1, 'V~O~LE~100', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[81, 1659, 'no_internal', 'vtiger_lettersin', 1, 1, 'no_internal', 'No internal', 1, 2, '', 100, 16, 254, 1, 'V~O~LE~100', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1660, 'parentid', 'vtiger_lettersin', 1, 10, 'parentid', 'Parent letters', 1, 2, '', 100, 17, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1661, 'lin_dimensions', 'vtiger_lettersin', 1, 16, 'lin_dimensions', 'Overall dimensions', 1, 2, '', 100, 18, 254, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[81, 1666, 'description', 'vtiger_crmentity', 1, 19, 'description', 'Notes', 1, 2, '', 100, 1, 255, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1667, 'number', 'vtiger_lettersout', 1, 4, 'number', 'Number', 1, 2, '', 100, 1, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1668, 'title', 'vtiger_lettersout', 1, 1, 'title', 'Title', 1, 2, '', 250, 2, 256, 1, 'V~M', 2, 1, 'BAS', 2, '', 1, '', null, 0, 0],
					[82, 1669, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 4, 256, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1670, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 5, 256, 2, 'DT~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[82, 1671, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 6, 256, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[82, 1672, 'date_adoption', 'vtiger_lettersout', 1, 5, 'date_adoption', 'Date of adoption', 1, 2, '', 100, 7, 256, 1, 'D~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1673, 'lout_type_ship', 'vtiger_lettersout', 1, 16, 'lout_type_ship', 'Type of shipping', 1, 2, '', 100, 8, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1674, 'lout_type_doc', 'vtiger_lettersout', 1, 33, 'lout_type_doc', 'Type of document', 1, 2, '', 100, 9, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1675, 'lout_status', 'vtiger_lettersout', 1, 15, 'lout_status', 'Status', 1, 2, '', 100, 10, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1677, 'relatedid', 'vtiger_lettersout', 1, 10, 'relatedid', 'Related', 1, 2, '', 100, 12, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1678, 'person_receiving', 'vtiger_lettersout', 1, 53, 'person_receiving', 'Person receiving', 1, 2, '', 100, 13, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1679, 'deadline_reply', 'vtiger_lettersout', 1, 5, 'deadline_reply', 'Deadline for reply', 1, 2, '', 100, 14, 256, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1680, 'cocument_no', 'vtiger_lettersout', 1, 1, 'cocument_no', 'Document no', 1, 2, '', 100, 15, 256, 1, 'V~O~LE~100', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[82, 1681, 'no_internal', 'vtiger_lettersout', 1, 1, 'no_internal', 'No internal', 1, 2, '', 100, 16, 256, 1, 'V~O~LE~100', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1682, 'parentid', 'vtiger_lettersout', 1, 10, 'parentid', 'Parent letters', 1, 2, '', 100, 17, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1683, 'lout_dimensions', 'vtiger_lettersout', 1, 16, 'lout_dimensions', 'Overall dimensions', 1, 2, '', 100, 18, 256, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[82, 1688, 'description', 'vtiger_crmentity', 1, 19, 'description', 'Notes', 1, 2, '', 100, 1, 257, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1689, 'buildingnumbera', 'vtiger_vendoraddress', 1, 1, 'buildingnumbera', 'LBL_BUILDING_NUMBER', 1, 2, '', 100, 10, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1690, 'buildingnumberb', 'vtiger_vendoraddress', 1, 1, 'buildingnumberb', 'LBL_BUILDING_NUMBER', 1, 2, '', 100, 10, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1691, 'buildingnumberc', 'vtiger_vendoraddress', 1, 1, 'buildingnumberc', 'LBL_BUILDING_NUMBER', 1, 2, '', 100, 10, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1692, 'localnumbera', 'vtiger_vendoraddress', 1, 1, 'localnumbera', 'LBL_LOCAL_NUMBER', 1, 2, '', 100, 11, 44, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1693, 'localnumberb', 'vtiger_vendoraddress', 1, 1, 'localnumberb', 'LBL_LOCAL_NUMBER', 1, 2, '', 100, 11, 43, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1694, 'localnumberc', 'vtiger_vendoraddress', 1, 1, 'localnumberc', 'LBL_LOCAL_NUMBER', 1, 2, '', 100, 11, 179, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 1708, 'subject', 'u_yf_announcement', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 258, 1, 'V~M', 1, 1, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 1709, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 259, 1, 'V~M', 1, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 1710, 'announcement_no', 'u_yf_announcement', 1, 4, 'announcement_no', 'LBL_NUMBER', 1, 2, '', 100, 1, 259, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[83, 1711, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 4, 259, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 1712, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 2, 259, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 1713, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 260, 1, 'V~O', 2, 3, 'BAS', 1, '', 0, '', '0', 0, 0],
					[16, 1714, 'state', 'vtiger_activity', 1, 16, 'state', 'LBL_STATE', 1, 2, 'PLL_OPAQUE', 100, 17, 39, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1715, 'state', 'vtiger_activity', 1, 16, 'state', 'LBL_STATE', 1, 2, 'PLL_OPAQUE', 100, 19, 19, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1716, 'title', 'vtiger_reservations', 1, 2, 'title', 'LBL_TITLE', 1, 2, '', 100, 1, 261, 1, 'V~M', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1717, 'reservations_no', 'vtiger_reservations', 1, 4, 'reservations_no', 'LBL_NUM', 1, 2, '', 100, 2, 261, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1718, 'reservations_status', 'vtiger_reservations', 1, 15, 'reservations_status', 'LBL_STATUS', 1, 2, 'PLL_TO_APPROVAL', 100, 2, 261, 1, 'V~M', 2, 3, 'BAS', 2, '', 1, '', null, 0, 0],
					[84, 1719, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 261, 1, 'V~M', 1, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1720, 'date_start', 'vtiger_reservations', 1, 6, 'date_start', 'LBL_START_DATE', 1, 0, '', 100, 3, 261, 1, 'D~M', 0, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1721, 'time_start', 'vtiger_reservations', 1, 14, 'time_start', 'LBL_TIME_START', 1, 0, '', 100, 5, 261, 1, 'T~M', 1, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1722, 'time_end', 'vtiger_reservations', 1, 14, 'time_end', 'LBL_END_TIME', 1, 0, '', 100, 6, 261, 1, 'T~M~OTH~GE~time_start~LBL_TIME_START', 1, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1723, 'due_date', 'vtiger_reservations', 1, 23, 'due_date', 'LBL_END_DATE', 1, 0, '', 100, 4, 261, 1, 'D~M~OTH~GE~date_start~LBL_START_DATE', 1, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1724, 'sum_time', 'vtiger_reservations', 1, 7, 'sum_time', 'LBL_TOTAL_TIME', 1, 2, '', 100, 8, 261, 10, 'NN~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[84, 1725, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 0, '', 100, 11, 261, 2, 'DT~O', 3, 2, 'BAS', 0, '', 0, '', null, 0, 0],
					[84, 1726, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 0, '', 100, 10, 261, 2, 'DT~O', 3, 2, 'BAS', 0, '', 0, '', null, 0, 0],
					[84, 1727, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'Closed Time', 1, 2, '', 100, 13, 261, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1728, 'was_read', 'vtiger_crmentity', 1, 56, 'was_read', 'Was read', 1, 2, '', 100, 12, 261, 2, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1729, 'type', 'vtiger_reservations', 1, 302, 'type', 'LBL_TYPE', 1, 2, '', 100, 14, 261, 1, 'V~M', 2, 2, 'BAS', 1, '', 0, '7', null, 0, 0],
					[84, 1730, 'relatedida', 'vtiger_reservations', 1, 10, 'relatedida', 'LBL_RELATEDA', 1, 2, '', 100, 1, 262, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1731, 'relatedidb', 'vtiger_reservations', 1, 10, 'relatedidb', 'LBL_RELATEDB', 1, 2, '', 100, 3, 262, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1732, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 263, 1, 'V~O', 2, 9, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1733, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 263, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[84, 1735, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 261, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1736, 'leads_relation', 'vtiger_leaddetails', 1, 16, 'leads_relation', 'LBL_RELATION', 1, 2, 'PLL_B2C', 100, 9, 13, 1, 'V~M', 1, 5, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1737, 'legal_form', 'vtiger_leaddetails', 1, 16, 'legal_form', 'LBL_LEGAL_FORM', 1, 2, '', 100, 5, 191, 1, 'V~O', 1, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1738, 'legal_form', 'vtiger_account', 1, 16, 'legal_form', 'LBL_LEGAL_FORM', 1, 2, '', 100, 7, 194, 1, 'V~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 1740, 'emailoptout', 'vtiger_users', 1, 56, 'emailoptout', 'Approval for email', 1, 0, '', 50, 22, 79, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 1741, 'projectmilestone_priority', 'vtiger_projectmilestone', 1, 15, 'projectmilestone_priority', 'LBL_PRIORITY', 1, 2, '', 100, 9, 101, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[42, 1742, 'estimated_work_time', 'vtiger_projecttask', 1, 7, 'estimated_work_time', 'LBL_ESTIMATED_WORK_TIME', 1, 2, '', 100, 9, 105, 1, 'NN~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 1743, 'projectmilestone_progress', 'vtiger_projectmilestone', 1, 1, 'projectmilestone_progress', 'LBL_PROGRESS', 1, 2, '', 100, 11, 101, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1744, 'jobtitle', 'vtiger_contactdetails', 1, 1, 'jobtitle', 'Job title', 1, 2, '', 100, 9, 4, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[54, 1745, 'date', 'vtiger_ossmailview', 1, 70, 'date', 'Date of receipt', 1, 2, '', 100, 14, 134, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1746, 'decision_maker', 'vtiger_contactdetails', 1, 56, 'decision_maker', 'Decision maker', 1, 2, '', 100, 9, 5, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1747, 'sum_time', 'vtiger_account', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 198, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1756, 'inventorybalance', 'vtiger_account', 1, 7, 'inventorybalance', 'LBL_INVENTORY_BALANCE', 1, 2, '', 100, 7, 198, 10, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1757, 'discount', 'vtiger_account', 1, 9, 'discount', 'LBL_DISCOUNT', 1, 2, '', 100, 8, 198, 1, 'N~O~2~2', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1758, 'creditlimit', 'vtiger_account', 1, 304, 'creditlimit', 'LBL_CREDIT_LIMIT', 1, 2, '', 100, 9, 198, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 1759, 'taxes', 'vtiger_products', 1, 303, 'taxes', 'LBL_TAXES', 1, 2, '', 100, 5, 32, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 1761, 'followup', 'vtiger_activity', 1, 10, 'followup', 'LBL_FOLLOWUP', 1, 2, '', 100, 4, 119, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1762, 'followup', 'vtiger_activity', 1, 10, 'followup', 'LBL_FOLLOWUP', 1, 2, '', 100, 4, 87, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[15, 1763, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 12, 37, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[19, 1764, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 0, '', 100, 10, 46, 1, 'V~M', 0, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[85, 1765, 'subject', 'u_yf_squoteenquiries', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 265, 1, 'V~M', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[85, 1766, 'category', 'u_yf_squoteenquiries', 1, 302, 'category', 'Category', 1, 2, '', 250, 2, 265, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '8', null, 0, 0],
					[85, 1767, 'salesprocessid', 'u_yf_squoteenquiries', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 3, 265, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[85, 1768, 'squoteenquiries_status', 'u_yf_squoteenquiries', 1, 15, 'squoteenquiries_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 5, 265, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[85, 1769, 'accountid', 'u_yf_squoteenquiries', 1, 10, 'accountid', 'Accounts', 1, 2, '', 100, 4, 265, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 1770, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 265, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[85, 1771, 'squoteenquiries_no', 'u_yf_squoteenquiries', 1, 4, 'squoteenquiries_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 265, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[85, 1772, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 265, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 1773, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 266, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 1774, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 266, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 1775, 'response_time', 'u_yf_squoteenquiries', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 267, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 1776, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 268, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[85, 1777, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 2, 268, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[85, 1778, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 3, 268, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 1779, 'subject', 'u_yf_ssalesprocesses', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 269, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[86, 1780, 'category', 'u_yf_ssalesprocesses', 1, 302, 'category', 'Category', 1, 2, '', 250, 6, 269, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '9', null, 0, 0],
					[86, 1781, 'related_to', 'u_yf_ssalesprocesses', 1, 10, 'related_to', 'SINGLE_Accounts', 1, 2, '', 100, 2, 269, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[86, 1782, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 269, 1, 'V~M', 1, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[86, 1783, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 269, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 1784, 'ssalesprocesses_no', 'u_yf_ssalesprocesses', 1, 4, 'ssalesprocesses_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 270, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[86, 1785, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 270, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[86, 1786, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 270, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[86, 1787, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 270, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 1788, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 271, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1789, 'subject', 'u_yf_srequirementscards', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 272, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[87, 1790, 'category', 'u_yf_srequirementscards', 1, 302, 'category', 'Category', 1, 2, '', 250, 5, 272, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '10', null, 0, 0],
					[87, 1791, 'salesprocessid', 'u_yf_srequirementscards', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 2, 272, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[87, 1792, 'quoteenquiryid', 'u_yf_srequirementscards', 1, 10, 'quoteenquiryid', 'SINGLE_SQuoteEnquiries', 1, 2, '', 100, 3, 272, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[87, 1793, 'srequirementscards_status', 'u_yf_srequirementscards', 1, 15, 'srequirementscards_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 6, 272, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[87, 1794, 'accountid', 'u_yf_srequirementscards', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 272, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1795, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 272, 1, 'V~M', 1, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[87, 1796, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 8, 272, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1797, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 273, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1798, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 273, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1799, 'response_time', 'u_yf_srequirementscards', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 274, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 1800, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 275, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[87, 1801, 'srequirementscards_no', 'u_yf_srequirementscards', 1, 4, 'srequirementscards_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 275, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[87, 1802, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 275, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[87, 1803, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 275, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1804, 'subject', 'u_yf_scalculations', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 276, 1, 'V~M', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[88, 1805, 'category', 'u_yf_scalculations', 1, 302, 'category', 'Category', 1, 2, '', 250, 5, 276, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '11', null, 0, 0],
					[88, 1806, 'salesprocessid', 'u_yf_scalculations', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 2, 276, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[88, 1807, 'srequirementscardsid', 'u_yf_scalculations', 1, 10, 'srequirementscardsid', 'SINGLE_SRequirementsCards', 1, 2, '', 100, 3, 276, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[88, 1808, 'scalculations_status', 'u_yf_scalculations', 1, 15, 'scalculations_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 6, 276, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[88, 1809, 'accountid', 'u_yf_scalculations', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 276, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1810, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 276, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[88, 1811, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 8, 276, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1812, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 277, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1813, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 277, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1814, 'response_time', 'u_yf_scalculations', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 278, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 1815, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 279, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[88, 1816, 'scalculations_no', 'u_yf_scalculations', 1, 4, 'scalculations_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 279, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[88, 1817, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 279, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[88, 1818, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 279, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1819, 'subject', 'u_yf_squotes', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 280, 1, 'V~M', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[89, 1820, 'category', 'u_yf_squotes', 1, 302, 'category', 'Category', 1, 2, '', 250, 5, 280, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '12', null, 0, 0],
					[89, 1821, 'salesprocessid', 'u_yf_squotes', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 2, 280, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1822, 'scalculationsid', 'u_yf_squotes', 1, 10, 'scalculationsid', 'SINGLE_SCalculations', 1, 2, '', 100, 3, 280, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1823, 'squotes_status', 'u_yf_squotes', 1, 15, 'squotes_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 6, 280, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1824, 'accountid', 'u_yf_squotes', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 280, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1825, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 280, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1826, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 8, 280, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1827, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 281, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1828, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 281, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1829, 'response_time', 'u_yf_squotes', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 282, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1830, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 283, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1831, 'squotes_no', 'u_yf_squotes', 1, 4, 'squotes_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 283, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[89, 1832, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 283, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[89, 1833, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 283, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1834, 'subject', 'u_yf_ssingleorders', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 284, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[90, 1835, 'category', 'u_yf_ssingleorders', 1, 302, 'category', 'Category', 1, 2, '', 250, 5, 284, 1, 'V~O', 2, 4, 'BAS', 1, '', 1, '13', null, 0, 0],
					[90, 1836, 'salesprocessid', 'u_yf_ssingleorders', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 2, 284, 1, 'V~O', 1, 3, 'BAS', 1, '', 1, '', '0', 0, 0],
					[90, 1837, 'squotesid', 'u_yf_ssingleorders', 1, 10, 'squotesid', 'SINGLE_SQuotes', 1, 2, '', 100, 3, 284, 1, 'V~O', 1, 2, 'BAS', 1, '', 1, '', '0', 0, 0],
					[90, 1838, 'ssingleorders_status', 'u_yf_ssingleorders', 1, 15, 'ssingleorders_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 9, 284, 1, 'V~O', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1839, 'accountid', 'u_yf_ssingleorders', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 284, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1840, 'date_start', 'u_yf_ssingleorders', 1, 5, 'date_start', 'LBL_REALIZATION_START_DATE', 1, 2, '', 100, 6, 284, 1, 'D~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1841, 'date_end', 'u_yf_ssingleorders', 1, 5, 'date_end', 'LBL_REALIZATION_COMPLETION_DATE', 1, 2, '', 100, 7, 284, 1, 'D~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1842, 'duedate', 'u_yf_ssingleorders', 1, 5, 'duedate', 'LBL_ACTUAL_COMPLETION_DATE', 1, 2, '', 100, 8, 284, 1, 'D~O', 2, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1843, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 10, 284, 1, 'V~M', 1, 9, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1844, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 11, 284, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1845, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 285, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1846, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 285, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1847, 'response_time', 'u_yf_ssingleorders', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 286, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1848, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 287, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[90, 1849, 'ssingleorders_no', 'u_yf_ssingleorders', 1, 4, 'ssingleorders_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 287, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[90, 1850, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 287, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[90, 1851, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 287, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1852, 'subject', 'u_yf_srecurringorders', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 288, 1, 'V~M', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[91, 1853, 'category', 'u_yf_srecurringorders', 1, 302, 'category', 'Category', 1, 2, '', 250, 5, 288, 1, 'V~O', 2, 4, 'BAS', 1, '', 1, '14', null, 0, 0],
					[91, 1854, 'salesprocessid', 'u_yf_srecurringorders', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 2, 288, 1, 'V~M', 2, 3, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1855, 'squotesid', 'u_yf_srecurringorders', 1, 10, 'squotesid', 'SINGLE_SQuotes', 1, 2, '', 100, 3, 288, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1856, 'srecurringorders_status', 'u_yf_srecurringorders', 1, 15, 'srecurringorders_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 9, 288, 10, 'V~O', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1857, 'accountid', 'u_yf_srecurringorders', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 288, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1858, 'date_start', 'u_yf_srecurringorders', 1, 5, 'date_start', 'LBL_REALIZATION_START_DATE', 1, 2, '', 100, 6, 288, 1, 'D~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1859, 'date_end', 'u_yf_srecurringorders', 1, 5, 'date_end', 'LBL_REALIZATION_COMPLETION_DATE', 1, 2, '', 100, 7, 288, 1, 'D~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1860, 'duedate', 'u_yf_srecurringorders', 1, 5, 'duedate', 'LBL_ACTUAL_COMPLETION_DATE', 1, 2, '', 100, 8, 288, 1, 'D~O', 2, 7, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1861, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 10, 288, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1862, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 11, 288, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1863, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 289, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1864, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 289, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1865, 'response_time', 'u_yf_srecurringorders', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 290, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1866, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 291, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1867, 'srecurringorders_no', 'u_yf_srecurringorders', 1, 4, 'srecurringorders_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 291, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[91, 1868, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 291, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[91, 1869, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 291, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1870, 'target_module', 'u_yf_recurring_info', 1, 301, 'target_module', 'LBL_TARGET_MODULE', 1, 0, '', 100, 1, 292, 1, 'V~M', 2, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1871, 'start_period', 'u_yf_recurring_info', 1, 5, 'start_period', 'LBL_START_PERIOD', 1, 0, '', 100, 3, 292, 1, 'D~M', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1872, 'end_period', 'u_yf_recurring_info', 1, 5, 'end_period', 'LBL_END_PERIOD', 1, 0, '', 100, 4, 292, 1, 'D~M~OTH~G~start_period~LBL_START_PERIOD', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1873, 'recurring_frequency', 'u_yf_recurring_info', 1, 16, 'recurring_frequency', 'LBL_FREQUENCY', 1, 0, '', 100, 2, 292, 1, 'V~M', 2, 8, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1874, 'date_start', 'u_yf_recurring_info', 1, 5, 'date_start', 'LBL_REALIZATION_START_DATE', 1, 0, '', 100, 5, 292, 10, 'D~0', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[91, 1875, 'date_end', 'u_yf_recurring_info', 1, 5, 'date_end', 'LBL_REALIZATION_COMPLETION_DATE', 1, 0, '', 100, 6, 292, 10, 'D~0', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[89, 1876, 'addresslevel1a', 'u_yf_squotes_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1877, 'addresslevel2a', 'u_yf_squotes_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1878, 'addresslevel3a', 'u_yf_squotes_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1879, 'addresslevel4a', 'u_yf_squotes_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1880, 'addresslevel5a', 'u_yf_squotes_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1881, 'addresslevel6a', 'u_yf_squotes_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1882, 'addresslevel7a', 'u_yf_squotes_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1883, 'addresslevel8a', 'u_yf_squotes_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1884, 'addresslevel1c', 'u_yf_squotes_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 10, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1885, 'addresslevel2c', 'u_yf_squotes_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 9, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1886, 'addresslevel3c', 'u_yf_squotes_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 8, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1887, 'addresslevel4c', 'u_yf_squotes_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 7, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1888, 'addresslevel5c', 'u_yf_squotes_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 6, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1889, 'addresslevel6c', 'u_yf_squotes_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 5, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1890, 'addresslevel7c', 'u_yf_squotes_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 4, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1891, 'addresslevel8c', 'u_yf_squotes_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 3, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1892, 'buildingnumbera', 'u_yf_squotes_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1893, 'localnumbera', 'u_yf_squotes_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1894, 'buildingnumberc', 'u_yf_squotes_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 1, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1895, 'localnumberc', 'u_yf_squotes_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1896, 'poboxa', 'u_yf_squotes_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 293, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1897, 'poboxc', 'u_yf_squotes_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 11, 294, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1898, 'addresslevel1a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1899, 'addresslevel2a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1900, 'addresslevel3a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1901, 'addresslevel4a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1902, 'addresslevel5a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1903, 'addresslevel6a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1904, 'addresslevel7a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1905, 'addresslevel8a', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1906, 'addresslevel1c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 10, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1907, 'addresslevel2c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 9, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1908, 'addresslevel3c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 8, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1909, 'addresslevel4c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 7, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1910, 'addresslevel5c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 6, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1911, 'addresslevel6c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 5, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1912, 'addresslevel7c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 4, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1913, 'addresslevel8c', 'u_yf_ssingleorders_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 3, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1914, 'buildingnumbera', 'u_yf_ssingleorders_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1915, 'localnumbera', 'u_yf_ssingleorders_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1916, 'buildingnumberc', 'u_yf_ssingleorders_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 1, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1917, 'localnumberc', 'u_yf_ssingleorders_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1918, 'poboxa', 'u_yf_ssingleorders_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 295, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1919, 'poboxc', 'u_yf_ssingleorders_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 11, 296, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1920, 'addresslevel1a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1921, 'addresslevel2a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1922, 'addresslevel3a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1923, 'addresslevel4a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1924, 'addresslevel5a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1925, 'addresslevel6a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1926, 'addresslevel7a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1927, 'addresslevel8a', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1928, 'addresslevel1c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 10, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1929, 'addresslevel2c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 9, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1930, 'addresslevel3c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 8, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1931, 'addresslevel4c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 7, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1932, 'addresslevel5c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 6, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1933, 'addresslevel6c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 5, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1934, 'addresslevel7c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 4, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1935, 'addresslevel8c', 'u_yf_srecurringorders_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 3, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1936, 'buildingnumbera', 'u_yf_srecurringorders_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1937, 'localnumbera', 'u_yf_srecurringorders_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1938, 'buildingnumberc', 'u_yf_srecurringorders_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 1, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1939, 'localnumberc', 'u_yf_srecurringorders_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1940, 'poboxa', 'u_yf_srecurringorders_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 297, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1941, 'poboxc', 'u_yf_srecurringorders_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 11, 298, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 1942, 'company', 'u_yf_squotes', 1, 1, 'company', 'LBL_COMPANY_NAME', 1, 2, '', 100, 9, 280, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 1943, 'company', 'u_yf_ssingleorders', 1, 1, 'company', 'LBL_COMPANY_NAME', 1, 2, '', 100, 12, 284, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 1944, 'company', 'u_yf_srecurringorders', 1, 1, 'company', 'LBL_COMPANY_NAME', 1, 2, '', 100, 12, 288, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 1945, 'products', 'vtiger_account', 1, 305, 'products', 'Products', 1, 2, '', 100, 10, 196, 5, 'C~O', 1, null, 'BAS', 1, '', 0, '{"module":"Products","field":"173","filterField":"-","filterValue":null}', null, 0, 0],
					[6, 1946, 'services', 'vtiger_account', 1, 305, 'services', 'Services', 1, 2, '', 100, 11, 196, 5, 'C~O', 1, null, 'BAS', 1, '', 0, '{"module":"Services","field":"560","filterField":"-","filterValue":null}', null, 0, 0],
					[37, 1947, 'pscategory', 'vtiger_assets', 1, 302, 'pscategory', 'Category', 1, 2, '', 100, 17, 95, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '15', null, 0, 0],
					[92, 1948, 'subject', 'u_yf_partners', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 299, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[92, 1949, 'vat_id', 'u_yf_partners', 1, 1, 'vat_id', 'LBL_VAT_ID', 1, 2, '', 100, 2, 299, 1, 'V~O', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[92, 1950, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 299, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[92, 1951, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 299, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1952, 'partners_no', 'u_yf_partners', 1, 4, 'partners_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 300, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[92, 1953, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 300, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[92, 1954, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 300, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[92, 1955, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 300, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1956, 'addresslevel1a', 'u_yf_partners_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1957, 'addresslevel2a', 'u_yf_partners_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1958, 'addresslevel3a', 'u_yf_partners_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1959, 'addresslevel4a', 'u_yf_partners_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1960, 'addresslevel5a', 'u_yf_partners_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1961, 'addresslevel6a', 'u_yf_partners_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1962, 'addresslevel7a', 'u_yf_partners_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1963, 'addresslevel8a', 'u_yf_partners_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1964, 'buildingnumbera', 'u_yf_partners_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1965, 'localnumbera', 'u_yf_partners_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1966, 'poboxa', 'u_yf_partners_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 301, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 1967, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 302, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1968, 'subject', 'u_yf_competition', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 303, 1, 'V~M', 1, 1, 'BAS', 0, '', 1, '', null, 0, 0],
					[93, 1969, 'vat_id', 'u_yf_competition', 1, 1, 'vat_id', 'LBL_VAT_ID', 1, 2, '', 100, 2, 303, 1, 'V~O', 1, null, 'BAS', 0, '', 1, '', null, 0, 0],
					[93, 1970, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 303, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[93, 1971, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 303, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1972, 'competition_no', 'u_yf_competition', 1, 4, 'competition_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 304, 1, 'V~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[93, 1973, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 304, 2, 'DT~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[93, 1974, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 304, 2, 'DT~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[93, 1975, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 304, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1976, 'addresslevel1a', 'u_yf_competition_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 10, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1977, 'addresslevel2a', 'u_yf_competition_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 9, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1978, 'addresslevel3a', 'u_yf_competition_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 8, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1979, 'addresslevel4a', 'u_yf_competition_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 7, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1980, 'addresslevel5a', 'u_yf_competition_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 6, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1981, 'addresslevel6a', 'u_yf_competition_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 5, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1982, 'addresslevel7a', 'u_yf_competition_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 4, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1983, 'addresslevel8a', 'u_yf_competition_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 3, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1984, 'buildingnumbera', 'u_yf_competition_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 1, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1985, 'localnumbera', 'u_yf_competition_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1986, 'poboxa', 'u_yf_competition_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 11, 305, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 1987, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 306, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[59, 1988, 'ssalesprocessesid', 'vtiger_outsourcedproducts', 1, 10, 'ssalesprocessesid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 31, 144, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 1989, 'ssalesprocessesid', 'vtiger_assets', 1, 10, 'ssalesprocessesid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 18, 95, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[57, 1990, 'ssalesprocessesid', 'vtiger_ossoutsourcedservices', 1, 10, 'ssalesprocessesid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 16, 138, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 1991, 'ssalesprocessesid', 'vtiger_osssoldservices', 1, 10, 'ssalesprocessesid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 10, 141, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 1992, 'subprocess', 'vtiger_activity', 1, 68, 'subprocess', 'FL_SUB_PROCESS', 1, 0, '', 100, 3, 87, 1, 'I~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[16, 1993, 'subprocess', 'vtiger_activity', 1, 68, 'subprocess', 'FL_SUB_PROCESS', 1, 0, '', 100, 3, 119, 1, 'I~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[51, 1994, 'process', 'vtiger_osstimecontrol', 1, 66, 'process', 'FL_PROCESS', 1, 2, '', 100, 11, 129, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1995, 'link', 'vtiger_osstimecontrol', 1, 67, 'link', 'FL_RELATION', 1, 2, '', 100, 12, 129, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[51, 1996, 'subprocess', 'vtiger_osstimecontrol', 1, 68, 'subprocess', 'FL_SUB_PROCESS', 1, 2, '', 100, 13, 129, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 1997, 'sum_time', 'vtiger_contactdetails', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 10, 5, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 1998, 'sum_time', 'vtiger_leaddetails', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 10, 14, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 1999, 'sum_time', 'vtiger_vendor', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 19, 42, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 2000, 'sum_time', 'u_yf_partners', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 300, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 2001, 'sum_time', 'u_yf_competition', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 304, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 2002, 'sum_time', 'vtiger_ossemployees', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 15, 151, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2003, 'sum_time', 'u_yf_ssalesprocesses', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 270, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[26, 2004, 'sum_time', 'vtiger_campaign', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 1, 74, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[41, 2005, 'sum_time', 'vtiger_projectmilestone', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 5, 102, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[85, 2006, 'sum_time', 'u_yf_squoteenquiries', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 267, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[87, 2007, 'sum_time', 'u_yf_srequirementscards', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 274, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 2008, 'sum_time', 'u_yf_scalculations', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 278, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2009, 'sum_time', 'u_yf_squotes', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 282, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2010, 'sum_time', 'u_yf_ssingleorders', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 286, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[91, 2011, 'sum_time', 'u_yf_srecurringorders', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 290, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 2012, 'email', 'u_yf_partners', 1, 13, 'email', 'Email', 1, 2, '', 100, 5, 299, 1, 'E~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 2013, 'email', 'u_yf_competition', 1, 13, 'email', 'Email', 1, 2, '', 100, 5, 303, 1, 'E~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 2014, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 11, 5, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 2015, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 12, 196, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 2016, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 11, 14, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 2017, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 7, 27, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 2018, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 20, 42, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 2019, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 17, 74, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 2020, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 2, 90, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[43, 2021, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 8, 108, 10, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 2022, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 16, 151, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[85, 2023, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 4, 268, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[86, 2024, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 6, 270, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[87, 2025, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 275, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[88, 2026, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 279, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[89, 2027, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 283, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[90, 2028, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 287, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[91, 2029, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 291, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[92, 2030, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 6, 300, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[93, 2031, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 6, 304, 10, 'NN~O', 1, null, 'BAS', 0, '', 0, '', null, 0, 0],
					[94, 2032, 'subject', 'u_yf_fbookkeeping', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 307, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2033, 'number', 'u_yf_fbookkeeping', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 308, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2034, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 307, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2035, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 308, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2036, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 308, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2037, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 309, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2038, 'related_to', 'u_yf_fbookkeeping', 1, 10, 'related_to', 'SINGLE_Accounts', 1, 2, '', 100, 2, 307, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[94, 2039, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 307, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2040, 'subject', 'u_yf_finvoice', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 310, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[95, 2041, 'paymentdate', 'u_yf_finvoice', 1, 5, 'paymentdate', 'FL_PAYMENT_DATE', 1, 2, '', 100, 6, 310, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2042, 'saledate', 'u_yf_finvoice', 1, 5, 'saledate', 'FL_SALE_DATE', 1, 2, '', 100, 7, 310, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2043, 'accountid', 'u_yf_finvoice', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 9, 310, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2044, 'finvoice_formpayment', 'u_yf_finvoice', 1, 15, 'finvoice_formpayment', 'FL_FORM_PAYMENT', 1, 2, '', 100, 10, 310, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2046, 'number', 'u_yf_finvoice', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 311, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2047, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 2, 311, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2048, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 3, 311, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2049, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 4, 311, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2050, 'poboxa', 'u_yf_finvoice_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 4, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2051, 'localnumbera', 'u_yf_finvoice_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2052, 'buildingnumbera', 'u_yf_finvoice_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 3, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2053, 'addresslevel8a', 'u_yf_finvoice_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2054, 'addresslevel7a', 'u_yf_finvoice_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2055, 'addresslevel6a', 'u_yf_finvoice_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2056, 'addresslevel5a', 'u_yf_finvoice_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 7, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2057, 'addresslevel4a', 'u_yf_finvoice_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2058, 'addresslevel3a', 'u_yf_finvoice_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 9, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2059, 'addresslevel2a', 'u_yf_finvoice_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 10, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2060, 'addresslevel1a', 'u_yf_finvoice_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 11, 312, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2061, 'addresslevel1c', 'u_yf_finvoice_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 11, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2062, 'addresslevel2c', 'u_yf_finvoice_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 10, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2063, 'addresslevel3c', 'u_yf_finvoice_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 9, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2064, 'addresslevel4c', 'u_yf_finvoice_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 8, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2065, 'addresslevel5c', 'u_yf_finvoice_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 7, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2066, 'addresslevel6c', 'u_yf_finvoice_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 6, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2067, 'addresslevel7c', 'u_yf_finvoice_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 5, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2068, 'addresslevel8c', 'u_yf_finvoice_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 1, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2069, 'buildingnumberc', 'u_yf_finvoice_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 3, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2070, 'localnumberc', 'u_yf_finvoice_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2071, 'poboxc', 'u_yf_finvoice_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 4, 313, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2072, 'sum_total', 'u_yf_finvoice', 1, 7, 'sum_total', 'FL_TOTAL', 1, 2, '', 100, 5, 311, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2073, 'sum_gross', 'u_yf_finvoice', 1, 7, 'sum_gross', 'FL_GROSS', 1, 2, '', 100, 6, 311, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2074, 'finvoice_status', 'u_yf_finvoice', 1, 15, 'finvoice_status', 'FL_STATUS', 1, 2, '', 100, 12, 310, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2075, 'subject', 'u_yf_knowledgebase', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 314, 1, 'V~M', 1, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[96, 2076, 'number', 'u_yf_knowledgebase', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 4, 314, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[96, 2077, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 314, 1, 'V~M', 1, 5, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2078, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 5, 314, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2079, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 6, 314, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2080, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'smcreatorid', 'Created By', 1, 2, '', 100, 7, 314, 2, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2081, 'content', 'u_yf_knowledgebase', 1, 300, 'content', 'FL_CONTENT', 1, 2, '', 100, 1, 315, 1, 'V~O', 2, 6, 'BAS', 1, '', 0, '', '0', 0, 0],
					[96, 2082, 'category', 'u_yf_knowledgebase', 1, 302, 'category', 'FL_CATEGORY', 1, 2, '', 100, 2, 314, 1, 'V~M', 2, 3, 'BAS', 1, '', 1, '16', '0', 0, 0],
					[97, 2083, 'subject', 'u_yf_istorages', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 316, 1, 'V~M', 1, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[97, 2084, 'parentid', 'u_yf_istorages', 1, 10, 'parentid', 'FL_PARENT_STORAGE', 1, 2, '', 100, 2, 316, 1, 'V~O', 2, 2, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2085, 'storage_status', 'u_yf_istorages', 1, 15, 'storage_status', 'FL_STATUS', 1, 2, '', 100, 3, 316, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[97, 2086, 'storage_type', 'u_yf_istorages', 1, 16, 'storage_type', 'FL_TYPE', 1, 2, 'PLL_INTERNAL', 100, 4, 316, 10, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[97, 2087, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 5, 316, 1, 'V~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2088, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 316, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2089, 'number', 'u_yf_istorages', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 317, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2090, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 317, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2091, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 317, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2092, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 318, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2093, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 343, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2094, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 319, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2095, 'estimated', 'u_yf_ssalesprocesses', 1, 71, 'estimated', 'FL_ESTIMATED', 1, 2, '', 100, 1, 320, 1, 'N~M', 1, 7, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2096, 'actual_sale', 'u_yf_ssalesprocesses', 1, 71, 'actual_sale', 'FL_ACTUAL_SALE', 1, 2, '', 100, 2, 320, 10, 'N~O', 2, 9, 'BAS', 1, '', 0, '', '0', 0, 0],
					[86, 2097, 'estimated_date', 'u_yf_ssalesprocesses', 1, 5, 'estimated_date', 'FL_ESTIMATED_DATE', 1, 2, '', 100, 12, 269, 1, 'D~M', 2, 6, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2098, 'actual_date', 'u_yf_ssalesprocesses', 1, 5, 'actual_date', 'FL_ACTUAL_DATE', 1, 2, '', 100, 13, 269, 10, 'D~O', 2, 8, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2099, 'probability', 'u_yf_ssalesprocesses', 1, 9, 'probability', 'FL_PROBABILITY', 1, 2, '', 100, 9, 269, 1, 'N~O~2~2', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2100, 'ssalesprocesses_source', 'u_yf_ssalesprocesses', 1, 16, 'ssalesprocesses_source', 'FL_SOURCE', 1, 2, '', 100, 8, 269, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2101, 'ssalesprocesses_type', 'u_yf_ssalesprocesses', 1, 16, 'ssalesprocesses_type', 'FL_TYPE', 1, 2, 'PLL_NEW_SALES', 100, 7, 269, 1, 'V~M', 1, 4, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2102, 'ssalesprocesses_status', 'u_yf_ssalesprocesses', 1, 15, 'ssalesprocesses_status', 'FL_STATUS', 1, 2, '', 100, 10, 269, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 2103, 'sum_total', 'u_yf_scalculations', 1, 7, 'sum_total', 'FL_TOTAL_PRICE', 1, 2, '', 100, 3, 278, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 2104, 'sum_marginp', 'u_yf_scalculations', 1, 7, 'sum_marginp', 'FL_MARGINP', 1, 2, '', 100, 4, 278, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[88, 2105, 'sum_margin', 'u_yf_scalculations', 1, 7, 'sum_margin', 'FL_MARGIN', 1, 2, '', 100, 5, 278, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2106, 'sum_total', 'u_yf_squotes', 1, 7, 'sum_total', 'FL_TOTAL_PRICE', 1, 2, '', 100, 3, 282, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2107, 'sum_marginp', 'u_yf_squotes', 1, 7, 'sum_marginp', 'FL_MARGINP', 1, 2, '', 100, 4, 282, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2108, 'sum_margin', 'u_yf_squotes', 1, 7, 'sum_margin', 'FL_MARGIN', 1, 2, '', 100, 5, 282, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2109, 'sum_gross', 'u_yf_squotes', 1, 7, 'sum_gross', 'FL_SUM_GROSS', 1, 2, '', 100, 6, 282, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[89, 2110, 'sum_discount', 'u_yf_squotes', 1, 7, 'sum_discount', 'FL_SUM_DISCOUNT', 1, 2, '', 100, 7, 282, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2111, 'sum_total', 'u_yf_ssingleorders', 1, 7, 'sum_total', 'FL_TOTAL_PRICE', 1, 2, '', 100, 3, 286, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2112, 'sum_marginp', 'u_yf_ssingleorders', 1, 7, 'sum_marginp', 'FL_MARGINP', 1, 2, '', 100, 4, 286, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2113, 'sum_margin', 'u_yf_ssingleorders', 1, 7, 'sum_margin', 'FL_MARGIN', 1, 2, '', 100, 5, 286, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2114, 'sum_gross', 'u_yf_ssingleorders', 1, 7, 'sum_gross', 'FL_SUM_GROSS', 1, 2, '', 100, 6, 286, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2115, 'sum_discount', 'u_yf_ssingleorders', 1, 7, 'sum_discount', 'FL_SUM_DISCOUNT', 1, 2, '', 100, 7, 286, 3, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2116, 'subject', 'u_yf_igrn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 321, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2117, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 321, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2118, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 321, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2119, 'number', 'u_yf_igrn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 322, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2120, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 322, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2121, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 322, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2122, 'subject', 'u_yf_finvoiceproforma', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 323, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[99, 2123, 'paymentdate', 'u_yf_finvoiceproforma', 1, 5, 'paymentdate', 'FL_PAYMENT_DATE', 1, 2, '', 100, 2, 323, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2124, 'saledate', 'u_yf_finvoiceproforma', 1, 5, 'saledate', 'FL_SALE_DATE', 1, 2, '', 100, 3, 323, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2125, 'accountid', 'u_yf_finvoiceproforma', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 4, 323, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[99, 2126, 'finvoiceproforma_formpayment', 'u_yf_finvoiceproforma', 1, 15, 'finvoiceproforma_formpayment', 'FL_FORM_PAYMENT', 1, 2, '', 100, 5, 323, 1, 'T~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2127, 'finvoiceproforma_status', 'u_yf_finvoiceproforma', 1, 15, 'finvoiceproforma_status', 'FL_STATUS', 1, 2, '', 100, 6, 323, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2128, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 324, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2129, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 4, 324, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2130, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 5, 324, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2131, 'number', 'u_yf_finvoiceproforma', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 324, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2132, 'sum_total', 'u_yf_finvoiceproforma', 1, 7, 'sum_total', 'FL_TOTAL', 1, 2, '', 100, 6, 324, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2133, 'sum_gross', 'u_yf_finvoiceproforma', 1, 7, 'sum_gross', 'FL_GROSS', 1, 2, '', 100, 7, 324, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2134, 'addresslevel1c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 11, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2135, 'addresslevel2c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 10, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2136, 'addresslevel3c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 9, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2137, 'addresslevel4c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2138, 'addresslevel5c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 7, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2139, 'addresslevel6c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2140, 'addresslevel7c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2141, 'addresslevel8c', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2142, 'poboxc', 'u_yf_finvoiceproforma_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 4, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2143, 'localnumberc', 'u_yf_finvoiceproforma_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2144, 'buildingnumberc', 'u_yf_finvoiceproforma_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 3, 325, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2145, 'addresslevel8a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2146, 'addresslevel1a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 11, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2147, 'addresslevel2a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 10, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2148, 'addresslevel3a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 9, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2149, 'addresslevel4a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2150, 'addresslevel5a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 7, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2151, 'addresslevel6a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2152, 'addresslevel7a', 'u_yf_finvoiceproforma_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2153, 'buildingnumbera', 'u_yf_finvoiceproforma_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 3, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2154, 'localnumbera', 'u_yf_finvoiceproforma_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[99, 2155, 'poboxa', 'u_yf_finvoiceproforma_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 4, 326, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2156, 'subject', 'u_yf_igdn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 327, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[100, 2157, 'storageid', 'u_yf_igdn', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 327, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[100, 2158, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 327, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2159, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 327, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2160, 'number', 'u_yf_igdn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 328, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2161, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 328, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2162, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 328, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2163, 'subject', 'u_yf_iidn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 329, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[101, 2164, 'storageid', 'u_yf_iidn', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 329, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[101, 2165, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 5, 329, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2166, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 329, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2167, 'number', 'u_yf_iidn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 330, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2168, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 330, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2169, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 330, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2170, 'storageid', 'u_yf_igrn', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 321, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[102, 2171, 'subject', 'u_yf_igin', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 331, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[102, 2172, 'storageid', 'u_yf_igin', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 331, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2173, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 5, 331, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2174, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 331, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2175, 'number', 'u_yf_igin', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 332, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2176, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 332, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2177, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 332, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2178, 'igrn_status', 'u_yf_igrn', 1, 15, 'igrn_status', 'FL_STATUS', 1, 2, '', 100, 4, 321, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[100, 2179, 'igdn_status', 'u_yf_igdn', 1, 15, 'igdn_status', 'FL_STATUS', 1, 2, '', 100, 4, 327, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[101, 2180, 'iidn_status', 'u_yf_iidn', 1, 15, 'iidn_status', 'FL_STATUS', 1, 2, '', 100, 3, 329, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[102, 2181, 'igin_status', 'u_yf_igin', 1, 15, 'igin_status', 'FL_STATUS', 1, 2, '', 100, 3, 331, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[103, 2182, 'subject', 'u_yf_ipreorder', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 333, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2183, 'storageid', 'u_yf_ipreorder', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 333, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[103, 2184, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 333, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2185, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 333, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2186, 'number', 'u_yf_ipreorder', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 334, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2187, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 334, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2188, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 334, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2189, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 335, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2190, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 336, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[98, 2191, 'vendorid', 'u_yf_igrn', 1, 10, 'vendorid', 'FL_VENDOR', 1, 2, '', 100, 3, 321, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[98, 2192, 'acceptance_date', 'u_yf_igrn', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 321, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[100, 2193, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 337, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2194, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 338, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2195, 'accountid', 'u_yf_igdn', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 3, 327, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[100, 2196, 'acceptance_date', 'u_yf_igdn', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 327, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[101, 2197, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 339, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2198, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 340, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[101, 2199, 'acceptance_date', 'u_yf_iidn', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 4, 329, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[102, 2200, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 341, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2201, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 342, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[102, 2202, 'acceptance_date', 'u_yf_igin', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 4, 331, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[103, 2203, 'ipreorder_status', 'u_yf_ipreorder', 1, 15, 'ipreorder_status', 'FL_STATUS', 1, 2, '', 100, 4, 333, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[103, 2204, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 344, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2205, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 345, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[103, 2206, 'accountid', 'u_yf_ipreorder', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 3, 333, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[103, 2207, 'acceptance_date', 'u_yf_ipreorder', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 333, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[14, 2208, 'ean', 'vtiger_products', 1, 1, 'ean', 'FL_EAN_13', 1, 2, '', 100, 28, 31, 1, 'V~O', 2, 3, 'BAS', 1, '', 1, '9999999999999', null, 0, 0],
					[98, 2209, 'sum_total', 'u_yf_igrn', 1, 7, 'sum_total', 'LBL_TOTAL_PRICE', 1, 2, '', 100, 4, 322, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2210, 'knowledgebase_view', 'u_yf_knowledgebase', 1, 16, 'knowledgebase_view', 'FL_VIEWS', 1, 2, '', 100, 8, 314, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', '0', 0, 0],
					[97, 2211, 'addresslevel1a', 'u_yf_istorages_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 8, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2212, 'addresslevel2a', 'u_yf_istorages_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 7, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2213, 'addresslevel3a', 'u_yf_istorages_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 6, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2214, 'addresslevel4a', 'u_yf_istorages_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 5, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2215, 'addresslevel5a', 'u_yf_istorages_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 4, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2216, 'addresslevel6a', 'u_yf_istorages_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 3, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2217, 'addresslevel7a', 'u_yf_istorages_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 2, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2218, 'addresslevel8a', 'u_yf_istorages_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2219, 'buildingnumbera', 'u_yf_istorages_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 10, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2220, 'localnumbera', 'u_yf_istorages_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 11, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[97, 2221, 'poboxa', 'u_yf_istorages_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 9, 346, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2222, 'subject', 'u_yf_istdn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 347, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2223, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 347, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2224, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 347, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2225, 'storageid', 'u_yf_istdn', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 347, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[104, 2226, 'istdn_status', 'u_yf_istdn', 1, 15, 'istdn_status', 'FL_STATUS', 1, 2, '', 100, 4, 347, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[104, 2227, 'accountid', 'u_yf_istdn', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 3, 347, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[104, 2228, 'acceptance_date', 'u_yf_istdn', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 347, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[104, 2229, 'number', 'u_yf_istdn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 348, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2230, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 348, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2231, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 348, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2232, 'sum_total', 'u_yf_istdn', 1, 7, 'sum_total', 'LBL_TOTAL_PRICE', 1, 2, '', 100, 4, 348, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2233, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 349, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2234, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 350, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2235, 'subject', 'u_yf_istn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 351, 1, 'V~M', 1, 1, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2236, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 351, 1, 'V~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2237, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 5, 351, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2238, 'istn_status', 'u_yf_istn', 1, 15, 'istn_status', 'FL_STATUS', 1, 2, '', 100, 2, 351, 1, 'V~M', 1, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[105, 2239, 'estimated_date', 'u_yf_istn', 1, 5, 'estimated_date', 'FL_ESTIMATED_DATE', 1, 2, '', 100, 4, 351, 1, 'D~O', 2, 4, 'BAS', 1, '', 1, '', null, 0, 0],
					[105, 2240, 'number', 'u_yf_istn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 352, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2241, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 352, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2242, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 352, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2243, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 353, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2244, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 354, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[105, 2245, 'istn_type', 'u_yf_istn', 1, 16, 'istn_type', 'FL_TYPE', 1, 2, '', 100, 3, 351, 1, 'V~O', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[106, 2246, 'subject', 'u_yf_istrn', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 355, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2247, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 355, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2248, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 355, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2249, 'storageid', 'u_yf_istrn', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 355, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[106, 2250, 'istrn_status', 'u_yf_istrn', 1, 15, 'istrn_status', 'FL_STATUS', 1, 2, '', 100, 4, 355, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[106, 2251, 'vendorid', 'u_yf_istrn', 1, 10, 'vendorid', 'FL_VENDOR', 1, 2, '', 100, 3, 355, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[106, 2252, 'acceptance_date', 'u_yf_istrn', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 355, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[106, 2253, 'number', 'u_yf_istrn', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 356, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2254, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 356, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2255, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 356, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2256, 'sum_total', 'u_yf_istrn', 1, 7, 'sum_total', 'LBL_TOTAL_PRICE', 1, 2, '', 100, 4, 356, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2257, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 357, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2258, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 358, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2259, 'process', 'u_yf_istdn', 1, 10, 'process', 'FL_PROCESS', 1, 2, '', 100, 8, 347, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2260, 'process', 'u_yf_istrn', 1, 10, 'process', 'FL_PROCESS', 1, 2, '', 100, 8, 355, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[104, 2261, 'subprocess', 'u_yf_istdn', 1, 10, 'subprocess', 'FL_SUB_PROCESS', 1, 2, '', 100, 9, 347, 10, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[106, 2262, 'subprocess', 'u_yf_istrn', 1, 10, 'subprocess', 'FL_SUB_PROCESS', 1, 2, '', 100, 9, 355, 1, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2263, 'knowledgebase_status', 'u_yf_knowledgebase', 1, 15, 'knowledgebase_status', 'FL_STATUS', 1, 2, '', 100, 9, 314, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[96, 2264, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, null, 100, 1, 359, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2265, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, null, 100, 1, 360, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[96, 2266, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, null, 100, 10, 314, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 2267, 'osssoldservices_renew', 'vtiger_osssoldservices', 1, 15, 'osssoldservices_renew', 'FL_RENEWAL', 1, 0, '', 100, 11, 141, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 2268, 'assets_renew', 'vtiger_assets', 1, 15, 'assets_renew', 'FL_RENEWAL', 1, 0, '', 100, 19, 95, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 2269, 'subunit', 'vtiger_products', 1, 16, 'subunit', 'FL_SUBUNIT', 1, 2, '', 100, 7, 33, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2270, 'subject', 'u_yf_fcorectinginvoice', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 361, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[107, 2271, 'paymentdate', 'u_yf_fcorectinginvoice', 1, 5, 'paymentdate', 'FL_PAYMENT_DATE', 1, 2, '', 100, 6, 361, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2272, 'saledate', 'u_yf_fcorectinginvoice', 1, 5, 'saledate', 'FL_SALE_DATE', 1, 2, '', 100, 7, 361, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2273, 'accountid', 'u_yf_fcorectinginvoice', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 9, 361, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2274, 'fcorectinginvoice_formpayment', 'u_yf_fcorectinginvoice', 1, 15, 'fcorectinginvoice_formpayment', 'FL_FORM_PAYMENT', 1, 2, '', 100, 10, 361, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2275, 'fcorectinginvoice_status', 'u_yf_fcorectinginvoice', 1, 15, 'fcorectinginvoice_status', 'FL_STATUS', 1, 2, '', 100, 12, 361, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2276, 'number', 'u_yf_fcorectinginvoice', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 362, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2277, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 2, 362, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2278, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 3, 362, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2279, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 4, 362, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2280, 'sum_total', 'u_yf_fcorectinginvoice', 1, 7, 'sum_total', 'FL_TOTAL', 1, 2, '', 100, 5, 362, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2281, 'sum_gross', 'u_yf_fcorectinginvoice', 1, 7, 'sum_gross', 'FL_GROSS', 1, 2, '', 100, 6, 362, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2282, 'poboxa', 'u_yf_fcorectinginvoice_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 4, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2283, 'localnumbera', 'u_yf_fcorectinginvoice_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2284, 'buildingnumbera', 'u_yf_fcorectinginvoice_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 3, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2285, 'addresslevel8a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2286, 'addresslevel7a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2287, 'addresslevel6a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2288, 'addresslevel5a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 7, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2289, 'addresslevel4a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2290, 'addresslevel3a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 9, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2291, 'addresslevel2a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 10, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2292, 'addresslevel1a', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 11, 363, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2293, 'addresslevel1c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 11, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2294, 'addresslevel2c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 10, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2295, 'addresslevel3c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 9, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2296, 'addresslevel4c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 8, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2297, 'addresslevel5c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 7, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2298, 'addresslevel6c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 6, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2299, 'addresslevel7c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 5, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2300, 'addresslevel8c', 'u_yf_fcorectinginvoice_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 1, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2301, 'buildingnumberc', 'u_yf_fcorectinginvoice_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 3, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2302, 'localnumberc', 'u_yf_fcorectinginvoice_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2303, 'poboxc', 'u_yf_fcorectinginvoice_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 4, 364, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[107, 2304, 'finvoiceid', 'u_yf_fcorectinginvoice', 1, 10, 'finvoiceid', 'FL_INVOICE', 1, 2, '', 100, 13, 361, 1, 'M~M', 2, null, 'BAS', 1, '', 0, '', '0', 0, 0],
					[108, 2305, 'subject', 'u_yf_igrnc', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 365, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2306, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 6, 365, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2307, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 365, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2308, 'storageid', 'u_yf_igrnc', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 365, 10, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[108, 2309, 'igrnc_status', 'u_yf_igrnc', 1, 15, 'igrnc_status', 'FL_STATUS', 1, 2, '', 100, 4, 365, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[108, 2310, 'vendorid', 'u_yf_igrnc', 1, 10, 'vendorid', 'FL_VENDOR', 1, 2, '', 100, 3, 365, 10, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[108, 2311, 'acceptance_date', 'u_yf_igrnc', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 365, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[108, 2312, 'number', 'u_yf_igrnc', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 366, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2313, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 366, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2314, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 366, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2315, 'sum_total', 'u_yf_igrnc', 1, 7, 'sum_total', 'LBL_TOTAL_PRICE', 1, 2, '', 100, 4, 366, 2, 'NN~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2316, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 367, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2317, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 368, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[108, 2318, 'igrnid', 'u_yf_igrnc', 1, 10, 'igrnid', 'FL_IGRN', 1, 2, '', 100, 8, 365, 1, 'I~M', 1, null, 'BAS', 1, '', 0, '', '0', 0, 0],
					[109, 2319, 'subject', 'u_yf_igdnc', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 369, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[109, 2320, 'storageid', 'u_yf_igdnc', 1, 10, 'storageid', 'FL_STORAGE', 1, 2, '', 100, 2, 369, 10, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[109, 2321, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 7, 369, 1, 'V~M', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2322, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 6, 369, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2323, 'igdnc_status', 'u_yf_igdnc', 1, 15, 'igdnc_status', 'FL_STATUS', 1, 2, '', 100, 4, 369, 1, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[109, 2324, 'accountid', 'u_yf_igdnc', 1, 10, 'accountid', 'FL_ACCOUNT', 1, 2, '', 100, 3, 369, 10, 'V~M', 1, null, 'BAS', 1, '', 1, '', null, 0, 0],
					[109, 2325, 'acceptance_date', 'u_yf_igdnc', 1, 5, 'acceptance_date', 'FL_ACCEPTANCE_DATE', 1, 2, '', 100, 5, 369, 1, 'D~O', 2, 1, 'BAS', 1, '', 1, '', null, 0, 0],
					[109, 2326, 'number', 'u_yf_igdnc', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 370, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2327, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 2, 370, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2328, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 370, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2329, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 371, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2330, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 1, 372, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[109, 2331, 'igdnid', 'u_yf_igdnc', 1, 10, 'igdnid', 'FL_IGDN', 1, 2, '', 100, 8, 369, 1, 'I~M', 1, null, 'BAS', 1, '', 0, '', '0', 0, 0],
					[13, 2332, 'contract_type', 'vtiger_troubletickets', 1, 16, 'contract_type', 'FL_SERVICE_CONTRACTS_TYPE', 1, 2, '', 100, 22, 25, 9, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 2333, 'contracts_end_date', 'vtiger_troubletickets', 1, 5, 'contracts_end_date', 'FL_SERVICE_CONTRACTS_DATE', 1, 2, '', 100, 23, 25, 9, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[14, 2334, 'renewable', 'vtiger_products', 1, 56, 'renewable', 'FL_RENEWABLE', 1, 2, '', 100, 29, 31, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[35, 2335, 'renewable', 'vtiger_service', 1, 56, 'renewable', 'FL_RENEWABLE', 1, 2, '', 100, 23, 91, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[37, 2336, 'renewalinvoice', 'vtiger_assets', 1, 10, 'renewalinvoice', 'FL_RENEWAL_INVOICE', 1, 2, '', 100, 20, 95, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[58, 2337, 'renewalinvoice', 'vtiger_osssoldservices', 1, 10, 'renewalinvoice', 'FL_RENEWAL_INVOICE', 1, 2, '', 100, 12, 141, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[90, 2344, 'ssingleorders_source', 'u_yf_ssingleorders', 1, 16, 'ssingleorders_source', 'FL_SOURCE', 1, 2, '', 100, 13, 284, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[100, 2345, 'ssingleordersid', 'u_yf_igdn', 1, 10, 'ssingleordersid', 'FL_SSIGNLEORDERS', 1, 2, '', 100, 8, 327, 1, 'M~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 2346, 'last_invoice_date', 'vtiger_account', 1, 5, 'last_invoice_date', 'FL_LAST_INVOICE_DATE', 1, 2, '', 100, 10, 198, 10, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 2347, 'report_time', 'vtiger_troubletickets', 1, 308, 'report_time', 'FL_REPORT_TIME', 1, 2, '', 100, 11, 27, 2, 'I~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[13, 2348, 'response_time', 'vtiger_troubletickets', 1, 308, 'response_time', 'FL_RESPONSE_TIME', 1, 2, '', 100, 12, 27, 3, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[4, 2349, 'active', 'vtiger_contactdetails', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 29, 4, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 2350, 'active', 'vtiger_account', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 24, 9, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[7, 2351, 'active', 'vtiger_leaddetails', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 24, 13, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 2352, 'active', 'u_yf_partners', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 6, 299, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[18, 2353, 'active', 'vtiger_vendor', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 21, 42, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[93, 2354, 'active', 'u_yf_competition', 1, 56, 'active', 'FL_ACTIVE', 1, 2, '', 100, 6, 303, 1, 'C~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[83, 2355, 'announcementstatus', 'u_yf_announcement', 1, 15, 'announcementstatus', 'FL_STATUS', 1, 2, 'PLL_DRAFT', 100, 2, 258, 1, 'V~M', 2, 4, 'BAS', 1, '', 1, '', '0', 0, 0],
					[83, 2356, 'interval', 'u_yf_announcement', 1, 7, 'interval', 'FL_INTERVAL', 1, 2, '', 100, 3, 258, 1, 'I~O', 1, null, 'BAS', 1, 'Edit,Detail,QuickCreateAjax', 0, '', null, 0, 0],
					[14, 2357, 'category_multipicklist', 'vtiger_products', 1, 309, 'category_multipicklist', 'LBL_CATEGORY_MULTIPICKLIST', 1, 1, null, 100, 31, 31, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '13', null, 0, 0],
					[89, 2358, 'valid_until', 'u_yf_squotes', 1, 5, 'valid_until', 'FL_VALID_UNTIL', 1, 2, '', 100, 10, 280, 1, 'D~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[92, 2381, 'category', 'u_yf_partners', 1, 302, 'category', 'FL_CATEGORY', 1, 2, '', 100, 7, 299, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '18', null, 0, 0],
					[61, 2382, 'secondary_phone', 'vtiger_ossemployees', 1, 11, 'secondary_phone', 'FL_SECONDARY_PHONE', 1, 2, '', 100, 5, 152, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 2383, 'position', 'vtiger_ossemployees', 1, 1, 'position', 'FL_POSITION', 1, 2, '', 100, 17, 151, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[61, 2384, 'rbh', 'vtiger_ossemployees', 1, 71, 'rbh', 'FL_RBH', 1, 2, '', 100, 18, 151, 1, 'N~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2385, 'campaignid', 'u_yf_ssalesprocesses', 1, 10, 'campaignid', 'FL_CAMPAIGN', 1, 2, '', 100, 15, 269, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 2386, 'ssalesprocessesid', 'vtiger_project', 1, 10, 'ssalesprocessesid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 14, 107, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2387, 'title', 'u_yf_notification', 1, 2, 'title', 'FL_TITLE', 1, 2, '', 100, 1, 374, 1, 'V~M', 1, 1, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2388, 'number', 'u_yf_notification', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 2, 374, 1, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2389, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 374, 1, 'V~M', 1, 3, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2390, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 10, 374, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2391, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 13, 374, 2, 'DT~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2392, 'description', 'vtiger_crmentity', 1, 300, 'description', 'FL_MESSAGE', 1, 2, '', 100, 1, 375, 1, 'V~O', 2, 8, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2393, 'link', 'u_yf_notification', 1, 67, 'link', 'FL_RELATION', 1, 2, '', 100, 6, 374, 1, 'I~O', 2, 5, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2395, 'notification_status', 'u_yf_notification', 1, 16, 'notification_status', 'FL_STATUS', 1, 2, 'PLL_UNREAD', 100, 9, 374, 10, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[111, 2396, 'notification_type', 'u_yf_notification', 1, 15, 'notification_type', 'FL_TYPE', 1, 2, 'PLL_USERS', 100, 5, 374, 10, 'V~O', 2, 4, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2397, 'process', 'u_yf_notification', 1, 66, 'process', 'FL_PROCESS', 1, 2, '', 100, 7, 374, 1, 'I~O', 2, 6, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2398, 'subprocess', 'u_yf_notification', 1, 68, 'subprocess', 'FL_SUB_PROCESS', 1, 2, '', 100, 8, 374, 1, 'I~O', 2, 7, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2399, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 374, 1, 'V~O', 2, 2, 'BAS', 1, '', 0, '', '0', 0, 0],
					[111, 2400, 'smcreatorid', 'vtiger_crmentity', 1, 52, 'smcreatorid', 'Created By', 1, 2, '', 100, 11, 374, 2, 'V~O', 1, null, 'BAS', 1, '', 0, '', null, 0, 0],
					[6, 2401, 'accounts_status', 'vtiger_account', 1, 15, 'accounts_status', 'FL_STATUS', 1, 2, '', 100, 25, 9, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2402, 'finvoice_paymentstatus', 'u_yf_finvoice', 1, 15, 'finvoice_paymentstatus', 'FL_PAYMENT_STATUS', 1, 2, '', 100, 13, 310, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2403, 'finvoice_type', 'u_yf_finvoice', 1, 15, 'finvoice_type', 'FL_INVOICE_TYPE', 1, 2, '', 100, 14, 310, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2404, 'parentid', 'u_yf_ssalesprocesses', 1, 10, 'parentid', 'FL_MEMBER_OF', 1, 2, '', 100, 4, 269, 1, 'I~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[6, 2405, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 26, 9, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[4, 2406, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 30, 4, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[7, 2407, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 25, 13, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[18, 2408, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 22, 42, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[14, 2409, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 32, 31, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[86, 2410, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 14, 269, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[97, 2411, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 316, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[26, 2412, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 18, 74, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 2413, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 25, 39, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[13, 2414, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 24, 25, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[105, 2415, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 351, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 2416, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 24, 91, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[106, 2417, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 10, 355, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[98, 2418, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 321, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[54, 2419, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 21, 134, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[108, 2420, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 365, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[100, 2421, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 327, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[109, 2422, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 369, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[9, 2423, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 27, 19, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[43, 2424, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 107, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[101, 2425, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 329, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[102, 2426, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 331, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[37, 2427, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 21, 95, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[61, 2428, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 19, 151, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[103, 2429, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 333, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[15, 2430, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 13, 37, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[74, 2431, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 12, 232, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[104, 2432, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 10, 347, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[34, 2433, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 18, 89, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[59, 2434, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 32, 144, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[51, 2435, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 128, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[19, 2436, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 11, 46, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[41, 2437, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 12, 101, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[8, 2438, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 13, 17, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[58, 2439, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 13, 141, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[84, 2440, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 261, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[60, 2441, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 13, 147, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[57, 2442, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 17, 138, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[83, 2443, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 4, 258, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[42, 2444, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 104, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[75, 2445, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 236, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[78, 2446, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 4, 247, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[79, 2447, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 250, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[80, 2448, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 252, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[81, 2449, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 19, 254, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[107, 2450, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 14, 361, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[82, 2451, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 19, 256, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[99, 2452, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 323, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[94, 2453, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 5, 307, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[95, 2454, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 15, 310, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[96, 2455, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 11, 314, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[111, 2456, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 14, 374, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[85, 2458, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 265, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[87, 2459, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 272, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[89, 2460, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 11, 280, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[91, 2461, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 13, 288, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[93, 2462, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 303, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[88, 2463, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 9, 276, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[90, 2464, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 14, 284, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[92, 2465, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 8, 299, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[33, 2466, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 18, 88, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[35, 2467, 'taxes', 'vtiger_service', 1, 303, 'taxes', 'FL_TAXES', 1, 2, '', 100, 3, 92, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 2468, 'available', 'vtiger_users', 1, 56, 'available', 'FL_AVAILABLE', 1, 0, '', 3, 24, 79, 1, 'C~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 2469, 'auto_assign', 'vtiger_users', 1, 56, 'auto_assign', 'FL_AUTO_ASSIGN_RECORDS', 1, 0, '', 3, 25, 79, 1, 'C~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[29, 2470, 'records_limit', 'vtiger_users', 1, 7, 'records_limit', 'FL_RECORD_LIMIT_IN_MODULE', 1, 0, '', 3, 23, 79, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2471, 'name', 'u_yf_emailtemplates', 1, 2, 'name', 'FL_NAME', 1, 2, '', 100, 0, 376, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2472, 'number', 'u_yf_emailtemplates', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 3, 378, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2473, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 1, 378, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2474, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 5, 378, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2475, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 6, 378, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2476, 'smcreatorid', 'vtiger_crmentity', 1, 53, 'created_user_id', 'Created By', 1, 2, '', 100, 4, 378, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[112, 2477, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 2, 378, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2478, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 2, '', 100, 7, 378, 1, 'C~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2479, 'email_template_type', 'u_yf_emailtemplates', 1, 16, 'email_template_type', 'FL_TYPE', 1, 2, '', 100, 0, 376, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2480, 'module', 'u_yf_emailtemplates', 1, 301, 'module_name', 'FL_MODULE', 1, 2, '', 100, 0, 376, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2481, 'subject', 'u_yf_emailtemplates', 1, 1, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 0, 377, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2482, 'content', 'u_yf_emailtemplates', 1, 300, 'content', 'FL_CONTENT', 1, 2, '', 100, 0, 377, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[112, 2483, 'email_template_priority', 'u_yf_emailtemplates', 1, 16, 'email_template_priority', 'FL_SMTP_PRIORITY', 1, 2, '', 100, 0, 376, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[86, 2484, 'startdate', 'u_yf_ssalesprocesses', 1, 5, 'startdate', 'FL_START_DATE', 1, 2, '', 100, 11, 269, 1, 'D~O', 2, 5, 'BAS', 1, '', 1, '', null, 0, 0],
					[113, 2485, 'subject', 'u_yf_cfixedassets', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 379, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2486, 'number', 'u_yf_cfixedassets', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 2, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2487, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 3, 379, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2488, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 18, 379, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2489, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 16, 379, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2490, 'smcreatorid', 'vtiger_crmentity', 1, 53, 'created_user_id', 'Created By', 1, 2, '', 100, 17, 379, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[113, 2491, 'fixed_assets_type', 'u_yf_cfixedassets', 1, 16, 'fixed_assets_type', 'FL_TYPE', 1, 2, '', 100, 5, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2492, 'fixed_assets_status', 'u_yf_cfixedassets', 1, 15, 'fixed_assets_status', 'FL_STATUS', 1, 2, '', 100, 6, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2493, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 4, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2494, 'producent_designation', 'u_yf_cfixedassets', 1, 1, 'producent_designation', 'FL_PRODUCENT_DESIGNATION', 1, 2, '', 100, 8, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2495, 'additional_designation', 'u_yf_cfixedassets', 1, 1, 'additional_designation', 'FL_ADDITIONAL_DESIGNATION', 1, 2, '', 100, 9, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2496, 'internal_designation', 'u_yf_cfixedassets', 1, 1, 'internal_designation', 'FL_INTERNAL_DESIGNATION', 1, 2, '', 100, 10, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2497, 'date_production', 'u_yf_cfixedassets', 1, 5, 'date_production', 'FL_DATE_PRODUCTION', 1, 2, '', 100, 11, 379, 1, 'D~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2498, 'date_acquisition', 'u_yf_cfixedassets', 1, 5, 'date_acquisition', 'FL_DATE_ACQUISITION', 1, 2, '', 100, 12, 379, 1, 'D~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2499, 'purchase_price', 'u_yf_cfixedassets', 1, 71, 'purchase_price', 'FL_PURCHASE_PRICE', 1, 2, '', 100, 14, 379, 1, 'N~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2500, 'actual_price', 'u_yf_cfixedassets', 1, 71, 'actual_price', 'FL_ACTUAL_PRICE', 1, 2, '', 100, 13, 379, 1, 'N~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2501, 'reservation', 'u_yf_cfixedassets', 1, 56, 'reservation', 'FL_RESERVATION', 1, 2, '', 100, 15, 379, 1, 'C~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2502, 'pscategory', 'u_yf_cfixedassets', 1, 302, 'pscategory', 'FL_CATEGORY', 1, 2, '', 100, 7, 379, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '19', null, 0, 0],
					[113, 2503, 'fixed_assets_fuel_type', 'u_yf_cfixedassets', 1, 16, 'fixed_assets_fuel_type', 'FL_FUEL_TYPE', 1, 2, '', 100, 0, 380, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2504, 'timing_change', 'u_yf_cfixedassets', 2, 7, 'timing_change', 'FL_TIMING_CHANGE', 1, 2, '', 100, 0, 380, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2505, 'oil_change', 'u_yf_cfixedassets', 1, 7, 'oil_change', 'FL_OIL_TYPE', 1, 2, '', 100, 0, 380, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2506, 'fuel_consumption', 'u_yf_cfixedassets', 1, 7, 'fuel_consumption', 'FL_AVARAGE_FUEL_CONSUPTION', 1, 2, '', 100, 0, 380, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2507, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 0, 381, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2508, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 382, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2509, 'subject', 'u_yf_cinternaltickets', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 0, 383, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2510, 'internal_tickets_status', 'u_yf_cinternaltickets', 1, 15, 'internal_tickets_status', 'FL_STATUS', 1, 2, '', 100, 0, 383, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2511, 'cinternaltickets_no', 'u_yf_cinternaltickets', 1, 4, 'cinternaltickets_no', 'FL_NUMBER', 1, 2, '', 100, 4, 384, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2512, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 1, 384, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2513, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 6, 384, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2514, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 5, 384, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2515, 'smcreatorid', 'vtiger_crmentity', 1, 53, 'created_user_id', 'Created By', 1, 2, '', 100, 3, 384, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[114, 2516, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 2, 384, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2517, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 0, 385, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2518, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 0, 386, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[114, 2519, 'resolution', 'u_yf_cinternaltickets', 1, 300, 'resolution', 'FL_RESOLUTION', 1, 2, '', 100, 0, 387, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2520, 'subject', 'u_yf_finvoicecost', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 388, 1, 'V~M', 1, 0, 'BAS', 1, '', 1, '', null, 0, 0],
					[115, 2521, 'paymentdate', 'u_yf_finvoicecost', 1, 5, 'paymentdate', 'FL_PAYMENT_DATE', 1, 2, '', 100, 2, 388, 1, 'D~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2522, 'saledate', 'u_yf_finvoicecost', 1, 5, 'saledate', 'FL_SALE_DATE', 1, 2, '', 100, 3, 388, 1, 'D~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2523, 'finvoicecost_formpayment', 'u_yf_finvoicecost', 1, 15, 'finvoicecost_formpayment', 'FL_FORM_PAYMENT', 1, 2, '', 100, 4, 388, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2524, 'finvoicecost_status', 'u_yf_finvoicecost', 1, 15, 'finvoicecost_status', 'FL_STATUS', 1, 2, '', 100, 5, 388, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2525, 'finvoicecost_paymentstatus', 'u_yf_finvoicecost', 1, 15, 'finvoicecost_paymentstatus', 'FL_PAYMENT_STATUS', 1, 2, '', 100, 6, 388, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2526, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 7, 388, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[115, 2527, 'pscategory', 'u_yf_finvoicecost', 1, 302, 'pscategory', 'FL_CATEGORY', 1, 2, '', 100, 8, 388, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '20', null, 0, 0],
					[115, 2528, 'number', 'u_yf_finvoicecost', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 1, 389, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2529, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 2, 389, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2530, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 3, 389, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2531, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 4, 389, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2532, 'sum_total', 'u_yf_finvoicecost', 1, 7, 'sum_total', 'FL_TOTAL', 1, 2, '', 100, 5, 389, 2, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2533, 'sum_gross', 'u_yf_finvoicecost', 1, 7, 'sum_gross', 'FL_GROSS', 1, 2, '', 100, 6, 389, 2, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2534, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 389, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2535, 'poboxa', 'u_yf_finvoicecost_address', 1, 1, 'poboxa', 'Po Box', 1, 2, '', 100, 4, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2536, 'localnumbera', 'u_yf_finvoicecost_address', 1, 1, 'localnumbera', 'Local number', 1, 2, '', 100, 2, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2537, 'buildingnumbera', 'u_yf_finvoicecost_address', 1, 1, 'buildingnumbera', 'Building number', 1, 2, '', 100, 3, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2538, 'addresslevel8a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel8a', 'AddressLevel8', 1, 2, '', 100, 1, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2539, 'addresslevel7a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel7a', 'AddressLevel7', 1, 2, '', 100, 5, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2540, 'addresslevel6a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel6a', 'AddressLevel6', 1, 2, '', 100, 6, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2541, 'addresslevel5a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel5a', 'AddressLevel5', 1, 2, '', 100, 7, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2542, 'addresslevel4a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel4a', 'AddressLevel4', 1, 2, '', 100, 8, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2543, 'addresslevel3a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel3a', 'AddressLevel3', 1, 2, '', 100, 9, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2544, 'addresslevel2a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel2a', 'AddressLevel2', 1, 2, '', 100, 10, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2545, 'addresslevel1a', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel1a', 'AddressLevel1', 1, 2, '', 100, 11, 390, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2546, 'addresslevel1c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel1c', 'AddressLevel1', 1, 2, '', 100, 11, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2547, 'addresslevel2c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel2c', 'AddressLevel2', 1, 2, '', 100, 10, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2548, 'addresslevel3c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel3c', 'AddressLevel3', 1, 2, '', 100, 9, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2549, 'addresslevel4c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel4c', 'AddressLevel4', 1, 2, '', 100, 8, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2550, 'addresslevel5c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel5c', 'AddressLevel5', 1, 2, '', 100, 7, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2551, 'addresslevel6c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel6c', 'AddressLevel6', 1, 2, '', 100, 6, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2552, 'addresslevel7c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel7c', 'AddressLevel7', 1, 2, '', 100, 5, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2553, 'addresslevel8c', 'u_yf_finvoicecost_address', 1, 1, 'addresslevel8c', 'AddressLevel8', 1, 2, '', 100, 1, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2554, 'buildingnumberc', 'u_yf_finvoicecost_address', 1, 1, 'buildingnumberc', 'Building number', 1, 2, '', 100, 3, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2555, 'localnumberc', 'u_yf_finvoicecost_address', 1, 1, 'localnumberc', 'Local number', 1, 2, '', 100, 2, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2556, 'poboxc', 'u_yf_finvoicecost_address', 1, 1, 'poboxc', 'Po Box', 1, 2, '', 100, 4, 391, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2557, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 0, 392, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[115, 2558, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 0, 393, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2559, 'subject', 'u_yf_cmileagelogbook', 1, 2, 'subject', 'FL_SUBJECT', 1, 2, '', 100, 1, 394, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2560, 'cmileage_logbook_status', 'u_yf_cmileagelogbook', 1, 15, 'cmileage_logbook_status', 'FL_STATUS', 1, 2, '', 100, 2, 394, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2561, 'number_kilometers', 'u_yf_cmileagelogbook', 2, 7, 'number_kilometers', 'FL_NUMBER_KILOMETERS', 1, 2, '', 100, 3, 394, 1, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2562, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 0, 395, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2563, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 0, 396, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2564, 'number', 'u_yf_cmileagelogbook', 1, 4, 'number', 'FL_NUMBER', 1, 2, '', 100, 3, 397, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2565, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 1, 397, 1, 'V~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2566, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 5, 397, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2567, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 4, 397, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[116, 2568, 'smcreatorid', 'vtiger_crmentity', 1, 53, 'created_user_id', 'Created By', 1, 2, '', 100, 6, 397, 2, 'V~O', 3, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[116, 2569, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 2, 397, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2570, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 7, 311, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2571, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 0, 399, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2572, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 0, 398, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[95, 2573, 'pscategory', 'u_yf_finvoice', 1, 302, 'pscategory', 'FL_CATEGORY', 1, 2, '', 100, 16, 310, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '21', null, 0, 0],
					[113, 2574, 'current_odometer_reading', 'u_yf_cfixedassets', 1, 7, 'current_odometer_reading', 'FL_CURRENT_ODOMETER_READING', 1, 2, '', 100, 0, 380, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2575, 'number_repair', 'u_yf_cfixedassets', 1, 7, 'number_repair', 'FL_NUMBER_REPAIR', 1, 2, '', 100, 0, 380, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[113, 2576, 'date_last_repair', 'u_yf_cfixedassets', 1, 5, 'date_last_repair', 'FL_DATE_OF_LAST_REPAIR', 1, 2, '', 100, 0, 380, 1, 'D~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2577, 'subject', 'u_yf_svendorenquiries', 1, 2, 'subject', 'LBL_SUBJECT', 1, 2, '', 100, 1, 400, 1, 'V~M', 1, 0, 'BAS', 0, '', 1, '', null, 0, 0],
					[117, 2578, 'category', 'u_yf_svendorenquiries', 1, 302, 'category', 'Category', 1, 2, '', 250, 2, 400, 1, 'V~O', 2, 5, 'BAS', 1, '', 1, '22', null, 0, 0],
					[117, 2579, 'salesprocessid', 'u_yf_svendorenquiries', 1, 10, 'salesprocessid', 'SINGLE_SSalesProcesses', 1, 2, '', 100, 3, 400, 1, 'V~M', 2, 2, 'BAS', 1, '', 1, '', null, 0, 0],
					[117, 2580, 'svendorenquiries_status', 'u_yf_svendorenquiries', 1, 16, 'svendorenquiries_status', 'LBL_STATUS', 1, 0, 'PLL_DRAFT', 100, 7, 400, 1, 'V~O', 2, 6, 'BAS', 1, '', 1, '', null, 0, 0],
					[117, 2581, 'accountid', 'u_yf_svendorenquiries', 1, 10, 'accountid', 'SINGLE_Accounts', 1, 2, '', 100, 4, 400, 10, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2582, 'smownerid', 'vtiger_crmentity', 1, 53, 'assigned_user_id', 'Assigned To', 1, 2, '', 100, 8, 400, 1, 'V~M', 1, 0, 'BAS', 1, '', 1, '', null, 0, 0],
					[117, 2583, 'shownerid', 'vtiger_crmentity', 1, 120, 'shownerid', 'Share with users', 1, 2, '', 100, 9, 400, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2584, 'private', 'vtiger_crmentity', 1, 56, 'private', 'FL_IS_PRIVATE', 1, 0, '', 1, 10, 400, 1, 'C~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[117, 2585, 'vendorid', 'u_yf_svendorenquiries', 1, 10, 'vendorid', 'FL_VENDOR', 1, 2, '', 100, 5, 400, 1, 'M~M', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2586, 'scalculationsid', 'u_yf_svendorenquiries', 1, 10, 'scalculationsid', 'FL_SCALCULATIONS', 1, 2, '', 100, 6, 400, 1, 'M~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2587, 'description', 'vtiger_crmentity', 1, 300, 'description', 'Description', 1, 2, '', 100, 1, 401, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2588, 'attention', 'vtiger_crmentity', 1, 300, 'attention', 'Attention', 1, 2, '', 100, 2, 401, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2589, 'response_time', 'u_yf_svendorenquiries', 1, 7, 'response_time', 'LBL_RESPONSE_TIME', 1, 2, '', 100, 1, 402, 10, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2590, 'sum_time', 'u_yf_svendorenquiries', 1, 7, 'sum_time', 'FL_TOTAL_TIME_H', 1, 2, '', 100, 2, 402, 10, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2591, 'sum_total', 'u_yf_svendorenquiries', 1, 7, 'sum_total', 'FL_TOTAL_PRICE', 1, 2, '', 100, 3, 402, 2, 'NN~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2592, 'sum_marginp', 'u_yf_svendorenquiries', 1, 7, 'sum_marginp', 'FL_MARGINP', 1, 2, '', 100, 4, 402, 3, 'N~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2593, 'sum_margin', 'u_yf_svendorenquiries', 1, 7, 'sum_margin', 'FL_MARGIN', 1, 2, '', 100, 5, 402, 3, 'N~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2594, 'createdtime', 'vtiger_crmentity', 1, 70, 'createdtime', 'Created Time', 1, 2, '', 100, 1, 403, 2, 'DT~O', 1, 0, 'BAS', 1, '', 1, '', null, 0, 0],
					[117, 2595, 'svendorenquiries_no', 'u_yf_svendorenquiries', 1, 4, 'svendorenquiries_no', 'LBL_NUMBER', 1, 2, '', 100, 2, 403, 1, 'V~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[117, 2596, 'modifiedtime', 'vtiger_crmentity', 1, 70, 'modifiedtime', 'Modified Time', 1, 2, '', 100, 3, 403, 2, 'DT~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[117, 2597, 'closedtime', 'vtiger_crmentity', 1, 70, 'closedtime', 'LBL_CLOSED_TIME', 1, 2, '', 100, 4, 403, 2, 'DT~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[117, 2598, 'crmactivity', 'vtiger_entity_stats', 1, 1, 'crmactivity', 'LBL_CRMACTIVITY', 1, 0, '', 100, 5, 403, 10, 'NN~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[16, 2600, 'reapeat', 'vtiger_activity', 1, 56, 'reapeat', 'FL_REAPEAT', 1, 2, '', 100, 7, 117, 1, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[16, 2601, 'recurrence', 'vtiger_activity', 1, 342, 'recurrence', 'FL_RECURRENCE', 1, 2, '', 100, 8, 117, 1, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 2602, 'reapeat', 'vtiger_activity', 1, 56, 'reapeat', 'FL_REAPEAT', 1, 2, '', 100, 0, 19, 3, 'I~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[9, 2603, 'recurrence', 'vtiger_activity', 1, 342, 'recurrence', 'FL_RECURRENCE', 1, 2, '', 100, 0, 19, 3, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[43, 2604, 'parentid', 'vtiger_project', 1, 10, 'parentid', 'FL_MEMBER_OF', 1, 2, '', 100, 10, 108, 1, 'I~O', 1, 0, 'BAS', 0, '', 0, '', null, 0, 0],
					[83, 2605, 'is_mandatory', 'u_yf_announcement', 1, 56, 'is_mandatory', 'FL_IS_MANDATORY', 1, 2, '', 100, 5, 258, 1, 'C~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
					[45, 2606, 'smsnotifier_status', 'vtiger_smsnotifier', 1, 16, 'smsnotifier_status', 'FL_STATUS', 1, 2, '', 100, 9, 110, 10, 'V~O', 1, 0, 'BAS', 1, '', 0, '', null, 0, 0],
				]
			],
			'vtiger_field_seq' => [
				'columns' => ['id'],
				'values' => [
					[2606],
				]
			],
			'vtiger_fieldmodulerel' => [
				'columns' => ['fieldid', 'module', 'relmodule', 'status', 'sequence'],
				'values' => [
					[546, 'ServiceContracts', 'Accounts', null, null],
					[580, 'Assets', 'Products', null, null],
					[601, 'ModComments', 'Leads', null, null],
					[601, 'ModComments', 'Contacts', null, null],
					[601, 'ModComments', 'Accounts', null, null],
					[603, 'ModComments', 'ModComments', null, null],
					[606, 'ProjectMilestone', 'Project', null, null],
					[617, 'ProjectTask', 'Project', null, null],
					[601, 'ModComments', 'ProjectTask', null, null],
					[635, 'Project', 'Accounts', null, null],
					[601, 'ModComments', 'Project', null, null],
					[601, 'ModComments', 'HelpDesk', null, null],
					[601, 'ModComments', 'Faq', null, null],
					[691, 'ModComments', 'Contacts', null, null],
					[157, 'HelpDesk', 'Accounts', null, 1],
					[601, 'ModComments', 'OSSTimeControl', null, null],
					[943, 'OSSEmployees', 'OSSEmployees', null, null],
					[601, 'ModComments', 'OSSEmployees', null, null],
					[72, 'Contacts', 'Accounts', null, null],
					[72, 'Contacts', 'Leads', null, null],
					[1044, 'Project', 'ServiceContracts', null, null],
					[1049, 'HelpDesk', 'ServiceContracts', null, null],
					[1314, 'Assets', 'Accounts', null, null],
					[1314, 'Assets', 'Leads', null, null],
					[1315, 'OutsourcedProducts', 'Accounts', null, 1],
					[1315, 'OutsourcedProducts', 'Leads', null, 2],
					[1316, 'OSSOutsourcedServices', 'Accounts', null, null],
					[1316, 'OSSOutsourcedServices', 'Leads', null, null],
					[1317, 'OSSSoldServices', 'Accounts', null, null],
					[1317, 'OSSSoldServices', 'Leads', null, null],
					[1318, 'ProjectTask', 'ProjectTask', null, null],
					[1319, 'ProjectTask', 'ProjectMilestone', null, null],
					[1344, 'OSSSoldServices', 'Services', null, null],
					[159, 'HelpDesk', 'Products', null, 1],
					[159, 'HelpDesk', 'Services', null, 2],
					[1482, 'HelpDesk', 'Assets', null, null],
					[1482, 'HelpDesk', 'OSSSoldServices', null, null],
					[1521, 'CallHistory', 'Contacts', null, null],
					[1521, 'CallHistory', 'Accounts', null, null],
					[1521, 'CallHistory', 'Leads', null, null],
					[1521, 'CallHistory', 'Vendors', null, null],
					[1521, 'CallHistory', 'OSSEmployees', null, null],
					[1522, 'CallHistory', 'HelpDesk', null, null],
					[601, 'ModComments', 'Ideas', null, null],
					[72, 'Contacts', 'Vendors', null, null],
					[1607, 'HolidaysEntitlement', 'OSSEmployees', null, null],
					[1622, 'PaymentsIn', 'Accounts', null, null],
					[1638, 'PaymentsOut', 'Accounts', null, null],
					[1640, 'PaymentsOut', 'PaymentsOut', null, null],
					[1655, 'LettersIn', 'Accounts', null, null],
					[1655, 'LettersIn', 'Leads', null, null],
					[1660, 'LettersIn', 'LettersIn', null, null],
					[601, 'ModComments', 'LettersIn', null, null],
					[1677, 'LettersOut', 'Accounts', null, null],
					[1677, 'LettersOut', 'Leads', null, null],
					[1682, 'LettersOut', 'LettersOut', null, null],
					[601, 'ModComments', 'LettersOut', null, null],
					[1730, 'Reservations', 'Accounts', null, null],
					[1730, 'Reservations', 'Leads', null, null],
					[1730, 'Reservations', 'Vendors', null, null],
					[1731, 'Reservations', 'Project', null, null],
					[1731, 'Reservations', 'HelpDesk', null, null],
					[601, 'ModComments', 'Reservations', null, null],
					[1677, 'LettersOut', 'Vendors', null, null],
					[1655, 'LettersIn', 'Vendors', null, null],
					[1677, 'LettersOut', 'OSSEmployees', null, null],
					[1655, 'LettersIn', 'OSSEmployees', null, null],
					[157, 'HelpDesk', 'Vendors', null, null],
					[635, 'Project', 'Vendors', null, null],
					[1761, 'Events', 'Calendar', null, null],
					[1762, 'Calendar', 'Calendar', null, null],
					[1767, 'SQuoteEnquiries', 'SSalesProcesses', null, null],
					[1769, 'SQuoteEnquiries', 'Accounts', null, null],
					[601, 'ModComments', 'SQuoteEnquiries', null, null],
					[1781, 'SSalesProcesses', 'Accounts', null, null],
					[601, 'ModComments', 'SSalesProcesses', null, null],
					[1791, 'SRequirementsCards', 'SSalesProcesses', null, null],
					[1792, 'SRequirementsCards', 'SQuoteEnquiries', null, null],
					[1794, 'SRequirementsCards', 'Accounts', null, null],
					[601, 'ModComments', 'SRequirementsCards', null, null],
					[1806, 'SCalculations', 'SSalesProcesses', null, null],
					[1807, 'SCalculations', 'SRequirementsCards', null, null],
					[1809, 'SCalculations', 'Accounts', null, null],
					[601, 'ModComments', 'SCalculations', null, null],
					[1821, 'SQuotes', 'SSalesProcesses', null, null],
					[1822, 'SQuotes', 'SCalculations', null, null],
					[1824, 'SQuotes', 'Accounts', null, null],
					[601, 'ModComments', 'SQuotes', null, null],
					[1836, 'SSingleOrders', 'SSalesProcesses', null, null],
					[1837, 'SSingleOrders', 'SQuotes', null, null],
					[1839, 'SSingleOrders', 'Accounts', null, null],
					[601, 'ModComments', 'SSingleOrders', null, null],
					[1854, 'SRecurringOrders', 'SSalesProcesses', null, null],
					[1855, 'SRecurringOrders', 'SQuotes', null, null],
					[1857, 'SRecurringOrders', 'Accounts', null, null],
					[601, 'ModComments', 'SRecurringOrders', null, null],
					[1638, 'PaymentsOut', 'Vendors', null, null],
					[1622, 'PaymentsIn', 'Vendors', null, null],
					[601, 'ModComments', 'Partners', null, null],
					[601, 'ModComments', 'Competition', null, null],
					[1988, 'OutsourcedProducts', 'SSalesProcesses', null, null],
					[1989, 'Assets', 'SSalesProcesses', null, null],
					[1990, 'OSSOutsourcedServices', 'SSalesProcesses', null, null],
					[1991, 'OSSSoldServices', 'SSalesProcesses', null, null],
					[601, 'ModComments', 'Vendors', null, null],
					[72, 'Contacts', 'Partners', null, null],
					[72, 'Contacts', 'Competition', null, null],
					[2038, 'FBookkeeping', 'Accounts', null, null],
					[2043, 'FInvoice', 'Accounts', null, null],
					[601, 'ModComments', 'FInvoice', null, null],
					[2084, 'IStorages', 'IStorages', null, null],
					[2125, 'FInvoiceProforma', 'Accounts', null, null],
					[601, 'ModComments', 'FInvoiceProforma', null, null],
					[2157, 'IGDN', 'IStorages', null, null],
					[601, 'ModComments', 'IGDN', null, null],
					[2164, 'IIDN', 'IStorages', null, null],
					[2170, 'IGRN', 'IStorages', null, null],
					[601, 'ModComments', 'IGRN', null, null],
					[601, 'ModComments', 'IIDN', null, null],
					[2172, 'IGIN', 'IStorages', null, null],
					[601, 'ModComments', 'IGIN', null, null],
					[2183, 'IPreOrder', 'IStorages', null, null],
					[601, 'ModComments', 'IPreOrder', null, null],
					[2191, 'IGRN', 'Vendors', null, null],
					[2195, 'IGDN', 'Accounts', null, null],
					[601, 'ModComments', 'IStorages', null, null],
					[2206, 'IPreOrder', 'Accounts', null, null],
					[601, 'ModComments', 'FBookkeeping', null, null],
					[2225, 'ISTDN', 'IStorages', null, null],
					[2227, 'ISTDN', 'Accounts', null, null],
					[601, 'ModComments', 'ISTDN', null, null],
					[601, 'ModComments', 'ISTN', null, null],
					[2249, 'ISTRN', 'IStorages', null, null],
					[2251, 'ISTRN', 'Vendors', null, null],
					[601, 'ModComments', 'ISTRN', null, null],
					[2259, 'ISTDN', 'ISTN', null, null],
					[2260, 'ISTRN', 'ISTN', null, null],
					[2261, 'ISTDN', 'ISTRN', null, null],
					[2262, 'ISTRN', 'ISTDN', null, null],
					[601, 'ModComments', 'KnowledgeBase', null, null],
					[2273, 'FCorectingInvoice', 'Accounts', null, null],
					[2304, 'FCorectingInvoice', 'FInvoice', null, null],
					[601, 'ModComments', 'FCorectingInvoice', null, null],
					[2308, 'IGRNC', 'IStorages', null, null],
					[2310, 'IGRNC', 'Vendors', null, null],
					[2318, 'IGRNC', 'IGRN', null, null],
					[2320, 'IGDNC', 'IStorages', null, null],
					[2324, 'IGDNC', 'Accounts', null, null],
					[2331, 'IGDNC', 'IGDN', null, null],
					[601, 'ModComments', 'IGDNC', null, null],
					[601, 'ModComments', 'IGRNC', null, null],
					[278, 'Faq', 'Products', null, null],
					[2336, 'Assets', 'FInvoice', null, null],
					[2337, 'OSSSoldServices', 'FInvoice', null, null],
					[2345, 'IGDN', 'SSingleOrders', null, null],
					[601, 'ModComments', 'Campaigns', null, null],
					[2385, 'SSalesProcesses', 'Campaigns', null, 0],
					[2386, 'Project', 'SSalesProcesses', null, 0],
					[2404, 'SSalesProcesses', 'SSalesProcesses', null, 0],
					[601, 'ModComments', 'CFixedAssets', null, 0],
					[601, 'ModComments', 'CInternalTickets', null, 0],
					[601, 'ModComments', 'FInvoiceCost', null, 0],
					[2579, 'SVendorEnquiries', 'SSalesProcesses', null, 0],
					[2581, 'SVendorEnquiries', 'Accounts', null, 0],
					[2585, 'SVendorEnquiries', 'Vendors', null, 0],
					[2586, 'SVendorEnquiries', 'SCalculations', null, 0],
					[601, 'ModComments', 'SVendorEnquiries', null, 0],
					[2604, 'Project', 'Project', null, 0],
				]
			],
			'vtiger_finvoice_formpayment' => [
				'columns' => ['finvoice_formpaymentid', 'finvoice_formpayment', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_TRANSFER', 0, 666, 1],
					[2, 'PLL_CASH', 0, 667, 2],
					[3, 'PLL_WIRE_TRANSFER', 0, 713, 3],
				]
			],
			'vtiger_finvoice_paymentstatus' => [
				'columns' => ['finvoice_paymentstatusid', 'finvoice_paymentstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_AWAITING_PAYMENT', 0, 714, 1],
					[2, 'PLL_PARTIALLY_PAID', 0, 715, 2],
					[3, 'PLL_FULLY_PAID', 0, 716, 3],
				]
			],
			'vtiger_finvoice_status' => [
				'columns' => ['finvoice_statusid', 'finvoice_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_AWAITING_REALIZATION', 1, 668, 2],
					[2, 'PLL_UNASSIGNED', 1, 766, 1],
					[3, 'PLL_FOR_PROCESSING', 1, 767, 3],
					[4, 'PLL_IN_PROGRESSING', 1, 768, 4],
					[5, 'PLL_SUBMITTED_COMMENTS', 1, 769, 5],
					[6, 'PLL_FOR_APPROVAL', 1, 770, 6],
					[7, 'PLL_CANCELLED', 1, 771, 7],
					[8, 'PLL_ACCEPTED', 1, 772, 8],
				]
			],
			'vtiger_finvoice_type' => [
				'columns' => ['finvoice_typeid', 'finvoice_type', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DOMESTIC_INVOICE', 0, 717, 1],
					[2, 'PLL_FOREIGN_INVOICE', 0, 718, 2],
					[3, 'PLL_IC_INVOICE', 0, 719, 3],
				]
			],
			'vtiger_finvoicecost_formpayment' => [
				'columns' => ['finvoicecost_formpaymentid', 'finvoicecost_formpayment', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_TRANSFER', 1, 746, 1],
					[2, 'PLL_CASH', 1, 747, 2],
					[3, 'PLL_WIRE_TRANSFER', 1, 748, 3],
				]
			],
			'vtiger_finvoicecost_paymentstatus' => [
				'columns' => ['finvoicecost_paymentstatusid', 'finvoicecost_paymentstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_AWAITING_PAYMENT', 1, 757, 1],
					[2, 'PLL_PARTIALLY_PAID', 1, 758, 2],
					[3, 'PLL_FULLY_PAID', 1, 759, 3],
				]
			],
			'vtiger_finvoicecost_status' => [
				'columns' => ['finvoicecost_statusid', 'finvoicecost_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_UNASSIGNED', 1, 749, 1],
					[2, 'PLL_AWAITING_REALIZATION', 1, 750, 2],
					[3, 'PLL_FOR_PROCESSING', 1, 751, 3],
					[4, 'PLL_IN_PROGRESSING', 1, 752, 4],
					[5, 'PLL_SUBMITTED_COMMENTS', 1, 753, 5],
					[6, 'PLL_FOR_APPROVAL', 1, 754, 6],
					[7, 'PLL_CANCELLED', 1, 755, 7],
					[8, 'PLL_ACCEPTED', 1, 756, 8],
				]
			],
			'vtiger_finvoiceproforma_formpayment' => [
				'columns' => ['finvoiceproforma_formpaymentid', 'finvoiceproforma_formpayment', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_TRANSFER', 1, 669, 1],
					[2, 'PLL_CASH', 1, 671, 2],
				]
			],
			'vtiger_finvoiceproforma_status' => [
				'columns' => ['finvoiceproforma_statusid', 'finvoiceproforma_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'None', 1, 670, 1],
				]
			],
			'vtiger_fixed_assets_fuel_type' => [
				'columns' => ['fixed_assets_fuel_typeid', 'fixed_assets_fuel_type', 'presence', 'sortorderid'],
				'values' => [
					[1, 'PLL_DIESEL', 1, 1],
				]
			],
			'vtiger_fixed_assets_status' => [
				'columns' => ['fixed_assets_statusid', 'fixed_assets_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_UNAVAILABLE', 1, 728, 1],
					[2, 'PLL_AVAILABLE', 1, 729, 2],
					[3, 'PLL_DEMAGED', 1, 730, 3],
					[4, 'PLL_IN_USE', 1, 731, 4],
					[5, 'PLL_UNDER_REPAIR', 1, 732, 5],
					[6, 'PLL_PENDING_FOR_REPROCESSING', 1, 733, 6],
					[7, 'PLL_PENDING_FOR_SALE', 1, 734, 7],
					[8, 'PLL_REPROCESSED', 1, 735, 8],
					[9, 'PLL_SOLD', 1, 736, 9],
				]
			],
			'vtiger_fixed_assets_type' => [
				'columns' => ['fixed_assets_typeid', 'fixed_assets_type', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DEVICE', 1, 723, 1],
					[2, 'PLL_MACHINE', 1, 724, 2],
					[3, 'PLL_VEHICLE', 1, 725, 3],
					[4, 'PLL_PROPERTY', 1, 726, 4],
					[5, 'PLL_INVENTORY', 1, 727, 5],
				]
			],
		];
	}
}
